#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include <iptables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_ROUTEMARK.h>

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"ROUTEMARK target v%s options:\n"
"  --set-wan value                    Set wan value\n"
"  --set-mark value                   Set mark value\n"
"  --set-vpn value		      Set vpn value\n"
"\n",
NETFILTER_VERSION);
}

static struct option opts[] = {
	{ "set-wan", 1, 0, '1' },
	{ "set-mark", 1, 0, '2' },
/* 2005/2/14 Steve ++ */
	{ "set-vpn", 1, 0, '3' },
/* Steve -- */
	{ 0 }
};

/* Initialize the target. */
static void
init(struct ipt_entry_target *t, unsigned int *nfcache)
{
}

static int
parse_to(char *arg, struct ipt_routemark_target_info *info)
{
	char *dot, *buf, *end;
	int i = 0;

	buf = arg;
	while (1) {
		dot = strchr(buf, ',');
		if (dot) {
			*dot = '\0';
			dot++;
		}

		info->element[i].id = strtoul(buf, &end, 0);
		if (*end != '\0' || end == buf)
			return -1;
		info->target_num++;
		if (dot) 
			buf = dot;
		else
			return 0;

		i++;
	}
}	

/* 2005/2/14 Steve ++ */
static int
parse_tovpn(char *arg, struct ipt_routemark_target_info *info)
{
	char *dot, *buf, *end, *name;
	int i = 0;

	buf = arg;
	while (1) {
		dot = strchr(buf, ',');
		if (dot) {
			*dot = '\0';
			dot++;
		}

		name = strchr(buf, ':');
		if (!name)
			return -1;
		*name = '\0';
		name++;
		strcpy(info->element[i].vpnname, name);

		info->element[i].id = strtoul(buf, &end, 0);
		if (*end != '\0' || end == buf)
			return -1;
		info->target_num++;
		if (dot) 
			buf = dot;
		else
			return 0;

		i++;
	}
}	
/* Steve -- */

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      const struct ipt_entry *entry,
      struct ipt_entry_target **target)
{
	struct ipt_routemark_target_info *markinfo
		= (struct ipt_routemark_target_info *)(*target)->data;

	switch (c) {
		char *end;
	case '1':
		if (*flags)
			exit_error(PARAMETER_PROBLEM,
				   "ROUTEMARK target: Can't specify --set-wan twice");

		markinfo->how = IPT_ROUTEMARK_SET_WAN;
		if (parse_to(optarg, markinfo) < 0)
			exit_error(PARAMETER_PROBLEM, "Bad ROUTEMARK value `%s'", optarg);
		*flags = 1;
                break;
/* 2004/5/12 Steve ++ */
	case '2':
		if (*flags)
			exit_error(PARAMETER_PROBLEM,
				   "ROUTEMARK target: Can't specify --set-mark twice");

		markinfo->how = IPT_ROUTEMARK_SET_MARK;
		markinfo->mark = strtoul(optarg, &end, 0);
                if (*end != '\0' || end == optarg)
		        exit_error(PARAMETER_PROBLEM, "Bad MARK value `%s'", optarg);

		*flags = 1;
		break;
/* Steve -- */
/* 2005/2/14 Steve ++ */
	case '3':
		if (*flags)
			exit_error(PARAMETER_PROBLEM,
				   "ROUTEMARK target: Can't specify --set-vpn twice");

		markinfo->how = IPT_ROUTEMARK_SET_VPN;
		if (parse_tovpn(optarg, markinfo) < 0)
			exit_error(PARAMETER_PROBLEM, "Bad ROUTEMARK value `%s'", optarg);
		*flags = 1;
                break;
/* Steve -- */
	default:
		return 0;
	}

	return 1;
}

static void
final_check(unsigned int flags)
{
	if (!flags)
		exit_error(PARAMETER_PROBLEM,
		           "ROUTEMARK target: Parameter --set-wan is required");
}

static void
print_data(const struct ipt_routemark_target_info *info)
{
	int i = 0;

	for (i = 0; i < info->target_num; i++)
		printf("%d ", info->element[i].id);
}

/* 2005/2/14 Steve ++ */
static void
print_vpndata(const struct ipt_routemark_target_info *info)
{
	int i = 0;

	for (i = 0; i < info->target_num; i++)
		printf("%s:%d ",
			info->element[i].vpnname,
			info->element[i].id);

}
/* Steve -- */

static void
print_mark(unsigned long mark, int numeric)
{
        printf("0x%lx ", mark);
}

/* Prints out the targinfo. */
static void
print(const struct ipt_ip *ip,
      const struct ipt_entry_target *target,
      int numeric)
{
	const struct ipt_routemark_target_info *markinfo =
		(const struct ipt_routemark_target_info *)target->data;
	
	switch (markinfo->how) {
	case IPT_ROUTEMARK_SET_WAN:
		printf("ROUTEMARK wan ");
		print_data(markinfo);
		break;
	case IPT_ROUTEMARK_SET_MARK:
		printf("ROUTEMARK mark ");
		print_mark(markinfo->mark, numeric);
		break;
/* 2005/2/14 Steve ++ */
	case IPT_ROUTEMARK_SET_VPN:
		printf("ROUTEMARK vpn ");
		print_vpndata(markinfo);
		break;
/* Steve -- */
        default:
            printf("ERROR: UNKNOWN CONNMARK MODE ");
            break;
	}		
}

/* Saves the union ipt_targinfo in parsable form to stdout. */
static void
save(const struct ipt_ip *ip, const struct ipt_entry_target *target)
{
	const struct ipt_routemark_target_info *markinfo =
		(const struct ipt_routemark_target_info *)target->data;

/* 2002/1/2 Steve ++ */
        switch (markinfo->how) {
	case IPT_ROUTEMARK_SET_WAN:
		printf("--set-wan %d ", markinfo->element[0].id);
		break;
	case IPT_ROUTEMARK_SET_MARK:
		printf("--set-mark 0x%lx ", markinfo->mark);
		break;
/* 2005/2/14 Steve ++ */
	case IPT_ROUTEMARK_SET_VPN:
		printf("--set-vpn %s:%d ",
		markinfo->element[0].name,
		markinfo->element[0].id);
		break;
/* Steve -- */
        default:
            printf("ERROR: UNKNOWN CONNMARK MODE ");
            break;
        }
/* Steve -- */
}

static
struct iptables_target routemark
= { NULL,
    "ROUTEMARK",
    NETFILTER_VERSION,
    IPT_ALIGN(sizeof(struct ipt_routemark_target_info)),
    IPT_ALIGN(sizeof(struct ipt_routemark_target_info)),
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    &save,
    opts
};

void _init(void)
{
	register_target(&routemark);
}
