/* Shared library add-on to iptables to add TCPMSS target support.
 *
 * Copyright (c) 2000 Marc Boucher
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>

#include <iptables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_TCPWINDOW.h>

struct windowinfo {
	struct ipt_entry_target t;
	struct ipt_tcpwindow_info window;
};

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"TCPWINDOW target v%s mutually-exclusive options:\n"
"  --set-window value               explicitly set WINDOW option to specified value\n",
NETFILTER_VERSION);
}

static struct option opts[] = {
	{ "set-window", 1, 0, '1' },
	{ 0 }
};

/* Initialize the target. */
static void
init(struct ipt_entry_target *t, unsigned int *nfcache)
{
}

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      const struct ipt_entry *entry,
      struct ipt_entry_target **target)
{
	struct ipt_tcpwindow_info *windowinfo
		= (struct ipt_tcpwindow_info *)(*target)->data;

	switch (c) {
		unsigned int mssval;

	case '1':
		if (*flags)
			exit_error(PARAMETER_PROBLEM,
			           "TCPWINDOW target: Only one option may be specified");
		if (string_to_number(optarg, 0, 65535, &mssval) == -1)
			exit_error(PARAMETER_PROBLEM, "Bad TCPWINDOW value `%s'", optarg);
		
		windowinfo->window = mssval;
		*flags = 1;
		break;

	default:
		return 0;
	}

	return 1;
}

static void
final_check(unsigned int flags)
{
	if (!flags)
		exit_error(PARAMETER_PROBLEM,
		           "TCPWINDOW target: At least one parameter is required");
}

/* Prints out the targinfo. */
static void
print(const struct ipt_ip *ip,
      const struct ipt_entry_target *target,
      int numeric)
{
	const struct ipt_tcpwindow_info *windowinfo =
		(const struct ipt_tcpwindow_info *)target->data;
	printf("TCPWINDOW set %u ", windowinfo->window);
}

/* Saves the union ipt_targinfo in parsable form to stdout. */
static void
save(const struct ipt_ip *ip, const struct ipt_entry_target *target)
{
	const struct ipt_tcpwindow_info *windowinfo =
		(const struct ipt_tcpwindow_info *)target->data;

	printf("--set-window %u ", windowinfo->window);
}

static
struct iptables_target window
= { NULL,
    "TCPWINDOW",
    NETFILTER_VERSION,
    IPT_ALIGN(sizeof(struct ipt_tcpwindow_info)),
    IPT_ALIGN(sizeof(struct ipt_tcpwindow_info)),
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    &save,
    opts
};

void _init(void)
{
	register_target(&window);
}
