#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <iptables.h>

#include<linux/netfilter_ipv4/ip_tables.h>
#include<linux/netfilter_ipv4/ipt_pElement.h>


static void help(void)
{
  printf("pElement match v%s options:\n"
  "--inaddr name	Internal Address Name\n"
  "--exaddr name	External Address Name\n"
  "--dmzaddr name	DMZ Address Name\n"
  "--service name	Service Name\n"
  "--schedule name	Schedule Name\n"
  "--qos name		QoS Name\n"
  "--vserver name	Vserver Name\n"
  "--authuser name      AuthUser Name\n"
  "\n", NETFILTER_VERSION);
}

static struct option opts[] = { { "inaddr", 1, 0, '1' }, 
				{ "exaddr", 1, 0, '2' },
				{ "dmzaddr", 1, 0, '3' },
				{ "service", 1, 0, '4' },
				{ "schedule", 1, 0, '5' },
				{ "qos", 1, 0, '6' },
				{ "vserver", 1, 0, '7' },
				{ "authuser", 1, 0, '8' },
				{ 0 } };

static void
init(struct ipt_entry_match *m, unsigned int *nfcache)
{
  *nfcache |= NFC_UNKNOWN;
}

static int
parse(int c,
      char **argv,
      int invert,
      unsigned int *flags,
      const struct ipt_entry *entry,
      unsigned int *nfcache,
      struct ipt_entry_match **match)
{
	struct pElement_info *info = (struct pElement_info*)(*match)->data;

	switch (c) {
	case '1':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "unexpected '!' with --inaddr");
		if (*flags & INTERNAL_ADDRESS_FLAG)
			exit_error(PARAMETER_PROBLEM,
				   "Can't specify --inaddr twice");
		*flags |= INTERNAL_ADDRESS_FLAG;
		info->flag |= INTERNAL_ADDRESS_FLAG;
		if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
			strcpy(info->inaddr_name, argv[optind-1]);
		else
			exit_error(PARAMETER_PROBLEM, "Internal Address name too long");

		break;
	case '2':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "unexpected '!' with --exaddr");
		if (*flags & EXTERNAL_ADDRESS_FLAG)
			exit_error(PARAMETER_PROBLEM,
				   "Can't specify --exaddr twice");
		*flags |= EXTERNAL_ADDRESS_FLAG;
		info->flag |= EXTERNAL_ADDRESS_FLAG;
		if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
			strcpy(info->exaddr_name, argv[optind-1]);
		else
			exit_error(PARAMETER_PROBLEM, "External Address name too long");

		break;
	case '3':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "unexpected '!' with --dmzaddr");
		if (*flags & DMZ_ADDRESS_FLAG)
			exit_error(PARAMETER_PROBLEM,
				   "Can't specify --dmzaddr twice");
		*flags |= DMZ_ADDRESS_FLAG;
		info->flag |= DMZ_ADDRESS_FLAG;
		if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
			strcpy(info->dmzaddr_name, argv[optind-1]);
		else
			exit_error(PARAMETER_PROBLEM, "DMZ Address name too long");

		break;
	case '4':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "unexpected '!' with --service");
		if (*flags & SERVICE_FLAG)
			exit_error(PARAMETER_PROBLEM,
				   "Can't specify --service twice");
		*flags |= SERVICE_FLAG;
		info->flag |= SERVICE_FLAG;
		if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
			strcpy(info->service_name, argv[optind-1]);
		else
			exit_error(PARAMETER_PROBLEM, "Service name too long");

		break;
	case '5':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "unexpected '!' with --schedule");
		if (*flags & SCHEDULE_FLAG)
			exit_error(PARAMETER_PROBLEM,
				   "Can't specify --schedule twice");
		*flags |= SCHEDULE_FLAG;
		info->flag |= SCHEDULE_FLAG;
		if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
			strcpy(info->schedule_name, argv[optind-1]);
		else
			exit_error(PARAMETER_PROBLEM, "Schedule name too long");

		break;
	case '6':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "unexpected '!' with --qos");
		if (*flags & QOS_FLAG)
			exit_error(PARAMETER_PROBLEM,
				   "Can't specify --qos twice");
		*flags |= QOS_FLAG;
		info->flag |= QOS_FLAG;
		if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
			strcpy(info->qos_name, argv[optind-1]);
		else
			exit_error(PARAMETER_PROBLEM, "QoS name too long");

		break;
	case '7':
		if (invert)
			exit_error(PARAMETER_PROBLEM,
				   "unexpected '!' with --vserver");
		if (*flags & VSERVER_FLAG)
			exit_error(PARAMETER_PROBLEM,
				   "Can't specify --vserver twice");
		*flags |= VSERVER_FLAG;
		info->flag |= VSERVER_FLAG;
		if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
			strcpy(info->vserver_name, argv[optind-1]);
		else
			exit_error(PARAMETER_PROBLEM, "vserver name too long");

		break;

        case '8':
                if (invert)
                        exit_error(PARAMETER_PROBLEM,
                                   "unexpected '!' with --authuser");
                if (*flags & AUTHUSER_FLAG)
                        exit_error(PARAMETER_PROBLEM,
                                   "Can't specify --authuser twice");
                *flags |= AUTHUSER_FLAG;
                info->flag |= AUTHUSER_FLAG;
                if (strlen(argv[optind-1]) < PELEMENT_NAME_LEN)
                        strcpy(info->authuser_name, argv[optind-1]);
                else
                        exit_error(PARAMETER_PROBLEM, "authuser name too long");
                break;

        default:
                return 0;
	}
	return 1;
}


static void
final_check(unsigned int flags)
{
	if(!flags)
		exit_error(PARAMETER_PROBLEM, "pElement match: must specify options");
}


static void
print(const struct ipt_ip *ip,
      const struct ipt_entry_match *match,
      int numeric)
{
	const struct pElement_info *info = (const struct pElement_info *)match->data;

	if (info->flag & INTERNAL_ADDRESS_FLAG)
		printf("--inaddr %s ", info->inaddr_name);
	if (info->flag & EXTERNAL_ADDRESS_FLAG)
		printf("--exaddr %s ", info->exaddr_name);
	if (info->flag & DMZ_ADDRESS_FLAG)
		printf("--dmzaddr %s ", info->dmzaddr_name);
	if (info->flag & SERVICE_FLAG)
		printf("--service %s ", info->service_name);
	if (info->flag & SCHEDULE_FLAG)
		printf("--schedule %s ", info->schedule_name);
	if (info->flag & QOS_FLAG)
		printf("--qos %s ", info->qos_name);
	if (info->flag & VSERVER_FLAG)
		printf("--vserver %s ", info->vserver_name);
        if (info->flag & AUTHUSER_FLAG)
                printf("--authuser %s ", info->authuser_name);
}


static void
save(const struct ipt_ip *ip,
     const struct ipt_entry_match *match)
{
	const struct pElement_info *info = (const struct pElement_info*)match->data;

	if (info->flag & INTERNAL_ADDRESS_FLAG)
		printf("--inaddr %s ", info->inaddr_name);
	if (info->flag & EXTERNAL_ADDRESS_FLAG)
		printf("--exaddr %s ", info->exaddr_name);
	if (info->flag & DMZ_ADDRESS_FLAG)
		printf("--dmzaddr %s ", info->dmzaddr_name);
	if (info->flag & SERVICE_FLAG)
		printf("--service %s ", info->service_name);
	if (info->flag & SCHEDULE_FLAG)
		printf("--schedule %s ", info->schedule_name);
	if (info->flag & QOS_FLAG)
		printf("--qos %s ", info->qos_name);
	if (info->flag & VSERVER_FLAG)
		printf("--vserver %s ", info->vserver_name);
        if (info->flag & AUTHUSER_FLAG)
                printf("--authuser %s ", info->authuser_name);
}


static struct iptables_match pElement = {
	NULL,
	"pElement",
	NETFILTER_VERSION,
	IPT_ALIGN(sizeof(struct pElement_info)),
	IPT_ALIGN(sizeof(struct pElement_info)),
	&help,
	&init,
	&parse,
	&final_check,
	&print,
	&save,
	opts
};


void _init(void)
{
  register_match(&pElement);
}
