/*
 * Author: Paul.Russell@rustcorp.com.au and mneuling@radlogic.com.au
 *
 * Based on the ipchains code by Paul Russell and Michael Neuling
 *
 *	iptables -- IP firewall administration for kernels with
 *	firewall table (aimed for the 2.3 kernels)
 *
 *	See the accompanying manual page iptables(8) for information
 *	about proper usage of this program.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <iptables.h>
/* 2004/3/11 Steve ++ */
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include "pidof.h"
/* Steve -- */

int
main(int argc, char *argv[])
{
	int ret;
	char *table = "filter";
	iptc_handle_t handle = NULL;
/* 2004/3/11 Steve ++ */
        struct progpid *pid, *tmp;
        pid_t mypid = getpid();
        struct timeval tv;
        int doing = 0;
/* Steve -- */

	program_name = "iptables";
	program_version = NETFILTER_VERSION;
/* 2004/3/11 Steve ++ */
        while (1) {
                doing = 0;
                if ((pid = pidof("iptables"))) {
                        for (tmp = pid; tmp; tmp = tmp->next) {
                                if (mypid > tmp->id)
                                        doing = 1;
                        }

                        freepid();
                }

                if (!doing)
                        break;

                tv.tv_sec = 0;
                tv.tv_usec = 150000;
                select(0, NULL, NULL, NULL, &tv);
        }
/* Steve -- */

#ifdef NO_SHARED_LIBS
	init_extensions();
#endif

	ret = do_command(argc, argv, &table, &handle);
	if (ret)
		ret = iptc_commit(&handle);

	if (!ret)
		fprintf(stderr, "iptables: %s\n",
			iptc_strerror(errno));

	exit(!ret);
}
