#include <errno.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>

#include "clamav.h"
char * safe_strncpy(char *dst, const char *src, size_t size)
{   
	dst[size-1] = '\0';
	return strncpy(dst, src, size-1);   
}

char * xstrdup (const char *s) {
	char *t;

	if (s == NULL)
		return NULL;

	t = strdup (s);

	return t;
}

char * xstrndup (const char *s, int n) {
	char *t;

	if (s == NULL)
		return NULL;

	t = malloc(++n);
	
	return safe_strncpy(t,s,n);
}

char * dirname(const char *path)
{
	const char *s;

	/* Go to the end of the string.  */
	s = path + strlen(path) - 1;

	/* Strip off trailing /s (unless it is also the leading /).  */
	while (path < s && s[0] == '/')
		s--;

	/* Strip the last component.  */
	while (path <= s && s[0] != '/')
		s--;

	while (path < s && s[0] == '/')
		s--;

	if (s < path)
		return xstrdup (".");
	else
		return xstrndup (path, s - path + 1);
}

int makedir (char *path, long mode, int flags)
{
	if (!(flags & FILEUTILS_RECUR)) {
#undef	mkdir()	
		if (mkdir (path, 0777) < 0) {
			cli_errmsg("Cannot create directory `%s'", path);
			return -1;
		}

		if (mode != -1 && chmod (path, mode) < 0) {
			cli_errmsg("Cannot set permissions of directory `%s'", path);
			return -1;
		}
	} else {
		struct stat st;

		if (stat (path, &st) < 0 && errno == ENOENT) {
			char *parent = dirname (path);
			mode_t mask = umask (0);
			umask (mask);

			if (makedir (parent, (0777 & ~mask) | 0300,
						FILEUTILS_RECUR) < 0)
				return -1;
			free (parent);

			if (makedir (path, mode, 0) < 0)
				return -1;
		}
	}

	return 0;
}
