/************************************************************************/
/*                                                                      */
/* Psionic PortSentry - Port Scan Detector				*/
/*                                                                      */
/* Author: Craig H. Rowland <crowland@psionic.com>                      */
/* Created: 10-12-97                                                    */
/* Modified: 04-12-99                                                   */
/*                                                                      */
/* Send all changes/modifications/bugfixes to the above address.        */
/*                                                                      */
/*                                                                      */
/* This software is Copyright(c) 1997-99 Craig H. Rowland               */
/*                                                                      */
/* Disclaimer:                                                          */
/*                                                                      */
/* All software distributed by Craig H. Rowland ("the author") and      */
/* Psionic Systems is distributed AS IS and carries NO WARRANTY or      */
/* GUARANTEE OF ANY KIND. End users of the software acknowledge that    */
/* they will not hold the author, Psionic Systems, and any employer of  */
/* the author liable for failure or non-function of a software          */
/* product. YOU ARE USING THIS PRODUCT AT YOUR OWN RISK                 */
/*                                                                      */
/* Licensing restrictions apply. See the license that came with this    */
/* file for more information or visit http://www.psionic.com for more   */
/* information.                                                         */
/*                                                                      */
/*$Id: init.c,v 1.2 2001/12/31 03:58:47 sukent Exp $ */
/************************************************************************/

#include "init.h"
#include "global.h"

/* Global variables...ugghhh.... */
char gblKillRoute[MAXBUF];
char gblKillHostsDeny[MAXBUF];
char gblKillRunCmd[MAXBUF];
int gblBlockTCP = 0;
int gblBlockUDP = 0;


/****************************************************************/
/* Reads generic config options into global variables           */
/****************************************************************/
int
InitConfig (char *ssmtp_conf)
{
  FILE *input;
  char configToken[MAXBUF];

  gblBlockTCP = 1;
  gblBlockUDP = 1;

  memset (gblKillRoute, '\0', MAXBUF);
  memset (gblKillHostsDeny, '\0', MAXBUF);
  memset (gblKillRunCmd, '\0', MAXBUF);


  if ((ConfigTokenRetrieve ("KILL_ROUTE", gblKillRoute)) == TRUE)
    {
#ifdef DEBUG
      Log ("debug: InitConfig: retrieved KILL_ROUTE option: %s \n",
	   gblKillRoute);
#endif
    }
  else
    {
#ifdef DEBUG
      Log ("debug: InitConfig: KILL_ROUTE option NOT FOUND.\n");
#endif
    }

  if ((ConfigTokenRetrieve ("KILL_HOSTS_DENY", gblKillHostsDeny)) == TRUE)
    {
#ifdef DEBUG
      Log ("debug: InitConfig: retrieved KILL_HOSTS_DENY option: %s \n",
	   gblKillHostsDeny);
#endif
    }
  else
    {
#ifdef DEBUG
      Log ("debug: InitConfig: KILL_HOSTS_DENY option NOT FOUND.\n");
#endif
    }

  if ((ConfigTokenRetrieve ("KILL_RUN_CMD", gblKillRunCmd)) == TRUE)
    {
#ifdef DEBUG
      Log ("debug: InitConfig: retrieved KILL_RUN_CMD option: %s \n",
	   gblKillRunCmd);
#endif
    }
  else
    {
#ifdef DEBUG
      Log ("debug: InitConfig: KILL_RUN_CMD option NOT FOUND.\n");
#endif
    }

  if ((ConfigTokenRetrieve ("SYN_FLOOD", configToken)) == TRUE)
  {
     Gsynflood = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("SYN_FLOOD_TH", configToken)) == TRUE)
  {
     Gsynlimit = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("UDP_FLOOD", configToken)) == TRUE)
  {
     Gudpflood = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("UDP_FLOOD_TH", configToken)) == TRUE)
  {
     Gudplimit = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("ICMP_FLOOD", configToken)) == TRUE)
  {
     Gicmpflood = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("ICMP_FLOOD_TH", configToken)) == TRUE)
  {
     Gicmplimit = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("PING_OF_DEATH", configToken)) == TRUE)
  {
     Gpingofdeath = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("PORT_SCAN", configToken)) == TRUE)
  {
     Gportscan = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("LAND", configToken)) == TRUE)
  {
     Gland = atoi(configToken);
  }
  if ((ConfigTokenRetrieve ("TEAR_DROP", configToken)) == TRUE)
  {
     Gteardrop = atoi(configToken);
  }

  /* 2001/09/16 Steve ++ */
  if ((FwConfigTokenRetrieve (ssmtp_conf, "email", configToken)) == TRUE)
  {
     gEmail = atoi(configToken);
  }

  if ((FwConfigTokenRetrieve (ssmtp_conf, "email_addr1", gEmail_addr1)) ==
TRUE)
    {
#ifdef DEBUG
      Log ("debug: InitConfig: retrieved Email_addr1 option: %s \n",
	   gEmail_addr1);
#endif
    }
  else
    {
#ifdef DEBUG
      Log ("debug: InitConfig: email_addr1 option NOT FOUND.\n");
#endif
    }
  if ((FwConfigTokenRetrieve (ssmtp_conf, "email_addr2", gEmail_addr2)) == TRUE)
    {
#ifdef DEBUG
      Log ("debug: InitConfig: retrieved email_addr2 option: %s \n",
	   gEmail_addr2);
#endif
    }
  else
    {
#ifdef DEBUG
      Log ("debug: InitConfig: email_addr2 option NOT FOUND.\n");
#endif
    }
    /* Steve -- */

}

/* kill the connection depending on config option */
int
DisposeConnect (char *target, int port)
{
  int status = TRUE;


#ifdef DEBUG
  Log ("debug: DisposeUDP: disposing of host %s on port %d with option: %d",
       target, port, gblBlockUDP);
#endif
/* Should we ignore TCP from active response? */
  if (gblBlockUDP == 1)
    {
      /* run external command first, hosts.deny second, dead route last */
      if (strlen (gblKillRunCmd) > 0)
	if (KillRunCmd (target, port, gblKillRunCmd) != TRUE)
	  status = FALSE;
      if (strlen (gblKillHostsDeny) > 0)
	if (KillHostsDeny (target, port, gblKillHostsDeny) != TRUE)
	  status = FALSE;
      if (strlen (gblKillRoute) > 0)
	if (KillRoute (target, port, gblKillRoute) != TRUE)
	  status = FALSE;
    }
  else if (gblBlockUDP == 2)
    {
      /* run external command only */
      if (strlen (gblKillRunCmd) > 0)
	if (KillRunCmd (target, port, gblKillRunCmd) != TRUE)
	  status = FALSE;
    }
  else
    Log ("attackalert: Ignoring UDP response per configuration file setting.");

  return (status);
}
