// $Id: exec.c,v 1.2 2001/12/31 03:22:57 perry Exp $
// $Log: exec.c,v $
// Revision 1.2  2001/12/31 03:22:57  perry
// Add Id & Log
//

#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define		DEFAULT_ELEMENT		50

static char strbuf[2048];
static char **argv;
static int chflag = 0;

void
*xrealloc(void *ptr, size_t bytes)
{
    char *cp;

    if (!ptr)
        cp = malloc (bytes);
    else
        cp = realloc (ptr, bytes);

    return (cp);
}

int
cmdParse(char *str)
{
	int i = 0, j = 0;
	char *cp;

	if (strlen(str) >= sizeof strbuf)
		return -1;
	strcpy(strbuf, str);

	for (cp = strbuf, i = 0; cp;) {
		if (i >= j) {
			j += DEFAULT_ELEMENT;
			argv = (char **)xrealloc((char *)argv, sizeof(char **) * j);
		}

		argv[i] = cp;
		while (*cp) {
			switch (*cp) {
				case '\'':
					cp++;
					if (chflag)
						goto next;
					else
						chflag = 1;
					break;
				case ' ' :
				case '\n':
					if (chflag) {
						cp++;
						break;
					} else
						goto next;
				default:
					cp++;
			}
		}

next:
		chflag = 0;
		if (*cp) {
			*cp++ = '\0';
			if (*argv[i])
				i++;
		} else
			break;
        }
	argv[++i] = (char *)0;
        return 0;
}

int
my_exec(char *cmdstring)
{
	pid_t pid;
	int status;
	struct sigaction ignore, saveintr, savequit;
	sigset_t chldmask, savemask;

	if (!cmdstring)
		return -1;

	if (cmdParse(cmdstring) < 0)
		return -1;

	ignore.sa_handler = SIG_IGN; /* ignore SIGINT and SIGQUIT */
	sigemptyset(&ignore.sa_mask);
	ignore.sa_flags = 0;

	if (sigaction(SIGINT, &ignore, &saveintr) < 0)
		return -1;
	if (sigaction(SIGQUIT, &ignore, &savequit) < 0)
		return -1;

	sigemptyset(&chldmask);
	sigaddset(&chldmask, SIGCHLD);

	if (sigprocmask(SIG_BLOCK, &chldmask, &savemask) < 0)
		return -1;

	if ((pid = fork()) < 0) {
		status = -1;
	} else if (pid == 0) {
		sigaction(SIGINT, &saveintr, NULL);
		sigaction(SIGQUIT, &saveintr, NULL);
		sigaction(SIG_SETMASK, &saveintr, NULL);

		execvp(argv[0], argv);
		_exit(127);
	} else {
		while (waitpid(pid, &status, 0) < 0) {
			if (errno != EINTR) {
				status = -1;
				break;
			}
		}
	}

	if (sigaction(SIGINT, &saveintr, NULL) < 0)
		return -1;
	if (sigaction(SIGQUIT, &saveintr, NULL) < 0)
		return -1;
	if (sigaction(SIG_SETMASK, &saveintr, NULL) < 0)
		return -1;

	return(status);
}
