/*            the pppd from the command line.
 *            C. Scott Ananian <cananian@alumni.princeton.edu>
 *
 * $Id: pptp.c,v 1.1 2002/07/05 06:14:11 sukent Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#if defined(__FreeBSD__)
#include <libutil.h>
#elif defined(__NetBSD__)
#include <util.h>
#else
#include <pty.h>
#endif
#ifdef USER_PPP
#include <fcntl.h>
#endif

/* Jack ++ 2004/02/25 */
#include <asm/socket.h>
#include <linux/limits.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/un.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <setjmp.h>
#include <errno.h>
#include <sys/wait.h>
#include <getopt.h>
#include <limits.h>
#include "config.h"
#include "pptp_callmgr.h"
#include "pptp_gre.h"
#include "version.h"
#include "inststr.h"
#include "util.h"
#include "pptp_quirks.h"

#include <syslog.h>

#ifndef PPPD_BINARY
#define PPPD_BINARY "pppd"
#endif

// Jack Tsai
char client_buf[300];
char pptp_server_ip[300];
int open_callmgr_pid = 0;

/* SuKent++, Mon 20 Oct 2003 12:31:36 PM CST */
int	use_interface = 1;
/* ******** */

struct in_addr get_ip_address(char *name);
int open_callmgr(struct in_addr inetaddr, char *phonenr, int argc,char **argv,char **envp);
void launch_callmgr(struct in_addr inetaddr, char *phonenr, int argc,char **argv,char **envp);
int get_call_id(int sock, pid_t gre, pid_t pppd, 
		 u_int16_t *call_id, u_int16_t *peer_call_id);
void launch_pppd(char *ttydev, int argc, char **argv);

void usage(char *progname) {
  fprintf(stderr,
	  "%s\n"
	  "Usage:\n"
	  " %s hostname [[--phone <phone number>] [--quirks ISP_NAME] -- ][ pppd options]\n"
	  "\nOr using pppd option pty: \n"
	  " pty \" %s hostname --nolaunchpppd [--phone <phone number>] [--quirks ISP_NAME]\"\n"
	  "Currently recognized ISP_NAMEs for quirks are BEZEQ_ISRAEL\n",
	  version, progname, progname);
  log("%s called with wrong arguments, program not started.", progname);
  
  exit(1);
}

static int signaled = 0;

void do_nothing(int sig) { 
    /* do nothing signal handler. Better than SIG_IGN. */
    signaled = 1;
}

sigjmp_buf env;
void sighandler(int sig) {
  siglongjmp(env, 1);
}

/* TODO: redesign to avoid longjmp/setjmp.  Several variables here
   have a volatile qualifier to silence warnings from gcc < 3.0.
   Remove the volatile qualifiers when longjmp/setjmp are removed. */

/* Jack ++ 2004/12/20 VPN_TRUNK */
int wan_port = 0;
/* Jack -- */

int main(int argc, char **argv, char **envp) {
  struct in_addr inetaddr;
  volatile int callmgr_sock = -1;
  char ttydev[PATH_MAX];
  int pty_fd, tty_fd, rc;
  volatile pid_t parent_pid, child_pid;
  u_int16_t call_id, peer_call_id;
  int pppdargc;
  char **pppdargv;
  char phonenrbuf[65]; /* maximum length of field plus one for the trailing
                        * '\0' */
  char * volatile phonenr = NULL;
  volatile int launchpppd = 1;
#if 1
/* Steve ++ */
	if (fork() > 0)
		exit(0);

	close(0);
	close(1);
	close(2);
/* Steve -- */
#endif

/* Jack ++ 2004/12/20 VPN_TRUNK */
{
  char *str_p;

  if(!(str_p = strstr(argv[0],"wan"))) {
        wan_port = 0;
  } else {
        str_p += 3;
        wan_port = atoi(str_p);
  }
}
/* Jack -- */

  if (argc < 2)
    usage(argv[0]);

/* SuKent++ Fri 11 Oct 2002 10:34:28 AM CST */
  openlog("pptp", LOG_PID, LOG_LOCAL7);
/* ******** */

// Jack Tsai
//strcpy(client_buf,argv[6]);
/* Jack ++ 2004/02/27 */
#if 0
sprintf(client_buf, "client.%s_%s", argv[3], argv[1]);
#else
if(!strncmp(argv[6],"_pptp",5))
	sprintf(client_buf, "client.%s", argv[6]);
else
	sprintf(client_buf, "client.%s_%s", argv[3], argv[1]);
#endif
/* Jack -- */
strcpy(pptp_server_ip,argv[1]);

/* SuKent++ Mon 28 Oct 2002 06:41:02 PM CST */
    {
	char	*p, buffer[100];
	
	
	if (argv[6] &&  (p = strstr(argv[6], "client."))) {
	    p = &p[strlen("client.")];
	    
	    sprintf(buffer, "/usr/local/apache/cgi-bin/renew_iptables.exe %s", p);
	    system(buffer);
	}	
    }	
/* ******** */

/* SuKent++, Mon 20 Oct 2003 12:33:59 PM CST, found use_interface */
    {
	int	i, j;
	
	for (i = 1;i < argc;i++) {
		if (argv[i] && !strncmp(argv[i], "WAN", 3)) {
			use_interface = atoi(&argv[i][3]);
			
			for (j = i;j < argc;j++)	argv[j] = argv[j+1];
			
			argc--;			
			break;
		}	
	}
    }
/* ******** */


  /* Step 1a: Get IP address for the hostname in argv[1] */
  inetaddr = get_ip_address(argv[1]);
// Jack ++
  if(inetaddr.s_addr == 0) {
      syslog(LOG_INFO, "get_ip_address failed, exiting");
      exit(-1);
  }
// Jack --

  /* step 1b: Find the ppp options, extract phone number */
  argc--;
  argv++;
  while(1){ 
      /* structure with all recognised options for pptp */
      static struct option long_options[] = {
          {"phone", 1, 0, 0},  
          {"nolaunchpppd", 0, 0, 0},  
	  {"quirks", 1, 0, 0},
          {0, 0, 0, 0}
      };
      int option_index = 0;
      int c;
      opterr=0; /* suppress "unrecognised option" message, here
                 * we assume that it is a pppd option */
      c = getopt_long (argc, argv, "", long_options, &option_index);
      if( c==-1) break;  /* no more options */
      switch (c) {
        case 0: 
            if(option_index == 0) { /* --phone specified */
                /* copy it to a buffer, as the argv's will be overwritten by 
                 * inststr() */
                strncpy(phonenrbuf,optarg,sizeof(phonenrbuf));
                phonenrbuf[sizeof(phonenrbuf)-1]='\0';
                phonenr=phonenrbuf;
            }else if(option_index == 1) {/* --nolaunchpppd specified */
                  launchpppd=0;
            }else if(option_index == 2) {/* --quirks specified */
                if (set_quirk_index(find_quirk(optarg)))
                    usage(argv[0]);
            }
            /* other pptp options come here */
            break;
        case '?': /* unrecognised option, treat it as the first pppd option */
            /* fall through */
        default:
            c = -1;
            break;
      }
      if( c==-1) break;  /* no more options for pptp */
    }
  pppdargc = argc - optind;
  pppdargv = argv + optind;
 
  /* Step 3: Find an open pty/tty pair. */
  if(launchpppd){
      rc = openpty (&pty_fd, &tty_fd, ttydev, NULL, NULL);
      if (rc < 0) { 
          close(callmgr_sock); 
          fatal("Could not find free pty.");
      }
  
      /* Step 4: fork and wait. */
      signal(SIGUSR1, do_nothing); /* don't die */
      parent_pid = getpid();
      switch (child_pid = fork()) {
      case -1:
        fatal("Could not fork pppd process");

      case 0: /* I'm the child! */
        close (tty_fd);
        signal(SIGUSR1, SIG_DFL);
        child_pid = getpid();
        break;
      default: /* parent */
        close (pty_fd);
        /*
         * There is still a very small race condition here.  If a signal
         * occurs after signaled is checked but before pause is called,
         * things will hang.
         */
        if (!signaled) {
            pause(); /* wait for the signal */
        }
        launch_pppd(ttydev, pppdargc, pppdargv); /* launch pppd */

        perror("Error");
        fatal("Could not launch pppd");
      }
  } else { /* ! launchpppd */
      pty_fd=tty_fd=0;
      child_pid=getpid();
      parent_pid=0; /* don't kill pppd */
  }

  do {
    /*
     * Step 2: Open connection to call manager
     *         (Launch call manager if necessary.)
     */
    callmgr_sock = open_callmgr(inetaddr, phonenr, argc, argv, envp);
  /* Step 5: Exchange PIDs, get call ID */
  } while (get_call_id(callmgr_sock, parent_pid, child_pid, 
	               &call_id, &peer_call_id) < 0);

  /* Step 5b: Send signal to wake up pppd task */
  if(launchpppd){
      kill(parent_pid, SIGUSR1);
      sleep(2);
  }
 
  {
    char buf[128];
    snprintf(buf, sizeof(buf), "pptp: GRE-to-PPP gateway on %s", ttyname(tty_fd));
    inststr(argc,argv,envp, buf);
  }

  if (sigsetjmp(env, 1)!=0) goto shutdown;
  signal(SIGINT,  sighandler);
  signal(SIGTERM, sighandler);
  signal(SIGKILL, sighandler);
 
  /* Step 6: Do GRE copy until close. */
  pptp_gre_copy(call_id, peer_call_id, pty_fd, inetaddr);

shutdown:
  /* on close, kill all. */
  if(launchpppd) {
      kill(parent_pid, SIGTERM);
  }      

  close(pty_fd);
  close(callmgr_sock);
  sleep(3);     /* give ctrl manager a chance to exit */
  exit(0);
}

void do_alarm(int sig) {
    /* the SIGALRM handler */    
    syslog(LOG_INFO,"Connect timeout..Exit()");            
    exit(-1);
}


struct in_addr get_ip_address(char *name) {
    struct in_addr retval;
    struct sockaddr_in dest;
    int s;

    if (inet_aton(name, &retval) == 0) {
	    struct hostent *host = gethostbyname(name);

            if (host==NULL) {
		    if (h_errno == HOST_NOT_FOUND)
	                  syslog(LOG_INFO, "gethostbyname: HOST NOT FOUND");
		    else if (h_errno == NO_ADDRESS)
	                  syslog(LOG_INFO, "gethostbyname: NO IP ADDRESS");
		    else
	                  syslog(LOG_INFO, "gethostbyname: name server error");
            }
	
	    if (host->h_addrtype != AF_INET)
		    syslog(LOG_INFO, "Host has non-internet address");
            
	    memcpy(&retval.s_addr, host->h_addr, sizeof(retval.s_addr));
    }

    bzero(&dest, sizeof(dest));
    dest.sin_family = AF_INET;
    dest.sin_port   = htons(PPTP_PORT);
    dest.sin_addr   = retval;

    syslog(LOG_INFO,"socket");    
    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	syslog(LOG_INFO,"sukent : Cannot get socket: %s", strerror(errno));    
	retval.s_addr = 0;
	return retval;
    }

/* Jack ++ 2004/12/20 VPN_TRUNK */
#if 0
/* SuKent++, Mon 20 Oct 2003 12:38:56 PM CST */
    setsockopt(s, SOL_SOCKET, SO_ROUTEMARK, &use_interface, 4);
/* ******** */
#endif

if(wan_port > 0) {
        int mark = wan_port;
        setsockopt(s, SOL_SOCKET, SO_ROUTEMARK, &mark, sizeof(int));
}
/* Jack -- */

    syslog(LOG_INFO, "Connect %s now...", inet_ntoa(retval));    
    
    signal(SIGALRM, do_alarm);
    alarm(15);
    if (connect(s, (struct sockaddr *) &dest, sizeof(dest)) != -1) {
	alarm(0);
        syslog(LOG_INFO, "Connect succeeded");    
	close(s);
	return(retval);
    }
    alarm(0);
    close(s);
    
    syslog(LOG_INFO, "Connect failed: %s",strerror(errno));    
 
    retval.s_addr = 0;
    return retval;
}


#if 0
struct in_addr get_ip_address(char *name) {
  struct in_addr retval;
  struct hostent *host = gethostbyname(name);
  if (host==NULL) {
    if (h_errno == HOST_NOT_FOUND)
      fatal("gethostbyname: HOST NOT FOUND");
    else if (h_errno == NO_ADDRESS)
      fatal("gethostbyname: NO IP ADDRESS");
    else
      fatal("gethostbyname: name server error");
  }
  
  if (host->h_addrtype != AF_INET)
    fatal("Host has non-internet address");
  
  memcpy(&retval.s_addr, host->h_addr, sizeof(retval.s_addr));
  return retval;
}
#endif

int open_callmgr(struct in_addr inetaddr, char *phonenr, int argc, char **argv, char **envp)
{
  /* Try to open unix domain socket to call manager. */
  struct sockaddr_un where;
  const int NUM_TRIES = 3;
  int i, fd;
  pid_t pid;
  int status;

  /* Open socket */
  if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
    fatal("Could not create unix domain socket: %s", strerror(errno));
  }

/* SuKent++, Mon 20 Oct 2003 12:38:56 PM CST */    
    setsockopt(fd, SOL_SOCKET, SO_ROUTEMARK, &use_interface, 4);
/* ******** */

  /* Make address */
  where.sun_family = AF_UNIX;
  snprintf(where.sun_path, sizeof(where.sun_path), 
	   PPTP_SOCKET_PREFIX "%s", inet_ntoa(inetaddr));

  for (i=0; i<NUM_TRIES; i++) {
    if (connect(fd, (struct sockaddr *) &where, sizeof(where)) < 0) {
      /* couldn't connect.  We'll have to launch this guy. */

      unlink (where.sun_path);	

      /* fork and launch call manager process */
      switch (pid=fork()) {
      case -1: /* failure */
	  fatal("fork() to launch call manager failed.");
      case 0: /* child */
	  {
	      close (fd);
	      launch_callmgr(inetaddr, phonenr, argc,argv,envp);
	  }
      default: /* parent */
	  waitpid(pid, &status, 0);
	  if (status!=0)
	      fatal("Call manager exited with error %d", status);
	  break;
      }
      sleep(1);
    }
    else return fd;
  }
  close(fd);
  fatal("Could not launch call manager after %d tries.", i);
  return -1;   /* make gcc happy */
}

void launch_callmgr(struct in_addr inetaddr, char *phonenr, int argc,
        char**argv,char**envp) 
{
      int callmgr_main(int argc, char**argv, char**envp);
      char *my_argv[3] = { argv[0], inet_ntoa(inetaddr), phonenr };
      char buf[128];
      snprintf(buf, sizeof(buf), "pptp: call manager for %s", my_argv[1]);
      inststr(argc,argv,envp,buf);
      exit(callmgr_main(3, my_argv, envp));
      /*
      const char *callmgr = PPTP_CALLMGR_BINARY;
      execlp(callmgr, callmgr, inet_ntoa(inetaddr), NULL);
      fatal("execlp() of call manager [%s] failed: %s", 
	  callmgr, strerror(errno));
      */
}

/* XXX need better error checking XXX */
int get_call_id(int sock, pid_t gre, pid_t pppd, 
		 u_int16_t *call_id, u_int16_t *peer_call_id)
{
  u_int16_t m_call_id, m_peer_call_id;
  /* write pid's to socket */
  /* don't bother with network byte order, because pid's are meaningless
   * outside the local host.
   */
  int rc;
  rc = write(sock, &gre, sizeof(gre));
  if (rc != sizeof(gre))
      return -1;
  rc = write(sock, &pppd, sizeof(pppd));
  if (rc != sizeof(pppd))
      return -1;
  rc = read(sock,  &m_call_id, sizeof(m_call_id));
  if (rc != sizeof(m_call_id))
      return -1;
  rc = read(sock,  &m_peer_call_id, sizeof(m_peer_call_id));
  if (rc != sizeof(m_peer_call_id))
      return -1;
  /*
   * XXX FIX ME ... DO ERROR CHECKING & TIME-OUTS XXX
   * (Rhialto: I am assuming for now that timeouts are not relevant
   * here, because the read and write calls would return -1 (fail) when
   * the peer goes away during the process. We know it is (or was)
   * running because the connect() call succeeded.)
   */
  *call_id = m_call_id;
  *peer_call_id = m_peer_call_id;

  return 0;
}

void launch_pppd(char *ttydev, int argc, char **argv) {
  char *new_argv[argc+4]; /* XXX if not using GCC, hard code a limit here. */
  int i = 0;

// Jack Tsai
  new_argv[0] = client_buf;

  new_argv[1] = ttydev;
  for (i=0; i<argc; i++)
    new_argv[i+2] = argv[i];

	#if 0 /* because pptp client UI will add linkname argument with	 client rank */
	// add by Jack Tsai
	    new_argv[i+2] = "linkname";
	    new_argv[i+3] = "client";
	    new_argv[i+4] = NULL;
	#else
	  new_argv[i+2] = "ipparam";
	  new_argv[i+3] = pptp_server_ip;
	  new_argv[i+4] = NULL;
	// end of add
	#endif

// Jack Tsai
  execvp(PPPD_BINARY, new_argv);
  
#if 0
/* ******************************************************* */
  new_argv[i++] = PPPD_BINARY;
#ifdef USER_PPP
  new_argv[i++] = "-direct";
  /* ppp expects to have stdin connected to ttydev */
  if ((j = open(ttydev, O_RDWR)) == -1)
    fatal("Cannot open %s: %s", ttydev, strerror(errno));
  if (dup2(j, 0) == -1)
    fatal("dup2 failed: %s", strerror(errno));
  close(j);
#else
  new_argv[i++] = ttydev;
  new_argv[i++] = "38400";
#endif
  for (j=0; j<argc; j++)
    new_argv[i++] = argv[j];
  new_argv[i] = NULL;
  execvp(new_argv[0], new_argv);
/* ************************************************************* */  
#endif  
}

/*************** COMPILE call manager into same binary *********/
#define main       callmgr_main
#define sighandler callmgr_sighandler
#define do_nothing callmgr_do_nothing
#define env        callmgr_env
#include "pptp_callmgr.c"
