/*
 * linux/arch/arm/mach-ixp2000/ixdp2800-time.h
 *
 * Author: Jeff Daly <jeffrey.daly@intel.com>
 * 
 * Generic IXP2000 time functions for boards without readable 
 * external clocks (i.e. not the IXDP2400).  This means that
 * CLOCK_TICK_RATE is predefined in timex.h
 *
 * TODO: It would be nice to have allow CPU speed to be learned
 * at run time.
 * 
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <linux/time.h>
#include <linux/init.h>
#include <linux/smp.h>

#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/irq.h>

#include <linux/timex.h>
#include <asm/hardware.h>

/* 
 * Ticks per second are picked up from include/asm-arm/arch-ixp2000/timex.h
 * LATCH is defined with CLOCK_TICK_RATE and HZ
 */

extern int setup_arm_irq(int, struct irqaction *);

static unsigned long ixdp2800_gettimeoffset (void)
{
	unsigned long offset, usec;

	/* Get ticks since last timer service */
	offset= LATCH - *IXP2000_T1_CSR;

	usec = (unsigned long)(offset*tick)/LATCH;
	return usec;
}

static void ixdp2800_timer_interrupt(int irq, void *dev_id, 
				     struct pt_regs *regs)
{
	/* clear timer 1 */
	*IXP2000_T1_CLR = 1;

	do_timer(regs);
}

extern unsigned long (*gettimeoffset)(void);
extern int setup_arm_irq(int, struct irqaction *);

static struct irqaction timer_irq = {
	name: "timer"
};

void __init setup_timer (void)
{
	gettimeoffset = ixdp2800_gettimeoffset;
	timer_irq.handler = ixdp2800_timer_interrupt;

	*IXP2000_T1_CLR = 0;

	/* Load timer 1 with latch value which is */
	/*  (tick rate + hz/2)/ hz  */
	
	//*IXP2000_T1_CLD = LATCH;
	*IXP2000_T1_CLD = LATCH;

	/* Enable Timer 1 in periodic mode */
	*IXP2000_T1_CTL = (1 << 7) ;

	/* register for interrupt */
	setup_arm_irq(IRQ_IXP2000_TIMER1, &timer_irq);

	printk("Using IXP2800 Timer0 as timer source\n");
}
