#include <asm/uaccess.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <asm/arch/gpio.h>
#include <asm/hardware.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/timer.h>
#include <linux/spinlock.h>
#include <linux/delay.h>
#include <linux/errno.h>


#define	IXP425_LED_PIN			IXP425_GPIO_PIN_9
#define	IXP425_SWITCH_PIN		IXP425_GPIO_PIN_10
#define	IXP425_PHY_PIN			IXP425_GPIO_PIN_11
/* 2005/8/15 Steve ++ */
#define	IXP425_UART_PIN			IXP425_GPIO_PIN_14
/* Steve -- */

static struct proc_dir_entry *proc_led, *proc_watchdog;
/* 2005/8/15 Steve ++ */
static struct proc_dir_entry *proc_uart;
/* Steve -- */
static struct timer_list led_timer, watchdog_timer;
static char led_status[20];
static char switch_status[20];
static char phy_status[20];
static char watchdog_status[20];

static spinlock_t ixp425_led_lock = SPIN_LOCK_UNLOCKED;
static spinlock_t ixp425_watchdog_lock = SPIN_LOCK_UNLOCKED;
/* 2005/8/15 Steve ++ */
static spinlock_t ixp425_uart_lock = SPIN_LOCK_UNLOCKED;
/* Steve -- */

/* For Watchdog */
static int proc_watchdog_write(struct file *file, const char *buffer,
                               unsigned long count, void *data)
{
        unsigned int len;
        char buf[20], *state;

	spin_lock(&ixp425_watchdog_lock);
	if (count > 20) {
		spin_unlock(&ixp425_watchdog_lock);
		return -EINVAL;
	} else
		len = count;

	if (copy_from_user(buf, buffer, len)) {
		spin_unlock(&ixp425_watchdog_lock);
		return -EFAULT;
	}

	buf[len - 1] = '\0';
	state = buf;
	if (!strncmp(buf, "WATCHDOG ", 9)) {
		state += strlen("WATCHDOG ");
		strcpy(watchdog_status, state);
	} else {
		spin_unlock(&ixp425_watchdog_lock);
		return -EINVAL;
	}

	spin_unlock(&ixp425_watchdog_lock); 
	return len;
}

static int proc_watchdog_read(char *buf , char **start, off_t offset,
                              int len, int *eof, void *data)
{
	int size = 0;
	unsigned int i = 0;
	off_t pos = 0;

	spin_lock(&ixp425_watchdog_lock);

	size = sprintf(buf, "%s IXP425_OSWT: %x IXP425_OSWE: %x\n",
		       watchdog_status,
		       *IXP425_OSWT,
		       *IXP425_OSWE);

	pos += size;
	i += size;
	if (pos <= offset)
		i = 0;
	*start = buf + i - (pos - offset);
	i = pos - offset;
	if (i > len)
		i = len;
	if (i < 0)
		i = 0;

	spin_unlock(&ixp425_watchdog_lock);
	return i;
}

void
update_ixp425_watchdog(void)
{
	spin_lock(&ixp425_watchdog_lock);
	if (!strncmp(watchdog_status, "ON", 2)) {
		/* Disable Watchdog */
		*IXP425_OSWK = 0x482e;
		*IXP425_OSWE = 0x00;
		/* Enable Watchdog */
		*IXP425_OSWK = 0x482e;
		*IXP425_OSWT = 0x50000000;
		*IXP425_OSWE = 0x07;
//		printk(KERN_EMERG "watchdog update %lu\n", jiffies);
	} else if (!strncmp(watchdog_status, "OFF", 3)) {
		/* Disable Watchdog */
		*IXP425_OSWK = 0x482e;
		*IXP425_OSWE = 0x00;
	}
	spin_unlock(&ixp425_watchdog_lock);
}

static void
watchdog_action(unsigned long timeout)
{
	spin_lock(&ixp425_watchdog_lock);
	if (!strncmp(watchdog_status, "ON", 2)) {
		/* Disable Watchdog */
		*IXP425_OSWK = 0x482e;
		*IXP425_OSWE = 0x00;
		/* Enable Watchdog */
		*IXP425_OSWK = 0x482e;
		*IXP425_OSWT = 0x50000000;
		*IXP425_OSWE = 0x07;
//		printk(KERN_EMERG "watchdog update %lu\n", jiffies);
	} else if (!strncmp(watchdog_status, "OFF", 3)) {
		/* Disable Watchdog */
		*IXP425_OSWK = 0x482e;
		*IXP425_OSWE = 0x00;
	}
	watchdog_timer.expires = jiffies + HZ;
	add_timer(&watchdog_timer);
	spin_unlock(&ixp425_watchdog_lock);
}
/* -- */

static void
led_on(void)
{
	gpio_line_config(IXP425_LED_PIN, IXP425_GPIO_OUT);
	gpio_line_set(IXP425_LED_PIN, IXP425_GPIO_LOW);
}

static void
led_off(void)
{
	gpio_line_config(IXP425_LED_PIN, IXP425_GPIO_OUT);
	gpio_line_set(IXP425_LED_PIN, IXP425_GPIO_HIGH);
}

static void
phy_reset(void)
{
	gpio_line_config(IXP425_PHY_PIN, IXP425_GPIO_OUT);
	gpio_line_set(IXP425_PHY_PIN, IXP425_GPIO_LOW);
	udelay(1000);
	gpio_line_set(IXP425_PHY_PIN, IXP425_GPIO_HIGH);
	gpio_line_config(IXP425_PHY_PIN, IXP425_GPIO_IN);
}

static void
switch_reset(void)
{
        int gp4_value, gp5_value, gp6_value;

        gpio_line_get(IXP425_GPIO_PIN_4, &gp4_value);
        gpio_line_get(IXP425_GPIO_PIN_5, &gp5_value);
        gpio_line_get(IXP425_GPIO_PIN_6, &gp6_value);
        gpio_line_config(IXP425_GPIO_PIN_4, IXP425_GPIO_IN);
        gpio_line_config(IXP425_GPIO_PIN_5, IXP425_GPIO_IN);
        gpio_line_config(IXP425_GPIO_PIN_6, IXP425_GPIO_IN);
        udelay(30*1000);

	gpio_line_config(IXP425_SWITCH_PIN, IXP425_GPIO_OUT);
	gpio_line_set(IXP425_SWITCH_PIN, IXP425_GPIO_LOW);
	udelay(1000);
	gpio_line_set(IXP425_SWITCH_PIN, IXP425_GPIO_HIGH);
	gpio_line_config(IXP425_SWITCH_PIN, IXP425_GPIO_IN);

        udelay(30*1000);
        gpio_line_config(IXP425_GPIO_PIN_4, IXP425_GPIO_OUT);
        gpio_line_config(IXP425_GPIO_PIN_5, IXP425_GPIO_OUT);
        gpio_line_config(IXP425_GPIO_PIN_6, IXP425_GPIO_OUT);
        gpio_line_set(IXP425_GPIO_PIN_4, gp4_value);
        gpio_line_set(IXP425_GPIO_PIN_5, gp5_value);
        gpio_line_set(IXP425_GPIO_PIN_6, gp6_value);
}

static void
led_action(unsigned long timeout)
{
	static unsigned int led_last_status = 0;

	spin_lock(&ixp425_led_lock);

	if (!strcmp(phy_status, "PHY ON"))
		phy_reset();
	strcpy(phy_status, "PHY OFF");

	if (!strcmp(switch_status, "SWITCH ON"))
		switch_reset();
	strcpy(switch_status, "SWITCH OFF");

	if (!strcmp(led_status, "LED ON")) {
		led_on();
		led_timer.expires = jiffies + HZ;
	} else if (!strcmp(led_status, "LED OFF")) {
		led_off();
		led_timer.expires = jiffies + HZ;
	} else if (!strcmp(led_status, "LED BLINKING")) {
		if ((led_last_status % 2))
			led_on();
		else
			led_off();

		led_last_status++;
		led_timer.expires = jiffies + HZ / 2;
	} else if (!strcmp(led_status, "LED FASTBLINKING")) {
		if ((led_last_status % 2))
			led_on();
		else
			led_off();

		led_last_status++;
		led_timer.expires = jiffies + HZ / 10;
	} else
		led_timer.expires = jiffies + HZ;

	add_timer(&led_timer);
	spin_unlock(&ixp425_led_lock);
}

static int proc_led_read(char *buf , char **start, off_t offset,
                         int len, int *eof, void *data)
{
	int size = 0;
	unsigned int i = 0;
	off_t pos = 0;

	spin_lock(&ixp425_led_lock);
	size += sprintf(buf, "%s\n%s\n%s\n",
			led_status,
			switch_status,
			phy_status);

	pos += size;
	i += size;
	if (pos <= offset)
		i = 0;
	*start = buf + i - (pos - offset);
	i = pos - offset;
	if (i > len)
		i = len;
	if (i < 0)
		i = 0;

	spin_unlock(&ixp425_led_lock);
	return i;
}

static int proc_led_write(struct file *file, const char *buffer,
                          unsigned long count, void *data)
{
        unsigned int len;
        char buf[20];

	spin_lock(&ixp425_led_lock);
	if (count > 20) {
		spin_unlock(&ixp425_led_lock);
		return -EINVAL;
	} else
		len = count;

	if (copy_from_user(buf, buffer, len)) {
		spin_unlock(&ixp425_led_lock);
		return -EFAULT;
	}

	buf[len - 1] = '\0';
	if (!strncmp(buf, "LED ", 4))
		strcpy(led_status, buf);
	else if (!strncmp(buf, "SWITCH ", 7))
		strcpy(switch_status, buf);
	else if (!strncmp(buf, "PHY ", 4))
		strcpy(phy_status, buf);
	else {
		spin_unlock(&ixp425_led_lock);
		return -EINVAL;
	}
	spin_unlock(&ixp425_led_lock); 
	return len;
}

/* 2005/8/15 Steve ++ */
static int proc_uart_read(char *buf , char **start, off_t offset,
                         int len, int *eof, void *data)
{
	int size = 0;
	unsigned int i = 0;
	off_t pos = 0;
	int value = 0;

	gpio_line_get(IXP425_UART_PIN, &value);

	spin_lock(&ixp425_uart_lock);
	size += sprintf(buf, "console: %d\n", value);

	pos += size;
	i += size;
	if (pos <= offset)
		i = 0;
	*start = buf + i - (pos - offset);
	i = pos - offset;
	if (i > len)
		i = len;
	if (i < 0)
		i = 0;

	spin_unlock(&ixp425_uart_lock);
	return i;
}
/* Steve -- */

static void __exit led_cleanup_module(void)
{
        if (proc_led)
                remove_proc_entry("driver/led", NULL);

        if (proc_watchdog)
                remove_proc_entry("driver/watchdog", NULL);

/* 2005/8/15 Steve ++ */
	if (proc_uart)
		remove_proc_entry("driver/console", NULL);
/* Steve -- */
}

static int __init led_init_module(void)
{
	int ret = 0;

	gpio_line_config(IXP425_LED_PIN, IXP425_GPIO_OUT);
	proc_led = create_proc_entry("driver/led", 600, 0);
	if (!proc_led) {
		printk("create_led_proc_entry fails\n");
		ret = -ENOMEM;
		goto proc_led_error;
	}
	proc_led->read_proc = proc_led_read;
	proc_led->write_proc = proc_led_write;
	proc_led->data = 0;

	proc_watchdog = create_proc_entry("driver/watchdog", 600, 0);
	if (!proc_watchdog) {
		printk("create_watchdog_proc_entry fails\n");
		ret = -ENOMEM;
		goto proc_watchdog_error;
	}
	proc_watchdog->read_proc = proc_watchdog_read;
	proc_watchdog->write_proc = proc_watchdog_write;
	proc_watchdog->data = 0;
	strcpy(watchdog_status, "ON");
	init_timer(&watchdog_timer);
	watchdog_timer.function = watchdog_action;
	watchdog_timer.expires = jiffies + HZ;
	watchdog_action(0);
//	add_timer(&watchdog_timer);

	memset(led_status, '\0', sizeof(led_status));
	memset(switch_status, '\0', sizeof(switch_status));
	memset(phy_status, '\0', sizeof(phy_status));
	strcpy(led_status, "LED BLINKING");
	strcpy(switch_status, "SWITCH ON");
	strcpy(phy_status, "PHY ON");

	init_timer(&led_timer);
	led_timer.function = led_action;
	led_timer.expires = jiffies + HZ / 2;
	add_timer(&led_timer);

/* 2005/8/15 Steve ++ */
#if 1
	gpio_line_config(IXP425_UART_PIN, IXP425_GPIO_OUT);
	proc_uart = create_proc_entry("driver/console", 600, 0);
	if (proc_uart)
		proc_uart->read_proc = proc_uart_read;
#endif
/* Steve -- */

	printk(KERN_INFO "LED Drivers\n");
	return ret;

proc_watchdog_error:
	remove_proc_entry("driver/led", NULL);
proc_led_error:
	return ret;
}

module_init(led_init_module);
module_exit(led_cleanup_module);
EXPORT_NO_SYMBOLS;
