/* 
 * FileName:    IxTskcCritSect.c
 * Author: Intel Corporation
 * Description: 
 *    This is the implementiation file for the Task controller critical
 *    section implementation.
 * 
 * Design Notes:
 *
 * -- Intel Copyright Notice --
 * INTEL CONFIDENTIAL
 *
 * Copyright 2000-2002 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

/*
 * Put the system defined include files required
 */

#include <stdio.h>
#include <stdlib.h>
#include <sysLib.h>
#include <vxWorks.h>
#include <semLib.h>


typedef struct _CompCritSecTable
{
    unsigned   count;
    SEM_ID **mutexes;
}
CompCritSecTable;

#define PRIVATE static

PRIVATE CompCritSecTable *critSecTable;

void
ixTskcCriticalSectionBegin (unsigned id)
{
    CompCritSecTable     *t;
    SEM_ID      *m;
    int stat;
    
    t = &critSecTable[0];

    m = *(t->mutexes + id);

    /* 
     *  Check if somebody tried to use a critical section which wasn't 
     *  registered for this component. 
     */

    stat = semTake (*m, WAIT_FOREVER);
}

void
ixTskcCriticalSectionEnd (unsigned id)
{
    CompCritSecTable     *t;
    SEM_ID      *m;
    int stat;

    t = &critSecTable[0];

    m = *(t->mutexes + id);

    /* 
     *  Check if somebody tried to use a critical section which wasn't 
     *  registered for this component. 
     */

    stat = semGive (*m);
}


