/* 
 * FileName:    IxAdslMgmtInfo.h
 * Author: Intel Corporation
 * Created:     29-April-2000
 * 
 * Description: 
 * Supports the IxAdslMgmt component. Contains the the mandatory
 * MIB objects to support RFC 2662 for an ATUR.
 * 
 * Design Notes:
 *
 * -- Intel Copyright Notice --
 * INTEL CONFIDENTIAL
 *
 * Copyright 2000-2002 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */


#ifndef IXADSLMGMTINFO_H
#define IXADSLMGMTINFO_H

#include "IxTypes.h"
#include "IxManagerInfoTypes.h"

#define START_OF_INTERVAL 1
#define SECONDS_IN_15MIN_INTERVAL 900
#define MAX_15MINUTEINTERVALS_PERDAY   ((60 / 15) * 24)

#define NO_INTERVALS_EXIST 1
#define INTERVALS_EXIST 2
#define LINK_UP_DOWN_TRAP_INDEXCOUNT 3

#define IF_ADMINSTATUS_UP 1
#define IF_ADMINSTATUS_DOWN 2
#define IF_ADMINSTATUS_TESTING 3


typedef enum _IxAdslManagerInfo
{
    /* ADSL Interface Support */
    IxAdslIfDescr_TRO,		                     /* OCTET */
    IxAdslIfType_TRO,		                     /* INT32 */
    IxAdslIfMtu_TRO,		                     /* INT32 */
    IxAdslIfSpeed_TRO,		                     /* UINT32 */
    IxAdslIfPhysAddress_TRO,		             /* OCTET */
    IxAdslIfAdminStatus_TRW,		             /* INT32 */
    IxAdslIfOperStatus_TRO,		             /* INT32 */
    IxAdslIfLastChange_TRO,		             /* UINT32 */
    IxAdslIfInOctets_TRO,		             /* UINT32 */
    IxAdslIfInUcastPkts_TRO,		             /* UINT32 */
    IxAdslIfInDiscards_TRO,		             /* UINT32 */
    IxAdslIfInErrors_TRO,		             /* UINT32 */
    IxAdslIfInUnknownProtos_TRO,	             /* UINT32 */
    IxAdslIfOutOctets_TRO,		             /* UINT32 */
    IxAdslIfOutUcastPkts_TRO,		             /* UINT32 */
    IxAdslIfOutDiscards_TRO,		             /* UINT32 */
    IxAdslIfOutErrors_TRO,		             /* UINT32 */
    IxAdslIfName_TRO,		                     /* OCTET */
    IxAdslIfInMulticastPkts_TRO,	             /* UINT32 */
    IxAdslIfInBroadcastPkts_TRO,	             /* UINT32 */
    IxAdslIfOutMulticastPkts_TRO,	             /* UINT32 */
    IxAdslIfOutBroadcastPkts_TRO,	             /* UINT32 */
    IxAdslIfHighSpeed_TRO,		             /* UINT32 */
    IxAdslIfPromiscuousMode_TRW,	             /* INT32 */
    IxAdslIfConnectorPresent_TRO,	             /* INT32 */
    IxAdslIfCounterDiscontinuityTime_TRO,            /* UINT32 */
    IxAdslIfStackLowerLevel_TRO,	             /* UINT32 */

    /* ADSL Line Support */
    IxAdslLineCoding_TRO,		             /* INT32 */
    IxAdslLineType_TRO,		                     /* INT32 */

    /* ADSL ATU-C Physical Layer Parameter Support */
    IxAdslAtucInvSerialNumber_TRO,                   /* OCTET */
    IxAdslAtucInvVendorID_TRO,                       /* OCTET */
    IxAdslAtucInvVersionNumber_TRO,                  /* OCTET */
    IxAdslAtucCurrSnrMgn_TRO,
    IxAdslAtucCurrAtn_TRO,
    IxAdslAtucCurrStatus_TRO,                        /* OCTET */
    IxAdslAtucCurrOutputPwr_TRO,                     /* INT32 */
    IxAdslAtucCurrAttainableRate_TRO,                /* UINT32 */
    
    /* ADSL ATU-R Physical Layer Parameter Support */
    IxAdslAturInvSerialNumber_TRO,                   /* OCTET */
    IxAdslAturInvVendorID_TRO,                       /* OCTET */
    IxAdslAturInvVersionNumber_TRO,                  /* OCTET */
    IxAdslAturCurrSnrMgn_TRO,		             /* INT32 */
    IxAdslAturCurrAtn_TRO,		             /* UINT32 */
    IxAdslAturCurrStatus_TRO,		             /* OCTET */
    IxAdslAturCurrOutputPwr_TRO,	             /* INT32 */
    IxAdslAturCurrAttainableRate_TRO,	             /* UINT32 */
    
    /* ADSL ATU-R Channel Support */
    IxAdslAturChanInterleaveDelay_TRO,               /* UINT32 */
    IxAdslAturChanCurrTxRate_TRO,                    /* UINT32 */
    IxAdslAturChanPrevTxRate_TRO,                    /* UINT32 */
    IxAdslAturChanCrcBlockLength_TRO,                /* UINT32 */

    /* ADSL ATU-R Performance Data Support */
    IxAdslAturPerfLofs_TRO,	 	             /* UINT32 */
    IxAdslAturPerfLoss_TRO,		             /* UINT32 */
    IxAdslAturPerfLprs_TRO,		             /* UINT32 */
    IxAdslAturPerfESs_TRO,		             /* UINT32 */ 
    IxAdslAturPerfValidIntervals_TRO,                /* INT32 */
    IxAdslAturPerfInvalidIntervals_TRO,              /* INT32 */
    IxAdslAturPerfCurr15MinTimeElapsed_TRO,          /* UINT32 */
    IxAdslAturPerfCurr15MinLofs_TRO,                 /* UINT32 */
    IxAdslAturPerfCurr15MinLoss_TRO,                 /* UINT32 */
    IxAdslAturPerfCurr15MinLprs_TRO,                 /* UINT32 */
    IxAdslAturPerfCurr15MinESs_TRO,                  /* UINT32 */
    IxAdslAturPerfCurr1DayTimeElapsed_TRO,           /* UINT32 */
    IxAdslAturPerfCurr1DayLofs_TRO,                  /* UINT32 */
    IxAdslAturPerfCurr1DayLoss_TRO,                  /* UINT32 */
    IxAdslAturPerfCurr1DayLprs_TRO,                  /* UINT32 */
    IxAdslAturPerfCurr1DayESs_TRO,                   /* UINT32 */
    IxAdslAturPerfPrev1DayMoniSecs_TRO,              /* INT32 */
    IxAdslAturPerfPrev1DayLofs_TRO,                  /* UINT32 */
    IxAdslAturPerfPrev1DayLoss_TRO,                  /* UINT32 */
    IxAdslAturPerfPrev1DayLprs_TRO,                  /* UINT32 */
    IxAdslAturPerfPrev1DayESs_TRO,                   /* UINT32 */

    /* ADSL ATU-R Interval Support */
    IxAdslAturIntervalLofs_TRO,                      /* UINT32 */
    IxAdslAturIntervalLoss_TRO,                      /* UINT32 */
    IxAdslAturIntervalLprs_TRO,                      /* UINT32 */
    IxAdslAturIntervalESs_TRO,                       /* UINT32 */
    IxAdslAturIntervalValidData_TRO,                 /* INT32 */

    /* ADSL ATU-R Channel Performance Data Support */
    IxAdslAturChanReceivedBlks_TRO,                  /* UINT32 */
    IxAdslAturChanTransmittedBlks_TRO,               /* UINT32 */
    IxAdslAturChanCorrectedBlks_TRO,                 /* UINT32 */
    IxAdslAturChanUncorrectBlks_TRO,               /* UINT32 */
    IxAdslAturChanPerfValidIntervals_TRO,            /* INT32 */
    IxAdslAturChanPerfInvalidIntervals_TRO,          /* INT32 */
    IxAdslAturChanPerfCurr15MinTimeElapsed_TRO,      /* UINT32 */
    IxAdslAturChanPerfCurr15MinReceivedBlks_TRO,     /* UINT32 */
    IxAdslAturChanPerfCurr15MinTransmittedBlks_TRO,  /* UINT32 */
    IxAdslAturChanPerfCurr15MinCorrectedBlks_TRO,    /* UINT32 */
    IxAdslAturChanPerfCurr15MinUncorrectBlks_TRO,  /* UINT32 */
    IxAdslAturChanPerfCurr1DayTimeElapsed_TRO,       /* UINT32 */
    IxAdslAturChanPerfCurr1DayReceivedBlks_TRO,      /* UINT32 */
    IxAdslAturChanPerfCurr1DayTransmittedBlks_TRO,   /* UINT32 */
    IxAdslAturChanPerfCurr1DayCorrectedBlks_TRO,     /* UINT32 */
    IxAdslAturChanPerfCurr1DayUncorrectBlks_TRO,   /* UINT32 */
    IxAdslAturChanPerfPrev1DayMoniSecs_TRO,          /* UINT32 */
    IxAdslAturChanPerfPrev1DayReceivedBlks_TRO,      /* UINT32 */
    IxAdslAturChanPerfPrev1DayTransmittedBlks_TRO,    /* UINT32 */
    IxAdslAturChanPerfPrev1DayCorrectedBlks_TRO,         /* UINT32 */
    IxAdslAturChanPerfPrev1DayUncorrectBlks_TRO,       /* UINT32 */

    /* ADSL ATU-R Channel Interval Support */
    IxAdslAturChanIntervalReceivedBlks_TRO,          /* UINT32 */
    IxAdslAturChanIntervalTransmittedBlks_TRO,        /* UINT32 */
    IxAdslAturChanIntervalCorrectedBlks_TRO,         /* UINT32 */
    IxAdslAturChanIntervalUncorrectBlks_TRO,       /* UINT32 */
    IxAdslAturChanIntervalValidData_TRO,             /* INT32 */

    /* ADSL ATU-R Line Configuration Profile Support */
    IxAdslAturThresh15MinLofs_TRW,                   /* INT32 */
    IxAdslAturThresh15MinLoss_TRW,                   /* INT32 */
    IxAdslAturThresh15MinLprs_TRW,                   /* INT32 */
    IxAdslAturThresh15MinESs_TRW,                    /* INT32 */
    IxAdslAturThreshFastRateUp_TRW,                  /* UINT32 */
    IxAdslAturThreshInterleaveRateUp_TRW,           /* UINT32 */
    IxAdslAturThreshFastRateDown_TRW,                /* UINT32 */
    IxAdslAturThreshInterleaveRateDown_TRW          /* UINT32 */
} IxAdslManagerInfo;

typedef enum _IxAdslManagerTraps
{
    IxAdslAtucInitFailureTrap,
    IxAdslAturPerfLofsThreshTrap,
    IxAdslAturPerfLossThreshTrap,
    IxAdslAturPerfLprsThreshTrap,
    IxAdslAturPerfESsThreshTrap,
    IxAdslAturRateChangeTrap,
    IxAdslLinkDown,
    IxAdslLinkUp
} IxAdslManagerTraps;


INT32
ixAdslMgmtManagementInfoGet (const IxAdslManagerInfo entity,
			     UINT32 *indexArray,
			     const UINT16 indexCount,
			     ManagerInfoData *dataBufferToWriteInto);

INT32
ixAdslMgmtManagementInfoGetNext (const IxAdslManagerInfo entity,
				 UINT32 *indexArray,
				 const UINT16 indexCount,
				 ManagerInfoData *dataBufferToWriteInto);

INT32
ixAdslMgmtManagementInfoSet (const IxAdslManagerInfo entity,
			     UINT32 *indexArray,
			     const UINT16 indexCount,
			     ManagerInfoData *dataBufferToRead);

INT32
ixAdslMgmtManagementInfoTest (const IxAdslManagerInfo entity,
			      UINT32 *indexArray,
			      const UINT16 indexCount,
			      ManagerInfoData *dataBufferToRead);

typedef void (*IxAdslMgmtTrapCallbackPtr_t)(const IxAdslManagerTraps entity, 
					    UINT32* indexArray, 
					    const UINT16 indexCount);

INT32
ixAdslMgmtTrapSubscriberBind(IxAdslMgmtTrapCallbackPtr_t funcPtr);

int
ixAdslMgmtInfoInit(void);

#endif /* IXADSLMGMTINFO_H */






