/*
* FileName:    IxManagerInfoTypes.h
 * Author: Intel Corporation
* Created:     14-APR-00
* Description: 
*    defines and structs used on the interface between management
*    components and application components.
* Design Notes:
*    See SNMP Functional Specification for details of how these
*    are used within the management interface of application
*    components.
*
 * -- Intel Copyright Notice --
 * INTEL CONFIDENTIAL
 *
 * Copyright 2000-2002 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

#ifndef __IXMANAGERINFOTYPES_H
#define __IXMANAGERINFOTYPES_H

#define MAX_STRING_LENGTH 256
#define MAX_OID_LENGTH 256

/*
 * Status passed back from Management Interface transactions. See additional
 * notes included below for their usage.
 * ok and failed status use standard ix defines.
 */
#define IX_MGRINF_INVALID 2	    /* Request invalid for unspecified reason. */
#define IX_MGRINF_NOSUCHINDEX 3     /* instance of index given does not exist. */
#define IX_MGRINF_NOSUCHENTITY 4    /* entity enum passed in does not exist.   */
#define IX_MGRINF_INDEXOUTOFRANGE 5 /* index passed is not within valid range. */
#define IX_MGRINF_NOMORE 6	    /* GetNext only. End of table reached.     */
#define IX_MGRINF_NOTSUPPORTED 7    /* Entity that will be supported in future.*/

/*
 * Please apply the following logic when determining what error status
 * to return:
 *    1. Does the ManagerInfo enum value passed exist?
 *           Yes. Go to check 2.
 *           No.  Return IX_MGRINF_NOSUCHENTITY
 *    2. Is the ManagerInfo enum passed to you currently supported (or
 *       is this data that you intend to provide in the future?)
 *           Yes. Go to check 3.
 *           No.  Return IX_MGRINF_NOTSUPPORTED
 *    3. Is the DataBuffer passed of the correct size?
 *           Yes. Go to check 4.
 *           No.  Return IX_MGRINF_INVALID
 *    4. Is the request GetNext?
 *           Yes. Go to check A1.
 *           No.  Go to check B1.
 *    A1. Is the enitity requested a tabular (as opposed to scalar) entity?
 *           Yes. Go to check A2.
 *           No.  Return IX_MGRINF_NOMORE
 *    A2. Tables Only. Is the index passed below the valid range?
 *           Yes. Set the index to the lowest valid value. Go to A3.
 *           No.  Go to check A3.
 *    A3. Tables Only. Is the index passed above the valid range?
 *           Yes. Return IX_MGRINF_INDEXOUTOFRANGE
 *           No.  Go to check A4.
 *    A4. Tables Only. Is the index passed equal to the last valid index?
 *           Yes. Return IX_MGRINF_NOMORE
 *           No.  Retrieve next valid value after given index & 
 *                return IX_SUCCESS.  
 *    B1. Is the request to change a read-only entity?
 *           Yes. Return IX_MGRINF_INVALID
 *           No.  Go to check B2
 *    B2. Tables Only. Is the index passed within the valid range?
 *           Yes. Go to check B3.
 *           No.  Return IX_MGRINF_INDEXOUTOFRANGE
 *    B3. Tables Only. Does index passed refer to a valid entity instance.
 *           Yes. Go to check B4.
 *           No.  Return IX_MGRINF_NOSUCHINDEX
 *    B4. For set/test requests, is proposed change acceptable?
 *           Yes. Go to check B5
 *           No.  Return IX_MGRINF_INVALID
 *    B5. Perform requested operation. Was this successful?
 *           Yes. Return IX_SUCESS
 *           No.  Return IX_FAILED
 */

/*
 * Struct for passing data in an untyped manner between application
 * components and management component. In most cases length is used
 * purely as a safety check to help confirm that the datatype is as
 * expected.
 */
typedef struct {
    void*  dataPtr;	/* A pointer to the data for/from management. */
    UINT32 dataLength; 	/* The length in bytes of the data pointed to. */
} ManagerInfoData;

/*
 * Defines for common enumerations on management interfaces, such as
 * operStates and admin states
 */
#define IX_MGRINF_ADMIN_UP 1
#define IX_MGRINF_ADMIN_DOWN 2
#define IX_MGRINF_ADMIN_TESTING 3

#define IX_MGRINF_OPER_UP 1
#define IX_MGRINF_OPER_DOWN 2
#define IX_MGRINF_OPER_TESTING 3
#define IX_MGRINF_OPER_UNKNOWN 4
#define IX_MGRINF_OPER_DORMANT 5
#define IX_MGRINF_OPER_NOTPRESENT 6
#define IX_MGRINF_OPER_LOWERLAYERDOWN 7

#define IX_MGRINF_TRUTH_TRUE 1
#define IX_MGRINF_TRUTH_FALSE 2

/* Defines for IfTypes */
#define IX_IFTYPE_OTHER 1

#endif /* __IXMANAGERINFOTYPES_H */
