/* ixAdslSoftReset.h - Macros, function definitions, etc. needed by ixAdslSoftReset.c */


#ifndef __INCixAdslSoftReset 
#define __INCixAdslSoftReset

#include "IxTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

#include "ixp425.h"

#define PERIPHERAL_BASE  IXP425_PERIPHERAL_BASE  /* physical address : (0xc8000000) */
#define GPIO_BASE        (PERIPHERAL_BASE + 0x00004000)
#define TMR_BASE         (PERIPHERAL_BASE + 0x00005000)
#define OSTS             (TMR_BASE + 0x0 )  /* Continuous TimeStamp Reg. */

#define PERIPHERAL_BUS_CLOCK (66)

#define  GPIO_PIN_MAX 15	/* 16 pins (incl. 0) */

/* GPIO pin mappings */
#define  GPIO_PIN_16 16
#define  GPIO_PIN_15 15
#define  GPIO_PIN_14 14
#define  GPIO_PIN_13 13
#define  GPIO_PIN_12 12
#define  GPIO_PIN_11 11
#define  GPIO_PIN_10 10
#define  GPIO_PIN_9  9
#define  GPIO_PIN_8  8
#define  GPIO_PIN_7  7
#define  GPIO_PIN_6  6
#define  GPIO_PIN_5  5
#define  GPIO_PIN_4  4
#define  GPIO_PIN_3  3
#define  GPIO_PIN_2  2
#define  GPIO_PIN_1  1

#define GPIO_GPOUTR      (GPIO_BASE + 0x0000)
#define GPIO_GPOER       (GPIO_BASE + 0x0004)
#define GPIO_GPINR       (GPIO_BASE + 0x0008)
#define GPIO_GPISR       (GPIO_BASE + 0x000C)
#define GPIO_GPIT1R      (GPIO_BASE + 0x0010)
#define GPIO_GPIT2R      (GPIO_BASE + 0x0014)
#define GPIO_GPCLKR      (GPIO_BASE + 0x0018)
#define GPIO_GPDBSELR    (GPIO_BASE + 0x001C)

/* GPIO signal types */
typedef enum _GPIO_SIG
{
     GPIO_LOW =0,
     GPIO_HIGH
}  GPIO_SIG;

/* GPIO pin types */
#define  GPIO_OUT (0x1)
#define  GPIO_IN  (0x2) 

/* GPIO interrupt types */
#define  GPIO_ACTIVE_HIGH	(0x4)		/* Default */
#define  GPIO_ACTIVE_LOW	(0x8) 
#define  GPIO_RISING_EDGE	(0x10)
#define  GPIO_FALLING_EDGE (0x20)
#define  GPIO_TRANSITIONAL (0x40)

/* Interrupt styles, these refer to actual values used in reg */
#define GPIO_STYLE_ACTIVE_HIGH   (0x0)
#define GPIO_STYLE_ACTIVE_LOW    (0x1)
#define GPIO_STYLE_RISING_EDGE   (0x2)
#define GPIO_STYLE_FALLING_EDGE  (0x3)
#define GPIO_STYLE_TRANSITIONAL  (0x4)

/* Mask used to clear interrupt styles */
#define GPIO_STYLE_CLEAR (0x7)
#define GPIO_INTSTYLE_MASK 0x7
#define GPIO_STYLE_SIZE  (3)

#define  GPIO_INTLOCK(key) 
#define  GPIO_INTUNLOCK(key)
#define  GPIO_KEYDECLARE

#define GPIO_REG_READ(regPtr,res) \
        ((res) = *(volatile UINT32 *)(regPtr))

#define GPIO_REG_WRITE(regPtr,val) \
        (*((volatile UINT32 *)(regPtr)) = (val))

#define  GPIO_LINE_SET(pin) \
	(*(volatile UINT32 *)  GPIO_GPOUTR) |= (0x1 << (pin));

#define  GPIO_LINE_CLEAR(pin) \
	(*(volatile UINT32 *)  GPIO_GPOUTR) &= ~(0x1 << (pin));

/* Function Declarations */
IX_STATUS GPIOLineConfig (UINT8 lineNo, UINT32 style);
IX_STATUS GPIOLineSet (UINT8 lineNo,  GPIO_SIG value);  
void MicrosecDelay(int microseconds);

#ifdef __cplusplus
}
#endif

#endif /*__INCixAdslSoftReset */ 
