/* 
 * FileName:    ixAdsl_p.h
 * Author: Intel Corporation
 * Created:     3-APR-2002
 * Description: 
 *    Local header file for the ADSL Driver
 * 
 * Design Notes:
 *
 * -- Intel Copyright Notice --
 * INTEL CONFIDENTIAL
 *
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */


#ifndef IX_ADSLP_H
#define IX_ADSLP_H

#include <IxAdsl.h>
#include <IxOsServices.h>

#define DEBUG 0


#define CPE_ALCATEL_ANALOG_FRONT_END_CODE (21) /* Defined by Alcatel */
#define CO_ALCATEL_ANALOG_FRONT_END_CODE (0x6) /* 0x6 for +20dbm max
						* power also may be
						* used 0x7 for 16.6
						* max power */

#if DEBUG
#define	CTRLE_ASSERT(test) ((void) \
		      ((test) ?	((void)	0) : \
		       printf("Assertion failed: %20s  file :%s	line:%d	/n"	\
				#test ,	__FILE__ , __LINE__ )))
#define	DEBUG_CMD(test)	{ test;	}

#define DEBUG_LOG printf

#else
#define	CTRLE_ASSERT(ignore)	((void)	0)
#define DEBUG_LOG printf
#define	DEBUG_CMD(test)		((void)	0)

#endif


typedef unsigned int ( * IX_ADSL_enableModemLine_fp) (unsigned int lineNum, T_adsl_ConfigurationData* configData );

typedef struct ixAdslVendorInfo_t
{
	unsigned char itu_vendorid_countrycode;
	unsigned char itu_vendorid_reserved;
	unsigned char itu_vendorid_vendorcode_1;
	unsigned char itu_vendorid_vendorcode_2;
	unsigned char itu_vendorid_vendorcode_3;
	unsigned char itu_vendorid_vendorcode_4;
	unsigned char itu_vendorid_vendorspecific_1;
	unsigned char itu_vendorid_vendorspecific_2;
} ixAdslVendorInfo_t;



typedef enum ixAdslLineAdminState
{
  ADSL_ADMIN_DOWN,
  ADSL_ADMIN_UP
}ixAdslLineAdminState;

typedef enum ixAdslLineOperState
{
  OPER_STATE_ADSL_DOWN,
  OPER_STATE_ADSL_UP,
  OPER_STATE_ADSL_NEGOCIATING
} ixAdslLineOperState;


typedef struct ixAdslState_t
{
  ixAdslLineAdminState adminState;
  t_ADSL_LINE_TYPE lineType;
} ixAdslState_t;

/*
 * There is a subtle difference between 'Loss of' seconds and failures.
 * A 'Loss of' second is a defect which has occured in the last second,
 * whereas a failure is declared when a defect has been active for the
 * last 2.5 seconds +/- 0.5 seconds. A failure is a more terminal event
 * and would normally cause a modem resync event. - DH 04/04/01 
 */
typedef struct IxadslAturPerfDataEntry
{
	UINT32 IxadslAturPerfLofs;	/* Defects */
	UINT32 IxadslAturPerfLcds;
	UINT32 IxadslAturPerfLoss;
	UINT32 IxadslAturPerfLprs;
	UINT32 IxadslAturPerfLoms;
	UINT32 IxadslAturPerfESs;
	UINT32 IxadslAturPerfFEC;
	UINT32 IxadslAturPerfHEC;

	BOOL IxadslAturPerfLoff;	/* Failures */
	BOOL IxadslAturPerfLosf;
	BOOL IxadslAturPerfLprf;
	BOOL IxadslAturPerfLcdf;
	BOOL IxadslAturPerfLomf;
} IxadslAturPerfDataEntry;

/*
 * The following structure is used to contain statistics on the cause
 * of a software initiated re-train, i.e. one of the failure flags 
 * defined in the IxadslAturPerfDataEntry structure detected.
 */
typedef struct _IxadslAturPerfRetrainStats
{
	UINT32 IxadslAturPerfLoffRetrainInitiated;
	UINT32 IxadslAturPerfLosfRetrainInitiated;
	UINT32 IxadslAturPerfLprfRetrainInitiated;
	UINT32 IxadslAturPerfLcdfRetrainInitiated;
	UINT32 IxadslAturPerfLomfRetrainInitiated;
	UINT32 IxadslAturPerfSwRetrainInitiated;
} IxadslAturPerfRetrainStats;

unsigned int IxADSL_CPE_OpenInternal( unsigned int lineNum, t_ADSL_LINE_TYPE lineType );
 

#define IX_ADSL_WAIT_FOR_OPEN_CNT 20
#define IX_ADSL_OPEN_POLL_SLEEP     { ixOsServTaskSleep(1000); } /* 1 sec */

#define IX_ADSL_DNLD_SLEEP     { ixOsServTaskSleep(2000); } /* 1 sec */



/*****************************
 * ADSL Task supervisory code.
 *****************************/

#define ADSL_TASK_NAME "ADSL Line Supervisory Task"
/* The ADSL task must run at a higher priority than the vxWorks netTask
 * as heavy ethernet traffic may block this task from catching changes 
 * in the ADSL line state */
#define ADSL_TASK_PRIORITY (25) 
#define ADSL_TASK_OPTIONS (0)
#define ADSL_TASK_STACK (8192)


#define ADSL_DOWN_DEBOUNCE_CNT (3)

extern const T_adsl_ConfigurationData  adslLineConfig_R;

extern unsigned char ixADSL_DownLoadDevice(unsigned int lineNum,
					   t_ADSL_PHY_TYPE phyType,
					   unsigned char* initFirmware,
					   unsigned int initFwLen,
					   unsigned char* phyFirmware,
					   unsigned int phyFwLen);

extern void ixStateChanged(unsigned int lineNum, t_ADSL_LINE_STATE state );

extern t_ADSL_PHY_TYPE ixAdslPhyTypeGet(unsigned int lineNum);
extern ixAdslState_t ixAdslStateGet(unsigned int lineNum);
T_adsl_ConfigurationData ixAdslAturConfigDataGet(unsigned int lineNum);

extern IxadslAturPerfDataEntry* ixAdslGetPerfData(unsigned int lineNum);
extern IxadslAturPerfRetrainStats* ixAdslPerfRetrainStatsGet(unsigned int lineNum);

extern void ixAdslManagementInfoInit(unsigned int lineNum);

#define ADSL_TASK_SLEEP { ixOsServTaskSleep(1000); }

#endif /* IX_ADSL_H */



