/** @file
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 1999-2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
*/

/*
 * Modification history
 *
 * Date       Author              Description
 * -------------------------------------------------------------
 * 04/29/99   pjb	Initial release.
 * 06/29/00   ts  Added an init complete code to determine
 *            when the init image has completed processing and the application
 *            image can be run.
 * 04/03/02   so	Modified for ixp425
 * -------------------------------------------------------------
 */

#if !defined(CTRLEREADWRITEHEADER_P)
#define	CTRLEREADWRITEHEADER_P

#include "ixp425.h"

#define DEBUG 0







#define	CTRLE_USE_SERIAL (0)

#if CTRLE_USE_SERIAL

/*
 * Prototypes for the file.
 */


bool ctrle_send_escaped_byte( uint8 octet );
bool ctrle_send_frame(unsigned char *framePtr,unsigned int numberBytes);


#define	CTRLE_BUFFER_SIZE (128)
#define	START_SYNC (0xA5)
#define	END_SYNC (0x5A)
#define	ESCAPE_CHAR (0xF7)


#define	CTRLE_MAX_BYTES	(64)

#define	CTRLE_READ_CMD	   (0x00)
#define	CTRLE_WRITE_CMD	   (0x01)
#define CTRLE_POLL_CMD     (0x03)
#define	CTRLE_INTRRUPT_CMD (0x0C)
#define CTRLE_RESET_CMD    (0x0f)

#define CTRLE_READ_RESP    (0x00)
#define CTRLE_WRITE_RESP   (0x01)
#define CTRLE_POLL_RESP    (0x03)
#define CTRLE_EXEC_FAIL    (0x07)
#define CTRLE_INTERRUPT    (0x0c)
#define CTRLE_RESET_ACK    (0x0f)

#define	RUNNING_XOR(data)   runningXOR = runningXOR ^ (data)

/*
 * Serial IO for device
 */

#define	CTRLE_SEND_BYTE( octet ) fdprintf(ioFD, "%c",octet);
#define	CTRLE_GETCHAR(ch) read(ioFD,&ch,1) 

#if 0
#define	CTRLE_SEND_BYTE( octet ) printf( "Raw Byte :	0x%02X\n",octet);
#define	CTRLE_GETCHAR(ch) read(0,&ch,1) 
#endif

#define VXWORKS_DEFAULT_SERIAL_DEVICE "/tyCo/0"


#define  MODULE_SIZE 0x200000





#if DEBUG
#define	CTRLE_ASSERT(test) ((void) \
		      ((test) ?	((void)	0) : \
		       printf("Assertion failed: %20s  file :%s	line:%d	/n"	\
				#test ,	__FILE__ , __LINE__ )))
#define	DEBUG_CMD(test)	{ test;	}

#else
#define	CTRLE_ASSERT(ignore)	((void)	0)
#define	DEBUG_CMD(test)		((void)	0)
#endif

typedef	struct t_ctrle_serial_stats
{
      int rxBytesRx;
      int rxStartFrameRx;
      int rxEndFrameRx;
      int rxFrameRx;
      int rxFrameDrops;
      int rxReadErrors;
      int rxCRCErrors;
} t_ctrle_serial_stats;

#endif /* CTRLE_USE_SERIAL */


#if ! (CTRLE_USE_SERIAL)
/******************************************************
 * 
 * Defines for CTRLE PARALLEL
 *
 ******************************************************/

/*
 * Used to determine when the init image has finished running and the
 * application image can be loaded.
 */
#define INIT_COMPLETE    0x80

#define CTRLE_READ_REG(x)                   *((volatile unsigned char *)(x))
#define CTRLE_WRITE_REG(x,val)              *((volatile unsigned char *)(x)) = ((unsigned char)(val));

#define CTRLE_BYTE_OFFSET (0x0)
#define CTRLE_ADDR_SHIFT (0) /* ixp425 Change */


#define CTRLE_THUNDER_CTRLE_BASE_ADDR         (IXP425_EXPANSION_BUS_CS1_BASE) /* ixp425 CHANGE */

#define CTRLE_TX_REG_ADDR(N)                  (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+(0x00 << CTRLE_ADDR_SHIFT ))
#define CTRLE_RX_REG_ADDR(N)                  (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+ (0x01 << CTRLE_ADDR_SHIFT ))
#define CTRLE_TX_COM_AV_ADDR(N)               (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+ (0x02 << CTRLE_ADDR_SHIFT ))
#define CTRLE_RX_COM_AV_ADDR(N)               (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+ (0x03 << CTRLE_ADDR_SHIFT ))
#define CTRLE_SEMAPHORE_ADDR(N)               (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+ (0x04 << CTRLE_ADDR_SHIFT ))
#define CTRLE_CONFIG_DATA_ADDR(N)             (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+ (0x40 << CTRLE_ADDR_SHIFT ))
#define CTRLE_MODEM_STATUS_ADDR(N)            (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+ (0x100 <<  CTRLE_ADDR_SHIFT ))
#define CTRLE_MODEM_AVAIL_DATA_ADDR(N)        (CTRLE_THUNDER_CTRLE_BASE_ADDR+N*MODULE_SIZE+CTRLE_BYTE_OFFSET+ (0x101 <<  CTRLE_ADDR_SHIFT ))



#define CTRLE_TX_REG(N)                       ( CTRLE_READ_REG(CTRLE_TX_REG_ADDR(N)))
#define CTRLE_RX_REG(N)                       ( CTRLE_READ_REG(CTRLE_RX_REG_ADDR(N)))
#define CTRLE_TX_COM_AV(N)                    ( CTRLE_READ_REG(CTRLE_TX_COM_AV_ADDR(N)))
#define CTRLE_RX_COM_AV(N)                    ( CTRLE_READ_REG(CTRLE_RX_COM_AV_ADDR(N)))
#define CTRLE_SEMAPHORE(N)                    ( CTRLE_READ_REG(CTRLE_SEMAPHORE_ADDR(N)))


#define CTRLE_SLEEP_TIME (10)      /* OS Ticks */
#define CTRLE_WAIT_FOR_CMD_REG_CNT ( 10 ) 

#define CTRLE_SLEEP { taskDelay(CTRLE_SLEEP_TIME); }


#endif /* ! CTRLE_USE_SERIAL */


#endif /* Header file */













