/** @file
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
*/
/* 
 * FileName:    ixAdslDrv.c
 * Author: Intel Corporation
 * Created:     04-Apr-2000
 *
 * Revised for ixp425:
 * Date:	3-Apr-2002
 *
 * Description: 
 *    Main source file for ADSL Driver
 *
 * Copyright 2000, Intel Corp.  All rights reserved. No part 
 * of this document may be reproduced, transmitted, transcribed, stored in 
 * a retrieval system, or compiled into computer source code, in any form 
 * or by any means, without the prior written consent of Intel Corp.
 *
 */

/* ixAdsl.c - Alcatel ADSL Driver definitions. */

#include <ix_ossl.h>
#include <IxOsServices.h> 

#include <string.h>

#include "IxAdsl.h"
#include "ixCtrle_c.h"
#include "ixAdsl_p.h"

#define IXADSL_SIZEOF_CALLBACK_LIST 10

#define NUM_ADSL_LINES 8

const T_adsl_ConfigurationData  adslLineConfig_C =
{
     0,  /*  MinNoiseMarginDnstr;	*/
     0,  /*  MinNoiseMarginUpstr;	*/
     31,  /*  MaxAddNoiseMarginDnstr;	*/
     31,  /*  MaxAddNoiseMarginUpstr;	*/
     6,  /*  TargetNoiseMarginDnstr;	*/
     6,  /*  TargetNoiseMarginUpstr;	*/
     -40,  /*  MaxPSDDnstr;	*/ /* 216=-40db 2s complement : Invert all bits and add 1 */
     20,  /*  MaxAggrPwrLvlDnstr;	*/
     13,  /*  MaxAggrPwrLvlUpstr;	*/

       { 0,0,0,0,0,0,0,0,  /*  CarrierMask[32];	*/	
         0,0,0,0,0,0,0,0,
         0,0,0,0,0,0,0,0,
         0,0,0,0,0,0,0,0
	},


     AUTOMATIC,  /*  T_RAMode RAModeDnstr;	*/
     AUTOMATIC,  /*  T_RAMode RAModeUpstr;	*/

     100,  /*  RARatioDnstr;	*/
     100,  /*  RARatioUpstr;	*/
     0,  /*  RADownshiftMarginDnstr;	*/
     0,  /*  RADownshiftMarginUpstr;	*/
     0,  /*  RAUpshiftMarginDnstr;	*/
     0,  /*  RAUpshiftMarginUpstr;	*/
     0,  /*  RADownshiftIntervalDnstr;	*/
     0,  /*  RADownshiftIntervalUpstr;	*/
     0,  /*  RAUpshiftIntervalDnstr;	*/
     0,  /*  RAUpshiftIntervalUpstr;	*/

     0,  /*  InterlMinBitrateDnstr;	*/
     0,  /*  InterlMinBitrateUpstr;	*/
     0,  /*  InterlPlannedBitrateDnstr;	*/
     0,  /*  InterlPlannedBitrateUpstr;	*/
     0,  /*  InterlMaxBitrateDnstr;	*/
     0,  /*  InterlMaxBitrateUpstr;	*/
     16,  /*  InterlMaxDelayDnstr;	*/
     16,  /*  InterlMaxDelayUpstr;	*/

     50,  /*  FastMinBitrateDnstr;	*/
     10,  /*  FastMinBitrateUpstr;	*/
     0,  /*  FastPlannedBitrateDnstr;	*/
     0,  /*  FastPlannedBitrateUpstr;	*/
     8000,  /*  FastMaxBitrateDnstr;	*/
     1000,  /*  FastMaxBitrateUpstr;	*/
     0x01,  /*  UtopiaLevel;	*/
     0,  /*  UtopiaAddrFast;	*/
     0,  /*  UtopiaAddrInterl;	*/
     CO_ALCATEL_ANALOG_FRONT_END_CODE,     /*  FunctionCode1FD;	*/ 		/*  :  Analog Front End Code */
     0,  /*  ExtraFunctionCode1FC;	*/
     0,  /*  itu_vendorid_countrycode;	*/
     0,  /*  itu_vendorid_reserved;	*/
     0,  /*  itu_vendorid_vendorcode_1;	*/
     0,  /*  itu_vendorid_vendorcode_2;	*/
     0,  /*  itu_vendorid_vendorcode_3;	*/
     0,  /*  itu_vendorid_vendorcode_4;	*/
     0,  /*  itu_vendorid_vendorspecific_1;	*/
     0   /*  itu_vendorid_vendorspecific_2;	*/

};

const T_adsl_ConfigurationData  adslLineConfig_C_interleaved =
{
     0,  /*  MinNoiseMarginDnstr;	*/
     0,  /*  MinNoiseMarginUpstr;	*/
     31,  /*  MaxAddNoiseMarginDnstr;	*/
     31,  /*  MaxAddNoiseMarginUpstr;	*/
     6,  /*  TargetNoiseMarginDnstr;	*/
     6,  /*  TargetNoiseMarginUpstr;	*/
     -40,  /*  MaxPSDDnstr;	*/ /* 216=-40db 2s complement : Invert all bits and add 1 */
     20,  /*  MaxAggrPwrLvlDnstr;	*/
     13,  /*  MaxAggrPwrLvlUpstr;	*/

       { 0,0,0,0,0,0,0,0,  /*  CarrierMask[32];	*/	
         0,0,0,0,0,0,0,0,
         0,0,0,0,0,0,0,0,
         0,0,0,0,0,0,0,0
	},


     AUTOMATIC,  /*  T_RAMode RAModeDnstr;	*/
     AUTOMATIC,  /*  T_RAMode RAModeUpstr;	*/

     100,  /*  RARatioDnstr;	*/
     100,  /*  RARatioUpstr;	*/
     0,  /*  RADownshiftMarginDnstr;	*/
     0,  /*  RADownshiftMarginUpstr;	*/
     0,  /*  RAUpshiftMarginDnstr;	*/
     0,  /*  RAUpshiftMarginUpstr;	*/
     0,  /*  RADownshiftIntervalDnstr;	*/
     0,  /*  RADownshiftIntervalUpstr;	*/
     0,  /*  RAUpshiftIntervalDnstr;	*/
     0,  /*  RAUpshiftIntervalUpstr;	*/

     50,  /*  InterlMinBitrateDnstr;	*/
     10,  /*  InterlMinBitrateUpstr;	*/
     0,  /*  InterlPlannedBitrateDnstr;	*/
     0,  /*  InterlPlannedBitrateUpstr;	*/
     8000,  /*  InterlMaxBitrateDnstr;	*/
     1000,  /*  InterlMaxBitrateUpstr;	*/
     16,  /*  InterlMaxDelayDnstr;	*/
     16,  /*  InterlMaxDelayUpstr;	*/

     0,  /*  FastMinBitrateDnstr;	*/
     0,  /*  FastMinBitrateUpstr;	*/
     0,  /*  FastPlannedBitrateDnstr;	*/
     0,  /*  FastPlannedBitrateUpstr;	*/
     0,  /*  FastMaxBitrateDnstr;	*/
     0,  /*  FastMaxBitrateUpstr;	*/
     0x01,  /*  UtopiaLevel;	*/
     0,  /*  UtopiaAddrFast;	*/
     0,  /*  UtopiaAddrInterl;	*/
     CO_ALCATEL_ANALOG_FRONT_END_CODE,     /*  FunctionCode1FD;	*/ 		/*  :  Analog Front End Code */
     0,  /*  ExtraFunctionCode1FC;	*/
     0,  /*  itu_vendorid_countrycode;	*/
     0,  /*  itu_vendorid_reserved;	*/
     0,  /*  itu_vendorid_vendorcode_1;	*/
     0,  /*  itu_vendorid_vendorcode_2;	*/
     0,  /*  itu_vendorid_vendorcode_3;	*/
     0,  /*  itu_vendorid_vendorcode_4;	*/
     0,  /*  itu_vendorid_vendorspecific_1;	*/
     0   /*  itu_vendorid_vendorspecific_2;	*/

};

ixAdslVendorInfo_t ixAdslVendorInfo[NUM_ADSL_LINES]={{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0}};



const T_adsl_ConfigurationData  adslLineConfig_R = /* ixp425 CPE line SETUP */
{
     0,  /*  MinNoiseMarginDnstr;	*/
     0,  /*  MinNoiseMarginUpstr;	*/
     0,  /*  MaxAddNoiseMarginDnstr;	*/
     0,  /*  MaxAddNoiseMarginUpstr;	*/
     0,  /*  TargetNoiseMarginDnstr;	*/
     0,  /*  TargetNoiseMarginUpstr;	*/
     0,  /*  MaxPSDDnstr;	*/
     0,  /*  MaxAggrPwrLvlDnstr;	*/
     0,  /*  MaxAggrPwrLvlUpstr;	*/

       { 0,0,0,0,0,0,0,0,  /*  CarrierMask[32];	*/	
         0,0,0,0,0,0,0,0,
         0,0,0,0,0,0,0,0,
         0,0,0,0,0,0,0,0
	},


     AUTOMATIC,  /*  T_RAMode RAModeDnstr;	*/
     AUTOMATIC,  /*  T_RAMode RAModeUpstr;	*/

     0,  /*  RARatioDnstr;	*/
     0,  /*  RARatioUpstr;	*/
     0,  /*  RADownshiftMarginDnstr;	*/
     0,  /*  RADownshiftMarginUpstr;	*/
     0,  /*  RAUpshiftMarginDnstr;	*/
     0,  /*  RAUpshiftMarginUpstr;	*/
     0,  /*  RADownshiftIntervalDnstr;	*/
     0,  /*  RADownshiftIntervalUpstr;	*/
     0,  /*  RAUpshiftIntervalDnstr;	*/
     0,  /*  RAUpshiftIntervalUpstr;	*/

     0,  /*  InterlMinBitrateDnstr;	*/
     0,  /*  InterlMinBitrateUpstr;	*/
     0,  /*  InterlPlannedBitrateDnstr;	*/
     0,  /*  InterlPlannedBitrateUpstr;	*/
     0,  /*  InterlMaxBitrateDnstr;	*/
     0,  /*  InterlMaxBitrateUpstr;	*/
     0,  /*  InterlMaxDelayDnstr;	*/
     0,  /*  InterlMaxDelayUpstr;	*/

     0,  /*  FastMinBitrateDnstr;	*/
     0,  /*  FastMinBitrateUpstr;	*/
     0,  /*  FastPlannedBitrateDnstr;	*/
     0,  /*  FastPlannedBitrateUpstr;	*/
     0,  /*  FastMaxBitrateDnstr;	*/
     0,  /*  FastMaxBitrateUpstr;	*/
     0x02,  /*  UtopiaLevel;	*/  	/* ixp425 level 2 changes */
     0x1,  /*  UtopiaAddrFast;	*/  /* ixp425 : For stacking adsl boards must be unique */
     0x2,  /*  UtopiaAddrInterl;	*/
     CPE_ALCATEL_ANALOG_FRONT_END_CODE,     /*  FunctionCode1FD;	*/ 		/*  :  Analog Front End Code */
     0,  /*  ExtraFunctionCode1FC;	*/
     0,  /*  itu_vendorid_countrycode;	*/
     0,  /*  itu_vendorid_reserved;	*/
     0,  /*  itu_vendorid_vendorcode_1;	*/
     0,  /*  itu_vendorid_vendorcode_2;	*/
     0,  /*  itu_vendorid_vendorcode_3;	*/
     0,  /*  itu_vendorid_vendorcode_4;	*/
     0,  /*  itu_vendorid_vendorspecific_1;	*/
     0   /*  itu_vendorid_vendorspecific_2;	*/

};



ixAdslState_t ixAdslState [NUM_ADSL_LINES]={{ADSL_ADMIN_DOWN, ADSL_NORMAL}, {ADSL_ADMIN_DOWN, ADSL_NORMAL}, {ADSL_ADMIN_DOWN, ADSL_NORMAL}, {ADSL_ADMIN_DOWN, ADSL_NORMAL}, {ADSL_ADMIN_DOWN, ADSL_NORMAL}, {ADSL_ADMIN_DOWN, ADSL_NORMAL}, {ADSL_ADMIN_DOWN, ADSL_NORMAL},{ADSL_ADMIN_DOWN, ADSL_NORMAL}};

/*
 * Debug strings for dump line state.
 */

const char *listStateStrs[]= 
{
  "Unknown Line State (error)",
  "Adsl Line UP",
  "Adsl Line Waiting for Activating",
  "Adsl Line Activating",
  "Adsl Line DOWN "
};

int ixADSL_moduleInit[NUM_ADSL_LINES]={0,0,0,0,0,0,0,0};
ix_ossl_thread_t ixADSL_taskId;
ix_error osError;
static int ixAdslCOInterleavedMode[NUM_ADSL_LINES]={FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE} ;
static int ixAdslEnableGasp[NUM_ADSL_LINES]={FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE};


#define IXADSL_CALLBACK_REG 1
#define IXADSL_CALLBACK_SEND 2

typedef enum adslTaskState_e
{
  ADSL_TASK_INIT,
  ADSL_LINE_OPEN_POLLING
} adslTaskState_e;

t_ADSL_PHY_TYPE ixAdslPhyType[NUM_ADSL_LINES];
unsigned int fwInitComplete = 0;


typedef struct
{
    t_ADSL_PHY_TYPE phyType;
    unsigned int lineNum;
} ixADSL_LineParams;


const char *
listStateStrsGet(int index){
    return listStateStrs[index];
}

/****************************************************************************/

#if CPU == XSCALE
ixAdslState_t 
ixAdslStateGet(unsigned int lineNum){
    return ixAdslState[lineNum];
}
#endif

#if CPU == XSCALE
T_adsl_ConfigurationData             
ixAdslAturConfigDataGet(unsigned int lineNum)
{
    return adslLineConfig_R;
}
#endif

void
ixAdslCOInterleavedModeSet(unsigned int lineNum, int on){
    if(on){
	ixAdslCOInterleavedMode[lineNum]=TRUE;
    } else {
	ixAdslCOInterleavedMode[lineNum]=FALSE;
    }
}

/***************************************************************************/

static unsigned int 
IxADSL_LineOpenInternal( unsigned int lineNum, 
			 t_ADSL_PHY_TYPE phyType, 
			 t_ADSL_LINE_TYPE lineType,
			 IX_ADSL_enableModemLine_fp enableModemFns[]
			 )
{
  unsigned int rc=IX_ADSL_SUCCESS;
  unsigned int alcatelRC = 0;
  T_adsl_ConfigurationData  adslLineConfig;
  unsigned char status;
  unsigned int cnt;
  
  /*  
   * Check arguments, 
   */
  if ( lineType >= ADSL_INVALID_MODE ) 
  {
    return(IX_ADSL_UNSUPPORTED_MODE);
  }
   
  if ( (phyType != IX_ADSL_CPE) && (phyType != IX_ADSL_CO) )
  {
    return (IX_ADSL_UNSUPPORTED_MODE);
  }

  DEBUG_CMD(DEBUG_LOG("ADSL Line Open\n"));
  
  /* Init local copy of adsl line config */
  if (phyType == IX_ADSL_CPE)
  {
    memcpy( &adslLineConfig,
	    &adslLineConfig_R,
	    sizeof(adslLineConfig_R) );
  } 
  else if (phyType == IX_ADSL_CO)
  {
    if(ixAdslCOInterleavedMode){
      memcpy( &adslLineConfig,
	      &adslLineConfig_C_interleaved,
	      sizeof(adslLineConfig_C_interleaved) );
    } 
    else 
    {
      memcpy( &adslLineConfig,
	      &adslLineConfig_C,
	      sizeof(adslLineConfig_C) );
    }
  }

  /* Copy Vendor info */
  

   adslLineConfig.itu_vendorid_countrycode = ixAdslVendorInfo[lineNum].itu_vendorid_countrycode;
   adslLineConfig.itu_vendorid_vendorcode_1 = ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_1;
   adslLineConfig.itu_vendorid_vendorcode_2 = ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_2;
   adslLineConfig.itu_vendorid_vendorcode_3 = ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_3;
   adslLineConfig.itu_vendorid_vendorcode_4 = ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_4;
   adslLineConfig.itu_vendorid_vendorspecific_1 = ixAdslVendorInfo[lineNum].itu_vendorid_vendorspecific_1;
   adslLineConfig.itu_vendorid_vendorspecific_2 = ixAdslVendorInfo[lineNum].itu_vendorid_vendorspecific_2;

  if (phyType == IX_ADSL_CPE)
  {
   adslLineConfig.UtopiaAddrFast = lineNum;
   adslLineConfig.UtopiaAddrInterl = lineNum + 1;
   if(lineNum == 7)
        adslLineConfig.UtopiaAddrInterl = 0;
  }

   /* Call the appropriate modem line function */

   alcatelRC = (enableModemFns[lineType])( lineNum, &adslLineConfig );

   if ( alcatelRC )
   {
     /*
      * Error Failed to open line.
      */
     DEBUG_CMD(DEBUG_LOG("ADSL: Line open failed\n"));
     rc = IX_ADSL_OPEN_FAILED;
   }


   ixAdslState[lineNum].adminState = ADSL_ADMIN_UP;
   ixAdslState[lineNum].lineType = lineType; 

   /*
    * Check return responce from Open up to 20sec 
    */
    
   /* for */
 
   for ( cnt = 0; cnt < IX_ADSL_WAIT_FOR_OPEN_CNT ; ++cnt )
   {
     DEBUG_CMD(DEBUG_LOG("."));
   
     if (!  ADSL_getRxCommand(lineNum, &status) )
     {
       /*
	* Got status.
	*/
       if (  status == ADSL_Connection_Opened ) 
       {
	 /* o.k */
	 DEBUG_CMD(DEBUG_LOG("ADSL: Line opened - success\n"));
	 ixStateChanged(lineNum, IX_ADSL_LINE_UP);
	 return(IX_ADSL_SUCCESS)	;  
       }
       else if ( status == ADSL_Open_Rejected )
       {
	 DEBUG_CMD(DEBUG_LOG("ADSL: Line open - REJECTED\n"));
	 return(IX_ADSL_OPEN_FAILED);	  
       }
       else if ( status == ADSL_Open_Failed )
       {
	 DEBUG_CMD(DEBUG_LOG("ADSL: Line open - FAILED\n"));
	 return(IX_ADSL_OPEN_FAILED);
       }
     }

     IX_ADSL_OPEN_POLL_SLEEP;

   }
	
   /*
    * We got here so that means there was NO responce from
    * the modem.
    */
   return(rc);        
}


#if CPU == XSCALE
unsigned int IxADSL_CPE_OpenInternal( unsigned int lineNum, t_ADSL_LINE_TYPE lineType )
{
  IX_ADSL_enableModemLine_fp enableModemFns[ADSL_INVALID_MODE] = 
  {
    NT_ADSL_enableModemLine,
    NT_ADSL_enableModemLine_glite,
    NT_ADSL_enableModemLine_gdmt,
    NT_ADSL_enableModemLine_ansi,
    NT_ADSL_enableModemLine_multimode
  };

  unsigned char enable=ADSL_DYING_GASP_ENABLE;

  if(ixAdslEnableGasp[lineNum]==TRUE)
  {
	wrctrle(lineNum, &enable, ADSL_DYING_GASP_REG, 1); 
  }

  if (ixAdslPhyType[lineNum] != IX_ADSL_CPE)
  {
      return (IX_ADSL_UNSUPPORTED_MODE);
  }
  return IxADSL_LineOpenInternal( lineNum, IX_ADSL_CPE, lineType, enableModemFns);
}
#endif

unsigned int IxADSL_CO_OpenInternal( unsigned int lineNum, t_ADSL_LINE_TYPE lineType )
{
  IX_ADSL_enableModemLine_fp enableModemFns[ADSL_INVALID_MODE] = 
  {
    LT_ADSL_enableModemLine,
    LT_ADSL_enableModemLine_glite,
    LT_ADSL_enableModemLine_gdmt,
    LT_ADSL_enableModemLine_ansi,
    LT_ADSL_enableModemLine_multimode
  };

  unsigned char enable=ADSL_DYING_GASP_ENABLE;
 
  if(ixAdslEnableGasp[lineNum]==TRUE)
  {
 	wrctrle(lineNum,&enable, ADSL_DYING_GASP_REG, 1); 
  }

  if (ixAdslPhyType[lineNum] != IX_ADSL_CO)
  {
    return (IX_ADSL_UNSUPPORTED_MODE);
  }
  return IxADSL_LineOpenInternal( lineNum, IX_ADSL_CO, lineType, enableModemFns);
}


/****************************************************************************/
static void 
ixAdslLineSupervisoryTask(ixADSL_LineParams* pLineParams)
{
  adslTaskState_e state = ADSL_TASK_INIT;
  unsigned char lineState;
  static unsigned char lastLineState = -1;
  int downPollCount=0;
  unsigned retval = 0;

  unsigned int lineNum = pLineParams->lineNum;
  t_ADSL_PHY_TYPE phyType = pLineParams->phyType;

  /*
   * Only need to do work if ADMIN State is UP
   */

  if ( (!fwInitComplete) ||
       ((phyType != IX_ADSL_CO) && (phyType != IX_ADSL_CPE)) )
  {
      DEBUG_CMD(DEBUG_LOG( "Error: Attempt to execute ADSL Supervisory"
			   "Task on Uninitialised line!" ) );
      ix_ossl_thread_exit(0x0,0x0); /* Kill this task */
  }

  while(1)
  {

   switch (state )
   {
    case ADSL_TASK_INIT:
      
      downPollCount=0;

      DEBUG_CMD(DEBUG_LOG("ADSL Line Task Super Init State\n"));
      if (ixAdslState[lineNum].adminState== ADSL_ADMIN_UP )
      {
	 if (phyType == IX_ADSL_CPE) 
	 {
	     retval = IxADSL_CPE_OpenInternal(lineNum, ixAdslState[lineNum].lineType);
	 }
	 else if (phyType == IX_ADSL_CO) 
	 {
	     retval = IxADSL_CO_OpenInternal(lineNum, ixAdslState[lineNum].lineType);
	 }
	 if ( retval == IX_ADSL_OPEN_FAILED )
	 {
	     /*
	      * Attempt a close...
	      */
	     if (phyType == IX_ADSL_CPE)
	     {

		 /* If the CPE side fails it indicates the fact via
		  * an alarm */
		 ADSL_disableModemLine_orderly_shutdown(lineNum);
	     }
	     else if (phyType == IX_ADSL_CO)
	     {  
		 /* The CO does not need to alarm */
		 ADSL_disableModemLine(lineNum);
	     }
	     DEBUG_CMD(DEBUG_LOG("ADSL Line Task Super Init State: Close Attempted\n"));
	 }
	 else
	 { 
	     ADSL_TASK_SLEEP   
	 }
      }

      state = ADSL_LINE_OPEN_POLLING;
      break;


     case ADSL_LINE_OPEN_POLLING:

       if (ixAdslState[lineNum].adminState== ADSL_ADMIN_UP )
       {
	 /* Want the link to be up. */
          lineState = IxADSL_GetLineState(lineNum);
	 
	  if (lastLineState != lineState)
	  {
	      printf("ADSL Line Task : ADSL State Changed: %s\n",
		  listStateStrs[lineState]);
	      lastLineState = lineState;
	  }
	   
	  /* Check State */
          if (lineState == IX_ADSL_LINE_UP ||
	      lineState == IX_ADSL_LINE_WAIT_FOR_ACTIVATING ||
	      lineState == IX_ADSL_LINE_ACTIVATING )
	  {
	    /* Line is up/trying nothing to do. */
	  }
	  else
	  {
	    /* 
	     * Line is not up or trying to get up, so change state and 
	     * retry..
	     */

	     if ( downPollCount == ADSL_DOWN_DEBOUNCE_CNT)
	     {
	        state = ADSL_TASK_INIT;
		DEBUG_CMD(DEBUG_LOG("ADSL Line task: Down timeout " 
				    "return to Init state.\n" ));
	     }
  	     else
	     {
	        downPollCount++;
	     }
	  }
       }
       break;

     default:
        state = ADSL_TASK_INIT;
	break;
	

   } /* of switch */

   ADSL_TASK_SLEEP   
    
  } /* end of while */
}

/***************************************************************************/


/* This function is called from the IxAdsl_CO_Open() and IxAdsl_CPE_Open()
 * functions.  Keeping the functions external means that unused firmware 
 * objects are excluded by the linker, therefore keeping the final image
 * size to a minimum. */
unsigned int IxADSL_LineOpen( unsigned int lineNum,
			      t_ADSL_PHY_TYPE phyType,
			      unsigned char *initFirmware,
			      unsigned int initFwLen,
			      unsigned char *phyFirmware,
			      unsigned int phyFwLen,
			      t_ADSL_LINE_TYPE lineType)
{

    static ixADSL_LineParams ADSL_LineParams;

    ADSL_LineParams.phyType = phyType; 
    ADSL_LineParams.lineNum = lineNum;

  if ( (phyType != IX_ADSL_CO) && (phyType != IX_ADSL_CPE) )
  {
     return IX_ADSL_UNSUPPORTED_MODE;
  }

  ixAdslPhyType[lineNum]=phyType;
  fwInitComplete = 1;
  

  if ( ! ixADSL_moduleInit[lineNum] )
  {
    /* Dont attempt to download the code if already opened.. */
    if (  ixADSL_DownLoadDevice(lineNum,
				phyType,
				initFirmware,
				initFwLen,
				phyFirmware,
				phyFwLen) ) 
    {
      DEBUG_CMD(DEBUG_LOG("IxADSL_Line_Open: Device Download failed\n"));
      return(IX_ADSL_OPEN_FAILED);
    }
  }

  /*  
   * Check arguments, 
   */
  if ( lineType >= ADSL_INVALID_MODE ) 
  {
    return(IX_ADSL_UNSUPPORTED_MODE);
  }

  if( ixAdslState[lineNum].adminState == ADSL_ADMIN_UP)
  {
      return(IX_ADSL_OPEN_FAILED);
  } 

  ixAdslState[lineNum].adminState  = ADSL_ADMIN_UP;
  ixAdslState[lineNum].lineType = lineType;

  if ( ixADSL_moduleInit[lineNum] )
  {
    /* Try to resume the Line super task */
  }
  else
  {
    ixADSL_moduleInit[lineNum] = 1;

                           
    osError = ix_ossl_thread_create((ix_ossl_thread_entry_point_t)ixAdslLineSupervisoryTask,&ADSL_LineParams,&ixADSL_taskId);
   
    if (osError != IX_OSSL_ERROR_SUCCESS)
    {
	ixOsServLog(LOG_ERROR, "Error creating ADSL Supervisory task\n",0,0,0,0,0,0);
	return IX_FAIL;
    }
  }

  return(0);
}

/***************************************************************************/

static unsigned int 
IxADSL_LineClose(unsigned int lineNum)
{
  unsigned int rc=IX_ADSL_SUCCESS;
  unsigned int alcatelRC = 0; 

  /* Set the Admin status to down */
  ixAdslState[lineNum].adminState = ADSL_ADMIN_DOWN;

  /*
   * Close down the line.
   */
   alcatelRC =  ADSL_disableModemLine(lineNum);

   if ( alcatelRC == C_CONF_DATA_ACK)
   {
     DEBUG_CMD(DEBUG_LOG("ADSL: Line closed - success\n"));
     rc = IX_ADSL_SUCCESS;
   }
   else if ( alcatelRC == C_CONF_DATA_REJ )
   {
     DEBUG_CMD(DEBUG_LOG("ADSL: Line closed - failed rejected\n"));
     rc = IX_ADSL_CLOSE_FAILED;
   }
   else if ( alcatelRC == C_CONF_DATA_COMMPROBLEM)
   {
     DEBUG_CMD(DEBUG_LOG("ADSL: Line closed - failed\n"));
     rc = IX_ADSL_CLOSE_FAILED;
   }
   else if ( alcatelRC == C_CONF_DATA_ALREADY_DOWN )
   {
     DEBUG_CMD(DEBUG_LOG("ADSL: Line closed - already down\n"));
     rc = IX_ADSL_ALREADY_DOWN;
   }

   /*
    * Notify Callback that the line state is now down.
    */
   if ( rc == IX_ADSL_SUCCESS ) 
   {
      ixStateChanged(lineNum, IX_ADSL_LINE_DOWN);
   }

   return(rc);
}

unsigned int IxADSL_CO_LineClose(unsigned int lineNum)
{
  return IxADSL_LineClose(lineNum);
}

#if CPU == XSCALE
unsigned int IxADSL_CPE_LineClose(unsigned int lineNum)
{
  return IxADSL_LineClose(lineNum);
}
#endif

/***************************************************************************/

#if CPU == XSCALE
unsigned int IxADSL_GetLineState(unsigned int lineNum)
{
  unsigned char lineState;
  unsigned char buffer[1];
  bool     asyncResponce;
  IxadslAturPerfDataEntry *ixadslAturPerfDataEntry;
  IxadslAturPerfRetrainStats *ixadslAturPerfRetrainStats=NULL;
  
  /*
   * First read command reg, to see if Loss of Signal 
   */

  asyncResponce =  rdctrle(lineNum, &buffer[0],RXCOMMANDREG,1) ;
  if ( asyncResponce )
  {
    /* There was a command available */
    if ( buffer[0] == ADSL_SIGNAL_LOST )
    {

      DEBUG_CMD(DEBUG_LOG("!! ADSL SIGNAL LOST!\n"));

      /* Need to close the line from this side ,
       * so it can be reopened later 
       */
      /* ADSL_disableModemLine_orderly_shutdown(); */
       ADSL_disableModemLine(lineNum);
       ixStateChanged(lineNum, IX_ADSL_LINE_DOWN);
      
    }
    else if ( buffer[0] == ADSL_SUICIDE_REQUEST )
    {

      DEBUG_CMD(DEBUG_LOG("!! ADSL SUICIDE REQUEST !!\n"));
     /*   ADSL_disableModemLine_orderly_shutdown(); */
     ADSL_disableModemLine(lineNum);
     ixStateChanged(lineNum, IX_ADSL_LINE_DOWN);
    }

    return(IX_ADSL_LINE_DOWN);
  }
 
  /* 
   * Finally get modem state as normal
   */  
   
  if ( ADSL_getModemState(lineNum,&lineState) )
  {
    return(IX_ADSL_LINE_UNKNOWN_STATE); /* Error State. */
  }

  if (lineState == MODEM_STATUS_DOWN)
  { 
    ixStateChanged(lineNum,IX_ADSL_LINE_DOWN);
    return(IX_ADSL_LINE_DOWN);
  }
  else if ( lineState == MODEM_STATUS_WAIT_FOR_INIT )
    return(IX_ADSL_LINE_WAIT_FOR_ACTIVATING);
  else if ( lineState == MODEM_STATUS_INITIALIZING )
    return( IX_ADSL_LINE_ACTIVATING );
  else if ( lineState == MODEM_STATUS_UP )
  {
      /* 
       * We need to do a pragmatic check for Modem failures.
       * This should be captured by the modem state but there are scarce 
       * occasions when the ADSL_getModemState 'lies'. -DH 04/04/01
       */
      if((ixadslAturPerfDataEntry=ixAdslGetPerfData(lineNum)) != NULL)
      {
	  if(ixadslAturPerfDataEntry->IxadslAturPerfLoff == TRUE ||
	     ixadslAturPerfDataEntry->IxadslAturPerfLosf == TRUE ||
	     ixadslAturPerfDataEntry->IxadslAturPerfLcdf == TRUE ||
	     ixadslAturPerfDataEntry->IxadslAturPerfLomf == TRUE ||
	     ixadslAturPerfDataEntry->IxadslAturPerfLprf == TRUE)
	  {
	      if ((ixadslAturPerfRetrainStats = ixAdslPerfRetrainStatsGet(lineNum)) != NULL)
	      {
		  /*
		   * Update the stats which show the number of software re-trains and
		   * the cause of the re-trains.
		   */
		  ixadslAturPerfRetrainStats->IxadslAturPerfSwRetrainInitiated++;

		  if (ixadslAturPerfDataEntry->IxadslAturPerfLoff == TRUE)
		  { 
		      ixadslAturPerfRetrainStats->IxadslAturPerfLoffRetrainInitiated++;
		  }
		  if (ixadslAturPerfDataEntry->IxadslAturPerfLosf == TRUE)
		  { 
		      ixadslAturPerfRetrainStats->IxadslAturPerfLosfRetrainInitiated++;
		  }
		  if (ixadslAturPerfDataEntry->IxadslAturPerfLcdf == TRUE)
		  { 
		      ixadslAturPerfRetrainStats->IxadslAturPerfLcdfRetrainInitiated++;
		  }
		  if (ixadslAturPerfDataEntry->IxadslAturPerfLomf == TRUE)
		  { 
		      ixadslAturPerfRetrainStats->IxadslAturPerfLomfRetrainInitiated++;
		  }
		  if (ixadslAturPerfDataEntry->IxadslAturPerfLprf == TRUE)
		  { 
		      ixadslAturPerfRetrainStats->IxadslAturPerfLprfRetrainInitiated++;
		  }
	      }	
	      ixStateChanged(lineNum,IX_ADSL_LINE_DOWN);
	      return (IX_ADSL_LINE_DOWN);
	  }
      }  
   
    ixStateChanged(lineNum,IX_ADSL_LINE_UP);
    return( IX_ADSL_LINE_UP );
  }
  else
    return( IX_ADSL_LINE_UNKNOWN_STATE );
    

}
#endif
/* CPU == XSCALE */

void IxADSL_setVendorCode(unsigned int lineNum,
			  unsigned char itu_vendorid_countrycode,
			  unsigned char itu_vendorid_vendorcode_1,
			  unsigned char itu_vendorid_vendorcode_2,
			  unsigned char itu_vendorid_vendorcode_3,
			  unsigned char itu_vendorid_vendorcode_4,
			  unsigned char itu_vendorid_vendorspecific_1,
			  unsigned char itu_vendorid_vendorspecific_2)
{

   ixAdslVendorInfo[lineNum].itu_vendorid_countrycode = itu_vendorid_countrycode;
   ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_1 = itu_vendorid_vendorcode_1;
   ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_2 = itu_vendorid_vendorcode_2;
   ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_3 = itu_vendorid_vendorcode_3;
   ixAdslVendorInfo[lineNum].itu_vendorid_vendorcode_4 = itu_vendorid_vendorcode_4;
   ixAdslVendorInfo[lineNum].itu_vendorid_vendorspecific_1 = itu_vendorid_vendorspecific_1;
   ixAdslVendorInfo[lineNum].itu_vendorid_vendorspecific_2 = itu_vendorid_vendorspecific_2;


}



unsigned int IxADSL_SelfTest(unsigned int lineNum)
{
  unsigned int rc = IX_ADSL_SELFTEST_UNIMPLEMENTED;
 

#if 0
	/* Self test currently unimplemented in Alcatel Firmware */
	rc = ADSL_SelfTest_Short();
#endif


	return(rc);
}


unsigned int IxADSL_GetLineRateUpstream(unsigned int lineNum)
{

 unsigned int rc;
 T_LTadslFarEndLineOperData farOperData;

   rc = LT_ADSL_get_FarEndLineOperData(lineNum, & farOperData);

   if ( rc == C_CONF_DATA_ACK )
   {    
        return(farOperData.interleavedBitRate + farOperData.fastBitRate);
   }    
   else
   {
        return(0);
   }


}

unsigned int IxADSL_GetLineRateDownstream(unsigned int lineNum)
{
 unsigned int rc;
 T_LTadslNearEndLineOperData nearOperData;

   rc = LT_ADSL_get_NearEndLineOperData(lineNum, & nearOperData);

   if ( rc == C_CONF_DATA_ACK )
   {    
        return(nearOperData.interleavedBitRate + nearOperData.fastBitRate);
   }    
   else
   {    
        return(0);
   }

}

#if CPU == XSCALE
t_ADSL_PHY_TYPE
ixAdslPhyTypeGet(unsigned int lineNum){
    return ixAdslPhyType[lineNum];
}
#endif


void IxADSL_DumpStats (unsigned int lineNum)
{

 struct T_Controller_SW_version controllerSwVersion;
 unsigned char lineState;
 unsigned int rc;
 T_LTadslNearEndLineOperData nearOperData;
 T_LTadslFarEndLineOperData farOperData;
 T_adsl_def_bitmap_set *ixDefectBitmap=NULL; 
 IxadslAturPerfDataEntry *ixadslAturPerfDataEntry=NULL;
 IxadslAturPerfRetrainStats *ixadslAturPerfRetrainStats=NULL;

 printf("Intel Communications - Alcatel ADSL Driver\n\n");

 /* Get Controller Software version */

 if ( ADSL_get_controller_SWVersion(lineNum, &controllerSwVersion) != C_CONF_DATA_ACK )
 {
   printf("Get Controller SW Version Failed");
   return;
 }
 else
 {
   printf("Controller SW Version: %d.%d.%d\n",
		controllerSwVersion.number1,
		controllerSwVersion.number2,
		controllerSwVersion.number3);

 }

  lineState = IxADSL_GetLineState(lineNum);
  printf("ADSL Line State : %-32.32s\n",listStateStrs[lineState]);
  printf("ADSL line type: %d\n",ixAdslState[lineNum].lineType);

  if(ixAdslEnableGasp[lineNum] == TRUE)
  {
      printf("ADSL Dying Gasp is Enabled.\n");
  }

  /*
   * GetLine Statistics.
   */

  rc = LT_ADSL_get_NearEndLineOperData(lineNum, & nearOperData);
  
  if ( rc == C_CONF_DATA_COMMPROBLEM )
  {
      printf("ADSL Get near line stats failed");
  }
  else if ( rc == C_CONF_DATA_REJ)
  {
      printf("ADSL No Near end stats available: Modem not in showtime\n");
  }
  else if ( rc == C_CONF_DATA_ACK )
  {
   /* The data is good */
      if(ixAdslPhyType[lineNum]==IX_ADSL_CO){
	 printf("Adsl CO side modem\n");
      } else {
	  printf("Adsl CPE side modem\n");
      }


   printf("\nADSL Near End Operational Data\n");
   printf("  Upstream Relative Capacity Occupancy : %d %%\n",
	  nearOperData.relCapacityOccupationUpstr);
  /*  printf("  Attainable Bitrate                : %d kbits/sec\n",
      nearOperData.attainableBitrate);  */
   printf("  Noise Margin Upstream                : %d dB \n",
	  nearOperData.noiseMarginUpstr);
   printf("  Output Pwr Downstream                : %d dBm\n",
	  nearOperData.outputPowerDnstr);
   printf("  Attenuation Upstream                 : %d dB\n",
	  nearOperData.attenuationUpstr);
   printf("  Downstream Fast Bitrate              : %d kbits/sec\n",
	  nearOperData.fastBitRate);
   printf("  Downstream Interleaved Bitrate       : %d kbits/sec\n",
	  nearOperData.interleavedBitRate);
  /* Print Carrier load later */
  }

  rc = ADSL_def_bitmap_poll(lineNum, ixDefectBitmap);
  if( rc == C_CONF_DATA_ACK )
  {
      printf("\n  Near-end defect bitmap \t\t: 0x%x\n",ixDefectBitmap->near_end.status); 
  }

  if(ixAdslPhyType[lineNum]==IX_ADSL_CPE)
  {
   ixadslAturPerfDataEntry=ixAdslGetPerfData(lineNum);

   printf("\n  Loss of Frame (secs)  \t\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfLofs);
   printf("  Loss of Cell delineation (secs)\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfLcds);
   printf("  Loss of Signal (secs)  \t\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfLoss);
   printf("  Loss of Power (secs)  \t\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfLprs);
   printf("  Loss of Margin (secs)  \t\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfLoms);
   printf("  Errored seconds       \t\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfESs);
   printf("  HEC Errors            \t\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfHEC);
   printf("  FEC Errors            \t\t: %u\n",ixadslAturPerfDataEntry->IxadslAturPerfFEC);
  }

 rc = LT_ADSL_get_FarEndLineOperData(lineNum, & farOperData);   

 if ( rc == C_CONF_DATA_COMMPROBLEM )
 {
   printf("\nADSL Get Far line stats failed\n");
 }
 else if ( rc == C_CONF_DATA_REJ)
 {
   printf("\nADSL No Far end stats available: Modem not in showtime\n");
 }
 else if ( rc == C_CONF_DATA_ACK )
 {
   /* The data is good */
   printf("\nADSL Far End Operational Data\n");
   printf("  Downstream Relative Capacity Occupancy : %d %%\n",
	  farOperData.relCapacityOccupationDnstr);
   /* printf("  Attainable Bitrate                  : %d kbits/sec\n",farOperData.attainableBitrate);  */
   printf("  Noise Margin Downstream                : %d dB \n",
	  farOperData.noiseMarginDnstr);
   printf("  Output Pwr Upstream                    : %d dBm\n",
	  farOperData.outputPowerUpstr);
   printf("  Attenuation Downstream                 : %d dB\n",
	  farOperData.attenuationDnstr);
   printf("  Upstream Fast Bitrate                  : %d kbits/sec\n",
	  farOperData.fastBitRate);
   printf("  Upstream Interleaved Bitrate           : %d kbits/sec\n",
	  farOperData.interleavedBitRate);
 } 

  rc = ADSL_def_bitmap_poll(lineNum, ixDefectBitmap);
  if( rc == C_CONF_DATA_ACK )
  {
      printf("\n  Far-end defect bitmap  \t\t: 0x%x\n", ixDefectBitmap->far_end.status);
  }

  if(ixAdslPhyType[lineNum]==IX_ADSL_CPE)
  {
      if ((ixadslAturPerfRetrainStats = ixAdslPerfRetrainStatsGet(lineNum)) != NULL)
      {
	  if (ixadslAturPerfRetrainStats->IxadslAturPerfSwRetrainInitiated > 0)
	  {
              printf("\nADSL re-train initiated %u time(s) due to the following:\n",
		     ixadslAturPerfRetrainStats->IxadslAturPerfSwRetrainInitiated);
              printf("  Loss of Frame Failure detected \t\t: %u time(s)\n",
	   	     ixadslAturPerfRetrainStats->IxadslAturPerfLoffRetrainInitiated);
              printf("  Loss of Cell Delineation Failure detected \t: %u time(s)\n",
		     ixadslAturPerfRetrainStats->IxadslAturPerfLcdfRetrainInitiated);
              printf("  Loss of Signal Failure detected \t\t: %u time(s)\n",
		     ixadslAturPerfRetrainStats->IxadslAturPerfLosfRetrainInitiated);
              printf("  Loss of Power Failure detected \t\t: %u time(s)\n",
		     ixadslAturPerfRetrainStats->IxadslAturPerfLprfRetrainInitiated);
              printf("  Loss of Margin Failure detected \t\t: %u time(s)\n",
		     ixadslAturPerfRetrainStats->IxadslAturPerfLomfRetrainInitiated);
          }
      }
  }

  printf("\n\n\n");
}


void ixAdslShow(unsigned int lineNum)
{
  IxADSL_DumpStats(lineNum);
}

void ixAdslStatsShow(unsigned int lineNum)
{
  IxADSL_DumpStats(lineNum);
}

PRIVATE int
ixAdslCallbackRegSend(unsigned int lineNum,
		      IX_ADSL_LINE_CHANGE_FNC callback_fn, 
		      t_ADSL_LINE_STATE linkState, 
		      int sendOrReg)
{
    static IX_ADSL_LINE_CHANGE_FNC lineStateCallbacks[NUM_ADSL_LINES][IXADSL_SIZEOF_CALLBACK_LIST];
    static unsigned int ixAdslLineStateCallbackPtr[NUM_ADSL_LINES];

    unsigned int funcPtr = 0;
    int i,j;
    
    for(i = 0; i < NUM_ADSL_LINES; i++)
    {
        ixAdslLineStateCallbackPtr[i] = 0;
        for(j = 0; j < IXADSL_SIZEOF_CALLBACK_LIST; j++)
	{
	    lineStateCallbacks[i][j] = NULL;
	}
    }
    
    if(sendOrReg == IXADSL_CALLBACK_REG)
    {
	if((callback_fn == NULL) ||
	   (ixAdslLineStateCallbackPtr[lineNum] == IXADSL_SIZEOF_CALLBACK_LIST))
	{
	    return IX_FAIL;
	}
	else if(lineStateCallbacks[ixAdslLineStateCallbackPtr[lineNum]] == NULL)
	{
	    lineStateCallbacks[lineNum][ixAdslLineStateCallbackPtr[lineNum]] = callback_fn;
	    ixAdslLineStateCallbackPtr[lineNum]++;
	    return IX_SUCCESS;
	}
	return IX_FAIL;
    }
    else if(sendOrReg == IXADSL_CALLBACK_SEND)
    {
	 if(ixAdslLineStateCallbackPtr[lineNum] == funcPtr)
	 {
	     return IX_SUCCESS;
	 }

	 for( ;funcPtr < ixAdslLineStateCallbackPtr[lineNum]; funcPtr++)
	 {
	     if(!lineStateCallbacks[lineNum][funcPtr]) 
	     {
		 return IX_FAIL;
	     }
	     else
	     {
		 lineStateCallbacks[lineNum][funcPtr](lineNum, linkState);
	     }
	 }
	 return IX_SUCCESS;
    }
    /* Function control should never reach this point */
    return IX_FAIL;
}

void ixStateChanged(unsigned int lineNum, t_ADSL_LINE_STATE state  )
{

  static t_ADSL_LINE_STATE prev_state = IX_ADSL_LINE_DOWN;
  
  if ( prev_state != state )
  {

      if((ixAdslCallbackRegSend(lineNum,
				NULL, 
				state, 
				IXADSL_CALLBACK_SEND)) != IX_SUCCESS)
      { 
      }

  }

  prev_state = state;

}

#if CPU == XSCALE
unsigned int IxADSL_LineStateReg(unsigned int lineNum, IX_ADSL_LINE_CHANGE_FNC callback_fn)
{
    
    return ixAdslCallbackRegSend(lineNum, callback_fn, IX_ADSL_INVALID_STATE, IXADSL_CALLBACK_REG);
}
#endif

/*
 * Function to enable 'Dying Gasp'. This feature is off by default on the
 * Alcatel Phy.
 */
void IxADSL_EnableDyingGasp(unsigned int lineNum)
{
    ixAdslEnableGasp[lineNum]=TRUE;
}

/***************************************************************************/


/* Dummy callback debug function for debug */

#if 1
void ixAdslTestCallback(unsigned int lineNum, t_ADSL_LINE_STATE lineState)
{

  if ( lineState == IX_ADSL_LINE_DOWN )
    printf("\nixAdslTestCallback TEST: :Line Down \n");
  if ( lineState == IX_ADSL_LINE_UP )
    printf("\nixAdslTestCallback TEST: :Line Up \n");

}

void
adsltest(unsigned int lineNum)
{
 IxADSL_LineStateReg(lineNum, ixAdslTestCallback);
}  

#endif
