 /* 
 * FileName:    ixAdslSoftReset.c
 * Author: Intel Corporation
 * Created:     22-July-2002
 * Description: 
 *    
 * Source file for Alcatel software reset to fix SCR 921
 *
 * Copyright 2000, Intel Corp.  All rights reserved. No part 
 * of this document may be reproduced, transmitted, transcribed, stored in 
 * a retrieval system, or compiled into computer source code, in any form 
 * or by any means, without the prior written consent of Intel Corp.
 *
 *
 */

#include "ixAdslSoftReset.h"

void IxADSLSoftReset(void )
{


    GPIOLineConfig( GPIO_PIN_12,  GPIO_OUT);
    GPIOLineSet( GPIO_PIN_12,  GPIO_HIGH); 
    MicrosecDelay(1); 
    GPIOLineSet( GPIO_PIN_12,  GPIO_LOW);
    MicrosecDelay(500);
    GPIOLineSet( GPIO_PIN_12,  GPIO_HIGH); 
    MicrosecDelay(1); 

}

/******************************************************************************
*
* GPIOLineConfig - configure a GPIO pin
*
* This routine configures a GPIO pin/line for use as either an input or output.
*
* RETURNS: OK when line is successfully configured; ERROR if the line number
*	   passed is out of range, or an unknown style is requested for an
*	   interrupt. 
*/
IX_STATUS GPIOLineConfig (UINT8 lineNo, UINT32 style)
    {
    volatile UINT32 enableReg;
    volatile UINT32 styleReg;
    volatile UINT32 intReg;
    UINT32 intStyle;

     GPIO_KEYDECLARE;

    if (lineNo >  GPIO_PIN_MAX)
        return (ERROR);

    GPIO_REG_READ( GPIO_GPOER, enableReg);

    if (style &  GPIO_OUT)
        {
	enableReg &= ~(0x1 << lineNo);
	}
    else if (style &  GPIO_IN)
	{	    

	switch (style &  GPIO_INTSTYLE_MASK)
	    {
	    case ( GPIO_ACTIVE_HIGH):
	        intStyle =  GPIO_STYLE_ACTIVE_HIGH; break;
	       
	    case ( GPIO_ACTIVE_LOW):
		intStyle =  GPIO_STYLE_ACTIVE_LOW; break;

	    case ( GPIO_RISING_EDGE):
		intStyle =  GPIO_STYLE_RISING_EDGE; break;

	    case ( GPIO_FALLING_EDGE):
		intStyle =  GPIO_STYLE_FALLING_EDGE; break;

	    case ( GPIO_TRANSITIONAL):
		intStyle =  GPIO_STYLE_TRANSITIONAL; break; 

	    default:
		intStyle =  GPIO_STYLE_ACTIVE_HIGH; break;
	    }

	enableReg |= (0x1 << lineNo);

         GPIO_INTLOCK (intKey);

	if(lineNo >= 8)	/* pins 8-15 */
	    {
	    lineNo -= 8;
	    intReg =  GPIO_GPIT2R;
	    }
	else		/* pins 0-7 */
	    {
	    intReg =  GPIO_GPIT1R;
	    }

        /* Clear the style for the appropriate pin */
	 GPIO_REG_READ(intReg, styleReg);
	styleReg &= ~( GPIO_STYLE_CLEAR << (lineNo* GPIO_STYLE_SIZE));
	 GPIO_REG_WRITE(intReg, styleReg);

	/* Set the new style */
	 GPIO_REG_READ(intReg, styleReg);
	styleReg |= (intStyle << (lineNo* GPIO_STYLE_SIZE));
	 GPIO_REG_WRITE(intReg, styleReg);

	 GPIO_INTUNLOCK (intKey);
	
	}
    else	/* Unknown style options */
	{
	return (ERROR);
	}

     GPIO_REG_WRITE( GPIO_GPOER, enableReg);
    return (OK);
    }

/******************************************************************************
*
* GPIOLineSet - set the value of a GPIO pin configured for output
*
* RETURNS: OK when pin is successfully set; ERROR if the pin number
*	   passed is out of range.
*/
IX_STATUS GPIOLineSet (UINT8 lineNo,  GPIO_SIG value)
    {
    if (lineNo >  GPIO_PIN_MAX)
	return (ERROR);

    if (value ==  GPIO_HIGH)
       {
        GPIO_LINE_SET(lineNo);
       }
    else if (value ==  GPIO_LOW)
       {
        GPIO_LINE_CLEAR(lineNo);
       }

    return (OK);
    }

/*****************************************************************************
 * MicrosecDelay - Microsecond delay
 *
 * RETURNS: N/A
 *
 */

void MicrosecDelay(int microseconds)
{
    UINT32 delay = 0;
    UINT32 lastTimestamp = *((volatile int *)OSTS);
    UINT32 currentTimestamp;
    UINT32 delta;

    while (delay < microseconds * PERIPHERAL_BUS_CLOCK)
    {
        currentTimestamp = *((volatile int *)OSTS);
        delta = currentTimestamp > lastTimestamp ? currentTimestamp - lastTimestamp : 0xffffffff - lastTimestamp + currentTimestamp;

        delay += delta;

        lastTimestamp = currentTimestamp;
    }
}
