/** @file
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
*/
/* 
 * FileName:    ixAdslswDnld.c
 * Author: Intel Corporation
 * Created:     29-APR-1999
 *
 * Revised for ixp425: 
 * Date: 	3-Apr-2002
 *
 * Description: 
 *    Software (firmware) download code for ADSL driver.
 * 
 * Design Notes:
 *
 *
 * Copyright 2000, Intel Corporation All rights reserved. No part 
 * of this document may be reproduced, transmitted, transcribed, stored in 
 * a retrieval system, or compiled into computer source code, in any form 
 * or by any means, without the prior written consent of Intel Corp.
 *
 */

#include <IxOsServices.h>

#include "IxAdsl.h"

#include "ixCtrle_c.h"
#include "ixCtrlereadwrite_p.h"
#include "ixCtrleaddressmap.h"

#include "ixAdsl_p.h" 


extern t_ADSL_PHY_TYPE ixAdslPhyType[8]; /* CO or CPE ... specified externally by user */

#define  MODULE_SIZE 0x200000
static unsigned char ixAdslAlcatelLoadBoot(unsigned int lineNum, 
					   unsigned char* initFirmware,
					   unsigned int initFwLen);
static unsigned char ixAdslAlcatelLoadFirmware(unsigned int lineNum,
					       unsigned char* phyFirmware,
					       unsigned int phyFwLen);

static unsigned char dnld(unsigned int lineNum,
			  unsigned char *localp,
			  unsigned int len,
			  unsigned char *remotep, 
			  unsigned char *jmpp );

static unsigned char writeblock(unsigned int lineNum,
				unsigned char *locp,
				unsigned char *remp,
				unsigned int number);


/* 
 * Entry point to Download the Alcatel ADSL controller.
 */
unsigned char 
ixADSL_DownLoadDevice(unsigned int lineNum,
		      t_ADSL_PHY_TYPE phyType,
		      unsigned char* initFirmware,
		      unsigned int initFwLen,
		      unsigned char* phyFirmware,
		      unsigned int phyFwLen)
{
  unsigned char rxAv,rxCom,txCom,txAv;
  unsigned int tmocnt;
  unsigned char initResult = 0;

#if ! (CTRLE_USE_SERIAL)

  if (phyType != ixAdslPhyType[lineNum])
  {
      /* Incorrect PHY Firmware registered */      
      printf("Incorrect PHYType  Registered");
      return (1);
  }

  if ( ixAdslAlcatelLoadBoot(lineNum,initFirmware, initFwLen) ) 
  {
    /* Failed */
        return(1);
  }

 /*
  * Wait for Boot to complete
  */

  while (1) {
      initResult = CTRLE_READ_REG(CTRLE_RX_REG_ADDR(lineNum));
      if (INIT_COMPLETE == initResult) break;
  }

  /* Clear all registers.. */

  rxCom = CTRLE_RX_REG(lineNum);
  txCom = CTRLE_TX_REG(lineNum);
  rxAv = CTRLE_RX_COM_AV(lineNum);
  txAv = CTRLE_TX_REG(lineNum);

  tmocnt= (unsigned int)1000;
  while (  (CTRLE_RX_COM_AV(lineNum) & 0x1) == 0x1 && tmocnt > 0 )
  {
      tmocnt--;
#if DEBUG
      printf("+");
#endif
      ixOsServTaskSleep(10);
  }

  if (  tmocnt == 0 ) 
  {
#if DEBUG
    printf("Timeout between boot load and application load.\n");
#endif
    return(1); /* Failed */
  }


  /* Clear all registers */
  rxCom = CTRLE_RX_REG(lineNum);
  txCom = CTRLE_TX_REG(lineNum);
  rxAv = CTRLE_RX_COM_AV(lineNum);
  txAv = CTRLE_TX_REG(lineNum);  
 
  if (ixAdslAlcatelLoadFirmware(lineNum, phyFirmware, phyFwLen))
  {
      /* Failed */
      return(1);
  }

  /* Ok. */

  IX_ADSL_DNLD_SLEEP


  return(0);

#else /* USE SERIAL */

  /* Serial Download not supported */

 return(1);
#endif

}


#if ! (CTRLE_USE_SERIAL)

static unsigned char 
ixAdslAlcatelLoadBoot(unsigned int lineNum, 
		      unsigned char* initFirmware,
		      unsigned int initFwLen )
{
   if ( (initFwLen == 0) || (initFirmware == NULL) )
   {
       /* Bad firmware specified */
       return (1);
   }

#if DEBUG
   printf("Init Load: %d bytes\n", initFwLen );
#endif
   


   /*
    * Download the Init code.
    */

  return(dnld( lineNum,
	       initFirmware,
	       initFwLen,
	       0,
	       0));

}

static unsigned char 
ixAdslAlcatelLoadFirmware( unsigned int lineNum,
			   unsigned char* phyFirmware,
			   unsigned int phyFwLen)
{

   if ( (phyFwLen == 0) || (phyFirmware == NULL) )
   {
       /* Bad firmware specified */
       return (1);
   }

#if DEBUG
   printf("Application Load: %d bytes\n",
	  phyFwLen );
#endif

   /*
    * Download the Init code.
    */
   return(dnld(lineNum,
	       phyFirmware,
	       phyFwLen,
	       0,
	       0));
}


static unsigned char 
dnld(unsigned int lineNum,
     unsigned char *localp,
     unsigned int len,
     unsigned char *remotep, 
     unsigned char *jmpp )
{

 unsigned char noproblem=1;
 int offset=0;
 int i;

/*********download***************************/  

  while (len>0 && noproblem)
  { 

#if DEBUG
     printf(".");
#endif
     printf("");
     printf("");
     printf("");
     if (len >= 0x80)
     { noproblem = writeblock(lineNum,localp+offset,remotep+offset,0x80);
	 ixOsServTaskSleep(10);
         offset+=0x80;
	 len-=0x80;
     }
     else
     { 
       noproblem = writeblock(lineNum,localp+offset,remotep+offset,len);
       len=0;
     }
    
  }

  if ( ! noproblem )
  {
    printf("Download failed..");
    return(1);
  }

  /*
   * Jump to relevant code.
   */

  /***if*download*ok***jump*****(no acknowledge)********/
  if (noproblem)
  {
    for( i=0;i<4;i++)
    {
      *( unsigned char *)(CTRLE_THUNDER_CTRLE_BASE_ADDR+ lineNum * MODULE_SIZE + ((BLCKBADDRREG+i)<< CTRLE_ADDR_SHIFT) ) =
	((unsigned int)jmpp) % 0x100; 
        jmpp = ((unsigned char*)(((unsigned int)jmpp)/0x100)); 
    }

    *(unsigned char *)( CTRLE_TX_REG_ADDR(lineNum) )= MODEM_JUMP_ADDR;
  }
  
  return(0); /* o.k */

}


static unsigned char 
writeblock( unsigned int lineNum,
	    unsigned char *locp,
	    unsigned char *remp,
	    unsigned int number)
{  
    unsigned char ok=1;
    unsigned int rempint=(unsigned int)remp;
    unsigned int lnumber=number;
    unsigned int tmocnt;
    unsigned int i;
    unsigned char rxResp;

/********issue dump block command****************/
    for(i=0;i<4;i++)
    {
	*(unsigned char *)(CTRLE_THUNDER_CTRLE_BASE_ADDR+lineNum*MODULE_SIZE+(BLCKBADDRREG+i))=rempint % 0x100; rempint/=0x100;
    }
    for(i=0;i<2;i++)
    {
	*(unsigned char *)(CTRLE_THUNDER_CTRLE_BASE_ADDR+lineNum*MODULE_SIZE+(BLCKLENREG+i)) = lnumber % 0x100; lnumber=lnumber / 0x100;
    }

    /********************************************/
    for(i=0;i<number;i++)
      *(unsigned char *)(CTRLE_THUNDER_CTRLE_BASE_ADDR+lineNum*MODULE_SIZE+(BLCKDATAREG+i)) = *(locp+i);
#if 0
    **these 2 lines time critical
    **therefore implemented in assembler:
    *********************************************/
    copyasm(locp,number);
#endif

    *(unsigned char *)(CTRLE_TX_REG_ADDR(lineNum)) = MODEM_DUMP_BLOCK;

/*********wait for acknowledge*******************/
    tmocnt=1000;
    while (  (CTRLE_RX_COM_AV(lineNum) & 0x1) != 0x1 && tmocnt > 0 )
    {
      tmocnt--;
    }

    if ( tmocnt == 0 )
    {
      /* Try a slower delay since quick polling did not do it
       */
        tmocnt = 100;
        while (  (CTRLE_RX_COM_AV(lineNum) & 0x1) != 0x1 && tmocnt > 0 )
	{
#if DEBUG
	  printf("!");
#endif
	  tmocnt--;
	  ixOsServTaskSleep(10);
	}
    }

    if (tmocnt==0) ok=0;

    if (ok ) /* No timeout. */
    {
      /* Check return code */
      rxResp = (*(unsigned char *)( CTRLE_RX_REG_ADDR(lineNum) ));
      if ( rxResp  != ADSL_Modem_Block_Dump_Ack )
      {
	ok=0;
      }
    }
    return ok;
}



#endif /* ! USE_SERIAL  */






