/** @file
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 1999-2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
*/

/*
 * Modification history
 *
 * Date       Author              Description
 * -------------------------------------------------------------
 * 04/29/99   pjb  	  Initial release.
 * 04/03/02   so	  Modified for ixp425
 * -------------------------------------------------------------
 */



#include <stdio.h>
#include <ioLib.h>
#include <taskLib.h>


#include "ixCtrlereadwrite.h"

#include "ixCtrlereadwrite_p.h"
#include "ixCtrleaddressmap.h"
#include "ixDetonator.h"
#include "ixp425.h"

#define EXP_CS1_REG  (IXP425_EXP_CONFIG_BASE + 0x04)
#define  MODULE_SIZE 0x200000

/**************************************************************

 #####     ##    #####     ##    #       #       ######  #
 #    #   #  #   #    #   #  #   #       #       #       #
 #    #  #    #  #    #  #    #  #       #       #####   #
 #####   ######  #####   ######  #       #       #       #
 #       #    #  #   #   #    #  #       #       #       #
 #       #    #  #    #  #    #  ######  ######  ######  ######

 CTRLE

 **************************************************************/

#if ! (CTRLE_USE_SERIAL)


void ixAdsldumpMem(unsigned int lineNum)
{

  unsigned char tx,rx,txcom,rxcom;
  unsigned int x;
  /*
   */
  rxcom = CTRLE_RX_COM_AV(lineNum);
  txcom = CTRLE_TX_COM_AV(lineNum);

  tx = CTRLE_TX_REG(lineNum);
  rx = CTRLE_RX_REG(lineNum);
 
 
  

  printf("ver 1\n");
  printf("tx : 0x%02x rx: 0x%02x txcom : 0x%02x rxcon : 0x%02x \n",tx,rx,txcom,rxcom);
  printf("Config Data *****************\n");

  for (x=0 ; x < 0x5F ; ++x )
  {
    printf("Data: 0x%02x : 0x%02x\n",x+0x40, CTRLE_READ_REG( CTRLE_CONFIG_DATA_ADDR(lineNum) + x ));

  }

  printf("Received Status *****************\n");
  for (x=0 ; x < 0x100 ; ++x )
  {
    printf("Data: 0x%02x : 0x%02x\n",x+0x100, CTRLE_READ_REG( CTRLE_MODEM_STATUS_ADDR(lineNum) + x ));
  }

}

bool wrctrle(unsigned int lineNum, const unsigned char* pval,const unsigned int address,const unsigned	int number)
{
   bool	rc=TRUE; /* Success by default.	*/
   unsigned int  addrOffset;
   volatile char *dataAddr;
   unsigned int numBytes=number;
 
   *((unsigned long *) EXP_CS1_REG) = 0xbfff3c03; 

   /*
    * Block on ctrle software seamphore 
    */

    
   /* 
    * Make sure Tx Command Register is available.
    */

   
  
   /*
    * Get Hardware semaphore 
     

   /*
    * Write Data to CTRLE Data Buffer
    */

   addrOffset = address;
  
   while ( numBytes )
   {  
     
     dataAddr = (volatile char *)(CTRLE_THUNDER_CTRLE_BASE_ADDR + lineNum * MODULE_SIZE + ( addrOffset << CTRLE_ADDR_SHIFT)) ;
     *dataAddr = *pval++;
     addrOffset++;
     numBytes--;
   }

   /*
    * Release Hardware semaphore 
    */


   /*
    * Release ctrle software semaphore.
    */


    return(rc);
}


/*
 * rdcrtle - parallel
 */

bool rdctrle(unsigned int lineNum, unsigned char *buffer,const	unsigned int address,const unsigned int	number)
{

  bool rc=TRUE; /* Success by default */
  unsigned int  addrOffset;
  volatile char *dataAddr;
  int x,rxCmdAvail;
  unsigned int numBytes=number;
  
   *((unsigned long *) EXP_CS1_REG) = 0xbfff3c03; 

   
  if ( address == RXCOMMANDREG && numBytes == 1)  
  {
    /*
     * This is a special register, RXCOM
     */
    
    /* Check if there is a new value waiting */
    rxCmdAvail = FALSE;

    x = CTRLE_WAIT_FOR_CMD_REG_CNT;
    while ( (x) && (! rxCmdAvail) ) 
    {
      if ( CTRLE_RX_COM_AV(lineNum) )
      {
	rxCmdAvail = TRUE;

      }
      --x;
      CTRLE_SLEEP;
    }

    if ( rxCmdAvail )
    {
      /* There is an Rx command available. */

      *buffer = CTRLE_RX_REG(lineNum);
      return(rc);
    }
    else
    {
      /* Error occured, stop waiting for command */
      return(FALSE);
    }

  }  /* End of RxCommand Register read */


  /*
   * Aquire hardware semaphore.
   */


  /*
   * Read dual-port register memory
   */

   addrOffset = address;
  
   while ( numBytes )
   {  
     
     dataAddr = (volatile char *)(CTRLE_THUNDER_CTRLE_BASE_ADDR + lineNum * MODULE_SIZE + (addrOffset << CTRLE_ADDR_SHIFT)) ;
     *buffer++ = *dataAddr ; 
     addrOffset++;
     --numBytes;
   }

  
   /*
    * Return hardware semaphore.
    */

 
  return(rc);

}









#endif


/******************************************************
  ####   ######  #####      #      ##    #
 #       #       #    #     #     #  #   #
  ####   #####   #    #     #    #    #  #
      #  #       #####      #    ######  #
 #    #  #       #   #      #    #    #  #
  ####   ######  #    #     #    #    #  ######
 
 CTRLE

  *****************************************************/

#if  CTRLE_USE_SERIAL

#define	 CTRLE_SEND_ESCAPED_BYTE( octet	) ctrle_send_escaped_byte(octet)


t_ctrle_serial_stats ctrleStats	; /* Init to zero */

/* File Descriptor For File IO */
int  ioFD=-1;



bool wrctrle(const char* pval,const unsigned int address,const unsigned	int number)
{


  /* 
   * Serial Frame Format. 
   *
   *   +-----------+
   *   | Sync	   |  -	A5
   *   +-----------+
   *   |RRRR| CMD  |  -	0x0 Read ; 0x1 Write ; 0xC interrupt.
   *   +-----------+
   *   |RRRR|RRR|a8|
   *   +-----------+
   *   | a7-a0	   | Address A8-A0 0x0..0x1FF
   *   +-----------+
   *   | byte count| Number of Data bytes, 0-64
   *   +-----------+
   *   |	   |
   *   | Data	   |
   *   | 0-64 bytes|
   *   +-----------+
   *   | LCR	   |  EXOR of previous bytes excluding sync.
   *   +-----------+
   *   | End Sync  | - 5A
   *   +-----------+
   *
   */


   bool	rc=TRUE; /* Success by default.	*/
   uint8 frameBuffer[CTRLE_BUFFER_SIZE]; /* Frame Buffer */
   uint8 n;


   frameBuffer[0] = CTRLE_WRITE_CMD;
   frameBuffer[1] = address>=256 ? (uint8)0x01 : (uint8)0x00;
   frameBuffer[2] = (uint8)(address & 0x00FF);
   frameBuffer[3] = (uint8) number ;  /* This is the actual number of data bytes,  */


   /*
    * Note: All	bytes a	copied and then	escaped, This is very inefficent,
    * Later the	Bytes shall be escaped as they are copied.
    */

   for ( n = 0 ; n < number && n < CTRLE_MAX_BYTES ; ++n)
   {
	frameBuffer[4+n] = (uint8)*pval++;
   }

   rc =	ctrle_send_frame(frameBuffer,number+4);

   return(rc);
}


/*
 * rdcrtle - serial */

bool rdctrle(char *buffer,const	unsigned int address,const unsigned int	number)
{

  bool rc=TRUE; /* Success by default */
  uint8	i;
  uint8	frameBuffer[CTRLE_BUFFER_SIZE];	/* Frame Buffer	*/


  /* Clear buffer */
    for	( i=0; i< number ; ++i )
  {
	buffer[i]= (uint8) 0;
  }

  /*
   * Perform a Serial Read request to the address in question.
   */


   frameBuffer[0] = CTRLE_READ_CMD;
   frameBuffer[1] = address>=256 ? (uint8)0x01 : (uint8)0x00;
   frameBuffer[2] = (uint8)(address & 0x00FF);
   frameBuffer[3] = (uint8) number ;  /* This is the actual number of data to read bytes,  */


   /*
    * Send Frame.
    */
   rc =	ctrle_send_frame(frameBuffer,4);

  /*
   * Get responce characters.
   */

   rc =	crtle_get_rx_frame(frameBuffer);


  /*
   * Decode responce into line codes.
   */
  
  printf("\n\n************\n* Need to VERIFY decode line responces \n******");

  if ( frameBuffer[3] < RECTRLE_MAX_DATA )
  /* copy the date in */
  {
     for(i=0;i<frameBuffer[3];i++)
       *buffer++ = frameBuffer[i+4];
     rc = TRUE; /* Success */
  }
  else
    rc = FALSE;

  return(rc);

}


/*
 *
 * CTRLE  : send a frame,
 *	Adds: Start and	end of frame sync pulses.
 *	Adds: XOR checksum to data.
 *	      Escapes any required data.
 *
 */

bool ctrle_send_frame(unsigned char *framePtr,unsigned int numberBytes)
{

  uint8	runningXOR=0; /* Running XOR value */
  unsigned int i;
  bool	rc=TRUE; /* Success by default */

   /*
    * Check arguments.
    */

/*    CTRLE_ASSERT(framePtr != NULL)
   CTRLE_ASSERT(numberBytes != 0 )
*/

   CTRLE_SEND_BYTE(START_SYNC);

   /*
    * Send data,  Escaped if necessary.
    */

   for ( i=0; i< numberBytes; ++i)
   {

	ctrle_send_escaped_byte(*framePtr);
	RUNNING_XOR(*framePtr++);
   }

   CTRLE_SEND_BYTE(runningXOR);
   CTRLE_SEND_BYTE(END_SYNC);


   return(rc);
}



bool ctrle_send_escaped_byte( uint8 octet )
{


   /*
    * Escape all the required bytes (except start and end sync )
    * 0xA5 -> 0xF7 0x25
    * 0x5A -> 0xF7 0x52
    * 0xF7 -> 0xF7 0xF7
    * 0x00 -> 0xF7 0x88
    */


  bool rc=TRUE;

     if	(octet	== 0xA5	)
     {
	 CTRLE_SEND_BYTE( 0xF7 );
	 CTRLE_SEND_BYTE( 0x25 );
     }
     else if ( octet ==	0x5A )
     {
	 CTRLE_SEND_BYTE( 0xF7 );
	 CTRLE_SEND_BYTE( 0x52 );
     }
     else if ( octet ==	0xF7 )
     {
	 CTRLE_SEND_BYTE( 0xF7);
	 CTRLE_SEND_BYTE( 0xF7);
     }
     else if ( octet ==	0x00 )
     {
	 CTRLE_SEND_BYTE( 0xF7);
	 CTRLE_SEND_BYTE( 0x88);
     }
     else
     {
	CTRLE_SEND_BYTE(octet);
     }

  return (rc);
}

typedef	enum t_ctrle_rx_state {
		WAIT_FOR_SYNC,
		RX_FRAME_BYTE,
		RX_ESCAPED_BYTE
} t_ctrle_rx_state;

testIn()
{
  unsigned char buffer[256];

  while(1)
 {
    crtle_get_rx_frame(buffer);
 }
}

bool crtle_get_rx_frame(uint8 *frameBuffer)
{

   /*
    * Read a frame, a message has been sent.
    */

   t_ctrle_rx_state  rx_state=WAIT_FOR_SYNC;

   uint8 frameAvail=FALSE;
   uint8 giveUP=FALSE;
   uint8 rxChar;
   uint8 numBytes=0;
   uint8 escapedMode=FALSE;
   uint8 runningXOR=0;

   printf("In crtle_get_rx_frame\n");

   while ( !( frameAvail || giveUP ) )
   {

    /* Get a character */


      if ( CTRLE_GETCHAR(rxChar) == -1 )
      { 
	 /* Error reading serial device. */
	 giveUP = TRUE;
	 ctrleStats.rxReadErrors++;
         break;
      }

      DEBUG_CMD(printf("Char Rx:0x%02X\n",rxChar));

      ctrleStats.rxBytesRx++;

      switch (rx_state)
      {
	case  WAIT_FOR_SYNC:
		if ( rxChar == START_SYNC)
		{
		   /* Start of Frame detected */
		   rx_state = RX_FRAME_BYTE;
		   ctrleStats.rxStartFrameRx++;
		   DEBUG_CMD(printf("Sync aquired"));
		}
		break;

	case  RX_FRAME_BYTE:

		/*
		 * Process frame characters,
		 */

		DEBUG_CMD(printf("In Rx normal byte receive\n "));


		/* Check if End	of Frame reached. */
		if ( rxChar == END_SYNC	)
		{
		    /* Set next	state. */
		    rx_state = WAIT_FOR_SYNC;
		    ctrleStats.rxEndFrameRx++;

		   /*
		    * Check frame, and return;
		    */
		   if ( runningXOR == 0)
		   {
		     frameAvail =	TRUE;
		     ctrleStats.rxFrameRx++;
		   }
		   else
		   {
		     frameAvail = FALSE;
		     giveUP=TRUE;
		     ctrleStats.rxCRCErrors++;
		   }

		   DEBUG_CMD(printf("End of Frame\n"));
		}
 
		/*
		 * Check if characer is	escaped.
		 */

		else if	( rxChar == ESCAPE_CHAR	)
		{
		   /* Change state to Rx Escaped character */
		   rx_state = RX_ESCAPED_BYTE;
		   DEBUG_CMD(printf("Escape Byte\n"));


		}
		else /*	Normal character arrived. */
		{

		  frameBuffer[numBytes++] = rxChar;
		  RUNNING_XOR(rxChar); 
		  DEBUG_CMD(printf("Added frame Byte\n"));

		  if ( numBytes	>= CTRLE_BUFFER_SIZE - 1) /* Leave some	head room */
		  {
		     /*	This frame is too big... drop it. */
		     DEBUG_CMD(printf("Ctrle Large Frame Dropped\n"));
		     ctrleStats.rxFrameDrops++;
		     giveUP = TRUE;
		     break;
		  }
		}
		break;

	case RX_ESCAPED_BYTE:

		DEBUG_CMD(printf("In escaped  Byte"));

		if ( rxChar == 0x88 )
		{
		  frameBuffer[numBytes++] = 0x00;
		  RUNNING_XOR(0x00); 
		}
		else if	( rxChar == 0x52 )
		{
		  frameBuffer[numBytes++] = 0x5A;
		  RUNNING_XOR(0x5A); 
		}
		else if	( rxChar == 0xF7 )
		{
		   frameBuffer[numBytes++] = 0xF7;
		   RUNNING_XOR(0xF7); 
		}
		else if	( rxChar == 0x25 )
		{
		   frameBuffer[numBytes++] = 0xA5;
		   RUNNING_XOR(0xA5); 
		}
		else
		{
		  /* Bad line condition. */
		   DEBUG_CMD(printf("CTRLE:Escape char failed\n"));
		   ctrleStats.rxFrameDrops++;
		   giveUP = TRUE;
		}

		rx_state = RX_FRAME_BYTE;

		break;

	default:
		giveUP = TRUE;
		DEBUG_CMD(printf("CTRLE:Invalid Case\n"));

      }
   }

   /*
    * Check if	Frame is complete.
    */
   if ( frameAvail )
   {
     uint8 i;
     
	DEBUG_CMD(printf("CTRLE: Good frame received\n"));
	for(i=0; i < numBytes ; ++i)
	{
	  DEBUG_CMD(printf("Frame byte[%4d]=0x%02X\n",i,frameBuffer[i]));
	}

        return(TRUE);
   }

   return(FALSE); /* Error no frame */

}


#endif


/***********************************************************
 * Iit routine - must be called be before any other call. 
 *
 *
 **********************************************************/

bool ctrleInit()
{
  
#if CTRLE_USE_SERIAL

  ioFD = open(VXWORKS_DEFAULT_SERIAL_DEVICE,2,0);

  if ( ioFD == -1 )
  {
    DEBUG_CMD(printf("CRTLE Init Failed\n"));
  }

  /* 
   * Setup Device to be raw
   */

  ioctl(ioFD,FIOSETOPTIONS,OPT_RAW);

#else /* CTRLE_USE_SERIAL  */


#endif

  return(1);
}


void ixCtrleStats()
{

#if CTRLE_USE_SERIAL

	printf("Ctrle Serial Interface Statistics\n");
	printf(" Rx Bytes: %d\n Frame Flag Rx: %d\n End Flag Rx: %d\n Frame Rx: %d\n Frame Drops: %d\n ReadErrors: %d\n rx CRC Errors: %d \n",
		ctrleStats.rxBytesRx,
		ctrleStats.rxStartFrameRx,
		ctrleStats.rxEndFrameRx,
		ctrleStats.rxFrameRx,
		ctrleStats.rxFrameDrops,
		ctrleStats.rxReadErrors,
		ctrleStats.rxCRCErrors);

#else /* CTRLE_USE_SERIAL */

	printf("Ctrle Parallel Interface Statistics\n");


#endif /* CTRLE_USE_SERIAL */

		
}

/************************************************************************/
/************************************************************************/
#if 0

bool ixCtrleSendPoll()
{
  bool rc=TRUE; /* Success by default */
  uint8	frameBuffer[CTRLE_BUFFER_SIZE];	/* Frame Buffer	*/

  /*
   * Perform a Serial Read request to the address in question.
   */


   frameBuffer[0] = CTRLE_POLL_CMD;
   frameBuffer[1] = (uint8)0x00;
   frameBuffer[2] = (uint8)0x00;
   frameBuffer[3] = (uint8)0x00;  

   /*
    * Send Frame.
    */
   rc =	ctrle_send_frame(frameBuffer,4);
   return(rc);

}

bool ixCtrleSendReset()
{

  bool rc=TRUE; /* Success by default */
  uint8	frameBuffer[CTRLE_BUFFER_SIZE];	/* Frame Buffer	*/

  /*
   * Perform a Serial Read request to the address in question.
   */


   frameBuffer[0] = CTRLE_RESET_CMD;
   frameBuffer[1] = (uint8)0x00;
   frameBuffer[2] = (uint8)0x00;
   frameBuffer[3] = (uint8)0x00;  

   /*
    * Send Frame.
    */
   rc =	ctrle_send_frame(frameBuffer,4);
   return(rc);


}


#endif /* endif 0 test code */

