/**
 * @file    IxAal5HssMain.c
 *
 * @date    06-Jun-2002
 *
 * @brief   API of the IXP425 Aal5/HSS Codelet
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

/*
 * System defined include files.
 */
#include <vxWorks.h>
#include <ioLib.h>
#include <logLib.h>
#include <stdio.h>

/*
 * User defined include files.
 */
#include "ixp425.h"

#include "ix_ossl.h"
#include "IxQMgr.h"
#include "IxNpeA.h"
#include "IxNpeMh.h"
#include "IxNpeDl.h"
#include "IxOsServices.h"
#include "IxDemoUtils.h"
#include "IxAal5CodeletMain.h"
#include "IxHssAccCodelet.h"
#include "IxAal5HssCodelet.h"

/*
 * Defines and macros
 */

#define IX_AAL5HSS_LOG printf

/*
 * This function macro is used to reduce the ammount of code needed to check
 * the return value of component initialisation functions. The
 * parameter FUNCTION_CALL is for example:
 * IX_AAL5HSS_COMP_INIT(ixQMgrInit ());
 */
#define IX_AAL5HSS_COMP_INIT(FUNCTION_CALL) do{\
  if (IX_SUCCESS != FUNCTION_CALL) \
  { \
     IX_AAL5HSS_LOG ("IxAal5Codelet: %s Failed\n", #FUNCTION_CALL); \
  } \
  else \
  { \
     IX_AAL5HSS_LOG ("IxAal5Codelet: %s Succeeded\n", #FUNCTION_CALL); \
  } \
}while(0)

/*
 * Variable declarations global to this file. Externs are followed by
 * statics.
 */

/*
 * Function definitions
 */

/* --------------------------------------------------------------
   This is the entry point for the IxQMgrDispatcher task.
   -------------------------------------------------------------- */
PRIVATE int
ixAal5HssDispatchTask (void *arg, void **ptrRetObj);

void
ixAal5HssCodeletMain (void)
{
#ifndef IX_AAL5_HSS_CODELET_QMGR_INTERRUPT_MODE
    ix_ossl_thread_t dispatchtid;
#endif

    /* set the debug level */
    logInit (ioTaskStdGet (0,1), 1);
    logFdSet (ioTaskStdGet (0,1));
    ixOsServLogLevelSet (LOG_ERROR); /* LOG_ALL, LOG_MESSAGE, ... */

    /**************** System initialisation ****************/
    /*
     * The IxQMgr component provides interfaces for configuring and
     * accessing the IXP425 AQM hardware queues used to facilitate
     * communication of data between the NPEs and the xscale. IxAtmdAcc and
     * IxHssAcc configure these queues. The IxQMgr component provides a
     * dispatcher that will call registered callback functions will
     * specified queue events occur. 
     */
    IX_AAL5HSS_COMP_INIT(ixQMgrInit ());

#ifdef IX_AAL5_HSS_CODELET_QMGR_INTERRUPT_MODE
    /* Running IxQMgrDispatcher from interrupt level */
     
    /* 
     * Bind the IxQMgr dispatcher to interrupt. The IX_QMGR_QUELOW_GROUP group
     * of queues concern ATM Transmit , Recieve, Transmit Done queues
     */
    IX_AAL5HSS_COMP_INIT(ixOsServIntBind(
	IXP425_INT_LVL_QM1,
	(IxVoidFnPtr)ixQMgrDispatcherLoopRun,
	(void *)IX_QMGR_QUELOW_GROUP));

    /*
     * Bind the IxQMgr dispatcher to interrupt. The IX_QMGR_QUELOW_GROUP group
     * of queues concern ATM Recieve Free queues.
     */
    IX_AAL5HSS_COMP_INIT(ixOsServIntBind(
	IXP425_INT_LVL_QM2,
	(IxVoidFnPtr)ixQMgrDispatcherLoopRun,
	(void *)IX_QMGR_QUEUPP_GROUP));
#else /* #ifdef IX_QM_INTERRUPT_MODE */
    /* This next section sets up how the IxQMgrDispatcher is called.
     */
    if (ix_ossl_thread_create ((ix_ossl_thread_entry_point_t) 
			       ixAal5HssDispatchTask,
			       NULL,
			       &dispatchtid) != IX_OSSL_ERROR_SUCCESS)
    {
	IX_AAL5HSS_LOG ("Error spawning dispatch task\n");
	return;
    }  	

    /* the IxQMgrDispatcher needs to operate at the lowest priority */
    if (ix_ossl_thread_set_priority (dispatchtid, 
				     IX_OSSL_THREAD_PRI_LOW)
	!= IX_OSSL_ERROR_SUCCESS)
    {
	IX_AAL5HSS_LOG ("Error setting dispatch task priority\n");
	return;
    }
#endif /* #ifdef IX_QM_INTERRUPT_MODE */

    /* Initialise IxNpeMh */
    IX_AAL5HSS_COMP_INIT(ixNpeMhInitialize (IX_NPEMH_NPEINTERRUPTS_YES));

    /* Download NPE image */
    /*     PSM_A_BUILD_0_3    HSS and ATM SPORT MPHY */
    IX_AAL5HSS_COMP_INIT(ixDemoUtilsNpeImageDownload (
        IX_NPEDL_NPEID_NPEA, /* npeId   */
        3));                 /* buildId */

    /* Start the NPE */
    IX_AAL5HSS_COMP_INIT(ixNpeDlNpeExecutionStart (IX_NPEDL_NPEID_NPEA));

    /* Initialise and start the HSS Codelet */
    ixHssAccCodeletChannelisedVerifySet (FALSE);
    ixHssAccCodeletPacketisedVerifySet (FALSE);

    ixHssAccCodeletInit (TRUE);

    /* Initialise and start the Aal5 Codelet */
    ixAal5CodeletMainSoftwareLoopback ();
}

/* --------------------------------------------------------------
   IxQMgrDispatcher task entry point.
   -------------------------------------------------------------- */
PRIVATE int
ixAal5HssDispatchTask (void *arg, void **ptrRetObj)
{
    /* This is an example of a task based dispatcher */
    while (TRUE)
    {
	/* ATM Tx, Rx, TxDone queues */
	ixQMgrDispatcherLoopRun (IX_QMGR_QUELOW_GROUP);

	/* ATM RxFree queues         */
	ixQMgrDispatcherLoopRun (IX_QMGR_QUEUPP_GROUP);
	
	ix_ossl_sleep(0); /* Sleep for 0 micro seconds */
    }
}
