/**
 * @file    IxAal5HssCodelet.h
 *
 * @date    17-Jun-2002
 *
 * @brief   API of the IXP4xx Aal5/HSS Codelet.
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

/* ------------------------------------------------------
   Doxygen group definitions
   ------------------------------------------------------ */
/**
 * @defgroup Codelets IXP4xx Codelets 
 *
 * @brief IXP4xx demo codelets
 *
 * @{
 */

/**
 * @ingroup Codelets
 *
 * @defgroup IxAal5HssCodelet IXP4xx Aal5 HSS Codelete (IxAal5HssCodelet) API
 * 
 * @brief API of the IXP4xx Aal5/HSS Codelet.
 *
 * This codelet demonstrates running the ATM(aal5Demo) and HSS(hssAcc) codelets
 * simultaneously. NOTE: Both codelets are run independantly, there is NO movement
 * of data between the codelets.
 *
 * There is one configuration parameter IX_AAL5_HSS_CODELET_QMGR_INTERRUPT_MODE. If
 * this is defined then the IxQMgrDispatcher is hooked to interrupt otherwise it is
 * periodically from a task. This parameter is contained wthin this file.
 *
 * The ATM codelet is setup for software loopback mode of operation.
 *
 * The HSS codelet is setup for running one channelised client using 16
 * voice channels, and four packetised clients each using 16 timeslots to
 * send 16K packets once every 200ms.
 *
 * @{
 */

#ifndef IXAAL5HSSCODELET_H
#define IXAAL5HSSCODELET_H

/*
 * System defined include files.
 */

/*
 * User defined include files.
 */
#include "IxTypes.h"

/*
 * Defines and macros
 */

/**
 * @ingroup IxAal5HssCodelet
 *
 * @def IX_AAL5_HSS_CODELET_QMGR_INTERRUPT_MODE
 *
 * @brief Run IxQMgrDispatcher from interrupt.
 *
 * This define specifies that the IxQMgrDispatcher is hooked to interrupt.
 */
#define IX_AAL5_HSS_CODELET_QMGR_INTERRUPT_MODE

/* Remove this line if it is required that the IxQMgrDispatcher be called from
 * a task.
 */
#undef  IX_AAL5_HSS_CODELET_QMGR_INTERRUPT_MODE

/*
 * Variable declarations global to this file. Externs are followed by
 * statics.
 */

/*
 * Function definitions
 */

/**
 *
 * @ingroup IxAal5HssCodelet
 *
 * @brief Start the IxAal5HssCodelet.
 *
 * @param - none
 *
 * @return - void
 */
PUBLIC void
ixAal5HssCodeletMain (void);

#endif /* IXAAL5HSSCODELET_H */

/**
 * @} defgroup IxAal5HssCodelet
 */

/**
 * @} defgroup Codelets
 */
