/**
 * @file    IxAtmCodeletMain.c
 *
 * @date    27-May-2000
 *
 * @brief   IxAtmCodelet scenarios.
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 *
 */

/*
 * System defined include files.
 */
#include <stdio.h>

/*
 * User defined include files.
 */

#include "IxOsServices.h"
#include "IxTypes.h"
#include "ix_ossl.h"
#include "IxAtmCodelet.h"
#include "IxAtmCodelet_p.h"
#include "IxAtmCodelet.h"

/*
 * Defines and macros
 */
#define NUM_AAL5_CHANNELS         12
#define NUM_AAL0_48_CHANNELS      10
#define NUM_AAL0_52_CHANNELS      10


/*
 * Variable declarations global to this file. Externs are followed by
 * statics.
 */
#define AAL5_CPCS_SDU_SEND_LENGTH  1500
#define AAL0_CELLS_SEND_LENGTH     32
#define AAL5_PACKET_SEND_COUNT     16
#define AAL0_PACKET_SEND_COUNT     22

/*
 * Function definitions
 */
PRIVATE void
ixAtmCodeletSduSendTask (void);

PUBLIC IX_STATUS
ixAtmCodeletMainUtopiaLoopback (void)
{
    IX_STATUS retval;
    unsigned int numPorts = 1;
    unsigned int rxToTxRatio = 1;
    ix_ossl_thread_t tid;

    IX_ATMCODELET_LOG ("*************************************\n");
    IX_ATMCODELET_LOG ("********** UTOPIA LOOPBACK **********\n");
    IX_ATMCODELET_LOG ("*************************************\n");

    retval = ixAtmCodeletInit (numPorts, rxToTxRatio);
    
    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }

    /* Provision some Aal5 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL5_CHANNELS,
					       IX_ATMDACC_AAL5);

    /* Provision some Aal0_48 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL0_48_CHANNELS,
					       IX_ATMDACC_AAL0_48);
 
    /* Provision some Aal0_52 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL0_52_CHANNELS,
					       IX_ATMDACC_AAL0_52);

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to provision channels\n");
	return IX_FAIL;
    }

    if (ix_ossl_thread_create((ix_ossl_thread_entry_point_t)  
 			      ixAtmCodeletSduSendTask, 
 			      NULL, 
 			      &tid) != IX_OSSL_ERROR_SUCCESS) 
      { 
	IX_ATMCODELET_LOG ("Error spawning SduSend task\n"); 
 	return IX_FAIL; 
      }  	 
    
    if (ix_ossl_thread_set_priority (tid,  
 				     IX_OSSL_THREAD_PRI_LOW) != IX_OSSL_ERROR_SUCCESS) 
      { 
 	IX_ATMCODELET_LOG ("Error setting ixAtmRxTxDemoAal5CpcsSdusSend task priority\n"); 
 	return IX_FAIL; 
      } 
    
    return IX_SUCCESS;
}

PUBLIC IX_STATUS
ixAtmCodeletMainUtopiaLoopbackRun (void)
{
    IX_STATUS retval;
    unsigned int numPorts = 1;
    IxAtmCodeletMode mode = IX_ATMCODELET_UTOPIA_LOOPBACK;

    /* Initialise system for the codelet */
    retval = ixAtmCodeletSystemInit (numPorts, mode);

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }

    retval = ixAtmCodeletMainUtopiaLoopback ();

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }

    return IX_SUCCESS;
}

/* --------------------------------------------------------------
   This scenario demonstrates that 32Mbps Rx/6 Mbps Tx can be
   achieved.
   -------------------------------------------------------------- */
PUBLIC IX_STATUS
ixAtmCodeletMainSoftwareLoopback (void)
{
    IX_STATUS retval;
    unsigned int numPorts = 1;
    unsigned int rxToTxRatio = 5;/* Transmit 1 PDU for every 5 PDUs recieved. */
    /* This will show 32Mbps Rx/6 Mbps Tx        */

    IX_ATMCODELET_LOG ("*************************************\n");
    IX_ATMCODELET_LOG ("********* SOFTWARE LOOPBACK *********\n");
    IX_ATMCODELET_LOG ("*************************************\n");

    /* Initialise the codelet */
    retval = ixAtmCodeletInit (numPorts, rxToTxRatio);
    
    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }

    /* Provision some Aal5 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL5_CHANNELS,
					       IX_ATMDACC_AAL5);

    /* Provision some Aal0_48 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL0_48_CHANNELS,
					       IX_ATMDACC_AAL0_48);
 
    /* Provision some Aal0_52 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL0_52_CHANNELS,
					       IX_ATMDACC_AAL0_52);

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to provision channels\n");
	return IX_FAIL;
    }

    return IX_SUCCESS;
}

PUBLIC IX_STATUS
ixAtmCodeletMainSoftwareLoopbackRun (void)
{
    IX_STATUS retval;
    unsigned int numPorts = 1;
    IxAtmCodeletMode mode = IX_ATMCODELET_SOFTWARE_LOOPBACK;

    /* Initialise system for the codelet */
    retval = ixAtmCodeletSystemInit (numPorts, mode);

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }

    retval = ixAtmCodeletMainSoftwareLoopback ();

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }
    
    return IX_SUCCESS;
}

PUBLIC IX_STATUS
ixAtmCodeletMainRemoteLoopback (void)
{
    IX_STATUS retval;
    unsigned int numPorts = 8;
    unsigned int rxToTxRatio = 1;
    ix_ossl_thread_t tid;
    IxAtmCodeletMode mode = IX_ATMCODELET_REMOTE_LOOPBACK;

    IX_ATMCODELET_LOG ("*************************************\n");
    IX_ATMCODELET_LOG ("*********** REMOTE LOOPBACK *********\n");
    IX_ATMCODELET_LOG ("*************************************\n");

    /* Initialise system for the codelet */
    retval = ixAtmCodeletSystemInit (numPorts, mode);

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }
    
    /* Initialise the codelet */
    retval = ixAtmCodeletInit (numPorts, rxToTxRatio); 
   
    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to initialise IxAtmCodelet\n");
	return IX_FAIL;
    }

    /* Provision some Aal5 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL5_CHANNELS,
					       IX_ATMDACC_AAL5);

    /* Provision some Aal0_48 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL0_48_CHANNELS,
					       IX_ATMDACC_AAL0_48);
 
    /* Provision some Aal0_52 channels */
    retval = ixAtmCodeletUbrChannelsProvision (numPorts,
					       NUM_AAL0_52_CHANNELS,
					       IX_ATMDACC_AAL0_52);

    if (IX_SUCCESS != retval)
    {
	IX_ATMCODELET_LOG ("Failed to provision channels\n");
	return IX_FAIL;
    }

    if (ix_ossl_thread_create((ix_ossl_thread_entry_point_t) 
 			      ixAtmCodeletSduSendTask, 
 			      NULL, 
 			      &tid) != IX_OSSL_ERROR_SUCCESS) 
      { 
 	IX_ATMCODELET_LOG ("Error spawning SduSend task\n"); 
 	return IX_FAIL; 
      }  	 
    
    if (ix_ossl_thread_set_priority (tid,  
				     IX_OSSL_THREAD_PRI_LOW) != IX_OSSL_ERROR_SUCCESS) 
      { 
	IX_ATMCODELET_LOG ("Error setting ixAtmRxTxDemoAal5CpcsSdusSend task priority\n"); 
	return IX_FAIL;
      } 
    
    return IX_SUCCESS;
}

/* --------------------------------------------------------------
   SduSend task.
   -------------------------------------------------------------- */
PRIVATE void
ixAtmCodeletSduSendTask (void)
{
    IX_STATUS retval;
    static BOOL toggle = TRUE; /* Toggle the type of packet to send */

    IX_ATMCODELET_LOG ("Alternatively sending %u Aal5 Packets and %u Aal0 Packets\n",
		       AAL5_PACKET_SEND_COUNT, AAL0_PACKET_SEND_COUNT);

    while (TRUE)
    {
	if (toggle)
	{
	    /* Send some Aal5 Cpcs SDUs */
	    retval = ixAtmCodeletAal5CpcsSdusSend (AAL5_CPCS_SDU_SEND_LENGTH,
						   AAL5_PACKET_SEND_COUNT);		
	    toggle = FALSE;
	}
	else
	{
	    /* Send some Aal0 Packets */
	    retval = ixAtmCodeletAal0PacketsSend (AAL0_CELLS_SEND_LENGTH, 
						  AAL0_PACKET_SEND_COUNT);
	    toggle = TRUE;
	}
			
	if (IX_SUCCESS != retval)
	{
	    IX_ATMCODELET_LOG ("Failed to send SDUs\n");
	    return;
	}
	
	ix_ossl_sleep (0);/* Sleep for 0 seconds, i.e. allow a task switch */
    }
}
