/**
 * @file    IxAtmCodeletRxTxDemo_p.h
 *
 * @date    20-May-2002
 *
 * @brief   The IxAtmCodeletRxTxDemo sub-component provides functions for
 * for setting up Rx and Tx transport and sending/receiving Atm Packets.
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 *
 */
#ifndef IXATMCODELETRXTXDEMO_P_H
#define IXATMCODELETRXTXDEMO_P_H

/*
 * User defined include files.
 */
#include "IxTypes.h"
#include "IxAtmCodelet_p.h"


/*
 * Defines and macros
 */

/*
 * Typedefs
 */

/*
 * Function declarations.
 */

/*
 * This function initializes Atm codelet transport subcomponent
 */
IX_STATUS
ixAtmRxTxDemoInit (IxAtmCodeletStats *stats);

/*
 * This function is used to provision a number of Atm transport channels
 */
IX_STATUS
ixAtmRxTxDemoChannelsProvision (unsigned int numPorts,
				unsigned int numChannels,
				IxAtmdAccAalType aalType);

/*
 * Get transport stats
 */
void
ixAtmRxTxDemoStatsGet (IxAtmCodeletStats *stats);

/*
 * Replenish buffers to the RxFree queue. Buffers are taken from
 * the mbuf pool
 */
void
ixAtmRxTxDemoRxFreeLowReplenish (IxAtmdAccUserId userId);

/* 
 * Send Aal5 CPCS SDUs
 */
IX_STATUS
ixAtmRxTxDemoAal5CpcsSdusSend (unsigned int sduSize,
			       unsigned int numSdus);

/*
 * Send Aal0 packets, where a packet is a number of cells
 */
IX_STATUS
ixAtmRxTxDemoAal0PacketsSend (unsigned int cellsPerPacket,
			      unsigned int numPackets);

#endif /* IXATMCODELETRXTXDEMO_P_H */

