/*
 * FileName:    IxDemoUtilsCodeletBufMan.c
 * Author:      Peter Mangan, Brendan Fennell, Pierre Laurant
 * Created:     15 May 2002
 * Description: Demo Utils Buffer Management
 *
 *
 * Design Notes:
 *
 *
 * -- Intel Copyright Notice --
 * INTEL CONFIDENTIAL
 *
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

#include "IxTypes.h"
#include "IxOsBuffMgt.h"
#include "IxOsBuffPoolMgt.h"
#include "IxOsServices.h"

typedef struct
{
    IX_MBUF_POOL *poolPtr;
    unsigned bufSize;
    unsigned numBufs;
} IxDemoUtilsBufManPoolInfo;


/*
 * Forward declarations
 */
void ixDemoUtilsMbufShow (void);

static IxDemoUtilsBufManPoolInfo poolInfo[] =   /* pool information table */
{
    /*
      poolPtr     bufSize  numBufs
      ---------   -----    -------
    */
    {NULL,             64,        5000},  /* Small packets, e.g. OAM */
    {NULL,             2048,      5200}
};

IX_STATUS
ixDemoUtilsMbufPoolInit(void)
{
    int i, numPools;

    numPools = NELEMENTS (poolInfo);

    for (i = 0; i < numPools; i++)
    {
        /* Initialize the pool */
	if (ixOsBuffPoolInit (&(poolInfo[i].poolPtr), poolInfo[i].numBufs, poolInfo[i].bufSize, "ixDemoUtils pool") != IX_SUCCESS)
        {
            return IX_FAIL;
        }
    }
    
    return IX_SUCCESS;
}

void
ixDemoUtilsMbufGet (unsigned int bufSize, IX_MBUF **buf)
{
    IX_MBUF_POOL *poolPtr = NULL;
    unsigned i,numPools;

    numPools = NELEMENTS (poolInfo);

    for (i = 0; i < numPools; i++)
    {
        if (poolInfo[i].bufSize >= bufSize)
	{
            poolPtr = poolInfo[i].poolPtr;
            break;
	}
    }

    if (poolPtr == NULL)
    {
        ixOsServLog (LOG_ERROR, "ixDemoUtilsMbufGet - Invalid bufSize specified\n", 0, 0, 0, 0, 0, 0);
        *buf = NULL;
        return;
    }

    ixOsBuffPoolUnchainedBufGet (poolPtr, buf);
    
    if (*buf)
    {
        IX_MBUF_MLEN(*buf) = bufSize;
	memset (IX_MBUF_MDATA(*buf), 0xBF, IX_MBUF_MLEN(*buf));
    }
    else
    {
	ixOsServLog (LOG_ERROR, "ixDemoUtilsMbufGet returning a NULL buffer\n", 0, 0, 0, 0, 0, 0);
	ixDemoUtilsMbufShow ();
    }
}

void
ixDemoUtilsMbufFree(IX_MBUF *buf)
{
    ixOsBuffPoolBufChainFree (buf);
}

void
ixDemoUtilsMbufShow (void)
{

    IX_MBUF_POOL *poolPtr = NULL;
    unsigned i,numPools;

    numPools = NELEMENTS (poolInfo);

    for (i = 0; i < numPools; i++)
    {
        poolPtr = poolInfo[i].poolPtr;
        if (poolPtr != NULL)
 	{
            ixOsBuffPoolShow (poolPtr);
	}
    }
}
