/*
 * FileName:    IxDemoUtilsNpe.c
 * Author: Intel Corporation
 * Created:     15 May 2002
 * Description: Codelet utils.
 *
 *
 * Design Notes:
 *
 *
 * -- Intel Copyright Notice --
 * INTEL CONFIDENTIAL
 *
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

/*
 * System defined include files
 */
#include <string.h>

/*
 * User defined include files
 */
#include "IxTypes.h"
#include "IxAtmm.h"
#include "IxAtmTypes.h"
#include "IxOsServices.h"
#include "IxNpeDl.h"
#include "IxAssert.h"
#include "IxNpeMicrocode.h"

/*
 * #defines and macros
 */
#define NO_UTOPIA_BUILD_ID_        1
#define SINGLE_PORT_SPHY_BUILD_ID  2
#define SINGLE_PORT_MPHY_BUILD_ID  3
#define MULTI_PORT_MPHY_BUILD_ID   4

/*
 * Typedefs
 */

/*
 * Variable declarations global to this file. Externs are followed by
 * statics.
 */

/*
 * Function definitions.
 */

PUBLIC IX_STATUS
ixDemoUtilsNpeImageDownload ( IxNpeDlNpeId npeId,
			      IxNpeDlBuildId buildId)
{
    UINT32 numVersions = 0;
    IxNpeDlVersionId versionIdList[IX_NPE_MICROCODE_AVAILABLE_VERSIONS_COUNT];
    IxNpeDlVersionId downloadVersionId;
    IxNpeDlVersionId uploadVersionId;
    UINT32 listSize = 0;
    IX_STATUS status;
    int i;

    downloadVersionId.npeId = npeId;
    downloadVersionId.buildId = buildId;
    downloadVersionId.major = 0x0;
    downloadVersionId.minor = 0x0;

    /* get the version count - there should be at most maxVersions */
    status = ixNpeDlAvailableVersionsCountGet (&numVersions);
    if (status != IX_SUCCESS)
    {
        ixOsServLog (LOG_ERROR, "Failed to get version count\n", 0, 0, 0, 0, 0, 0);
        return IX_FAIL;
    }
    else if (numVersions > IX_NPE_MICROCODE_AVAILABLE_VERSIONS_COUNT)
    {
        ixOsServLog (LOG_ERROR, "Version count is not equal to %d\n", numVersions, 0, 0, 0, 0, 0);
        return IX_FAIL;
    }

    listSize = numVersions;
    /* get the list of available versions */
    status = ixNpeDlAvailableVersionsListGet (versionIdList, &listSize);
    if (status != IX_SUCCESS)
    {
        ixOsServLog (LOG_ERROR, "Failed to get version list\n", 0, 0, 0, 0, 0, 0);
        return IX_FAIL;
    }

    IX_ASSERT(numVersions == listSize);

    /* Get the latest version of NPE code for this feature */
    for (i = 0; i < listSize; i++)
    {
        if (versionIdList[i].npeId == npeId)
        {
            if (versionIdList[i].buildId == downloadVersionId.buildId)
	    {
		if (versionIdList[i].major > downloadVersionId.major)
                {
		    downloadVersionId.major = versionIdList[i].major;
		    downloadVersionId.minor = versionIdList[i].minor;
                }
		else if (versionIdList[i].major == downloadVersionId.major
                         && versionIdList[i].minor >= downloadVersionId.minor)
                {
		    downloadVersionId.minor = versionIdList[i].minor;
                }
            }
	}
    }

    printf("NPE%c Version information\n", downloadVersionId.npeId  + 'A');
    printf("Major ID:\t 0x%x\n", downloadVersionId.major);
    printf("Minor ID:\t 0x%x\n", downloadVersionId.minor);
    printf("Build ID:\t 0x%x\n", downloadVersionId.buildId);

    /* stop and reset the NPE */
    status = ixNpeDlNpeStopAndReset (npeId);
    if (status != IX_SUCCESS)
    {
        ixOsServLog (LOG_ERROR, "Failed to stop and reset NPE\n", 0, 0, 0, 0, 0, 0);
        return IX_FAIL;
    }

    /* download the version */
    status = ixNpeDlVersionDownload (
        &downloadVersionId, /* versionIdPtr */
        TRUE);              /* verify */

    if (status != IX_SUCCESS)
    {
        ixOsServLog (LOG_ERROR, "Failed to download the version\n", 0, 0, 0, 0, 0, 0);
        return IX_FAIL;
    }

    /* get the downloaded version */
    status = ixNpeDlLoadedVersionGet (npeId, &uploadVersionId);
    if (status != IX_SUCCESS)
    {
        ixOsServLog (LOG_ERROR, "Failed to get the downloaded version\n", 0, 0, 0, 0, 0, 0);
        return IX_FAIL;
    }

    /* verify the downloaded version is the one we downloaded */
    if (memcmp(&uploadVersionId, &downloadVersionId, sizeof (downloadVersionId)) != 0)
    {
        ixOsServLog (LOG_ERROR, "Version ID downloaded not as expected\n", 0, 0, 0, 0, 0, 0);
        return IX_FAIL;
    }

    return IX_SUCCESS;
}
