 /**
 * @file IxDmaAccCodelet_p.h
 *
 * @author Intel Corporation
 * @date 18 November 2002
 *
 * @brief This file contains some private data structures and
 * defines for the Dma Access Codelet.
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * -- End Intel Copyright Notice --
 *
 */

#ifndef IXDMAACCCODELET_P_H
#define IXDMAACCCODELET_P_H

/** Test pattern content and array size */
#define IX_DMA_CODELET_TESTPATTERN_LIST { 0x12,0x34,0x56,0x78,0x9A,0xBC,0xDE,0xF0 } 
#define IX_DMA_CODELET_TESTPATTERN_LENGTH (8) 

/** Number of milliiseconds to poll the dispatcher */
#define IX_DMA_CODELET_MS_SLEEP (5)

/** Number of words for test transfer */
#define IX_DMA_CODELET_TEST_MAXLENGTH (65535)

/** Number of bytes to show when doing memory dump */
#define IX_DMA_CODELET_MEMDUMPSIZE (32)

/** Priority of queue manager dispatch loop */
#define IX_DMA_CODELET_QMR_PRIORITY (150)

/** Version ID for NPE A Dma Code */
#define IX_DMA_CODELET_NPE_A_BUILDID (21)

/** Version ID for NPE B Dma Code */
#define IX_DMA_CODELET_NPE_B_BUILDID (2)

/** Version ID for NPE C Dma Code */
#define IX_DMA_CODELET_NPE_C_BUILDID (8)

/**
 * @fn ixDmaAccCodeletShow()
 *
 * This function performs a memory dump at the source and destination   
 * addresses. This function is called before and after a Dma transfer
 * to enable the user to check that a transfer has taken place and
 * allows the user to examine the data at the destination after it has 
 * been transfered.   
 *
 * This function is used by ixDmaAccCodeletTestPerform()  
 *
 * @param none 
 *
 * @return none 
 */
/**
 * @fn ixDmaAccCodeletTestPatternReset()
 *
 * This function formats source and destination address blocks
 * with test patterns 
 *
 * This function is used by ixDmaAccCodeletTestPerform()  
 *
 * @param none 
 *
 * @return none 
 */

/**
 * @fn ixDmaAccCodeletDispatcherPoll()
 *
 * The function runs the dispatcher for a group of queues. 
 * Callbacks are made for interrupts that have occurred on 
 * queues within the group that have registered callbacks. 
 * This function may be called from interrupt or task context. 
 *
 * This function is used by ixDmaAccCodeletInit()  
 *
 * @param  void* argUnused Not used
 * @param  void** ptrRetObjUnused Not used
 *
 * @return none 
 */
/**
 * @fn ixDmaAccCodeletDispatcherStart()
 *
 * This function starts the queue dispatcher 
 * This function is used by ixDmaAccCodeletInit()  
 *
 * @param useInterrupt TRUE sets the dispatcher to be interrupt driven
 * @param useInterrupt FALSE sets the dispatcher to poll mode 
 *
 * @return none 
 */

/**
 * @fn ixDmaAccCodeletNpeInit()
 *
 * This function initialises the Npe 
 * This function is used by ixDmaAccCodeletInit()  
 *
 * @param npeId IX_NPEDL_NPEID_NPEA for Npe A 
 * @param npeId IX_NPEDL_NPEID_NPEB for Npe B 
 * @param npeId IX_NPEDL_NPEID_NPEC for Npe C 
 *
 * @return IX_SUCCESS Npe succesfully initialised 
 * @return IX_FAIL Npe failed to initialise 
 */

/**
 * @fn ixDmaAccCodeletCallback()
 *
 * This function is called by Dma Acc component when 
 * Dma transfer is done  
 *
 * @param IX_STATUS IX_SUCCESS Dma transfer succesful 
 * @param IX_STATUS IX_FAIL Dma transfer failed 
 *
 * @return none 
 */


#endif /* IXDMAACCCODELET_P_H */

