/**
 * @file IxEthAal5App.h
 *
 * @author Intel Corporation
 * @date 3 Apr 2002
 *
 * @brief This file contains the declaration of the IxEthAal5App
 * component's main functions, etc.
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
*/

/**
 * @defgroup Codelets IXP4xx Codelets
 *
 * @brief IXP4xx demo codelets
 *
 * @{
 */

/**
 * @ingroup Codelets
 *
 * @defgroup IxEthAal5App IXP4xx Ethernet Aal5 (IxEthAal5App) API 
 *
 * @brief IXP4xx demo Ethernet Aal5 driver component API
 *
 * @{
 */

#ifndef IXETHAAL5APP_H
#define IXETHAAL5APP_H

#include "IxEthAcc.h"
#include "IxTypes.h"
#include "IxAtmTypes.h"

#include "EthAal5User.h"


/**
 * @def IX_EAA_NUM_BUFFERS_PER_ETH
 * @brief  This is the number of buffers, which can be stored in free buffer queue for each ethernet port 
 */
#ifndef IX_EAA_NUM_BUFFERS_PER_ETH
#define IX_EAA_NUM_BUFFERS_PER_ETH  64
#endif

/**
 * @def IX_EAA_NUM_ATM_PORTS
 * @brief Define number of supported atm ports by this application
 *
 * User can define them in EthAal5User.h file to overwrite definitions below
 */
#ifndef  IX_EAA_NUM_ATM_PORTS
#define IX_EAA_NUM_ATM_PORTS     8
#endif

/**
 * @def IX_EAA_NUM_BUFFERS_PER_VC
 * @brief This is the number of buffers per atm port 
 *
 * User can define them in EthAal5User.h file to overwrite definitions below
 */
#ifndef IX_EAA_NUM_BUFFERS_PER_VC
#define IX_EAA_NUM_BUFFERS_PER_VC 16 
#endif

/**
 * @brief Define default VPI and VCI for 8 ports. 
 * User can define them in EthAal5User.h file to overwrite definitions below
 */
#ifndef IX_EAA_PORT1_VPI
#define IX_EAA_PORT1_VPI 1
#endif
#ifndef IX_EAA_PORT1_VCI
#define IX_EAA_PORT1_VCI 100
#endif
#ifndef IX_EAA_PORT2_VPI
#define IX_EAA_PORT2_VPI 2
#endif
#ifndef IX_EAA_PORT2_VCI
#define IX_EAA_PORT2_VCI 100
#endif
#ifndef IX_EAA_PORT3_VPI
#define IX_EAA_PORT3_VPI 3
#endif
#ifndef IX_EAA_PORT3_VCI
#define IX_EAA_PORT3_VCI 100
#endif
#ifndef IX_EAA_PORT4_VPI
#define IX_EAA_PORT4_VPI 4
#endif
#ifndef IX_EAA_PORT4_VCI
#define IX_EAA_PORT4_VCI 100
#endif
#ifndef IX_EAA_PORT5_VPI
#define IX_EAA_PORT5_VPI 5
#endif
#ifndef IX_EAA_PORT5_VCI
#define IX_EAA_PORT5_VCI 100
#endif
#ifndef IX_EAA_PORT6_VPI
#define IX_EAA_PORT6_VPI 6
#endif
#ifndef IX_EAA_PORT6_VCI
#define IX_EAA_PORT6_VCI 100
#endif
#ifndef IX_EAA_PORT7_VPI
#define IX_EAA_PORT7_VPI 7
#endif
#ifndef IX_EAA_PORT7_VCI
#define IX_EAA_PORT7_VCI 100
#endif
#ifndef IX_EAA_PORT8_VPI
#define IX_EAA_PORT8_VPI 8
#endif
#ifndef IX_EAA_PORT8_VCI
#define IX_EAA_PORT8_VCI 100
#endif

/* entry point */

/**
 * Initialize MAC data base, to contain 0xffs - no valid Mac addresses present.Also
 * initialize QMGR, NpeMh, Eth phys - 100Mbit, FULL DUPLEX, NO AUTONEGOTIATION (User 
 * can change those settings accordingly to required configuration), ATM, and Utopia 
 * interface
 *
 * If Linux is used, use interrupt mode - much faster under Linux than polling
 *
 * If vxWorks is used use poll mode - much faster under vxWorks than interrupts and 
 * start background QMgr queues poll
*/
int ixEAAMain(void);

/**
 *  display the current counters 
 */
int ixEAAShow(void);

/**
 * This function can be used to add MAC address to Atm data base and assign it
 * to particular port. Valid ports are 0-7 
 */
void ixEAAAddMAC(int port, UINT8 mac1, UINT8 mac2, UINT8 mac3, UINT8
		 mac4, UINT8 mac5, UINT8 mac6);

#endif 
/* IXETHAAL5APP_H */

/** @} defgroup IxEthAal5App*/

/** @} defgroup Codelets*/
