/**
 * @file IxEthAccCodeletSymbols.c
 *
 * @author Intel Corporation
 * @date 14-Oct-2002
 *
 * @brief This file declares exported symbols for linux kernel module builds.
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

#ifdef __linux

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <asm/semaphore.h>

#include "IxTypes.h"
#include "IxEthAccCodelet.h"

EXPORT_SYMBOL(ixEthAccCodeletInit);
EXPORT_SYMBOL(ixEthAccCodeletRxSink);
EXPORT_SYMBOL(ixEthAccCodeletSwLoopback);
EXPORT_SYMBOL(ixEthAccCodeletTxGenRxSinkLoopback);
EXPORT_SYMBOL(ixEthAccCodeletBridge);
EXPORT_SYMBOL(ixEthAccCodeletDBLearning);

int demo;
MODULE_PARM(demo, "i");

DECLARE_MUTEX_LOCKED(__codlet_getchar);
DECLARE_MUTEX_LOCKED(__codlet_end);

static int demo_func(void *unused)
{
    ixEthAccCodeletInit();
    switch (demo)
    {
    case 1: ixEthAccCodeletRxSink(); break;
    case 2: ixEthAccCodeletSwLoopback(); break;
    case 3: ixEthAccCodeletTxGenRxSinkLoopback(); break;
    case 4: ixEthAccCodeletBridge(); break;
    case 5: ixEthAccCodeletDBLearning(); break;
    default: break;
    }
    up(&__codlet_end);
    return 0;
}

static int __init eth_init_module(void)
{
    printk("Load Intel CSR Codlet: Ethernet Demo.\n");
    kernel_thread(demo_func, NULL, CLONE_SIGHAND);
    return 0;
}

static void __init eth_cleanup_module(void)
{
    ixEthAccDataPlaneShow();
    up(&__codlet_getchar);
    down(&__codlet_end);
    printk("Unload Intel CSR Codlet: Ethernet Demo.\n");
}

module_init(eth_init_module);
module_exit(eth_cleanup_module);

#endif
