 /**
 * @file IxEthAccCodelet_p.h
 *
 * @author Intel Corporation
 * @date 22 April 2002
 *
 * @brief This file contains some private data structures and
 * defines for the Ethernet Access Codelet.
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
*
 */

#ifndef IXETHACCCODELET_P_H
#define IXETHACCCODELET_P_H

/**< Recommended priority of queue manager dispatch loop */
#define IX_ETHACC_CODELET_QMR_PRIORITY 150 

/**< Recommended priority of Ethernet DB Maintenance task */
#define IX_ETHACC_CODELET_DB_PRIORITY 91 

/* Buffer management utilities */


/**< Macro to check if the free buffer queue is empty */
#define IX_ETHACC_CODELET_IS_Q_EMPTY(mbuf_list) (mbuf_list == NULL)


/**< Macro to add an MBUF to the queue. The queue uses the m_nextpkt pointer 
 * which is not used by the Ethernet Access component */ 
#define IX_ETHACC_CODELET_ADD_MBUF_TO_Q_TAIL(mbuf_list,mbuf_to_add)   	\
  {									\
    int lockVal = ixOsServIntLock();					\
    IX_MBUF* mBufptr;							\
    mBufptr = mbuf_list;						\
    if ((mbuf_list) != NULL) 						\
    {									\
        while(IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((mBufptr)) != NULL)		\
        {                                                               \
            mBufptr = (IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((mBufptr)));	\
        }                                                               \
        IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((mBufptr)) = (mbuf_to_add);	\
        IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((mbuf_to_add)) = NULL;		\
    } 									\
    else								\
    {									\
        (mbuf_list) = (mbuf_to_add);					\
        IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((mbuf_to_add)) = NULL;		\
    } 									\
    ixOsServIntUnlock(lockVal);						\
  }


/**< Macro to remove and MBUF from the queue */
#define IX_ETHACC_CODELET_REMOVE_MBUF_FROM_Q_HEAD(mbuf_list,mbuf_to_rem)\
  {									\
    int lockVal = ixOsServIntLock();					\
    if ((mbuf_list) != NULL ) 						\
    {									\
        (mbuf_to_rem) = (mbuf_list);					\
        (mbuf_list) = (IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((mbuf_to_rem)));	\
    } 									\
    else								\
    {									\
        (mbuf_to_rem) = NULL;						\
    } 									\
    ixOsServIntUnlock(lockVal);						\
  }


/**< Macro to check if the codelet has been initialised */
#define IX_ETHACC_IS_CODELET_INITIALISED()				\
  {									\
    if(!ixEthAccCodeletInitialised)					\
    {									\
	printf("Ethernet Access Codelet not yet initialised!\n");	\
	return (IX_FAIL);						\
    }									\
  }


/**< Macro to generate random payload data */
#define IX_ETHACC_CODELET_DATAGEN(compPtr)				\
  {									\
	int i = 0;							\
	srand(ixOsServTimestampGet());					\
	for(i=0; i<IX_ETHACC_CODELET_TXGEN_PCK_LEN; i++)		\
	 {								\
	     compPtr[i] = rand();					\
	 }								\
  }


/**< Macro to verify payload */
#define IX_ETHACC_CODELET_DATA_VERIFY(m_data, compPtr)			\
  {									\
      if(memcmp(&compPtr[0], m_data, IX_ETHACC_CODELET_TXGEN_PCK_LEN) != 0)  \
	{								\
	    ixOsServLog(LOG_ERROR, "Mbuf data verification failed\n",   \
                        0, 0, 0, 0, 0, 0);                              \
	}								\
  }

#endif /* IXETHACCCODELET_P_H */
