/**
 * @file  IxFpathAccCodeletDefines_p.h
 * @date  May 29 2002
 * @brief Defines constants of system wide significance
 *        for the IxFpathAcc demo codelet
 *
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

#ifndef IXFPATHACCCODELETDEFINES_P_H
#define IXFPATHACCCODELETDEFINES_P_H

#include <stdio.h>
#include "IxTypes.h"
#include "IxAtmTypes.h"
#include "ixp425.h"
#include "IxOsServices.h"

/**
 * Define the space that should be available before m_data
 * in an mbuf when first allocated ( to be used for pre-pending
 * headers ).
 */
#define MBUF_HEADROOM 32
#if( MBUF_HEADROOM < SIZEOF_IEEE802_3_HDR )
#error MBUF HEADROOM TOO SMALL
#endif

/**
 * Define the size of mbufs that will be used to populate
 * the AtmdAcc rx free queue.
 */
#define ATM_RX_MBUF_SIZE (2048 - MBUF_HEADROOM)

/**
 * Define the size of the mbufs that will be used to populate
 * the fast path free queue. This cannot be larger than
 * the ATM rx mbuf size.
 */
#define FASTPATH_MBUF_SIZE 1600

#if( ATM_RX_MBUF_SIZE < FASTPATH_MBUF_SIZE )
#error  INVALID FPATH/ATM MBUF SIZES
#endif

/**
 * Define the size of the mbufs that will be used to populate
 * the ethernet rx free queue
 */
#define ETH_RX_MBUF_SIZE 1024

/**
 * Define the initial number of mbufs to be supplied to the
 * EthAcc rx free queues
 */
#define ETH_INITIAL_RX_REPLENISH_NUM 32

/**
 * Define the MAC addresses for the ethernet
 * ports associated with NPE-B and NPE-C
 */
#define ETH1_MAC_ADDRESS {0x00,0x00,0x00,0x00,0x00,0x01}
#define ETH2_MAC_ADDRESS {0x00,0x00,0x00,0x00,0x00,0x02}


/**
 * The routed IP fast path demonstration does not include ARP
 * functionality. All received pdus are transmitted to this MAC
 * address. If necessary this should be re-defined to the suit the test
 * environment, typically the MAC address of an attached ethernet
 * device
 */
#define FPATH_ROUTEDIP_TARGET_MAC {0x00,0x00,0x00,0x00,0x00,0x03}


/**
 * For the Routed IP demonstration define the
 * time in milliseconds that a flow is allowed to persist on the fast
 * path before it must be learned again.
 * (Accurate to 1 OS tick)
 */
#define IP_FLOW_AGEING_INTERVAL_MS  2000

/**
 * For the Routed IP demonstration define the number of pdus
 * that must be received within the IP_FLOW_LEARNING_INTERVAL
 * to qualify for being moved from the slow path to the fast path
 */
#define IP_FLOW_LEARNING_PDU_COUNT_THRESHOLD 300

/**
 * Defined the number of IP flows that can be learned
 * per VC at any time.
 */
#define NUM_LEARNING_FLOWS 8

/**
 * For the Routed IP demonstration define the interval
 * in milliseconds during which IP flow activity is measured.
 * This number must be a multiple of the number of
 * learning flows, and of the OS tick
 */
#define IP_FLOW_ACCOUNTING_INTERVAL_MS 512

#if (IP_FLOW_ACCOUNTING_INTERVAL % NUM_LEARNING_FLOWS)
#error IP ACCOUNTING INTERVAL MUST BE A MULTIPLE OF NUM_LEARNING_FLOWS
# endif
#if (IP_FLOW_ACCOUNTING_INTERVAL % (1000/IXP425_OSST_TICKS_PER_SECOND))
#error IP ACCOUNTING INTERVAL MUST BE A MULTIPLE OF OS TICK
# endif

/**
 * This control determines if WAN MAC address learning
 * is enabled for the ethernet bridge demonstration code
 */
#define WAN_MAC_LEARNING_ENABLED TRUE

/**
 * Define how often the WAN MAC learning history will be displayed
 * as a number of seconds
 */
#define WAN_MAC_LEARNING_DB_DISPLAY_INTERVAL_S 5

/* Defines how many learned WAN MAC addresses should be displayed
*/
#define WAN_MAC_LEARNING_DB_DISPLAY_NUM_ENTRIES 16


/**
 * Define the ATM UTOPIA port, VPI and VCI to be used for
 * simple bridge demo.
 */
#define FPATH_BRIDGED_TEST_UTOPIA_PORT IX_UTOPIA_PORT_0
#define FPATH_BRIDGED_TEST_VPI 1
#define FPATH_BRIDGED_TEST_VCI 32

/**
 * Define the ATM UTOPIA port, VPI and VCI to be used for
 * simple routed demo.
 */
#define FPATH_ROUTED_TEST_UTOPIA_PORT IX_UTOPIA_PORT_0
#define FPATH_ROUTED_TEST_VPI 1
#define FPATH_ROUTED_TEST_VCI 33

/**
 * Define the NPE Versions that will be loaded
 */
#define NPEA_BUILDID 0x04
#define NPEA_MAJOR 0
#define NPEA_MINOR 0

#define NPEB_BUILDID 0x1
#define NPEB_MAJOR 1
#define NPEB_MINOR 0

#define NPEC_BUILDID 0x0
#define NPEC_MAJOR 1
#define NPEC_MINOR 0

/**
 * @brief Log text using printf style formatting
 *
 * e.g. IXFPATH_CODELET_TRACE((traceBuf, "One plus One = %d", 2 ));
 *
 * @note double parenthesis.
 */
#define IXFPATH_CODELET_TRACE(string) do { \
    static char traceBuf[80];\
    sprintf string;\
    ixOsServLog( LOG_MESSAGE,"%s",(int)traceBuf,0,0,0,0,0);\
    /* task sleep to prevent log output from flooding */\
    ixOsServTaskSleep(1000/IXP425_PERIPHERAL_BUS_CLOCK);\
} while (0)

/**
 * @brief Log errors with file and line number
 *
 * e.g. IXFPATH_CODELET_ERROR(("Something went wrong here"));
 *
 */
#define IXFPATH_CODELET_ERROR(string) do {\
    ixOsServLog( LOG_ERROR,\
        "IxFpathAcc Codelet Error - %s"\
        "\n%s LINE %d",\
        (int) string,\
        (int) __FILE__,\
        (int) __LINE__,\
        0,0,0 );\
} while (0)

#endif /* IXFPATHACCCODELETDEFINES_P_H */



