/**
 * @file  IxFpathAccCodeletMain_p.h
 * @date  June 18 2002
 * @brief Defines the interface to the main codelet engine.
 *
 * Design Notes:
 *              It is used by both the Bridge and Routed example
 *              code. See IxFpCodeletBridgeDemo.c and
 *              IxFpCodeletRoutedDemo.c.
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

#ifndef IXFPATHACCCODELETMAIN_P_H
#define IXFPATHACCCODELETMAIN_P_H

#include "IxAtmTypes.h"
#include "IxAtmdAcc.h"
#include "IxFpathAccCodeletTemplates_p.h"
#include "IxFpathAccCodeletDefines_p.h"

/**
 * Types For Gathering Statistics
 */
typedef struct
{
    unsigned int totalPduCount;
    unsigned int validPduCount;
    unsigned int errorCount;
} Aal5Statistics;

typedef struct
{
    unsigned int lengthExceptionCount;
    unsigned int otherExceptionCount;
    unsigned int newWanMacCount;
} EtherBridgeStatistics;

typedef struct
{
    unsigned int ethernetTxErrorCount;
    unsigned int flowSetErrorCount;
    unsigned int templateSetErrorCount;
    unsigned int macLookupErrorCount;
} RoutedIPStatistics;

/**
 * @typedef AtmRxCallback
 * encapsulation type specific ATM rx callback registered
 * with the codelet aal5 receive handler.
 */
typedef void (*AtmRxCallback) (IxAtmdAccUserId userId, IX_MBUF *buf);

/**
 * @typedef IpFlowRequestData
 * Basic Ip flow tuple
 */
typedef struct
{
    UINT32 ipAddress;
    UINT8 macAddress[6];
} IpFlowRequestData;

/** Struct to hold IP flow information while learning */
typedef struct
{
    BOOL         inUse;         /** Learning is taking place  */
    UINT32       ipAddress;     /** Destination IP address    */
    UINT8        macAddress[6]; /** Corresponding Mac Address */
    unsigned int counter;       /** Current Packet Count      */
} LearningIpFlow;

/** Struct to hold IP flow information while active */
typedef struct
{
    BOOL         aged;          /** Flow may be superceded         */
    UINT32       ipAddress;     /** Destination IP address         */
    UINT8        macAddress[6]; /** Corresponding Mac Address      */
    int          age;           /** In units of Counting Intervals */
} ActiveIpFlow;

/** Main structure to hold connection information */
typedef struct
{
    BOOL               inUse;
    VcNumber           vc;

    IxAtmLogicalPort   port;
    unsigned int       vpi;
    unsigned int       vci;

    EncapsulationType  encapsulationType; /** See IxFpathCodeletTemplates_p.h*/

    AtmRxCallback      atmRxHandler;
    IxAtmNpeRxVcId     npeVcId; /**
                                 * N.B. Obtained from IxAtmdAcc at
                                 * ixAtmdAccRxVcConnect().
                                 */

    /** IP Flow Learning and Aging Information */
    LearningIpFlow          learningIpFlow[NUM_LEARNING_FLOWS];
    ActiveIpFlow            activeIpFlow[MAX_FLOWS_PER_VC];
    BOOL                    ipFlowRequest;
    IpFlowRequestData       ipFlowRequestData;

    /** Statistics */
    Aal5Statistics          aal5Stats;
    EtherBridgeStatistics   etherBridgeStats;
    RoutedIPStatistics      routedIpStats;
} IxFpCodeletFastPathConnection;

extern BOOL ixFpCodeletTraceEnabled;

/** Do the main intialisation for the codelet */
IX_STATUS
ixFpCodeletInit( void );

/**
 * Creates a FP connection and invokes the templating mechanism
 * to create a basic fast path template for the chosen VC encapsulation
 * type.
 */
IX_STATUS
ixFpCodeletFastPathVcCreate( VcNumber vc );

/**
 * Delete the FP and ATM connection
 */
IX_STATUS
ixFpCodeletFastPathVcDelete( VcNumber vc );

/**
 * Show codelet information
 */
void
ixFpCodeletShow(void);

/** Gets a pointer to the connection for the specified VC */
IxFpCodeletFastPathConnection *
ixFpCodeletFastPathConnectionGet( VcNumber vc );

#endif /* IXFPATHACCCODELETMAIN_P_H */

