/**
 * @file    IxOamITU610Types_p.h
 *
 * @date    20-July-2002
 *
 * @brief   This header file defines ITU-610 OAM cell format, offsets and fields.
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 *
 */
#ifndef IXOAMITU610TYPES_P_H
#define IXOAMITU610TYPES_P_H

/*
 * Defines and macros
 */

/*
 * #defines and macros used in this file.
 */

/* VCI values */
#define IX_OAM_ITU610_F4_SEG_VCI     3
#define IX_OAM_ITU610_F4_ETE_VCI     4

/* PTI values */
#define IX_OAM_ITU610_F5_SEG_PTI                    4
#define IX_OAM_ITU610_F5_ETE_PTI                    5

/*  Length of correlation tag bytes in LB cell */
#define IX_OAM_ITU610_LB_CORRELATION_TAG_LEN 4


/* Size of all location ID attributes in bytes. */
#define IX_OAM_ITU610_LOCATION_ID_LEN 16

/* Reserved bytes in OAM cells */
#define IX_OAM_ITU610_RESERVED_BYTE_VALUE         0x6a
#define IX_OAM_ITU610_GENERIC_PAYLOAD_RESERVED_BYTES_LEN   45
#define IX_OAM_ITU610_LB_RESERVED_BYTES_LEN                 8

/* Reserved and CRC10 length */
#define IX_OAM_ITU610_RESERVED_AND_CRC10_LEN 2

/* Loopback timeout */
#define IX_OAM_ITU610_LB_TIMEOUT_PERIOD_MSECS   5000

/*
 * OAM Function Types
 */
#define IX_OAM_ITU610_TYPE_FAULT_MAN_LB     0x18

/* Values for Loopback Indication field */
#define IX_OAM_ITU610_LB_INDICATION_PARENT  1
#define IX_OAM_ITU610_LB_INDICATION_CHILD   0

/*
 * Typedefs 
 */

/*
 * Oam cells payload typedefs
 */
typedef struct
{
    UINT8 oamTypeAndFunction;
    UINT8 loopbackIndication;
    UINT8 correlationTag[IX_OAM_ITU610_LB_CORRELATION_TAG_LEN];
    UINT8 llid[IX_OAM_ITU610_LOCATION_ID_LEN];
    UINT8 sourceId[IX_OAM_ITU610_LOCATION_ID_LEN];
    UINT8 reserved[IX_OAM_ITU610_LB_RESERVED_BYTES_LEN];
    UINT8 reservedAndCrc10[IX_OAM_ITU610_RESERVED_AND_CRC10_LEN];
} IxOamITU610LbPayload;

/*
 * Generic payload isn't a real payload but is used
 * for checking which payload type a received OAM cell is
 */
typedef struct
{
    UINT8 oamTypeAndFunction;    
    UINT8 reserved[IX_OAM_ITU610_GENERIC_PAYLOAD_RESERVED_BYTES_LEN];
    UINT8 reservedAndCrc10[IX_OAM_ITU610_RESERVED_AND_CRC10_LEN];
} IxOamITU610GenericPayload;

typedef union
{
    IxOamITU610LbPayload lbPayload;
    IxOamITU610GenericPayload genericPayload;
    /* Other payload types could be defined here, e.g. Fault Management etc.  */
} IxOamITU610Payload;

typedef struct
{
    IxNpeA_AtmCellHeader header;
    IxOamITU610Payload payload;
} IxOamITU610Cell;


#endif /* IXOAMITU610TYPES_P_H */
