/**
 * @file IxCryptoAccCodelet.c
 *
 * @date October-31-2002
 *
 * @brief This codelet file contains the demonstration codes for the 
 *	  Crypto Access Compenent.
 *
 *
 * -- Intel Copyright Notice -- 
 *
 * @par
 * INTEL CONFIDENTIAL
 * 
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 * 
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material]
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 * 
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 * 
 * -- End Intel Copyright Notice -- 
 */

/*
 * System include files.
 */

#include <stdio.h>
#include <stdlib.h>

/*
 * User include files.
 */

#include "IxQMgr.h"
#include "IxNpeDl.h"
#include "IxOsServices.h"
#include "IxOsCacheMMU.h"
#include "ix_ossl.h"
#include "IxTypes.h"
#include "ixp425.h"
#include "IxCryptoAccCodelet.h"
#include "IxCryptoAcc.h"

#ifdef COEXIST

/*****************************************
 * ethAcc Codelets inclusion
 */

#include "IxEthDB.h"
#include "IxEthAccCodelet.h"
#include "IxEthAccCodelet_p.h"
#include "IxNpeMh.h"

#ifdef IX_ETHACC_CODELET_USE_NVRAM_MAC 
#include "config.h"
#endif

/*
 * ethAcc variable declarations global to this file only.
 */

PRIVATE IX_MBUF *ixEthAccCodeletBufPool;

PRIVATE INT8 *ixEthAccCodeletBufData;

PRIVATE UINT8 compData[IX_ETHACC_CODELET_PCK_LEN];

PRIVATE IX_MBUF *ixEthAccCodeletFreeBufQ;
PRIVATE IX_MBUF *ixEthAccCodeletTxFreeBufQ;

PRIVATE UINT32 memPoolFreeCnt = 0;

PRIVATE UINT32 phyAddresses[IX_ETHACC_CODELET_MAX_PORT];

PRIVATE IxEthAccCodeletPhyConf phyConf = 
{
    TRUE,	/* 100 Mbits */
    TRUE,	/* Full duplex */
    TRUE	/* Autonegotiate */
};

PRIVATE struct _IxEthAccCodeletStats
{
    UINT32 rxCount;
    UINT32 txCount;
} IxEthAccCodeletStats[IX_ETHACC_CODELET_MAX_PORT];
  
PRIVATE BOOL ixEthAccCodeletInitialised = FALSE;
PRIVATE BOOL ixEthAccCodeletNpeInitialised = FALSE;

PRIVATE volatile BOOL ixEthAccCodeletPollEnabled = FALSE;

/*
 * Static function prototypes.
 */

PRIVATE IX_STATUS ixEthAccCodeletDispatcherStart(BOOL useInterrupt);
PRIVATE void ixEthAccCodeletDispatcherPoll(void* arg, void** ptrRetObj);
PRIVATE IX_STATUS ixEthAccCodeletPhyInit(void);
PRIVATE IX_STATUS ixEthAccCodeletNpeInit(IxNpeDlNpeId npeId);
PRIVATE IX_STATUS ixEthAccCodeletMemPoolInit(void);
PRIVATE IX_STATUS ixEthAccCodeletMacSet(void);
PRIVATE IX_STATUS ixEthAccCodeletRecoverBuffers(void);
PRIVATE IX_STATUS ixEthAccCodeletReplenishBuffers(IxEthAccPortId portNo, UINT32 num); 
PRIVATE IX_STATUS ixEthAccCodeletLoop(void);
PRIVATE void ixEthAccCodeletStatsPollTask(void* arg, void** ptrRetObj);

/* Ethernet callbacks */

PRIVATE void ixEthAccCodeletSwLoopbackRxCB(UINT32 cbTag, IX_MBUF* mBufPtr, IxEthAccPortId portId);
PRIVATE void ixEthAccCodeletSwLoopbackTxCB(UINT32 cbTag, IX_MBUF* mBufPtr);
PRIVATE void ixEthAccCodeletMemPoolFreeRxCB(UINT32 cbTag, IX_MBUF* mBufPtr, IxEthAccPortId portId);
PRIVATE void ixEthAccCodeletMemPoolFreeTxCB(UINT32 cbTag, IX_MBUF* mBufPtr);


/* End of ethAcc codelets include and declarations */
#endif /* End of #ifdef COEXIST */

/**< Size of MBuf pool, ie number of buffers to circulate */
#define IX_CRYPTOACC_CODELET_MBUF_POOL_SIZE 20

/**<  Packet count to use between time measurements */
#define IX_CRYPTOACC_CODELET_BATCH_LEN 1000

/**< Max number of NPE uCode versions */
#define IX_CRYPTOACC_CODELET_NPE_LIST 20

/**< Recommended priority of queue manager dispatch loop */
#define IX_CRYPTOACC_CODELET_QMR_PRIORITY 150 

/**< Maximum time (in ms) to wait before exiting the program  */
#define IX_CRYPTOACC_CODELET_MAX_TIME_WAIT 200

/**< The NPE ID that will be used in this codelet */
#define IX_CRYPTOACC_CODELET_NPE_ID IX_NPEDL_NPEID_NPEC

/**< The QMgr mode, whether in interrupt (TRUE) or poll (FALSE) mode */
#define IX_CRYPTOACC_CODELET_QMGR_MODE FALSE

/**< The built ID for non AES and AES */
#define IX_CRYPTOACC_CODELET_AES_BUILT_ID 1
#define CRYPTO_START_BUILD_ID 2  /* need to be changed according to NPE
                                  * image master list
                                  */
#define CRYPTO_END_BUILD_ID   7  /* need to be changed according to NPE
                                  * image master list
                                  */

/**< The XScale tick is 66 MHz*/
#define IX_CRYPTOACC_CODELET_XSCALE_TICK 66

/**< The codelet dispatcher poll wait time in ms */
#define IX_CRYPTOACC_CODELET_DISPATCHER_POLL_WAIT_TIME 1

/**< The codelet register wait time a for loop */
#define IX_CRYPTOACC_CODELET_REGISTER_WAIT_TIME 500

/**< The codelet perform wait time in a for loop */
#define IX_CRYPTOACC_CODELET_PERFORM_WAIT_TIME 500

/**< The maximum packet size */
#define IX_CRYPTOACC_CODELET_MAX_PCKT_SIZE 65408

/**< The codelet mode of operation */
#define IX_CRYPTOACC_CODELET_ENCRYPT 1
#define IX_CRYPTOACC_CODELET_AUTH 2
#define IX_CRYPTOACC_CODELET_ENCRYPT_AUTH 3
#define IX_CRYPTOACC_CODELET_QUIT 4

/**< The codelet mode of operation for linux */
#define IX_CRYPTOACC_CODELET_LINUX_OP IX_CRYPTOACC_CODELET_ENCRYPT

/**< Length of MBuf payload for Linux (in bytes) */
#define IX_CRYPTOACC_CODELET_LINUX_PCKT_LEN 64

/**< Length of MBuf payload (in bytes) */
PRIVATE UINT32 ixCryptoAccCodeletPcktLen;

/* Mbuf Pool */
PRIVATE IX_MBUF *ixCryptoAccCodeletBufPool;
PRIVATE IX_MBUF *ixCryptoAccCodeletFreeBufQHead = NULL;
PRIVATE IX_MBUF *ixCryptoAccCodeletFreeBufQTail = NULL;
PRIVATE INT8 *ixCryptoAccCodeletBufData;
PRIVATE UINT32 ixCryptoAccCodeletFreeBufQSize = 0;

/* Guard for once-only init code, when initialised this variable is set to 
 * TRUE 
 */
PRIVATE BOOL ixCryptoAccCodeletInitialised = FALSE;

/* Flag used when shutting down services, when TRUE, the function will 
 * shutdown 
 */
PRIVATE BOOL trafficFlowHalt = FALSE;

/* Used in performance calculation */
PRIVATE UINT32 totalPacket = 0;
PRIVATE UINT32 timeStamp = 0;
PRIVATE UINT32 rateRatio;

/* Used for Linux callback count */
PRIVATE UINT32 linuxCbCount = 1;

/* Registration results  */
PRIVATE BOOL forwardRegisterCBCalled = FALSE;
PRIVATE BOOL reverseRegisterCBCalled = FALSE;

/* Initialised both CtxId to an invalid number as 0 (zero) is a valid ID.
 * The ID ranges from 0 - (IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS - 1), hence
 * IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS is invalid.
 */
PRIVATE UINT32 forwardCtxId = IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS;
PRIVATE UINT32 reverseCtxId = IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS;

/* Random IV value for this demo and IV should remain constant on both
 * directions; encrypt and decrypt.
 */
PRIVATE UINT8 IV[IX_CRYPTO_ACC_MAX_CIPHER_IV_LENGTH] = {0,2,4,6,
							1,3,5,7,
							3,4,1,2,
							6,7,5,1}; 

/* Random DES key */
UINT8 desKeyToUse[IX_CRYPTO_ACC_DES_KEY_64] = {0x01,0x23,0x45,0x67,
					       0x89,0xAB,0xCD,0xEF};

/* SHA1 key from test case 1 in RFC 2202 */
UINT8 sha1KeyToUse[IX_CRYPTO_ACC_SHA1_KEY_160]=  {0x0B,0x0B,0x0B,0x0B,
						  0x0B,0x0B,0x0B,0x0B,
						  0x0B,0x0B,0x0B,0x0B,
						  0x0B,0x0B,0x0B,0x0B,
						  0x0B,0x0B,0x0B,0x0B};

/* Forward and reverse context declaration and initilisation */

PRIVATE IxCryptoAccCtx forwardCtx, reverseCtx; 

/* Define the Register callback function */
#define IxCryptoAccRegisterCompleteCallback ixCryptoAccCodeletRegisterCompleteCB

/* Define the Perform callback function */
#define IxCryptoAccPerformCompleteCallback ixCryptoAccCodeletPerformCB

/**
 * @fn ixCryptoAccCodeletInit()
 *
 * @brief This function is the entry point to the Crypto Access codelet.
 * 	  It must be called before any of the demos as it initialises the 
 *	  neccessary component. The sequence of initilisation is illustrated
 *	  in the header file.
 */
IX_STATUS ixCryptoAccCodeletInit (void)
{
    if (ixCryptoAccCodeletInitialised) 
    {
	printk ("Crypto codelet already initialised\n");
	return (IX_SUCCESS);
    }

#ifndef COEXIST
    /* Initialise NPEs */
    if (IX_SUCCESS != ixCryptoAccCodeletNpeInit (IX_CRYPTOACC_CODELET_NPE_ID))
    {
	printk ("Error initialising NPE!\n");
	return (IX_FAIL);
    }

    /* Initialise Queue Manager */
    if (IX_SUCCESS != ixQMgrInit ())
    {
	printk ("Error initialising queue manager!\n");
	return (IX_FAIL);
    }
#endif /* End of ifndef COEXIST */

    /* Initialise cryptoAcc component */
    if (IX_CRYPTO_ACC_STATUS_SUCCESS != ixCryptoAccInit ())
    {
	printk ("Error initialising Crypto access component!\n");
	return (IX_FAIL);
    }

#ifndef COEXIST
    /* Start NPE C */
    if (IX_SUCCESS != ixNpeDlNpeExecutionStart (IX_CRYPTOACC_CODELET_NPE_ID))
    {
	printk ("Error starting NPE C!\n");
	return (IX_FAIL);
    }

    /* Start the Queue Manager dispatcher */   
    if ( IX_SUCCESS != 
	 ixCryptoAccCodeletDispatcherStart (IX_CRYPTOACC_CODELET_QMGR_MODE))
    {
	printk("Error starting queue manager dispatcher!\n");
	return (IX_FAIL);
    }
#endif /* End of ifndef COEXIST */

    ixCryptoAccCodeletInitialised = TRUE;
    printk ("\n\ncryptoAcc Codelet Initialiasation complete!\n\n");

    /* Set up initialize crypto context, both the forward and reverse 
     * context are the same. The only difference is the operation which
     * will be initialize in ixCryptoAccCodeletRegisterCtx () 
     */
    
    forwardCtx.cipherCtx.cipherAlgo = 
	reverseCtx.cipherCtx.cipherAlgo = IX_CRYPTO_ACC_CIPHER_DES;
    forwardCtx.cipherCtx.cipherMode = 
	reverseCtx.cipherCtx.cipherMode = IX_CRYPTO_ACC_MODE_CBC;
    forwardCtx.cipherCtx.cipherKeyLen =
	reverseCtx.cipherCtx.cipherKeyLen = IX_CRYPTO_ACC_DES_KEY_64;
    memcpy(forwardCtx.cipherCtx.key.desKey,desKeyToUse,IX_CRYPTO_ACC_DES_KEY_64);
    memcpy(reverseCtx.cipherCtx.key.desKey,desKeyToUse,IX_CRYPTO_ACC_DES_KEY_64);
    forwardCtx.cipherCtx.cipherBlockLen = 
	reverseCtx.cipherCtx.cipherBlockLen = IX_CRYPTO_ACC_DES_BLOCK_64;
    forwardCtx.cipherCtx.cipherInitialVectorLen =
	reverseCtx.cipherCtx.cipherInitialVectorLen = IX_CRYPTO_ACC_DES_IV_64;
    forwardCtx.cipherCtx.securityParameterIndex = 
	reverseCtx.cipherCtx.securityParameterIndex = 0; /* a random number */
    forwardCtx.authCtx.authAlgo = 
	reverseCtx.authCtx.authAlgo = IX_CRYPTO_ACC_AUTH_SHA1;
    forwardCtx.authCtx.authDigestLen = 
	reverseCtx.authCtx.authDigestLen = IX_CRYPTO_ACC_SHA1_DIGEST_160;
    forwardCtx.authCtx.authKeyLen =
	reverseCtx.authCtx.authKeyLen = IX_CRYPTO_ACC_SHA1_KEY_160;
    /* Using test case 1 in rfc 2202 */
    memcpy(forwardCtx.authCtx.key.sha1Key,sha1KeyToUse,IX_CRYPTO_ACC_SHA1_KEY_160);
    memcpy(reverseCtx.authCtx.key.sha1Key,sha1KeyToUse,IX_CRYPTO_ACC_SHA1_KEY_160);
    forwardCtx.useDifferentSrcAndDestMbufs =
	reverseCtx.useDifferentSrcAndDestMbufs = FALSE;

    return (IX_SUCCESS);
} /* End of ixCryptoAccCodeletInit () function */


/**
 * @fn ixCryptoAccCodeletBufIsQEmpty
 *
 * @brief This inline function is used to check whether all the buffer
 *        queue is empty.
 */

INLINE BOOL
ixCryptoAccCodeletBufIsQEmpty(void)
{
    BOOL status = FALSE;
    if (0 == ixCryptoAccCodeletFreeBufQSize)
    { 
	status = TRUE;
    }
    return status;
}


/**
 * @fn ixCryptoAccCodeletBufIsQFull
 *
 * @brief This inline function is used to check whether all the buffer
 *        queue is full.
 */

INLINE BOOL
ixCryptoAccCodeletBufIsQFull(void)
{
    BOOL status = FALSE;
    if (IX_CRYPTOACC_CODELET_MBUF_POOL_SIZE == ixCryptoAccCodeletFreeBufQSize)
    { 
	status = TRUE;
    }
    return status;
}

/**
 * @fn ixCryptoAccCodeletMbufToQTailAdd()
 *
 * @brief This function will add an mbuf to the queue. The mbuf will
 *	  be added to the end of the queue.  . 
 */
void ixCryptoAccCodeletMbufToQTailAdd (IX_MBUF *pMbufToAdd)
{
    IX_MBUF *pMbuf;
    if (!ixCryptoAccCodeletBufIsQFull())
    {
	pMbuf = ixCryptoAccCodeletFreeBufQTail;
	if (NULL != ixCryptoAccCodeletFreeBufQTail) 
	{
	    /*Walk the mbuf chain */
	    while (NULL != IX_MBUF_NEXT_PKT_IN_CHAIN_PTR(pMbuf))
	    { 
		pMbuf = IX_MBUF_NEXT_PKT_IN_CHAIN_PTR(pMbuf);
	    }
	    /* Add pMbufToAdd in the last mbuf in the chain */
	    IX_MBUF_NEXT_PKT_IN_CHAIN_PTR(pMbuf) = pMbufToAdd;

	}/* End of if ixCryptoAccCodeletFreeBufQ */
	else
	{
	    /* The Queue Head pointer will point to the head of the queue 
	     * and it is initialised when the first mbuf is added to an 
	     * empty queue */	
	    ixCryptoAccCodeletFreeBufQHead = pMbufToAdd;
		
	} /* End of if-else ixCryptoAccCodeletFreeBufQTail */

	/* Update the ixCryptoAccCodeletFreeBufQ with the last mbuf */
	ixCryptoAccCodeletFreeBufQTail = pMbufToAdd;
	IX_MBUF_NEXT_PKT_IN_CHAIN_PTR(pMbufToAdd) = NULL;
	ixCryptoAccCodeletFreeBufQSize++;
    } /* if ixCryptoAccCodeletFreeBufQSize is full */
    else /* Buffer queue is full */
    {
	printk ("\nBuffer Queue is full\n");
    } /* End of if-else ixCryptoAccCodeletFreeBufQSize */

} /* End of ixCryptoAccCodeletMbufToQTailAdd () function */


/**
 * @fn ixCryptoAccCodeletMbufFromQHeadRemove()
 *
 * @brief This function will remove a mbuf from the queue. The mbuf will
 *	  be removed from the head of the queue.
 */

void ixCryptoAccCodeletMbufFromQHeadRemove(IX_MBUF **pMbufToRem)
{
    if (!ixCryptoAccCodeletBufIsQEmpty())
    {
	*pMbufToRem = ixCryptoAccCodeletFreeBufQHead;
	ixCryptoAccCodeletFreeBufQHead = IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((*pMbufToRem));
	ixCryptoAccCodeletFreeBufQSize--;
    } 
    else
    {
	*pMbufToRem = NULL;
	printk ("\nBuffer Queue is empty\n");	
    } /* if-else ixCryptoAccCodeletBufIsQEmpty */
} /* End of ixCryptoAccCodeletMbufFromQHeadRemove () function */


/**
 * @fn IX_STATUS ixCryptoAccCodeletMemPoolInit()
 *
 * @brief Initialise Crypto Access Codelet MBUF pool
 */

PRIVATE IX_STATUS ixCryptoAccCodeletMemPoolInit (void)
{
    UINT32 bufNo;
    IX_MBUF *pMbuf;
    ixCryptoAccCodeletBufPool = NULL;
    ixCryptoAccCodeletBufData = NULL;

    if (NULL ==
	(ixCryptoAccCodeletBufPool = 
	 IX_ACC_DRV_DMA_MALLOC(IX_CRYPTOACC_CODELET_MBUF_POOL_SIZE*sizeof(IX_MBUF))))
    {
	printk("Error allocating mBuf pool for Crypto Codelet!\n");
	return (IX_FAIL);
    }
 
    pMbuf = ixCryptoAccCodeletBufPool;

    if (NULL == (ixCryptoAccCodeletBufData = 
		 IX_ACC_DRV_DMA_MALLOC(
		     IX_CRYPTOACC_CODELET_MBUF_POOL_SIZE*
		     (ixCryptoAccCodeletPcktLen+IX_CRYPTO_ACC_SHA1_DIGEST_160))))
    {
	printk("Error allocating mBuf data pool for Crypto Codelet!\n");
	return (IX_FAIL);
    }

    /* Format our mBufs */
    for (bufNo = 0;
	 bufNo < IX_CRYPTOACC_CODELET_MBUF_POOL_SIZE;
	 bufNo++)
    {
	/* Initialise payload */ 
	IX_MBUF_MDATA (pMbuf) = 
	    &ixCryptoAccCodeletBufData[bufNo*
				      (ixCryptoAccCodeletPcktLen+
				       IX_CRYPTO_ACC_SHA1_DIGEST_160)];

	/* Initialise length */
	IX_MBUF_MLEN(pMbuf) = (ixCryptoAccCodeletPcktLen+
			IX_CRYPTO_ACC_SHA1_DIGEST_160);
 	
	/* No chains */
	IX_MBUF_NEXT_BUFFER_IN_PKT_PTR(pMbuf)= NULL;

	/* Add newly formated buffer to our free queue */
	ixCryptoAccCodeletMbufToQTailAdd (pMbuf);
	pMbuf++;
    }

    return (IX_SUCCESS);
} /* End of ixCryptoAccCodeletMemPoolInit () function */

/**
 * @fn IX_STATUS ixCryptoAccCodeletMemPoolFree()
 *
 * @brief Free all resources in Crypto Access Codelet MBUF pool
 */
PRIVATE void ixCryptoAccCodeletMemPoolFree (void)
{
    IX_CRYPTO_ACC_CODELET_DRV_DMA_FREE(ixCryptoAccCodeletBufPool);
    IX_CRYPTO_ACC_CODELET_DRV_DMA_FREE(ixCryptoAccCodeletBufData);
    ixCryptoAccCodeletFreeBufQHead = NULL;
    ixCryptoAccCodeletFreeBufQTail = NULL;
    ixCryptoAccCodeletFreeBufQSize = 0;

} /* End of ixCryptoAccCodeletMemPoolFree () function */


/**
 * @fn IX_STATUS ixCryptoAccCodeletNpeInit()
 *
 * @brief Download microcode to the NPE. 
 *
 * @note November 25th 2002: The current file downloads the latest crypto
 *	 firmware. When the combined firmware is available the codes in this
 * 	 function needs to be changed.
 */

PRIVATE IX_STATUS ixCryptoAccCodeletNpeInit (IxNpeDlNpeId npeId)
{
    IxNpeDlVersionId npeIdList[IX_CRYPTOACC_CODELET_NPE_LIST];
    UINT32 npeListSize = IX_CRYPTOACC_CODELET_NPE_LIST;

    IxNpeDlVersionId tmpIdList;
    UINT32 i;

    /* Crypto only handled on NPE C */
    tmpIdList.npeId = npeId;

#ifndef AES_ENABLED
    tmpIdList.buildId = CRYPTO_START_BUILD_ID;
#else
    tmpIdList.buildId = IX_CRYPTOACC_CODELET_AES_BUILT_ID;
#endif /*ndef AES_ENABLED*/

    /* Initialise the major and minor struct member */
    tmpIdList.major = 0; 
    tmpIdList.minor = 0; 
 
    /* Stop the NPE before attempting a download */
    if (IX_SUCCESS != ixNpeDlNpeStopAndReset (npeId))
    {
	printk("NPE stop and reset failed\n");
	return (IX_FAIL);
    }

    /* Get the latest version of NPE code available */
    if (IX_SUCCESS != ixNpeDlAvailableVersionsListGet (npeIdList,
						       &npeListSize))
    {
	printk ("Unable to get NPE version information\n");
	return (IX_FAIL);
    }

   for(i = 0; i < npeListSize; i++)
    {
	if((npeIdList[i].buildId > tmpIdList.buildId) &&
	   (npeIdList[i].buildId <= CRYPTO_END_BUILD_ID))
	{
	    tmpIdList.buildId = npeIdList[i].buildId;
	    tmpIdList.major = npeIdList[i].major;
	    tmpIdList.minor = npeIdList[i].minor;
	}
	else if(npeIdList[i].buildId == tmpIdList.buildId)
	{
	    if(npeIdList[i].major > tmpIdList.major)
	    {
		tmpIdList.major = npeIdList[i].major;
		tmpIdList.minor = npeIdList[i].minor;
	    }
	    else if(npeIdList[i].major == tmpIdList.major)
	    {
		if(npeIdList[i].minor >= tmpIdList.minor)
		{
		    tmpIdList.minor = npeIdList[i].minor;
		}
	    }
	}
    }

    printk ("\nNPE %d Version information\n", tmpIdList.npeId);
    printk ("Build ID:\t 0x%x\n", tmpIdList.buildId);
    printk ("Major ID:\t 0x%x\n", tmpIdList.major);
    printk ("Minor ID:\t 0x%x\n", tmpIdList.minor);


    /* Download NPE code. The second argument is ON/OFF option to verify
     * the download. If ON (verify == TRUE), the Downloader will read back
     * each word written to the NPE registers to ensure the download operation
     * was successful
     */
    if (IX_SUCCESS != ixNpeDlVersionDownload (&tmpIdList, TRUE))
    {
	printk("NPE download failed\n");
	return(IX_FAIL);
    }

    return(IX_SUCCESS);
} /* End of ixCryptoAccCodeletNpeInit () function */


/**
 * @fn void ixCryptoAccCodeletDispatcherPoll
 *
 * @brief This function polls the Queue manager loop.
 *
 * @return void
 */
PRIVATE void ixCryptoAccCodeletDispatcherPoll (void)
{
    while (1)
    {
	/* The crypto NPE uses queues 29 and 30. */
	ixQMgrDispatcherLoopRun (IX_QMGR_QUELOW_GROUP);
	
	/* Sleep 1/1000 s */
        ixOsServTaskSleep (IX_CRYPTOACC_CODELET_DISPATCHER_POLL_WAIT_TIME); 
    }
} /* End of ixCryptoAccCodeletDispatcherPoll */

/**
 * @fn IX_STATUS ixCryptoAccCodeletDispatcherStart()
 *
 * @brief  This function starts the Queue manager dispatcher.
 */
PRIVATE IX_STATUS ixCryptoAccCodeletDispatcherStart (BOOL useInterrupt)
{
   UINT32 dispatchtid;

    if (useInterrupt)	/* Interrupt mode */
    {
	printk ("\nStarting Interrupt mode for cryptoAcc!\n\n");
	/* 
	 * Hook the QM QLOW dispatcher to the interrupt controller. 
	 * The crypto NPE uses queues 29 and 30.
	 */
	if (IX_SUCCESS != ixOsServIntBind (IXP425_INT_LVL_QM1,
					    (IxVoidFnPtr)ixQMgrDispatcherLoopRun,
					    (void *)IX_QMGR_QUELOW_GROUP))
	{
	    printk ("Failed to bind to QM1 interrupt\n");
	    return (IX_FAIL);
	}
    }
    else  /* Polled mode */
    {
	printk ("\nStarting Polling mode for cryptoAcc!\n\n");
	if (IX_OSSL_ERROR_SUCCESS != 
	    ix_ossl_thread_create ((ix_ossl_thread_entry_point_t)
				   ixCryptoAccCodeletDispatcherPoll,
				   NULL,
				   &dispatchtid))
	{
	    printk ("Error spawning dispatch task\n");
	    return (IX_FAIL);
	}

	ix_ossl_thread_set_priority (dispatchtid, IX_CRYPTOACC_CODELET_QMR_PRIORITY);
    }

    return (IX_SUCCESS);
} /* End of ixCryptoAccCodeletDispatcherStart () function */


/**
 * @fn ixCryptoAccCodeletRegisterCompleteCB()
 *
 * @brief  Cryptographic Context registration completed callback notification.
 */
void ixCryptoAccCodeletRegisterCompleteCB (UINT32 cryptoCtxId,
					   IX_MBUF *pMbuf, 
					   IxCryptoAccStatus status)
{
    BOOL *flagToUse;

    if (cryptoCtxId == forwardCtxId)
    {
	flagToUse = &forwardRegisterCBCalled;
    }
    else
    {	
	if (cryptoCtxId == reverseCtxId)
	{
	    flagToUse = &reverseRegisterCBCalled;
	}
	else
	{
	    ixOsServLog(LOG_ERROR, "\nInvalid ContextId %d\n", 
			cryptoCtxId, 0, 0, 0, 0, 0);
	    return;
	}
    }

    /* We are expecting two callbacks for authentication and combined service 
     * The pMbuf could be null for encrypt/decrypt ONLY operation.
     */
    if (NULL != pMbuf)
    {
	/* Must return the buffer to the pool irrespective of the status */
	ixCryptoAccCodeletMbufToQTailAdd (pMbuf);
    }

    if (IX_CRYPTO_ACC_STATUS_SUCCESS == status)
    {
	*flagToUse = TRUE;
	ixOsServLog(LOG_MESSAGE, "\nRegistrationCB is successful. Context %d\n", 
		    cryptoCtxId, 0, 0, 0, 0, 0);
    } /* End of if (IX_CRYPTO_ACC_STATUS_SUCCESS == status) */
    else 
    {
	if (IX_CRYPTO_ACC_STATUS_WAIT == status)
	{
	    ixOsServLog(LOG_MESSAGE, "\nRegister not completed yet; Context %d\n", 
			cryptoCtxId, 0, 0, 0, 0, 0);
	}
	else
	{
	    ixOsServLog(LOG_WARNING, "\nRegistration failed; Context %d\n", 
			cryptoCtxId, 0, 0, 0, 0, 0);
	}
    } /* End of if else (IX_CRYPTO_ACC_STATUS_SUCCESS == status) */
} /* End of ixCryptoAccCodeletRegisterCompleteCB () function */


 /**
 *
 * @fn ixCryptoAccCodeletPerformCB
 *
 * @brief  Security perform process completed callback notification.
 */

void ixCryptoAccCodeletPerformCB (UINT32 cryptoCtxId,
				  IX_MBUF *pSrcMbuf, 
				  IX_MBUF *pDestMbuf,
				  IxCryptoAccStatus status)
{
    switch (status)
    {
	case IX_CRYPTO_ACC_STATUS_SUCCESS:
	    ixOsServLog(LOG_MESSAGE, "\nPerform Callback success! %d\n",
			linuxCbCount, 0, 0, 0, 0, 0);
	    linuxCbCount++;

	    break;
  
	case IX_CRYPTO_ACC_STATUS_AUTH_FAIL:
	    ixOsServLog(LOG_WARNING, "\nAuthentication Failed\n",
			    0, 0, 0, 0, 0, 0);
	    break;

	case IX_CRYPTO_ACC_STATUS_FAIL:
	    ixOsServLog(LOG_WARNING, "\nOperation Failed\n", 
			    0, 0, 0, 0, 0, 0);
	    break;

	default:
	    ixOsServLog(LOG_WARNING, "\nError in status message, %d\n", 
			status, 0, 0, 0, 0, 0);
    } /* End of switch (status) */

	/* Return the buffer(s) to the pool */
	if (NULL != pSrcMbuf)
	{
	    ixCryptoAccCodeletMbufToQTailAdd (pSrcMbuf);
{
// Jack ++
int ii;
	    for (ii=0; ii<ixCryptoAccCodeletPcktLen; ii++)
	      {
		printk("%x\n",pSrcMbuf->mBlkHdr.mData[ii]);
	      }

}
	}
	if (NULL != pDestMbuf)
	{
	    ixCryptoAccCodeletMbufToQTailAdd (pDestMbuf);
	}
	return;
} /* End of ixCryptoAccCodeletPerformCB () function */


/* Register Security Contexts */
IX_STATUS ixCryptoAccCodeletRegisterCtx (UINT32 opType)
{
    IxCryptoAccStatus status;
    IX_MBUF *pPrimaryMbuf = NULL;
    IX_MBUF *pSecondaryMbuf = NULL;
    UINT16 i = 0;
    
    /* Set up forward and reverse context and then register it */
    switch (opType)
    {
	case IX_CRYPTOACC_CODELET_ENCRYPT:
	    forwardCtx.operation = IX_CRYPTO_ACC_OP_ENCRYPT;
	    reverseCtx.operation = IX_CRYPTO_ACC_OP_DECRYPT;
	    break;
	    
	case IX_CRYPTOACC_CODELET_AUTH:
	    forwardCtx.operation = IX_CRYPTO_ACC_OP_AUTH_CALC;
	    reverseCtx.operation = IX_CRYPTO_ACC_OP_AUTH_CHECK;
	    break;
	    
	case IX_CRYPTOACC_CODELET_ENCRYPT_AUTH:
	    forwardCtx.operation = IX_CRYPTO_ACC_OP_ENCRYPT_AUTH;
	    reverseCtx.operation = IX_CRYPTO_ACC_OP_AUTH_DECRYPT;
	    break;
	    
	default:
	    printk ("\nInvalid Operation Type, %d\n", opType);
	    return (IX_FAIL);
    } /* End of switch (opType) */
  

    /* Primary Mbuf and Secondary Mbuf is required for authentication 
     * and combined service for forward context
     */
    if ((IX_CRYPTOACC_CODELET_AUTH == opType) || 
	(IX_CRYPTOACC_CODELET_ENCRYPT_AUTH == opType))
    {
	ixCryptoAccCodeletMbufFromQHeadRemove (&pPrimaryMbuf);
	ixCryptoAccCodeletMbufFromQHeadRemove (&pSecondaryMbuf);

	/* Check if the mbufs are NULL */
	if ((NULL == pPrimaryMbuf) || (NULL == pSecondaryMbuf))
	{
	    printk ("\nInvalid mbufs!\n");
	    return (IX_FAIL);
	}
    }
  
    /* Register the forward context */
    status = ixCryptoAccCtxRegister (&forwardCtx,pPrimaryMbuf,pSecondaryMbuf,
				     ixCryptoAccCodeletRegisterCompleteCB,
				     ixCryptoAccCodeletPerformCB,
				     &forwardCtxId);
    if (IX_CRYPTO_ACC_STATUS_SUCCESS != status)
    {
	printk ("Registration Failed. Status = %d\n", status);
	return (IX_FAIL);
    }

    /* Wait for the forward register callback to return */
    while (!forwardRegisterCBCalled)
    {
	i=0;
	
	/* Sleep 1/10 s */
	ixOsServTaskSleep (IX_CRYPTOACC_CODELET_REGISTER_WAIT_TIME);
	
	/* Wait for 200 times before return fail status */
	if (IX_CRYPTOACC_CODELET_MAX_TIME_WAIT < i)
	{
	    printk ("\nRegistration incomplete, timeout for forward callback\n");
	    return (IX_FAIL);
	}
	i++;
    }

    /* Primary Mbuf and Secondary Mbuf is required for authentication 
     * and combined service for reverse context
     */
    if ((IX_CRYPTOACC_CODELET_AUTH == opType) || 
	(IX_CRYPTOACC_CODELET_ENCRYPT_AUTH == opType))
    {
	ixCryptoAccCodeletMbufFromQHeadRemove (&pPrimaryMbuf);
	ixCryptoAccCodeletMbufFromQHeadRemove (&pSecondaryMbuf);

	/* Check if the mbufs are NULL */
	if ((NULL == pPrimaryMbuf) || (NULL == pSecondaryMbuf))
	{
	    printk ("\nInvalid mbufs!\n");
	    return (IX_FAIL);
	}
    }
    /* Register the reverse context */
    status = ixCryptoAccCtxRegister (&reverseCtx,pPrimaryMbuf,pSecondaryMbuf,
				     ixCryptoAccCodeletRegisterCompleteCB,
				     ixCryptoAccCodeletPerformCB,
				     &reverseCtxId);
    if (IX_CRYPTO_ACC_STATUS_SUCCESS != status)
    {
	printk ("Registration Failed. Status = %d\n", status);
	return (IX_FAIL);
    }

    /* Wait for the reverse register callback to return */
    while (!reverseRegisterCBCalled)
    {
	i=0;
	
	/* Sleep 1/10 s */
	ixOsServTaskSleep (IX_CRYPTOACC_CODELET_REGISTER_WAIT_TIME);
	
	if (i > IX_CRYPTOACC_CODELET_MAX_TIME_WAIT)
	{
	    printk ("\nRegistration incomplete, timeout for reverse callback\n");
	    return (IX_FAIL);
	}
	i++;
    }
    
    return IX_SUCCESS;
} /* End of ixCryptoAccCodeletRegisterCtx () function */

void ixCryptoAccCodeletUnregisterCtx(void)
{

    if (IX_CRYPTO_ACC_STATUS_SUCCESS != 
	ixCryptoAccCtxUnregister (forwardCtxId))
    {
	printk("\nFailed to unregister context %d\n",forwardCtxId);
    }

    if (IX_CRYPTO_ACC_STATUS_SUCCESS != 
	ixCryptoAccCtxUnregister (reverseCtxId))
    {
	printk("\nFailed to unregister context %d\n",reverseCtxId);
    }

    forwardRegisterCBCalled = FALSE;
    reverseRegisterCBCalled = FALSE;
    
    /* Initialised to an invalid number, zero is a valid ID */
    forwardCtxId = IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS;
    reverseCtxId = IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS;
}

#ifdef COEXIST

/*******************************************************************************
 * The ethAcc codelet are included in this section for temporary co-exist testing
 * with ethAcc component running on Port 1  using NPE C. Basically just one port
 * will be used to receive packets from Smartbit (SB) and then retransmit the 
 * packet unmodified back to the same source. 
 * SB 7110 cards will be used.
 *
 * ethAcc addition to be added:
 *
 * 1.	Include both ethAcc codelet header files:IxEthAccCodelet.h and
 *	IxEthAccCodelet_p.h, IxEthDB.h.
 * 2.	Other header files to be included are: 
 *	#ifdef IX_ETHACC_CODELET_USE_NVRAM_MAC 
 *	#include "config.h"
 *	#endif
 * 3.	All the declaration stated on top of this file.
 * 4.	Functions required for the co-exist test in NPE B:
 *	a.ixEthAccCodeletInit
 *	b.ixEthAccCodeletNpeInit
 *	c.ixEthAccCodeletPhyInit
 *	d.ixEthAccCodeletMemPoolInit
 *	e.ixEthAccCodeletDispatcherPoll
 *	f.ixEthAccCodeletDispatcherStart
 *	g.ixEthAccCodeletMacSet
 *	h.ixEthAccCodeletSwLoopback
 *	i.ixEthAccCodeletShow
 *	j.ixEthAccCodeletRecoverBuffers
 *	k.ixEthAccCodeletReplenishBuffers
 *	l.ixEthAccCodeletStatsPollTask
 *	m.ixEthAccCodeletLoop
 *	n.ixEthAccCodeletSwLoopbackTxCB
 *	o.ixEthAccCodeletSwLoopbackRxCB
 *	p.ixEthAccCodeletMemPoolFreeCB
 *	q.ixEthAccCodeletStart
 *	r.ixEthAccCodeletStop
 *	
 *
 */


/*
 * Function definition: ixEthAccCodeletInit()
 *
 * See header file for documentation.
 */

IX_STATUS ixEthAccCodeletInit(void)
{  
    unsigned int statstid;

    if(ixEthAccCodeletInitialised) 
    {
	printk("Ethernet codelet already initialised\n");
	return(IX_SUCCESS);
    }

    /* Initialise Queue Manager */
    printk("Initialising Queue Manager...\n");
    if (IX_SUCCESS != ixQMgrInit())
    {
	printk("Error initialising queue manager!\n");
	return (IX_FAIL);
    }

    /* Start the Queue Manager dispatcher loop */   
    if(IX_SUCCESS != ixEthAccCodeletDispatcherStart(IX_CRYPTOACC_CODELET_QMGR_MODE))
    {
	printk("Error starting queue manager dispatch loop!\n");
	return (IX_FAIL);
    }
    
    /* Initialise NPEs, Both NPEs are initialize because 
     * ethAcc component needs both the NPE initialized
     */
    printk("Initialising NPEs...\n");
    if(IX_SUCCESS != ixEthAccCodeletNpeInit(IX_NPEDL_NPEID_NPEB))
    {
	printk("Error initialising NPE B!\n");
	return (IX_FAIL);
    }
 
    if(IX_SUCCESS != ixEthAccCodeletNpeInit(IX_NPEDL_NPEID_NPEC))
    {
	printk("Error initialising NPE C!\n");
	return (IX_FAIL);
    }

    /* Initialise NPE Message handler */
    printk("\nStarting NPE message handler...\n");
    if(IX_SUCCESS != ixNpeMhInitialize(IX_NPEMH_NPEINTERRUPTS_YES))
    {
	printk("Error initialising NPE Message handler!\n");
	return (IX_FAIL);
    }

    /* Start NPEs */
    printk("Starting NPEs...\n");
    if(IX_SUCCESS != ixNpeDlNpeExecutionStart(IX_NPEDL_NPEID_NPEB))
    {
	printk("Error starting NPE B!\n");
	return (IX_FAIL);
    }

    if(IX_SUCCESS != ixNpeDlNpeExecutionStart(IX_NPEDL_NPEID_NPEC))
    {
	printk("Error starting NPE C!\n");
	return (IX_FAIL);
    }

    ixEthAccCodeletNpeInitialised = TRUE;

    /***********************************************************************
     *
     * System initialisation done. Now initialise Ethernet Access components. 
     *
     ***********************************************************************/

    if (IX_ETH_ACC_SUCCESS != ixEthAccInit())
    {
	printk("Error initialising Ethernet access driver!\n");
	return (IX_FAIL);
    }

    /* Initialise Ethernet ports */
    if(IX_ETH_ACC_SUCCESS != ixEthAccPortInit(IX_ETH_PORT_1))
    {
	printk("Error initialising Ethernet port 1!\n");	
	return (IX_FAIL);
    }

    if(IX_ETH_ACC_SUCCESS != ixEthAccPortInit(IX_ETH_PORT_2))
    {
	printk("Error initialising Ethernet port 2!\n");	
	return (IX_FAIL);
    }

    /* Find and initialise all available PHYs */
    if(IX_SUCCESS != ixEthAccCodeletPhyInit())
    {
	printk("Error initialising Ethernet phy(s)!\n");
	return (IX_FAIL);
    }

    /* Program MAC addresses for available PHYs */
    if(IX_SUCCESS != ixEthAccCodeletMacSet())
    {
	printk("Error programming MAC address for Ethernet phy(s)!\n");
	return (IX_FAIL);
    }

    /* Set ports to promiscuous mode  */
    if(IX_ETH_ACC_SUCCESS != ixEthAccPortPromiscuousModeSet(IX_ETH_PORT_1))
    {
	printk("Error setting promiscuous mode for port 1\n");
	return (IX_FAIL);
    }

    if(IX_ETH_ACC_SUCCESS != ixEthAccPortPromiscuousModeSet(IX_ETH_PORT_2))
    {
	printk("Error setting promiscuous mode for port 2\n");
	return (IX_FAIL);
    }

    /* Initialise MBUF pool */
    if(IX_SUCCESS != ixEthAccCodeletMemPoolInit())
    {
	printk("Error initialising mBuf pool\n");
	return (IX_FAIL);
    }

    /* Set scheduling discipline */
    ixEthAccTxSchedulingDisciplineSet(IX_ETH_PORT_1, FIFO_NO_PRIORITY);
    ixEthAccTxSchedulingDisciplineSet(IX_ETH_PORT_2, FIFO_NO_PRIORITY);

    ixEthAccCodeletPollEnabled = FALSE;
    if (IX_OSSL_ERROR_SUCCESS != 
	ix_ossl_thread_create((ix_ossl_thread_entry_point_t)
                              ixEthAccCodeletStatsPollTask,
                              NULL,
                              &statstid))
    {
        printk("Error spawning stats task\n");
        return (IX_FAIL);
    }

    ixEthAccCodeletInitialised = TRUE;
    return (IX_SUCCESS);

} /* End of ixEthAccCodeletInit */


/**
 * @fn IX_STATUS ixEthAccCodeletNpeInit(IxNpeDlNpeId npeId)
 *
 * @param npeId - ID of NPE to initialise
 *
 * Download microcode to the NPE. This function always downloads the latest
 * version of NPE microcode available.
 *
 * @return IX_SUCCESS - NPE successfuly initialised
 * @return IX_FAIL - Error initialising NPE
 */

PRIVATE IX_STATUS ixEthAccCodeletNpeInit(IxNpeDlNpeId npeId)
{
    IxNpeDlVersionId npeIdList[IX_CRYPTOACC_CODELET_NPE_LIST];
    UINT32 npeListSize = IX_CRYPTOACC_CODELET_NPE_LIST;

    IxNpeDlVersionId tmpIdList;
    UINT32 i;
    UINT32 endBuiltId = 0;

    tmpIdList.npeId = npeId;
    if (IX_NPEDL_NPEID_NPEC == npeId)
    {
	tmpIdList.buildId = CRYPTO_START_BUILD_ID;
	endBuiltId = CRYPTO_END_BUILD_ID;
    }
    else
    {
	tmpIdList.buildId = 0;
    }
    tmpIdList.major = 0; 
    tmpIdList.minor = 0;
    
    /* Stop the NPE before attempting a download */
    if(IX_SUCCESS != ixNpeDlNpeStopAndReset(npeId))
    {
	printk("NPE stop and reset failed\n");
	return (IX_FAIL);
    }

    /* Get the latest version of NPE code available */
    if(IX_SUCCESS != ixNpeDlAvailableVersionsListGet(npeIdList,
						     &npeListSize))
    {
	printk("Unable to get NPE version information\n");
	return (IX_FAIL);
    }

    for(i = 0; i < npeListSize; i++)
    {
	if(npeIdList[i].npeId == npeId)
	{
	    if((npeIdList[i].buildId > tmpIdList.buildId) &&
	       (npeIdList[i].buildId <= endBuiltId))
	    {
		tmpIdList.buildId = npeIdList[i].buildId;
		tmpIdList.major = npeIdList[i].major;
		tmpIdList.minor = npeIdList[i].minor;
	    }
	    else if(npeIdList[i].buildId == tmpIdList.buildId)
	    {
		if(npeIdList[i].major > tmpIdList.major)
		{
		    tmpIdList.major = npeIdList[i].major;
		    tmpIdList.minor = npeIdList[i].minor;
		}
		else if(npeIdList[i].major == tmpIdList.major)
		{
		    if(npeIdList[i].minor >= tmpIdList.minor)
		    {
			tmpIdList.minor = npeIdList[i].minor;
		    }
		}
	    }
	}
    }

    printk("\nNPE %d Version information\n", tmpIdList.npeId);
    printk("Build ID:\t 0x%x\n", tmpIdList.buildId);
    printk("Major ID:\t 0x%x\n", tmpIdList.major);
    printk("Minor ID:\t 0x%x\n", tmpIdList.minor);


     /* Download NPE code */
    if(IX_SUCCESS != ixNpeDlVersionDownload(&tmpIdList, TRUE))
    {
	printk("NPE download failed\n");
	return(IX_FAIL);
    }

    return(IX_SUCCESS);
}


/**
 * @fn IX_STATUS ixEthAccCodeletPhyInit()
 *
 * This function scans for and then initialises any available PHYs on
 * the board. It uses the EthAcc MII library routines to do so.
 * 
 * @return IX_SUCCESS - PHY(s) detected and initialised
 * @return IX_FAIL - Error initialising PHY(s)
 */

PRIVATE IX_STATUS ixEthAccCodeletPhyInit(void)
{
   BOOL phyPresent[IXP425_ETH_ACC_MII_MAX_ADDR];

   UINT32 phyNo;
   UINT32 phyNoAddr;

   /* Scan for available Ethernet PHYs on the board */
   if(ERROR == ixEthAccMiiPhyScan(phyPresent))
   {	
       return (IX_FAIL);
   }
   else
   {
       for(phyNoAddr=0, phyNo=0; phyNoAddr<IXP425_ETH_ACC_MII_MAX_ADDR; phyNoAddr++)
       {
	   if(phyPresent[phyNoAddr] == TRUE)
	   {
	       phyAddresses[phyNo] = phyNoAddr;

	       if(++phyNo == IX_ETHACC_CODELET_MAX_PORT)
	       {
		   break;
	       }
	   }
       }
   }
    
   for(phyNo=0; phyNo<IX_ETHACC_CODELET_MAX_PORT; phyNo++)
   {
       /* Reset each phy */
       ixEthAccMiiPhyReset(phyAddresses[phyNo]);

       /* Set each phy properties */
       ixEthAccMiiPhyConfig(phyAddresses[phyNo],
			    phyConf.speed100, 
			    phyConf.fullDuplex, 
			    phyConf.autonegotiate);

       /* Set the MAC to the same duplex mode as the phy */
       if(phyConf.fullDuplex)
       {
	   ixEthAccPortDuplexModeSet (phyNo, IX_ETH_ACC_FULL_DUPLEX);
       }
       else
       {
	   ixEthAccPortDuplexModeSet (phyNo, IX_ETH_ACC_HALF_DUPLEX);
       }

       printk("\nPHY %d configuration:\n", phyNo); 
       ixEthAccMiiShow(phyAddresses[phyNo]);
   }

   return (IX_SUCCESS);
} /* End of ixEthAccCodeletPhyInit */


/**
 * @fn IX_STATUS ixEthAccCodeletMemPoolInit()
 *
 * Initialise Ethernet Access Codelet MBUF pool
 *
 * @return IX_SUCCESS - MBuf pool successfully initialised
 * @return IX_FAIL - Error initialising MBuf pool
 */

PRIVATE IX_STATUS ixEthAccCodeletMemPoolInit(void)
{
    UINT32 bufNo;
    IX_MBUF *mBufPtr;

    if (NULL == 
	(ixEthAccCodeletBufPool = 
	 IX_ACC_DRV_DMA_MALLOC(IX_ETHACC_CODELET_MBUF_POOL_SIZE*sizeof(IX_MBUF))))
    {
	printk("Error allocating mBuf pool for Ethernet Codelet!\n");
	return (IX_FAIL);
    }
    
    mBufPtr = ixEthAccCodeletBufPool;

    if (NULL == (ixEthAccCodeletBufData = 
	 IX_ACC_DRV_DMA_MALLOC(IX_ETHACC_CODELET_MBUF_DATA_POOL_SIZE)))
    {
	printk("Error allocating mBuf data pool for Ethernet Codelet!\n");
	return (IX_FAIL);
    }

    /* Format our Receive mBufs */
    for(bufNo = 0;
	bufNo < IX_ETHACC_CODELET_RX_MBUF_POOL_SIZE*IX_ETHACC_CODELET_MAX_PORT;
	bufNo++)
    {
	/* Initialise payload */ 
	IX_MBUF_MDATA(mBufPtr) = 
	    (char *)&ixEthAccCodeletBufData[bufNo*IX_ETHACC_CODELET_PCK_SIZE];

        /* Initialise length */
	IX_MBUF_MLEN(mBufPtr) = IX_MBUF_PKT_LEN(mBufPtr) = IX_ETHACC_CODELET_PCK_SIZE;
	
	/* No chains */
        IX_MBUF_NEXT_BUFFER_IN_PKT_PTR(mBufPtr) = NULL;
        mBufPtr->m_flags = 0x0;

	/* Add newly formated buffer to our Receive free queue */
	IX_ETHACC_CODELET_ADD_MBUF_TO_Q_TAIL(ixEthAccCodeletFreeBufQ, mBufPtr);
	mBufPtr++;
    }

    /* Format our Transmit mBufs */
    for(bufNo = IX_ETHACC_CODELET_RX_MBUF_POOL_SIZE*IX_ETHACC_CODELET_MAX_PORT; 
	bufNo < (IX_ETHACC_CODELET_MBUF_POOL_SIZE); 
	bufNo++)
    {
	/* Initialise payload */ 
	IX_MBUF_MDATA(mBufPtr) = (char *)&ixEthAccCodeletBufData[bufNo*IX_ETHACC_CODELET_PCK_SIZE];

        /* Initialise length */
	IX_MBUF_MLEN(mBufPtr) = IX_MBUF_PKT_LEN(mBufPtr) = IX_ETHACC_CODELET_PCK_SIZE;
	
        /* No chains */
        IX_MBUF_NEXT_BUFFER_IN_PKT_PTR(mBufPtr) = NULL;
        mBufPtr->m_flags = 0x0;

	/* Add newly formated buffer to our free queue */
	IX_ETHACC_CODELET_ADD_MBUF_TO_Q_TAIL(ixEthAccCodeletTxFreeBufQ, mBufPtr);
	mBufPtr++;
    }

    return (IX_SUCCESS);

} /* End of ixEthCodeletMemPoolInit */


/**
 * @fn void ixEthAccCodeletDispatcherPoll
 *
 * This function polls the Queue manager loop.
 * 
 * @return void
 */

PRIVATE void ixEthAccCodeletDispatcherPoll (void* arg, void** ptrRetObj)
{
    while (1)
    {
	ix_ossl_sleep(1);
	ixQMgrDispatcherLoopRun (IX_QMGR_QUELOW_GROUP);
    }
} /* End of ixEthAccCodeletDispatcherPoll */



/**
 * @fn IX_STATUS ixEthAccCodeletDispatcherStart()
 *
 * @param BOOL useInterrupt - start in interrupt or polled mode
 *
 * This function starts the Queue manager dispatch timer.
 * 
 * @return IX_SUCCESS - Dispatch timer successfully started
 * @return IX_FAIL - Error starting dispatch timer
 */

PRIVATE IX_STATUS ixEthAccCodeletDispatcherStart(BOOL useInterrupt)
{
    unsigned int dispatchtid;

    if(useInterrupt)	/* Interrupt mode */
    {
	/* 
	 * Hook the QM QLOW dispatcher to the interrupt controller. 
	 */
	if (IX_SUCCESS != 
	    ixOsServIntBind(IXP425_INT_LVL_QM1,
			    (IxVoidFnPtr)(ixQMgrDispatcherLoopRun),
			    (void *)IX_QMGR_QUELOW_GROUP))
	{
	    printk("Failed to bind to QM1 interrupt\n");
	    return (IX_FAIL);
	}
    }
    else	/* Polled mode */
    {
 	if (IX_OSSL_ERROR_SUCCESS != 
	    ix_ossl_thread_create((ix_ossl_thread_entry_point_t)
				  ixEthAccCodeletDispatcherPoll,
				  NULL,
				  &dispatchtid))
	{
	    printk("Error spawning dispatch task\n");
	    return (IX_FAIL);
	}

	ix_ossl_thread_set_priority(dispatchtid, IX_ETHACC_CODELET_QMR_PRIORITY);
    }

    return (IX_SUCCESS);
} /* End of ixEthAccCodeletDispatcherStart */


/**
 * @fn IX_STATUS ixEthAccCodeletMacSet()
 *
 * Set the MAC address for available Ports on the board.
 *
 * @return IX_SUCCESS - MAC address successfuly set
 * @return IX_FAIL - Error setting MAC address
 */

PRIVATE IX_STATUS ixEthAccCodeletMacSet(void)
{
    IxEthAccMacAddr npeMacAddr2 = IX_ETHACC_CODELET_NPE2_MAC;

#ifdef IX_ETHACC_CODELET_USE_NVRAM_MAC
    printk("\nReading MAC address information from non-volatile
storage...\n");

    if(ERROR != sysNvRamGet((UINT8 *)&npeMacAddr2.macAddress, 
			    IX_IEEE803_MAC_ADDRESS_SIZE, 
			    NV_MAC_ADRS_NPE2))
    {
	printk("Unable to read NPE2 MAC address from non-volatile
storage!\n");
	return (IX_FAIL);
    }
#endif

    if(IX_ETH_ACC_SUCCESS != 
       ixEthAccPortUnicastMacAddressSet(IX_ETH_PORT_2, &npeMacAddr2))
    {
	return (IX_FAIL);
    }

    printk("PHY %d MAC address is:\t", IX_ETH_PORT_2);
    ixEthAccPortUnicastAddressShow(IX_ETH_PORT_2);

    return (IX_SUCCESS);

} /* End of ixEthAccCodeletMacSet */


/*
 * Function definition: ixEthAccCodeletStart()
 *
 * See header file for documentation.
 */

IX_STATUS ixEthAccCodeletStart(IxEthAccPortRxCallback port1RxCB,
			       IxEthAccPortRxCallback port2RxCB,
			       IxEthAccPortTxDoneCallback port1TxDoneCB,
			       IxEthAccPortTxDoneCallback port2TxDoneCB)
{
    IX_ETHACC_IS_CODELET_INITIALISED();

    /* Clear stats */
    memset(&IxEthAccCodeletStats[0], 0, sizeof(IxEthAccCodeletStats[0]));
    memset(&IxEthAccCodeletStats[1], 0, sizeof(IxEthAccCodeletStats[1]));

    /* Initialize CB for port 2 only, as port 1 is not used */
   
    if (port2RxCB != NULL)
    {
	if(IX_ETH_ACC_SUCCESS != 
	   ixEthAccPortRxCallbackRegister(IX_ETH_PORT_2,
					  port2RxCB,
					  IX_ETH_PORT_2))
	{
	    printk("Failed to register Rx callback for port 2\n");
	    return (IX_FAIL);
	}
    }

    if(port2TxDoneCB != NULL)
    {
	if(IX_ETH_ACC_SUCCESS != 
	   ixEthAccPortTxDoneCallbackRegister(IX_ETH_PORT_2,
					      port2TxDoneCB,
					      IX_ETH_PORT_2))
	{
	    printk("Failed to register Tx done callback for port 2\n");
	    return (IX_FAIL);
	}
    }

    if (ixEthAccCodeletNpeInitialised == FALSE)
    {
	/* Initialise NPEs, Both NPEs are initialize because 
	 * ethAcc component needs both the NPE initialized
	 */
	printk("Initialising NPEs...\n");
	if(IX_SUCCESS != ixEthAccCodeletNpeInit(IX_NPEDL_NPEID_NPEB))
	{
	    printk("Error initialising NPE B!\n");
	    return (IX_FAIL);
	}
	
	if(IX_SUCCESS != ixEthAccCodeletNpeInit(IX_NPEDL_NPEID_NPEC))
	{
	    printk("Error initialising NPE C!\n");
	    return (IX_FAIL);
	}
	
	/* Start NPEs */
	printk("Starting NPEs...\n");
	if(IX_SUCCESS != ixNpeDlNpeExecutionStart(IX_NPEDL_NPEID_NPEB))
	{
	    printk("Error starting NPE B!\n");
	    return (IX_FAIL);
	}
	
	if(IX_SUCCESS != ixNpeDlNpeExecutionStart(IX_NPEDL_NPEID_NPEC))
	{
	    printk("Error starting NPE C!\n");
	    return (IX_FAIL);
	}
	ixEthAccCodeletNpeInitialised = TRUE;
    }

    if(IX_SUCCESS != 
       ixEthAccCodeletReplenishBuffers(IX_ETH_PORT_2,
				       IX_ETHACC_CODELET_RX_MBUF_POOL_SIZE))
    {
	printk("Error replenish port 2\n");
	return (IX_FAIL);
    }

    /* 
     * The receive path does not strip FCS so we disable appending of FCS on the
     * transmit path
     */
  
    if(IX_ETH_ACC_SUCCESS != ixEthAccPortTxFrameAppendFCSDisable(IX_ETH_PORT_2))
    {
	printk("Error disabling FCS for port 2\n");
	return (IX_FAIL);
    }

    /* Enable port 2 only */
  
    if(IX_ETH_ACC_SUCCESS != ixEthAccPortEnable(IX_ETH_PORT_2))
    {
	printk("Error enabling port 2\n");
	return (IX_FAIL);
    }

    return IX_SUCCESS;
} /* End of ixEthAccCodeletStart */


IX_STATUS ixEthAccCodeletStop(IxEthAccPortRxCallback port1RxCB,
			       IxEthAccPortRxCallback port2RxCB,
			       IxEthAccPortTxDoneCallback port1TxDoneCB,
			       IxEthAccPortTxDoneCallback port2TxDoneCB)
{
    if (port2RxCB != NULL)
    {
	if(IX_ETH_ACC_SUCCESS != 
	   ixEthAccPortRxCallbackRegister(IX_ETH_PORT_2,
					  port2RxCB,
					  IX_ETH_PORT_2))
	{
	    printk("Failed to register Rx callback for port 2\n");
	    return (IX_FAIL);
	}
    }

    if(port2TxDoneCB != NULL)
    {
	if(IX_ETH_ACC_SUCCESS != 
	   ixEthAccPortTxDoneCallbackRegister(IX_ETH_PORT_2,
					      port2TxDoneCB,
					      IX_ETH_PORT_2))
	{
	    printk("Failed to register Tx done callback for port 2\n");
	    return (IX_FAIL);
	}
    }

    if (TRUE == ixEthAccCodeletNpeInitialised)
    {
	/* Disable port 2 - unused MBufs will be returned */
	
	if(IX_ETH_ACC_SUCCESS != ixEthAccPortDisable(IX_ETH_PORT_2))
	{
	    printk("Error disabling port 2\n");
	    return (IX_FAIL);
	}
    
	ixEthAccCodeletNpeInitialised = FALSE;
    }

    return IX_SUCCESS;
} /* End of ixEthAccCodeletStop */


/*
 * Function definition: ixEthAccCodeletSwLoopback()
 *
 * See header file for documentation.
 */

IX_STATUS ixEthAccCodeletSwLoopback(void)
{
    IX_ETHACC_IS_CODELET_INITIALISED();

    if (IX_SUCCESS !=  
	ixEthAccCodeletStart(ixEthAccCodeletSwLoopbackRxCB, 
			     ixEthAccCodeletSwLoopbackRxCB, 
			     ixEthAccCodeletSwLoopbackTxCB,
			     ixEthAccCodeletSwLoopbackTxCB))
    {
	printk("Failed to start the Sw Loopback Demo\n");
	return (IX_FAIL);
    }


    printk("Software loopback successfully started. Begin sending packets
from external source\n");  

    if(IX_SUCCESS != ixEthAccCodeletLoop())
    {
	return (IX_FAIL);
    }

    /* Recover buffers */
    if(IX_SUCCESS != ixEthAccCodeletRecoverBuffers())
    {
	printk("Error recovering buffers for Sw Loopback demo\n");
	return (IX_FAIL);
    }    

    return (IX_SUCCESS);
} /* End of ixEthAccCodeletSwLoopback */


/*
 * Function definition: ixEthAccCodeletShow()
 *
 * See header file for documentation.
 */

void ixEthAccCodeletShow(void)
{
    static IxEthEthObjStats *portStats = NULL;
    UINT32 portNo;

    if(NULL == portStats)
    {
	/*
	 * The statistics are gathered by the NPE therefore we use DMA_MALLOC
	 * to make it cache safe for us to read.
	 */
	if(NULL == (portStats = IX_ACC_DRV_DMA_MALLOC(sizeof(IxEthEthObjStats))))
	{
	    printk("Unable to create port stats!\n");
	    return;
	}
    }

    for(portNo=0; portNo<IX_ETHACC_CODELET_MAX_PORT; portNo++)
    {
	memset(portStats, 0, sizeof(IxEthEthObjStats));
	
	printk("Statistics for port %d:\n", portNo);
	if(IX_ETH_ACC_SUCCESS != ixEthAccMibIIStatsGetClear(portNo, portStats))
	{
	    printk("Unable to retrieve statistics for port %d!\n",portNo);
	}
	else
	{
	    printk("dot3StatsAlignmentErrors: \t\t%u\n",
		   portStats->dot3StatsAlignmentErrors);
	    printk("dot3StatsFCSErrors: \t\t\t%u\n",
		   portStats->dot3StatsFCSErrors);
	    printk("dot3StatsFrameTooLongs: \t\t%u\n",
		   portStats->dot3StatsFrameTooLongs);
	    printk("dot3StatsInternalMacReceiveErrors: \t%u\n",
		   portStats->dot3StatsInternalMacReceiveErrors);
	    printk("dot3StatsSingleCollisionFrames: \t%u\n",
		   portStats->dot3StatsSingleCollisionFrames);
	    printk("dot3StatsMultipleCollisionFrames: \t%u\n",
		   portStats->dot3StatsMultipleCollisionFrames);
	    printk("dot3StatsDeferredTransmissions: \t%u\n",
		   portStats->dot3StatsDeferredTransmissions);
	    printk("dot3StatsLateCollisions: \t\t%u\n",
		   portStats->dot3StatsLateCollisions);
	    printk("dot3StatsExcessiveCollisions: \t\t%u\n",
		   portStats->dot3StatsExcessiveCollsions);
	    printk("dot3StatsInternalMacTransmitErrors: \t%u\n",
		   portStats->dot3StatsInternalMacTransmitErrors);
	    printk("dot3StatsCarrierSenseErrors: \t\t%u\n",
		   portStats->dot3StatsCarrierSenseErrors);
	    printk("LearnedEntryDiscards: \t\t\t%u\n",
		   portStats->LearnedEntryDiscards);

	    printk("\n");
	}
    }
} /* End of ixEthAccCodeletShow function */

/**
 * @fn IX_STATUS ixEthAccCodeletRecoverBuffers()
 *
 * Recover buffers used by a demo into the free queue.
 *
 * @return IX_SUCCESS - Recovery succeeded
 * @return IX_FAIL - Error recovering buffers
 */

IX_STATUS ixEthAccCodeletRecoverBuffers(void)
{
    memPoolFreeCnt = 0;

    /* Register callbacks to recover the buffers used */
    if(IX_SUCCESS != ixEthAccCodeletStop(ixEthAccCodeletMemPoolFreeRxCB,
					 ixEthAccCodeletMemPoolFreeRxCB,
					 ixEthAccCodeletMemPoolFreeTxCB,
					 ixEthAccCodeletMemPoolFreeTxCB))
    {
	printk("Failed to stop the codelet\n");
	return (IX_FAIL);
    }

    /* Sleep 1 sec */
    ix_ossl_sleep(1000);
    
    printk("Recovering buffers... "
	   "%d buffers recovered\n", memPoolFreeCnt);

    return (IX_SUCCESS);
} /* End of ixEthAccCodeletRecoverBuffers */

/**
 * @fn IX_STATUS ixEthAccCodeletReplenishBuffers()
 *
 * @param portNo - port number to replenish buffers into.
 * @param num - number of buffers to replenish.
 *
 * Function to replenish buffers into the Rx free queue.
 *
 * @return IX_SUCCESS - Replenish succeeded
 * @return IX_FAIL - Error replenishing buffers in Rx queue for port
 */

PRIVATE IX_STATUS ixEthAccCodeletReplenishBuffers(IxEthAccPortId portNo,
						 UINT32 num)
{
    IX_MBUF *mBufPtr;
    UINT32 numBufs;

    for(numBufs=0; numBufs<num; numBufs++)
    {
	if(IX_ETHACC_CODELET_IS_Q_EMPTY(ixEthAccCodeletFreeBufQ))
	{
	    printk("Buffer queue empty."
		   "Not enough free buffers for port %d!\n", portNo+1);
	    return (IX_FAIL);
	}
	
	IX_ETHACC_CODELET_REMOVE_MBUF_FROM_Q_HEAD(ixEthAccCodeletFreeBufQ, mBufPtr);
	IX_MBUF_MLEN(mBufPtr) = IX_MBUF_PKT_LEN(mBufPtr) = IX_ETHACC_CODELET_PCK_SIZE;
	IX_ACC_DATA_CACHE_INVALIDATE(IX_MBUF_MDATA(mBufPtr), IX_MBUF_MLEN(mBufPtr));
	
	if(IX_SUCCESS != ixEthAccPortRxFreeReplenish(portNo, mBufPtr))
	{
	    printk("Error replenishing free queue on port %d\n",
portNo+1);
	    return (IX_FAIL);
	}
    }

    printk("Port %d Rx Free pool has %d buffers\n", portNo+1, numBufs);
    return (IX_SUCCESS);
} /* End of ixEthAccCodeletReplenishBuffers */


/**
 * @fn void ixEthAccCodeletStatsPollTask
 *
 * This task polls the Codelet Stats and displays the rate of Rx and 
 * Tx packets per second.
 * 
 * @return void
 */

PRIVATE void ixEthAccCodeletStatsPollTask (void* arg, void** ptrRetObj)
{
    static char stillRunning[] = "|/-\\";
    static int  stillRunningIndex = 0;

    while (1)
    {
    while (FALSE == ixEthAccCodeletPollEnabled)
    {
	/* Sleep 1 sec */
	ix_ossl_sleep(1000);
    }

    printk("\nPort1 Rates:              | Port2 Rates:\n"
	   "=====================================================\n");
    while (TRUE == ixEthAccCodeletPollEnabled)
    {
	/* Clear stats */
	IxEthAccCodeletStats[0].rxCount=0;
	IxEthAccCodeletStats[0].txCount=0;
	IxEthAccCodeletStats[1].rxCount=0;
	IxEthAccCodeletStats[1].txCount=0;

	/* Sleep 1 sec */
	ix_ossl_sleep(1000);
        if  (ixEthAccCodeletPollEnabled == TRUE)
        {
        /* Display stats */
        printk("\rRx: %6u     Tx: %6u | Rx: %6u     Tx: %6u %c",
	       IxEthAccCodeletStats[0].rxCount,
	       IxEthAccCodeletStats[0].txCount,
	       IxEthAccCodeletStats[1].rxCount,
	       IxEthAccCodeletStats[1].txCount,
               stillRunning[stillRunningIndex++ & 3]);
        }
    }
    printk("\n");
    }
} /* End of ixEthAccCodeletStatsPollTask */



/**
 * @fn void ixEthAccCodeletLoop()
 *
 * This function is called once a demo has started. It spawns the
 * stats poll task and waits for use interrupt to end the codelet.
 * 
 * @return IX_SUCCESS - demo successfully run
 * @return IX_FAIL - error spawning stats poll task
 */

PRIVATE IX_STATUS ixEthAccCodeletLoop(void)
{
    int ch = 0;

    printk("Press ESC at any time to stop the demo ... \n");

    ixEthAccCodeletPollEnabled = TRUE;

    do
    {
	ch = getchar();
    }while(ch != 27);
    
    ixEthAccCodeletPollEnabled = FALSE;

    /* Dump Stats */
    ixEthAccCodeletShow();

    return (IX_SUCCESS);
} /* End of ixEthAccCodeletLoop */

/******************************************************************************
 *
 * Ethernet Codelet callback functions
 *
 ******************************************************************************/

/**
 * @fn void ixEthAccCodeletSwLoopbackTxCB()
 *
 * Transmit callback for Sw Loopback demo. Transmitted frames are put on
 * the receive free pool.
 * 
 * @return void
 */

PRIVATE void ixEthAccCodeletSwLoopbackTxCB(UINT32 cbTag, IX_MBUF* mBufPtr)
{
    /* Put the mbuf back on the rx free pool */
    if(IX_ETH_PORT_1 == cbTag || IX_ETH_PORT_2 == cbTag)
    {
	IxEthAccCodeletStats[cbTag].txCount++;

	IX_MBUF_MLEN(mBufPtr) = IX_MBUF_PKT_LEN(mBufPtr) = IX_ETHACC_CODELET_PCK_SIZE;
	if(IX_SUCCESS != ixEthAccPortRxFreeReplenish(cbTag, mBufPtr))
	{
	    ixOsServLog(LOG_ERROR, "Error replenishing RX free queue on port %d\n", 
			cbTag+1, 0, 0, 0, 0, 0);
	}    
    }
    else
    {
	ixOsServLog(LOG_ERROR, "Invalid Tx callback tag %d in RxSink demo\n", 
		    cbTag, 0, 0, 0, 0, 0);
    }
} /* End of ixEthAccCodeletSwLoopbackTxCB */

/**
 * @fn void ixEthAccCodeletSwLoopbackRxCB()
 *
 * Receive callback for the Sw Loopback demo. Any frames received are transmited
 * back out on the same port.
 * 
 * @return void
 */

PRIVATE void ixEthAccCodeletSwLoopbackRxCB(UINT32 cbTag, IX_MBUF* mBufPtr, IxEthAccPortId portId)
{
    /* Transmit the buffer back */
    if(IX_ETH_PORT_1 == cbTag || IX_ETH_PORT_2 == cbTag)
    {
	IxEthAccCodeletStats[cbTag].rxCount++;

	if(IX_ETH_ACC_SUCCESS != 
	   ixEthAccPortTxFrameSubmit(cbTag, mBufPtr,
				     IX_ETH_ACC_TX_DEFAULT_PRIORITY))
	{
	    ixOsServLog(LOG_ERROR, "Tx buffer submission failure port %d\n", 
			cbTag+1, 0, 0, 0, 0, 0);
	}
    }
    else
    {
	ixOsServLog(LOG_ERROR, "Invalid Rx callback tag %d in Sw Loopback demo\n", 
		    cbTag, 0, 0, 0, 0, 0);
    }
} /* End of ixEthAccCodeletSwLoopbackRxCB */



/*
 * @fn ixEthAccCodeletMemPoolFreeRxCb
 *
 * This callback is used at the end of each ethernet demo.
 * It recovers the buffers used and places them back in the free queue.
 * This way the MBUF pool is free to be used by another ethernet demo.
 *
 * @return void
 */

PRIVATE void ixEthAccCodeletMemPoolFreeRxCB(UINT32 cbTag, IX_MBUF* mBufPtr, IxEthAccPortId portId)
{
    IX_ETHACC_CODELET_ADD_MBUF_TO_Q_TAIL(ixEthAccCodeletFreeBufQ, mBufPtr);
    memPoolFreeCnt++;
} /* End of ixEthAccCodeletMemPoolFreeRxCB */

/*
 * @fn ixEthAccCodeletMemPoolFreeTxCb
 *
 * This callback is used at the end of each ethernet demo.
 * It recovers the buffers used and places them back in the free queue.
 * This way the MBUF pool is free to be used by another ethernet demo.
 *
 * @return void
 */

PRIVATE void ixEthAccCodeletMemPoolFreeTxCB(UINT32 cbTag, IX_MBUF* mBufPtr)
{
    IX_ETHACC_CODELET_ADD_MBUF_TO_Q_TAIL(ixEthAccCodeletFreeBufQ, mBufPtr);
    memPoolFreeCnt++;
} /* End of ixEthAccCodeletMemPoolFreeTxCB */



/* End of ethAcc codelet functions */
#endif /* End of #ifdef COEXIST */

/* Main function for linux */
void test_main(void)
{
    IxCryptoAccStatus status;
    UINT32 mode;
    IX_MBUF *pMbuf;
    UINT16 i,j = 0;
    char data;


#ifdef COEXIST
/* For ethAcc coexist functionality in NPE C
 *
 * 1. Initialise the ethAcc component and codelet - ixEthAccCodeletInit
 * 2. Start SwLoopback - ixEthAccCodeletSwLoopback
 */
#error here in COEXIST

    /* ethAcc Initialisation */
    if (IX_SUCCESS !=  ixEthAccCodeletInit ())
    {
	printk ("ethAcc Initialisation Failed.\n");
	return;
    }

#endif /* End of ifdef COEXIST */

    /* Initialisation */
    if (IX_SUCCESS !=  ixCryptoAccCodeletInit ())
    {
	printk ("Initialisation Failed.\n");
	return;
    }

    /* Initialisation of the mode and packet length */
    mode = IX_CRYPTOACC_CODELET_LINUX_OP;
    ixCryptoAccCodeletPcktLen = IX_CRYPTOACC_CODELET_LINUX_PCKT_LEN;
    
    /* Alloc Mbufs pool */
    if (IX_SUCCESS != ixCryptoAccCodeletMemPoolInit ())
    {
	printk ("Mbuf pool init failed\n");
	return;
    }

    /* Calculate rate ratio used in performance calculation based on buffer size chosen */
    rateRatio = (ixCryptoAccCodeletPcktLen*8*IX_CRYPTOACC_CODELET_BATCH_LEN);
    
    /* Registration */
    if (IX_SUCCESS != ixCryptoAccCodeletRegisterCtx(mode))
    {
	printk ("Registration Failed.\n");
	return;
    }

	/* Send Off Mbufs based on mix parameters */  
	for(i=0;i<IX_CRYPTOACC_CODELET_MBUF_POOL_SIZE;i++)
	{
	    /* Get an mbuf */
	    ixCryptoAccCodeletMbufFromQHeadRemove (&pMbuf);

	    /* Check if the mbufs are NULL */
	    if (NULL == pMbuf)
	    {
		printk ("\nInvalid mbuf!\n");
		return;
	    }
	    
	    /* copy data into mBuf from the ASCII table 0 (0x30) to z (0x7a)*/
	    data = (char) 0x30;
	    for (j=0; j<ixCryptoAccCodeletPcktLen; j++)
	    {
		pMbuf->mBlkHdr.mData[j] = data++;
		
		if((char) 0x7b == data)
		{
		    data =(char) 0x30;
		}
	    }
	    /* Reset the icv field */
	    if ((IX_CRYPTOACC_CODELET_AUTH == mode) || 
		(IX_CRYPTOACC_CODELET_ENCRYPT_AUTH == mode))
	    {	    
		memset (&(pMbuf->mBlkHdr.mData[ixCryptoAccCodeletPcktLen]), 
			0, 
			IX_CRYPTO_ACC_SHA1_DIGEST_160);
	    }
	    
	    /* This while loop will always call perform until it is  not 
	     * successful but will continue to retry perform if the status 
	     * is queue_full. Start with forwardCtxId and the reverseCtxId
	     * will be used in the perform callback.
	     */

// Jack ++
{
int ii = 0;
		printk("before : \n");
	    for (ii=0; ii<ixCryptoAccCodeletPcktLen; ii++)
	      {

		printk("%x\n",pMbuf->mBlkHdr.mData[ii]);

	      }
}
	    /* Start the timeStamp before Perform */
	    timeStamp = ixOsServTimestampGet();
	    status = 
		ixCryptoAccAuthCryptPerform (forwardCtxId,
					     pMbuf,
					     NULL,
					     0,
					     ixCryptoAccCodeletPcktLen,    
					     0,	
					     ixCryptoAccCodeletPcktLen,
					     ixCryptoAccCodeletPcktLen,
					     IV);
	    
	    if (IX_CRYPTO_ACC_STATUS_QUEUE_FULL == status)
	    {
		/* Sleep 1/10 s */
		ixOsServTaskSleep (IX_CRYPTOACC_CODELET_PERFORM_WAIT_TIME);
	    }
	   
	    printk("Placed %dth Packet in to flow\n",i+1);

	} /* End of for loop */
	

#ifdef COEXIST	
	/* ethAcc Loopback */
	if (IX_SUCCESS !=  ixEthAccCodeletSwLoopback ())
	{
	    printk ("Software loopback demo  Failed.\n");
	    return;
	}
#endif /* End of ifdef COEXIST */

// Jack ++	    
#if 0
{
int ii = 0;

		printk("after : \n");
	    for (ii=0; ii<ixCryptoAccCodeletPcktLen; ii++)
	      {
		printk("%x\n",pMbuf->mBlkHdr.mData[ii]);
	      }

}
#else
		printk("%x\n",&pMbuf->mBlkHdr.mData[0]);

#endif


} /* End of test_main () function for linux */

/* End of IxCryptoAccCodelet.c */
