/**
 * @file    IxNpeA.h
 *
 * @date    22-Mar-2002
 *
 * @brief   Header file for the IXP4xx ATM NPE API
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

/**
 * @defgroup IxNpeA IXP4xx NPE-A (IxNpeA) API
 *
 * @brief The Public API for the IXP4xx NPE-A
 *
 * @{
 */

#ifndef IX_NPE_A_H
#define IX_NPE_A_H

#include "IxTypes.h"
#include "IxOsBuffMgt.h"

/* General Message Ids */

/* ATM Message Ids */

/**
 * @def IX_NPE_A_MSSG_ATM_UTOPIA_CONFIG_WRITE       
 *
 * @brief ATM Message ID command to write the data to the offset in the 
 * Utopia Configuration Table 
 */
#define IX_NPE_A_MSSG_ATM_UTOPIA_CONFIG_WRITE       0x20

/**
 * @def IX_NPE_A_MSSG_ATM_UTOPIA_CONFIG_LOAD        
 *
 * @brief ATM Message ID command triggers the NPE to copy the Utopia 
 * Configuration Table to the Utopia coprocessor
 */
#define IX_NPE_A_MSSG_ATM_UTOPIA_CONFIG_LOAD        0x21

/**
 * @def IX_NPE_A_MSSG_ATM_UTOPIA_STATUS_UPLOAD      
 *
 * @brief ATM Message ID command triggers the NPE to read-back the Utopia 
 * status registers and update the Utopia Status Table.   
 */
#define IX_NPE_A_MSSG_ATM_UTOPIA_STATUS_UPLOAD      0x22

/**
 * @def IX_NPE_A_MSSG_ATM_UTOPIA_STATUS_READ        
 *
 * @brief ATM Message ID command to read the Utopia Status Table at the 
 * specified offset.  
 */
#define IX_NPE_A_MSSG_ATM_UTOPIA_STATUS_READ        0x23

/**
 * @def IX_NPE_A_MSSG_ATM_FP_TEMPLATE_WRITE         
 *
 * @brief ATM Message ID command to write the FastPath Template for 
 * fpIndex a word at a time.  
 */
#define IX_NPE_A_MSSG_ATM_FP_TEMPLATE_WRITE         0x24

/**
 * @def IX_NPE_A_MSSG_ATM_TX_ENABLE                 
 *
 * @brief ATM Message ID command triggers the NPE to re-enable processing 
 * of any entries on the TxVcQ for this port. 
 *
 * This command will be ignored for a port already enabled 
 */
#define IX_NPE_A_MSSG_ATM_TX_ENABLE                 0x25

 /**
 * @def IX_NPE_A_MSSG_ATM_TX_DISABLE                
 *
 * @brief ATM Message ID command triggers the NPE to disable processing on 
 * this port
 *
 * This command will be ignored for a port already disabled
 */
#define IX_NPE_A_MSSG_ATM_TX_DISABLE                0x26

/**
 * @def IX_NPE_A_MSSG_ATM_RX_ENABLE
 *
 * @brief ATM Message ID command triggers the NPE to process any received  
 * cells for this VC according to the VC Lookup Table.
 *
 * Re-issuing this command with different contents for a VC that is not 
 * disabled will cause unpredictable behavior.
 */
#define IX_NPE_A_MSSG_ATM_RX_ENABLE                 0x27

/**
 * @def IX_NPE_A_MSSG_ATM_RX_DISABLE                
 *
 * @brief ATM Message ID command triggers the NPE to disable processing for 
 * this VC.
 *
 * This command will be ignored for a VC already disabled
 */
#define IX_NPE_A_MSSG_ATM_RX_DISABLE                0x28

/**
 * @def IX_NPE_A_MSSG_ATM_STATUS_READ               
 *
 * @brief ATM Message ID command to read the ATM status. The data is returned via
 * a response message
 */
#define IX_NPE_A_MSSG_ATM_STATUS_READ               0x29

/**
 * @def IX_NPE_A_MSSG_ATM_FP_ENABLE                 
 *
 * @brief ATM Message ID command triggers the NPE to begin FastPath processing 
 * for this VC at the first cell of the next PDU.
 *
 * Re-issuing this command for a vc already enabled for fastpath may cause 
 * erroneous behavior
 */
#define IX_NPE_A_MSSG_ATM_FP_ENABLE                 0x2a

/**
 * @def IX_NPE_A_MSSG_ATM_FP_DISABLE      
 *
 * @brief ATM Message ID command triggers the NPE to stop FastPath processing 
 * with the completion of any current PDU
 *
 * This command will be ignored for a vc already disabled for fastpath.
 */
#define IX_NPE_A_MSSG_ATM_FP_DISABLE                0x2b

/**
 * @def IX_NPE_A_MSSG_ATM_FP_STATISTICS_READ
 *
 * @brief ATM Message ID command to read the FastPath status word for the 
 * FastPath template identified by fpIndex.  
 *
 * 
 */
#define IX_NPE_A_MSSG_ATM_FP_STATISTICS_READ        0x2c

/** 
 * @def IX_NPE_A_MSSG_ATM_FP_WAN_MAC
 *
 * @brief ATM Message ID command generated by the NPE whenever a new
 * MAC address is detected based upon monitoring of the 6 bytes in a
 * PDU identified by the MacWanLearning operation. 
 */
#define IX_NPE_A_MSSG_ATM_FP_WAN_MAC                0x2d

/** 
 * @def IX_NPE_A_MSSG_ATM_FP_STATISTICS_CLEAR
 *
 * @brief ATM Message ID command to clear the FastPath status word for the 
 * FastPath template
 */
#define IX_NPE_A_MSSG_ATM_FP_STATISTICS_CLEAR       0x2e

/*--------------------------------------------------------------------------
 * HSS Message IDs 
 *--------------------------------------------------------------------------*/

/**
 * @def IX_NPE_A_MSSG_HSS_PORT_CONFIG_WRITE         
 *
 * @brief HSS Message ID command writes the ConfigWord value to the location 
 * in the HSS_CONFIG_TABLE specified by offset for HSS port hPort.
 */
#define IX_NPE_A_MSSG_HSS_PORT_CONFIG_WRITE         0x40

/**
 * @def IX_NPE_A_MSSG_HSS_PORT_CONFIG_LOAD          
 *
 * @brief HSS Message ID command triggers the NPE to copy the contents of the 
 * HSS Configuration Table to the appropriate configuration registers in the 
 * HSS coprocessor for the port specified by hPort. 
 */
#define IX_NPE_A_MSSG_HSS_PORT_CONFIG_LOAD          0x41

/**
 * @def IX_NPE_A_MSSG_HSS_PORT_ERROR_READ           
 *
 * @brief HSS Message ID command triggers the NPE to return an HssErrorReadResponse 
 * message for HSS port hPort.
 */
#define IX_NPE_A_MSSG_HSS_PORT_ERROR_READ           0x42

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_FLOW_ENABLE          
 *
 * @brief HSS Message ID command triggers the NPE to reset internal status and 
 * enable the HssChannelized operation on the HSS port specified by hPort.
 */
#define IX_NPE_A_MSSG_HSS_CHAN_FLOW_ENABLE          0x43

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_FLOW_DISABLE         
 *
 * @brief HSS Message ID command triggers the NPE to disable the HssChannelized 
 * operation on the HSS port specified by hPort.
 */
#define IX_NPE_A_MSSG_HSS_CHAN_FLOW_DISABLE         0x44

/**
 * @def  IX_NPE_A_MSSG_HSS_CHAN_IDLE_PATTERN_WRITE  
 * 
 * @brief HSS Message ID command writes the HSSnC_IDLE_PATTERN value for HSS 
 * port hPort. (n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_CHAN_IDLE_PATTERN_WRITE   0x45

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_NUM_CHANS_WRITE      
 *
 * @brief HSS Message ID command writes the HSSnC_NUM_CHANNELS value for HSS 
 * port hPort. (n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_CHAN_NUM_CHANS_WRITE      0x46

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_RX_BUF_ADDR_WRITE    
 *
 * @brief HSS Message ID command writes the HSSnC_RX_BUF_ADDR value for HSS 
 * port hPort. (n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_CHAN_RX_BUF_ADDR_WRITE    0x47

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_RX_BUF_CFG_WRITE     
 *
 * @brief HSS Message ID command  writes the HSSnC_RX_BUF_SIZEB and  
 * HSSnC_RX_TRIG_PERIOD values for HSS port hPort.  (n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_CHAN_RX_BUF_CFG_WRITE     0x48

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_TX_BLK_CFG_WRITE     
 *
 * @brief HSS Message ID command writes the HSSnC_TX_BLK1_SIZEB, 
 * HSSnC_TX_BLK1_SIZEW, HSSnC_TX_BLK2_SIZEB, and HSSnC_TX_BLK2_SIZEW  values
 * for HSS port hPort. (n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_CHAN_TX_BLK_CFG_WRITE     0x49

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_TX_BUF_ADDR_WRITE    
 * @brief HSS Message ID command writes the HSSnC_TX_BUF_ADDR value for HSS 
 * port hPort. (n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_CHAN_TX_BUF_ADDR_WRITE    0x4A

/**
 * @def IX_NPE_A_MSSG_HSS_CHAN_TX_BUF_SIZE_WRITE    
 *
 * @brief HSS Message ID command writes the HSSnC_TX_BUF_SIZEN value for HSS 
 * port hPort. (n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_CHAN_TX_BUF_SIZE_WRITE    0x4B

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_PIPE_FLOW_ENABLE      
 *
 * @brief HSS Message ID command triggers the NPE to reset internal status and 
 * enable the HssPacketized operation for the flow specified by pPipe on 
 * the HSS port specified by hPort.
 */
#define IX_NPE_A_MSSG_HSS_PKT_PIPE_FLOW_ENABLE      0x50

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_PIPE_FLOW_DISABLE     
 * @brief HSS Message ID command triggers the NPE to disable the HssPacketized 
 * operation for the flow specified by pPipe on the HSS port specified by hPort. 
 */
#define IX_NPE_A_MSSG_HSS_PKT_PIPE_FLOW_DISABLE     0x51

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_NUM_PIPES_WRITE
 * @brief HSS Message ID command writes the HSSnP_NUM_PIPES value for HSS 
 * port hPort.(n=hPort)
 */
#define IX_NPE_A_MSSG_HSS_PKT_NUM_PIPES_WRITE       0x52

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_PIPE_FIFO_SIZEW_WRITE 
 *
 * @brief HSS Message ID command writes the HSSnP_PIPEp_FIFOSIZEW value for 
 * packet-pipe pPipe on HSS port hPort.  (n=hPort, p=pPipe)
 */
#define IX_NPE_A_MSSG_HSS_PKT_PIPE_FIFO_SIZEW_WRITE 0x53

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_PIPE_HDLC_CFG_WRITE   
 *
 * @brief HSS Message ID command writes the HSSnP_PIPEp_HDLC_RXCFG and  
 * HSSnP_PIPEp_HDLC_TXCFG values for packet-pipe pPipe on HSS port hPort. 
 * (n=hPort, p=pPipe)
 */
#define IX_NPE_A_MSSG_HSS_PKT_PIPE_HDLC_CFG_WRITE   0x54

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_PIPE_IDLE_PATTERN_WRITE 
 *
 * @brief HSS Message ID command writes the HSSnP_PIPEp_IDLE_PATTERN value 
 * for packet-pipe pPipe on HSS port hPort.  (n=hPort, p=pPipe)
 */
#define IX_NPE_A_MSSG_HSS_PKT_PIPE_IDLE_PATTERN_WRITE 0x55

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_PIPE_RX_SIZE_WRITE    
 *
 * @brief HSS Message ID command writes the HSSnP_PIPEp_RXSIZEB value for 
 * packet-pipe pPipe on HSS port hPort.  (n=hPort, p=pPipe)
 */
#define IX_NPE_A_MSSG_HSS_PKT_PIPE_RX_SIZE_WRITE    0x56

/**
 * @def IX_NPE_A_MSSG_HSS_PKT_PIPE_MODE_WRITE       
 *
 * @brief HSS Message ID command writes the HSSnP_PIPEp_MODE value for 
 * packet-pipe pPipe on HSS port hPort.  (n=hPort, p=pPipe)
 */
#define IX_NPE_A_MSSG_HSS_PKT_PIPE_MODE_WRITE       0x57



/* Queue Entry Masks */

/*--------------------------------------------------------------------------
 *  ATM Descriptor Structure offsets 
 *--------------------------------------------------------------------------*/

/**
 * @def IX_NPE_A_RXDESCRIPTOR_STATUS_OFFSET          
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Status field 
 *
 * It is used for descriptor error reporting.  
 */
#define IX_NPE_A_RXDESCRIPTOR_STATUS_OFFSET          0

/**
 * @def IX_NPE_A_RXDESCRIPTOR_VCID_OFFSET            
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor VC ID field
 *
 * It is used to hold an identifier number for this VC
 */
#define IX_NPE_A_RXDESCRIPTOR_VCID_OFFSET            1

/**
 * @def IX_NPE_A_RXDESCRIPTOR_CURRMBUFSIZE_OFFSET    
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Current Mbuf
 * Size field
 *
 * Number of bytes the current mbuf data buffer can hold
 */
#define IX_NPE_A_RXDESCRIPTOR_CURRMBUFSIZE_OFFSET    2

/**
 * @def IX_NPE_A_RXDESCRIPTOR_ATMHEADER_OFFSET 
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor ATM Header
 */
#define IX_NPE_A_RXDESCRIPTOR_ATMHEADER_OFFSET       4

/**
 * @def IX_NPE_A_RXDESCRIPTOR_PSLOWPATH_OFFSET       
 *
 * @brief  ATM Descriptor structure offset for Receive Descriptor Slow Path
 * Pointer
 *
 * Pointer to a slowpath descriptor which can be used by the Xscale to replace the 
 * first FastPath mbuf in the associated chain.
 */
#define IX_NPE_A_RXDESCRIPTOR_PSLOWPATH_OFFSET       8

/**
 * @def IX_NPE_A_RXDESCRIPTOR_CURRMBUFLEN_OFFSET    
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Current MBuf length
 * 
 *
 * RX - Initialized to zero.  The NPE updates this field as each cell is received and 
 * zeroes it with every new mbuf for chaining. Will not be bigger than currBbufSize. 
 */
#define IX_NPE_A_RXDESCRIPTOR_CURRMBUFLEN_OFFSET    12

/**
 * @def IX_NPE_A_RXDESCRIPTOR_FPINDEX_OFFSET        
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Fast Path Index 
 *
 * Contains the FastPath index number of this VC
 */
#define IX_NPE_A_RXDESCRIPTOR_FPINDEX_OFFSET        14

/**
 * @def IX_NPE_A_RXDESCRIPTOR_FPMATCHINDEX_OFFSET   
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Fast Path Match Index 
 *
 * Contains the index of the matching array element of either the IP or TCP/UDP address.
 * The field shall be ignored if the FP_index Classify/Modify templates are not IP or NAPT.
 */
#define IX_NPE_A_RXDESCRIPTOR_FPMATCHINDEX_OFFSET   15

/**
 * @def IX_NPE_A_RXDESCRIPTOR_FPMBUFDATA_OFFSET     
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Fast MBuf Data 
 *
 * Points to the beginning of the FastPath MBuf data block. 
 */
#define IX_NPE_A_RXDESCRIPTOR_FPMBUFDATA_OFFSET     16

/**
 * @def IX_NPE_A_RXDESCRIPTOR_PCURRMBUFF_OFFSET     
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Current MBuf Pointer
 *
 * The current mbuf pointer of a chain of mbufs. 
 */
#define IX_NPE_A_RXDESCRIPTOR_PCURRMBUFF_OFFSET     20

/**
 * @def IX_NPE_A_RXDESCRIPTOR_PCURRMBUFDATA_OFFSET  
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Current MBuf Pointer
 *
 * Pointer to the next byte to be read or next free location to be written. 
 */
#define IX_NPE_A_RXDESCRIPTOR_PCURRMBUFDATA_OFFSET  24

/**
 * @def IX_NPE_A_RXDESCRIPTOR_PNEXTMBUF_OFFSET      
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Next MBuf Pointer
 *
 * Pointer to the next MBuf in a chain of MBufs. 
 */
#define IX_NPE_A_RXDESCRIPTOR_PNEXTMBUF_OFFSET      28

/**
 * @def IX_NPE_A_RXDESCRIPTOR_TOTALLENGTH_OFFSET    
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Total Length 
 *
 * Total number of bytes written to the chain of MBufs by the NPE
 */
#define IX_NPE_A_RXDESCRIPTOR_TOTALLENGTH_OFFSET    32

/**
 * @def IX_NPE_A_RXDESCRIPTOR_AAL5CRCRESIDUE_OFFSET 
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor AAL5 CRC Residue 
 *
 * Current CRC value for a PDU
 */
#define IX_NPE_A_RXDESCRIPTOR_AAL5CRCRESIDUE_OFFSET 36

/**
 * @def IX_NPE_A_RXDESCRIPTOR_SIZE
 *
 * @brief ATM Descriptor structure offset for Receive Descriptor Size
 *
 * The size of the Receive descriptor
 */
#define IX_NPE_A_RXDESCRIPTOR_SIZE                  40

/**
 * @def IX_NPE_A_TXDESCRIPTOR_PORT_OFFSET
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor Port
 *
 * Port identifier.  
 */
#define IX_NPE_A_TXDESCRIPTOR_PORT_OFFSET            0

/**
 * @def IX_NPE_A_TXDESCRIPTOR_RSVD_OFFSET    
 *        
 * @brief ATM Descriptor structure offset for Transmit Descriptor RSVD
 */
#define IX_NPE_A_TXDESCRIPTOR_RSVD_OFFSET            1

/**
 * @def IX_NPE_A_TXDESCRIPTOR_CURRMBUFLEN_OFFSET     
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor Current MBuf Length
 *
 * TX - Initialized by the XScale to the number of bytes in the current MBuf data buffer.  
 * The NPE decrements this field for every transmitted cell.  Thus, when the NPE writes a 
 * descriptor the TxDone queue, this field will equal zero.
 */
#define IX_NPE_A_TXDESCRIPTOR_CURRMBUFLEN_OFFSET     2

/**
 * @def IX_NPE_A_TXDESCRIPTOR_ATMHEADER_OFFSET       
 * @brief ATM Descriptor structure offset for Transmit Descriptor ATM Header
 */
#define IX_NPE_A_TXDESCRIPTOR_ATMHEADER_OFFSET       4

/**
 * @def IX_NPE_A_TXDESCRIPTOR_PCURRMBUFF_OFFSET      
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor Pointer to the current MBuf chain
 */
#define IX_NPE_A_TXDESCRIPTOR_PCURRMBUFF_OFFSET      8

/**
 * @def IX_NPE_A_TXDESCRIPTOR_PCURRMBUFDATA_OFFSET  
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor Pointer to the current MBuf Data
 *
 * Pointer to the next byte to be read or next free location to be written.
 */
#define IX_NPE_A_TXDESCRIPTOR_PCURRMBUFDATA_OFFSET  12

/**
 * @def IX_NPE_A_TXDESCRIPTOR_PNEXTMBUF_OFFSET      
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor Pointer to the Next MBuf chain
 */
#define IX_NPE_A_TXDESCRIPTOR_PNEXTMBUF_OFFSET      16

/**
 * @def IX_NPE_A_TXDESCRIPTOR_TOTALLENGTH_OFFSET    
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor Total Length
 *
 * Total number of bytes written to the chain of MBufs by the NPE
 */
#define IX_NPE_A_TXDESCRIPTOR_TOTALLENGTH_OFFSET    20

/**
 * @def IX_NPE_A_TXDESCRIPTOR_AAL5CRCRESIDUE_OFFSET 
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor AAL5 CRC Residue
 *
 * Current CRC value for a PDU
 */
#define IX_NPE_A_TXDESCRIPTOR_AAL5CRCRESIDUE_OFFSET 24

/**
 * @def IX_NPE_A_TXDESCRIPTOR_SIZE 
 *
 * @brief ATM Descriptor structure offset for Transmit Descriptor Size
 */
#define IX_NPE_A_TXDESCRIPTOR_SIZE                  28

/* define this macro to enable MPHY mode */
/* undefine this macro to enable SPHY mode */
#undef  IX_NPE_MPHY /** @brief Define this macro to enable MPHY mode */
#define IX_NPE_MPHY 1

/*--------------------------------------------------------------------------
 *  Hardware queue IDs relating to Atm Part-I 
 *--------------------------------------------------------------------------*/

/**
 * @def IX_NPE_A_TXDONE_QUEUE_HIGHWATERMARK
 *
 * @brief The NPE reserves the High Watermark for its operation. But it must be set by the Xscale
 */
#define IX_NPE_A_TXDONE_QUEUE_HIGHWATERMARK  2

/**
 * @def IX_NPE_A_QMQ_FPATH_FREE_Q      
 *
 * @brief Queue ID for FastPath Free Queue
 */
#define IX_NPE_A_QMQ_FPATH_FREE_Q      0

/**
 * @def IX_NPE_A_QMQ_ATM_TX_DONE       
 *
 * @brief Queue ID for ATM Transmit Done queue
 */
#define IX_NPE_A_QMQ_ATM_TX_DONE       1

/**
 * @def IX_NPE_A_QMQ_ATM_TX0       
 *    
 * @brief Queue ID for ATM transmit Queue in a single phy configuration
 */
#define IX_NPE_A_QMQ_ATM_TX0           2


/**
 * @def IX_NPE_A_QMQ_ATM_TXID_MIN
 *    
 * @brief Queue Manager Queue ID for ATM transmit Queue with minimum number of queue
 *
 */
/**
 * @def IX_NPE_A_QMQ_ATM_TXID_MAX
 *    
 * @brief Queue Manager Queue ID for ATM transmit Queue with maximum number of queue
 *
 */
/**
 * @def IX_NPE_A_QMQ_ATM_RX_HI
 *    
 * @brief Queue Manager Queue ID for ATM Receive high Queue
 *
 */
/**
 * @def IX_NPE_A_QMQ_ATM_RX_LO
 *    
 * @brief Queue Manager Queue ID for ATM Receive low Queue
 */
#ifdef IX_NPE_MPHY
/**
 * @def IX_NPE_A_QMQ_ATM_TX1
 *
 * @brief Queue ID for ATM transmit Queue Multiphy from 1 to 11
 */
#define IX_NPE_A_QMQ_ATM_TX1           IX_NPE_A_QMQ_ATM_TX0+1
#define IX_NPE_A_QMQ_ATM_TX2           IX_NPE_A_QMQ_ATM_TX1+1
#define IX_NPE_A_QMQ_ATM_TX3           IX_NPE_A_QMQ_ATM_TX2+1
#define IX_NPE_A_QMQ_ATM_TX4           IX_NPE_A_QMQ_ATM_TX3+1
#define IX_NPE_A_QMQ_ATM_TX5           IX_NPE_A_QMQ_ATM_TX4+1
#define IX_NPE_A_QMQ_ATM_TX6           IX_NPE_A_QMQ_ATM_TX5+1
#define IX_NPE_A_QMQ_ATM_TX7           IX_NPE_A_QMQ_ATM_TX6+1
#define IX_NPE_A_QMQ_ATM_TX8           IX_NPE_A_QMQ_ATM_TX7+1
#define IX_NPE_A_QMQ_ATM_TX9           IX_NPE_A_QMQ_ATM_TX8+1
#define IX_NPE_A_QMQ_ATM_TX10          IX_NPE_A_QMQ_ATM_TX9+1
#define IX_NPE_A_QMQ_ATM_TX11          IX_NPE_A_QMQ_ATM_TX10+1
#define IX_NPE_A_QMQ_ATM_TXID_MIN      IX_NPE_A_QMQ_ATM_TX0
#define IX_NPE_A_QMQ_ATM_TXID_MAX      IX_NPE_A_QMQ_ATM_TX11
#define IX_NPE_A_QMQ_ATM_RX_HI         21
#define IX_NPE_A_QMQ_ATM_RX_LO         22
#else
#define IX_NPE_A_QMQ_ATM_TXID_MIN      IX_NPE_A_QMQ_ATM_TX0
#define IX_NPE_A_QMQ_ATM_TXID_MAX      IX_NPE_A_QMQ_ATM_TX0
#define IX_NPE_A_QMQ_ATM_RX_HI         10
#define IX_NPE_A_QMQ_ATM_RX_LO         11
#endif /* MPHY */

/*--------------------------------------------------------------------------
 * Hardware queue IDs relating to HSS 
 *--------------------------------------------------------------------------*/

/**
 * @def IX_NPE_A_QMQ_HSS_CHL_RX_TRIG   
 *
 * @brief Hardware QMgr Queue ID for HSS Channelized Receive trigger
 */
#define IX_NPE_A_QMQ_HSS_CHL_RX_TRIG   12

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_RX
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Receive
 */
#define IX_NPE_A_QMQ_HSS_PKT_RX        13

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_TX0       
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Transmit queue 0
 */
#define IX_NPE_A_QMQ_HSS_PKT_TX0       14

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_TX1       
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Transmit queue 1
 */
#define IX_NPE_A_QMQ_HSS_PKT_TX1       15

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_TX2       
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Transmit queue 2
 */
#define IX_NPE_A_QMQ_HSS_PKT_TX2       16

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_TX3     
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Transmit queue 3
 */
#define IX_NPE_A_QMQ_HSS_PKT_TX3       17

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_RX_FREE0
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Receive Free queue 0
 */
#define IX_NPE_A_QMQ_HSS_PKT_RX_FREE0  18

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_RX_FREE1
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Receive Free queue 1
 */
#define IX_NPE_A_QMQ_HSS_PKT_RX_FREE1  19

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_RX_FREE2  
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Receive Free queue 2
 */
#define IX_NPE_A_QMQ_HSS_PKT_RX_FREE2  20

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_RX_FREE3
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Receive Free queue 3
 */
#define IX_NPE_A_QMQ_HSS_PKT_RX_FREE3  21

/**
 * @def IX_NPE_A_QMQ_HSS_PKT_TX_DONE
 *
 * @brief Hardware QMgr Queue ID for HSS Packetized Transmit Done queue
 */
#define IX_NPE_A_QMQ_HSS_PKT_TX_DONE   22

/*--------------------------------------------------------------------------
 * Hardware queue IDs relating to Atm Part-II
 *--------------------------------------------------------------------------*/

/**
 * @def IX_NPE_A_QMQ_ATM_FREE_VC0
 *
 * @brief Hardware QMgr Queue ID for ATM Free VC Queue. 
 *
 * There are 32 Hardware QMgr Queue ID; from IX_NPE_A_QMQ_ATM_FREE_VC1 to
 * IX_NPE_A_QMQ_ATM_FREE_VC30
 */
#define IX_NPE_A_QMQ_ATM_FREE_VC0      32
#define IX_NPE_A_QMQ_ATM_FREE_VC1      IX_NPE_A_QMQ_ATM_FREE_VC0+1
#define IX_NPE_A_QMQ_ATM_FREE_VC2      IX_NPE_A_QMQ_ATM_FREE_VC1+1
#define IX_NPE_A_QMQ_ATM_FREE_VC3      IX_NPE_A_QMQ_ATM_FREE_VC2+1
#define IX_NPE_A_QMQ_ATM_FREE_VC4      IX_NPE_A_QMQ_ATM_FREE_VC3+1
#define IX_NPE_A_QMQ_ATM_FREE_VC5      IX_NPE_A_QMQ_ATM_FREE_VC4+1
#define IX_NPE_A_QMQ_ATM_FREE_VC6      IX_NPE_A_QMQ_ATM_FREE_VC5+1
#define IX_NPE_A_QMQ_ATM_FREE_VC7      IX_NPE_A_QMQ_ATM_FREE_VC6+1
#define IX_NPE_A_QMQ_ATM_FREE_VC8      IX_NPE_A_QMQ_ATM_FREE_VC7+1
#define IX_NPE_A_QMQ_ATM_FREE_VC9      IX_NPE_A_QMQ_ATM_FREE_VC8+1
#define IX_NPE_A_QMQ_ATM_FREE_VC10     IX_NPE_A_QMQ_ATM_FREE_VC9+1
#define IX_NPE_A_QMQ_ATM_FREE_VC11     IX_NPE_A_QMQ_ATM_FREE_VC10+1
#define IX_NPE_A_QMQ_ATM_FREE_VC12     IX_NPE_A_QMQ_ATM_FREE_VC11+1
#define IX_NPE_A_QMQ_ATM_FREE_VC13     IX_NPE_A_QMQ_ATM_FREE_VC12+1
#define IX_NPE_A_QMQ_ATM_FREE_VC14     IX_NPE_A_QMQ_ATM_FREE_VC13+1
#define IX_NPE_A_QMQ_ATM_FREE_VC15     IX_NPE_A_QMQ_ATM_FREE_VC14+1
#define IX_NPE_A_QMQ_ATM_FREE_VC16     IX_NPE_A_QMQ_ATM_FREE_VC15+1
#define IX_NPE_A_QMQ_ATM_FREE_VC17     IX_NPE_A_QMQ_ATM_FREE_VC16+1
#define IX_NPE_A_QMQ_ATM_FREE_VC18     IX_NPE_A_QMQ_ATM_FREE_VC17+1
#define IX_NPE_A_QMQ_ATM_FREE_VC19     IX_NPE_A_QMQ_ATM_FREE_VC18+1
#define IX_NPE_A_QMQ_ATM_FREE_VC20     IX_NPE_A_QMQ_ATM_FREE_VC19+1
#define IX_NPE_A_QMQ_ATM_FREE_VC21     IX_NPE_A_QMQ_ATM_FREE_VC20+1
#define IX_NPE_A_QMQ_ATM_FREE_VC22     IX_NPE_A_QMQ_ATM_FREE_VC21+1
#define IX_NPE_A_QMQ_ATM_FREE_VC23     IX_NPE_A_QMQ_ATM_FREE_VC22+1
#define IX_NPE_A_QMQ_ATM_FREE_VC24     IX_NPE_A_QMQ_ATM_FREE_VC23+1
#define IX_NPE_A_QMQ_ATM_FREE_VC25     IX_NPE_A_QMQ_ATM_FREE_VC24+1
#define IX_NPE_A_QMQ_ATM_FREE_VC26     IX_NPE_A_QMQ_ATM_FREE_VC25+1
#define IX_NPE_A_QMQ_ATM_FREE_VC27     IX_NPE_A_QMQ_ATM_FREE_VC26+1
#define IX_NPE_A_QMQ_ATM_FREE_VC28     IX_NPE_A_QMQ_ATM_FREE_VC27+1
#define IX_NPE_A_QMQ_ATM_FREE_VC29     IX_NPE_A_QMQ_ATM_FREE_VC28+1
#define IX_NPE_A_QMQ_ATM_FREE_VC30     IX_NPE_A_QMQ_ATM_FREE_VC29+1
#define IX_NPE_A_QMQ_ATM_FREE_VC31     IX_NPE_A_QMQ_ATM_FREE_VC30+1

/**
 * @def IX_NPE_A_QMQ_ATM_RXFREE_MIN  
 *
 * @brief The minimum queue ID for FreeVC queue
 */
#define IX_NPE_A_QMQ_ATM_RXFREE_MIN  IX_NPE_A_QMQ_ATM_FREE_VC0

/**
 * @def IX_NPE_A_QMQ_ATM_RXFREE_MAX 
 *
 * @brief The maximum queue ID for FreeVC queue
 */
#define IX_NPE_A_QMQ_ATM_RXFREE_MAX  IX_NPE_A_QMQ_ATM_FREE_VC31

/**
 * @def IX_NPE_A_QMQ_OAM_FREE_VC
 * @brief OAM Rx Free queue ID 
 */
#ifdef IX_NPE_MPHY
#define IX_NPE_A_QMQ_OAM_FREE_VC       14
#else
#define IX_NPE_A_QMQ_OAM_FREE_VC       3
#endif

/**
 * @def IX_NPE_A_CHAIN_DESC_COUNT_MAX
 *
 * @brief Maximum number of chained MBufs that can be chained together
 */
#define IX_NPE_A_CHAIN_DESC_COUNT_MAX            256

/* ATM Descriptor Structure */
#ifdef LITTLE_ENDIAN
#error  NO_LITTLE_ENDIAN_SUPPORT
#else

/* N.B. Changes to field sizes or orders must be reflected in the offset 
 * definitions above. */

/**
 * @brief Atm cell header definition
 */
typedef struct
{
    unsigned int gfc:4;
    unsigned int vpi:8;
    unsigned int vci:16;
    unsigned int pti:3;
    unsigned int clp:1;
} IxNpeA_AtmCellHeader;

/* N.B. Changes to field sizes or orders must be reflected in the offset 
 * definitions above. */

/**
 * @brief Tx Descriptor definition
 */
typedef struct
{
    UINT8 port;
    UINT8 aalType; /* Added for CSR1.1 it was a resrvd field for CSR1.0 */
    UINT16 currMbufLen;

    IxNpeA_AtmCellHeader atmCellHeader;

    IX_MBUF *pCurrMbuf;
    unsigned char *pCurrMbufData;
    IX_MBUF *pNextMbuf;
    UINT32  totalLen;
    UINT32  aal5CrcResidue;
} IxNpeA_TxAtmVc;

/* N.B. Changes to field sizes or orders must be reflected in the offset 
 * definitions above. */

/**
 * @brief Rx Descriptor definition
 */
typedef struct 
{
    struct
    {
        unsigned int status:1;
        unsigned int port:7;
        unsigned int vcId:8;
        unsigned int currMbufSize:16;
    } rxBitField; 

    IxNpeA_AtmCellHeader atmCellHeader;
    UINT32  pSlowPathDesc;
    UINT16  currMbufLen;
    UINT8   fpIndex; /* shared with timeLimit for CSR1.1 */
    UINT8   fpMatchIndex;
    unsigned char *pFpMbufData;
    IX_MBUF *pCurrMbuf;
    unsigned char *pCurrMbufData;
    IX_MBUF *pNextMbuf;
    UINT32  totalLen;
    UINT32  aal5CrcResidue;
} IxNpeA_RxAtmVc;
#endif

/**
 * @brief  Atm Descriptor structure used for fpath 
 */
typedef struct
{
    IxNpeA_RxAtmVc npe;
    UINT32       masterMbufLen;
    IX_MBUF      *pRootMbuf;
} IxNpeA_AtmVcFp;

/** 
 * @brief NPE-A AAL Type
 */
typedef enum
{
    IX_NPE_A_AAL_TYPE_INVALID = 0,
    IX_NPE_A_AAL_TYPE_0_48    = 0x1,
    IX_NPE_A_AAL_TYPE_0_52    = 0x2,
    IX_NPE_A_AAL_TYPE_5       = 0x5,
    IX_NPE_A_AAL_TYPE_OAM     = 0xF
} IxNpeA_AalType;

/** 
 * @brief NPE-A Payload format 52-bytes & 48-bytes
 */
typedef enum
{
    IX_NPE_A_52_BYTE_PAYLOAD = 0,
    IX_NPE_A_48_BYTE_PAYLOAD
} IxNpeA_PayloadFormat;

/**
 * @brief  HSS Packetized NpePacket Descriptor Structure 
 */
typedef struct
{
    UINT8   status;
    UINT8   errorCount;
    UINT8   chainCount;
    UINT8   rsvdByte0;

    UINT16  packetLength;
    UINT16  rsvdShort0;

    IX_MBUF *pRootMbuf;
    IX_MBUF *pNextMbuf;
    unsigned char *pMbufData;
    UINT32  mbufLength;

} IxNpeA_NpePacketDescriptor;


#endif
/**
 *@}
 */
