/*
 * -- Intel Copyright Notice --
 * INTEL CONFIDENTIAL
 *
 * Copyright 2002 Intel Corporation All Rights Reserved.
 *
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
*/
#include <mbuf.h>
#include <simple_pool.h>

#ifdef DEBUG_POOL
void simple_pool_dump(char * name, simple_netpool_t *pool)
{
    mbuf_t *mbuf;
    int count;
    printk("---- %s -------\nPOOL: %p -> %p : ",
	name, pool, pool->free);

    for (mbuf = pool->first, count = pool->count; count; count--, mbuf++)
	printk("{%p -> %p} ", mbuf, mbuf->m_next);
    printk("\n");
    printk("\n");
}
#define POOL_DUMP(name, pool) simple_pool_dump(name, pool)
#else
#define POOL_DUMP(name, pool)
#endif


/* in our simple mbuf pool, we use m_next, to chain the free buffers.
 * free pointer will point to the first free mbuf.
 * It should point to the first mbuf of the pool when this function
 * is called.
 */
int simple_pool_init(simple_netpool_t *pool, int count, int size,
    __u8 *data)
{
    int i;
    mbuf_t *mbuf;
    
#ifdef DEBUG_POOL
    pool->first = pool->free;
    pool->count = count;
    POOL_DUMP(__FUNCTION__, pool);
#endif
    
    mbuf = pool->free;

    for (i = 0; i < count; i++)
    {
	mbuf[i].m_net_pool = pool;
	mbuf[i].m_next = &mbuf[i + 1];

	if (data)
	{
	    mbuf[i].m_data = data;
	    data += size;
	}
    }
    mbuf[count - 1].m_next = NULL;

    POOL_DUMP(__FUNCTION__"-end", pool);

    return 0;
}

mbuf_t *simple_pool_get(simple_netpool_t *pool, int *has_more)
{
    int irq_state;
    mbuf_t *ret;

    spin_lock_irqsave(&pool->lock, irq_state);
    POOL_DUMP(__FUNCTION__, pool);
    
    ret = pool->free;
    if (ret)
    {
	pool->free = ret->m_next;
	ret->m_next = NULL;
    }
    if (has_more)
	*has_more = (int)pool->free;
    
    POOL_DUMP(__FUNCTION__"-end", pool);
    spin_unlock_irqrestore(&pool->lock, irq_state);

    return ret;
}

void simple_pool_put(mbuf_t *buf)
{
    int irq_state;
    simple_netpool_t *pool = buf->m_net_pool;

    spin_lock_irqsave(&pool->lock, irq_state);
    POOL_DUMP(__FUNCTION__, pool);
    
    buf->m_next = pool->free;
    pool->free = buf;
    
    POOL_DUMP(__FUNCTION__"-end", pool);
    spin_unlock_irqrestore(&pool->lock, irq_state);
}

