/**
 * @file usbprivate.h
 *
 * @author Intel Corporation
 * @date 30-OCT-2001
 *
 * @brief This file containes the private USB Driver API and global data
 *
 * -- Intel Copyright Notice --
 *
 * @par
 * INTEL CONFIDENTIAL
 *
 * @par
 * Copyright 2001-2002 Intel Corporation All Rights Reserved.
 *
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.  The Material
 * contains trade secrets and proprietary and confidential information of
 * Intel or its suppliers and licensors.  The Material is protected by
 * worldwide copyright and trade secret laws and treaty provisions. No
 * part of the Material may be used, copied, reproduced, modified,
 * published, uploaded, posted, transmitted, distributed, or disclosed in
 * any way without Intel's prior express written permission.
 *
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel or otherwise.  Any license under
 * such intellectual property rights must be express and approved by
 * Intel in writing.
 *
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * -- End Intel Copyright Notice --
 */

/**
 * @addtogroup SupportAPI
 *
 * @brief Internal support API, part of the USB driver
 *
 * @{
 */

#ifndef usbprivate_H

#ifndef __doxygen_HIDE

#define usbprivate_H

#endif /* __doxygen_HIDE */

#include "usbprivatetypes.h"

PRIVATE void 
ixUSBInterruptHandler(USBDevice *device);

PRIVATE void 
ixUSBINInterruptHandler(EPStatusData *epData);

PRIVATE void 
ixUSBOUTInterruptHandler(EPStatusData *epData);

PRIVATE void 
ixUSBEP0InterruptHandler(EPStatusData *epData);

PRIVATE void 
ixUSBEP0TokenDecode(USBDevice *device);

PRIVATE void 
ixUSBEP0SetupPacketDecode(USBDevice *device);

PRIVATE void 
ixUSBEP0StateReset(USBDevice *device);

PRIVATE void 
ixUSBEP0RequestSend(USBDevice *device);

PRIVATE void 
ixUSBEP0DataDeliver(USBDevice *device);

PRIVATE void 
ixUSBEP0SendCleanup(USBDevice *device);

PRIVATE void 
ixUSBRequestSend(EPStatusData *epData);

PRIVATE void 
ixUSBSendCleanup(EPStatusData *epData);

PRIVATE void 
ixUSBReceiveProcess(EPStatusData *epData);

PRIVATE void
ixUSBQueueInit(EPStatusData *epData);

PRIVATE BOOL 
ixUSBBufferEnqueue(EPStatusData *epData, IX_USB_MBLK *buf);

PRIVATE BOOL 
ixUSBBufferDequeue(EPStatusData *epData);

PRIVATE void
ixUSBQueueDiscard(EPStatusData *epData);

PRIVATE void
ixUSBTransferAbort(EPStatusData *epData);

PRIVATE UINT16
ixUSBTransferDirectionGet(EPStatusData *epData);

PRIVATE IX_USB_MBLK* 
ixUSBBufferAlloc(size_t size);

#ifdef IX_USB_HAS_TIMESTAMP_CHECKS

#ifndef IX_USB_HAS_CUSTOM_TIMESTAMP_GET

PRIVATE UINT32
ixUSBTimestampGet(void);

#endif /* IX_USB_HAS_CUSTOM_TIMESTAMP_GET */

PRIVATE BOOL
ixUSBTimestampCheck(EPStatusData *epData);

#endif /* IX_USB_HAS_TIMESTAMP_CHECKS */

PRIVATE void
ixUSBNullReceiveCallback(
    USBDevice *device, 
    UINT16 sourceEndpoint, 
    IX_USB_MBLK *receiveBuffer);

PRIVATE void 
ixUSBNullEventCallback(USBDevice *device, USBEventSet events);

PRIVATE void 
ixUSBNullSetupCallback(USBDevice *device, const char *packet);

typedef void (*EPInterruptHandler) (EPStatusData *);

static EPInterruptHandler EPInterruptHandlerTable[] =
{
    /* EP_0 */ ixUSBEP0InterruptHandler,
    /* EP_1 */ ixUSBINInterruptHandler,
    /* EP_2 */ ixUSBOUTInterruptHandler,
    /* EP_3 */ ixUSBINInterruptHandler,
    /* EP_4 */ ixUSBOUTInterruptHandler,
    /* EP_5 */ ixUSBINInterruptHandler,
    /* EP_6 */ ixUSBINInterruptHandler,
    /* EP_7 */ ixUSBOUTInterruptHandler,
    /* EP_8 */ ixUSBINInterruptHandler,
    /* EP_9 */ ixUSBOUTInterruptHandler,
    /* EP_10 */ ixUSBINInterruptHandler,
    /* EP_11 */ ixUSBINInterruptHandler,
    /* EP_12 */ ixUSBOUTInterruptHandler,
    /* EP_13 */ ixUSBINInterruptHandler,
    /* EP_14 */ ixUSBOUTInterruptHandler,
    /* EP_15 */ ixUSBINInterruptHandler,
};

#endif /* usbprivate_H */

/**
 * @} addtogroup SupportAPI
 */
