/**
 * @file  IxFpathAccCodeletDefines_p.h
 * @date  May 29 2002
 * @brief Defines constants of system wide significance
 *        for the IxFpathAcc codelet
 *
 *
 * -- Intel Copyright Notice --
 * 
 * @par
 * Copyright 2002-2003 Intel Corporation All Rights Reserved.
 * 
 * @par
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.
 * 
 * @par
 * The Material is protected by worldwide copyright and trade secret laws
 * and treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way except in accordance with the
 * applicable license agreement .
 * 
 * @par
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel, except in accordance with the
 * applicable license agreement.
 * 
 * @par
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 * 
 * @par
 * For further details, please see the file README.TXT distributed with
 * this software.
 * 
 * @par
 * -- End Intel Copyright Notice --
 */

#ifndef IXFPATHACCCODELETDEFINES_P_H
#define IXFPATHACCCODELETDEFINES_P_H

#include <stdio.h>
#include "IxTypes.h"
#include "IxAtmTypes.h"
#include "ixp425.h"
#include "IxOsServices.h"
#include "IxAtmdAcc.h"
 



/**
 * Define the space that should be available before m_data
 * in an mbuf when first allocated ( to be used for pre-pending
 * headers ).
 */

#define MBUF_HEADROOM 32
#if( MBUF_HEADROOM < SIZEOF_IEEE802_3_HDR )
#error MBUF HEADROOM TOO SMALL
#endif

/**
 * Define the size of mbufs that will be used to populate
 * the AtmdAcc rx free queue.
 */
#define ATM_RX_MBUF_SIZE (2048 - MBUF_HEADROOM)

/**
 * Define the size of the mbufs that will be used to populate
 * the fast path free queue. This cannot be larger than
 * the ATM rx mbuf size.
 */
#define FASTPATH_MBUF_SIZE 1600

#if( ATM_RX_MBUF_SIZE < FASTPATH_MBUF_SIZE )
#error  INVALID FPATH/ATM MBUF SIZES
#endif

/**
 * Define the size of the mbufs that will be used to populate
 * the ethernet rx free queue
 */
#define ETH_RX_MBUF_SIZE 1024

/**
 * Define the initial number of mbufs to be supplied to the
 * EthAcc rx free queues
 */
#define ETH_INITIAL_RX_REPLENISH_NUM 32

/**
 * Define the MAC addresses for the ethernet
 * ports associated with NPE-B and NPE-C
 */
#define ETH1_MAC_ADDRESS {0x00,0x00,0x00,0x00,0x00,0x01}
#define ETH2_MAC_ADDRESS {0x00,0x00,0x00,0x00,0x00,0x02}


/**
 * The routed IP fast path codelet does not include ARP
 * functionality. All received pdus are transmitted to this MAC
 * address. If necessary this should be re-defined to the suit the test
 * environment, typically the MAC address of an attached ethernet
 * device
 */
#define FPATH_ROUTEDIP_TARGET_MAC {0x00,0x00,0x00,0x00,0x00,0x03}


/**
 * For the Routed IP codelet define the
 * time in milliseconds that a flow is allowed to persist on the fast
 * path before it must be learned again.
 * (Accurate to 1 OS tick)
 */
#define IP_FLOW_AGEING_INTERVAL_MS  2000

/**
 * For the Routed IP codelet define the number of pdus
 * that must be received within the IP_FLOW_LEARNING_INTERVAL
 * to qualify for being moved from the slow path to the fast path
 */
#define IP_FLOW_LEARNING_PDU_COUNT_THRESHOLD 300

/**
 * Defined the number of IP flows per VC that can be selected as a 
 * candidate to be learned as active flow at any time.  The maximum 
 * number of active IP flows at any one time is 4.
 */
#define NUM_LEARNING_FLOWS 8

/**
 * For the Routed IP codelet define the interval
 * in milliseconds during which IP flow activity is measured.
 * This number must be a multiple of the number of
 * learning flows, and of the OS tick
 */
#define IP_FLOW_ACCOUNTING_INTERVAL_MS 512

#if (IP_FLOW_ACCOUNTING_INTERVAL % NUM_LEARNING_FLOWS)
#error IP ACCOUNTING INTERVAL MUST BE A MULTIPLE OF NUM_LEARNING_FLOWS
# endif
#if (IP_FLOW_ACCOUNTING_INTERVAL % (1000/IXP425_OSST_TICKS_PER_SECOND))
#error IP ACCOUNTING INTERVAL MUST BE A MULTIPLE OF OS TICK
# endif

/**
 * This control determines if WAN MAC address learning
 * is enabled for the ethernet bridge codelet code
 */
#define WAN_MAC_LEARNING_ENABLED TRUE

/**
 * Define how often the WAN MAC learning history will be displayed
 * as a number of seconds
 */
#define WAN_MAC_LEARNING_DB_DISPLAY_INTERVAL_S 5

/* Defines how many learned WAN MAC addresses should be displayed
*/
#define WAN_MAC_LEARNING_DB_DISPLAY_NUM_ENTRIES 16


/**
 * Define the ATM UTOPIA port, VPI and VCI to be used for
 * simple bridge codelet.
 */
#define FPATH_BRIDGED_TEST_UTOPIA_PORT IX_UTOPIA_PORT_0
#define FPATH_BRIDGED_TEST_VPI 1
#define FPATH_BRIDGED_TEST_VCI 32

/**
 * Define the ATM UTOPIA port, VPI and VCI to be used for
 * simple routed codelet.
 */
#define FPATH_ROUTED_TEST_UTOPIA_PORT IX_UTOPIA_PORT_0
#define FPATH_ROUTED_TEST_VPI 1
#define FPATH_ROUTED_TEST_VCI 33

/**
 * Define the NPE Versions that will be loaded
 */
#define NPEA_IMAGEID IX_NPEDL_NPEIMAGE_NPEA_ATM_MPHY_12_PORT

#define NPEB_IMAGEID IX_NPEDL_NPEIMAGE_NPEB_ETH_FPATH

#define NPEC_IMAGEID IX_NPEDL_NPEIMAGE_NPEC_ETH

/**
 * @brief Log text using printf style formatting
 *
 * e.g. IXFPATH_CODELET_TRACE((traceBuf, "One plus One = %d", 2 ));
 *
 * @note double parenthesis.
 */
#define IXFPATH_CODELET_TRACE(string) do { \
    static char traceBuf[80];\
    sprintf string;\
    ixOsServLog( LOG_MESSAGE,"%s",(int)traceBuf,0,0,0,0,0);\
    /* task sleep to prevent log output from flooding */\
    ixOsServTaskSleep(1000/IXP425_PERIPHERAL_BUS_CLOCK);\
} while (0)

/**
 * @brief Log errors with file and line number
 *
 * e.g. IXFPATH_CODELET_ERROR(("Something went wrong here"));
 *
 */
#define IXFPATH_CODELET_ERROR(string) do {\
    ixOsServLog( LOG_ERROR,\
        "IxFpathAcc Codelet Error - %s"\
        "\n%s LINE %d",\
        (int) string,\
        (int) __FILE__,\
        (int) __LINE__,\
        0,0,0 );\
} while (0)


/**********************************************************
**
**  IxFpathAccCodeletTemplates.c header - #define
**
**********************************************************/
/** Header defines c.f. Rfc1483 Multiprotocol Encapsulation
    over ATM Adaptation Layer 5 */
#define SIZEOF_1483_LLC_IEEE802_3_HDR   (10)
#define SIZEOF_1483_VCMUX_IEEE802_3_HDR (2)
#define SIZEOF_1483_LLC_ROUTED_IP_HDR   (8)
#define SIZEOF_1483_VCMUX_ROUTED_IP_HDR (0)

/** Ethernet Frame Size Defines */
#define SIZEOF_IEEE802_3_DEST_MAC_ADDR  (6)
#define SIZEOF_IEEE802_3_SRC_MAC_ADDR   (6)
#define SIZEOF_IEEE802_3_TYPE_FIELD     (2)
#define MAXSIZEOF_IEEE802_3_PAYLOAD     (1500)
#define SIZEOF_IEEE802_3_LAN_FCS        (0x02)

#define SIZEOF_IEEE802_3_HDR \
            (SIZEOF_IEEE802_3_DEST_MAC_ADDR + \
             SIZEOF_IEEE802_3_SRC_MAC_ADDR + \
             SIZEOF_IEEE802_3_TYPE_FIELD)

#define SIZEOF_IEEE802_3_OVERHEAD \
            (SIZEOF_IEEE802_3_HDR + \
             SIZEOF_IEEE802_3_LAN_FCS)

#define MAXSIZEOF_IEEE802_3_FRAME \
            (SIZEOF_IEEE802_3_OVERHEAD + \
             MAXSIZEOF_IEEE802_3_PAYLOAD)

#define IEEE802_3_DEST_MAC_ADDR_OFFSET 0

/** Offset from start of IP Packet */
#define SIZEOF_VER_PLUS_HL               (0x01)
#define SIZEOF_TYPE_OF_SERVICE           (0x01)
#define SIZEOF_IP_TOTAL_LEN              (0x02)
#define SIZEOF_IP_ID                     (0x02)
#define SIZEOF_IP_FLAGS_PLUS_FRAG_OFFSET (0x02)
#define SIZEOF_IP_TTL                    (0x01)
#define SIZEOF_IP_PROTOCOL               (0x01)
#define SIZEOF_IP_HDR_CHECKSUM           (0x02)
#define SIZEOF_IP_SRC_ADDR               (0x04)
#define SIZEOF_IP_DEST_ADDR              (0x04)

/** @note the IP header we use is always 20 bytes for
    simplicity. */
#define SIZEOF_IP_HDR_WITHOUT_OPTIONS    (0x14)
#define NUMBEROF_SHORTS_TO_CHECKSUM_IN_IP_HDR (0x0a)

/** Offsets into the IP HDR */
#define IP_DEST_ADDR_OFFSET (0x10)
#define IP_TOTAL_LEN_OFFSET (0x02)
#define IP_TTL_OFFSET       (0x08)


/** First byte of a typical IP packet */
#define IPV4_HDR_ID          (0x45)


/**********************************************************
**
**  IxFpathAccMain.c header - #define
**
**********************************************************/
/*Interval of 1 sec for OS Services Task Sleep */
#define   IX_FPATHACC_SLEEP_IN_S  1000



/**********************************************************
**
**  IxFpathAccCodeletUtopia.c header - #define
**
**********************************************************/
#define MPHY_UTOPIA_PHY_ADDR 0
#define UTOPIA_PORT_RATE (8*1000*1000)

/***********************************************************
**
**	Enumeration Definition
**
***********************************************************/

/**
 * Supported channel encapsulation type.
 */
typedef enum
{
    TYPE_NONE = 0,          /* Raw VC - always slow path*/
    TYPE_1483BLLC,
    TYPE_1483BVCMUX,
    TYPE_ROUTEDIPLLC,
    TYPE_ROUTEDIPVCMUX,
    NUM_ENCAP_TYPES	/* N.B. Add any new encapsulation types before
		         * this line */
} EncapsulationType;

/** Allow a maximum of 8 VCs */
typedef enum
{
    VC_0 = 0,
    VC_1,
    VC_2,
    VC_3,
    VC_4,
    VC_5,
    VC_6,
    VC_7,
    MAX_NUM_VCS
} VcNumber;

/** Four IP flows per VC */
typedef enum
{
    IP_FLOW0 = 0,
    IP_FLOW1,
    IP_FLOW2,
    IP_FLOW3,
    MAX_FLOWS_PER_VC
}FlowId;


/* Supported Fastpath Codelet Type */
typedef enum
{
    ETHERNET_BRIDGE = 0,
    ROUTED_IP
} IxFpathCodeletType;

extern BOOL ixFpCodeletTraceEnabled;


/*************************************************************
**
**	Structure Definition
**
*************************************************************/

/**
 * This type describes a classifier or modifier template
 */
typedef struct
{
    UINT8 *data;        /**< pointer to the template data */
    unsigned int len;   /**< the length of the template data */
} IxFpCodeletTemplate;


/**
 * Types For Gathering Statistics
 */
typedef struct
{
    unsigned int totalPduCount;
    unsigned int validPduCount;
    unsigned int errorCount;
} Aal5Statistics;

typedef struct
{
    unsigned int lengthExceptionCount;
    unsigned int otherExceptionCount;
    unsigned int newWanMacCount;
} EtherBridgeStatistics;

typedef struct
{
    unsigned int ethernetTxErrorCount;
    unsigned int flowSetErrorCount;
    unsigned int templateSetErrorCount;
    unsigned int macLookupErrorCount;
} RoutedIPStatistics;

/**
 * @typedef AtmRxCallback
 * encapsulation type specific ATM rx callback registered
 * with the codelet aal5 receive handler.
 */
typedef void (*AtmRxCallback) (IxAtmdAccUserId userId, IX_MBUF *buf);

/**
 * @typedef IpFlowRequestData
 * Basic Ip flow tuple
 */
typedef struct
{
    UINT32 ipAddress;
    UINT8 macAddress[6];
} IpFlowRequestData;

/** Struct to hold IP flow information while learning */
typedef struct
{
    BOOL         inUse;         /**< Learning is taking place  */
    UINT32       ipAddress;     /**< Destination IP address    */
    UINT8        macAddress[6]; /**< Corresponding Mac Address */
    unsigned int counter;       /**< Current Packet Count      */
} LearningIpFlow;

/** Struct to hold IP flow information while active */
typedef struct
{
    BOOL         aged;          /**< Flow may be superceded         */
    UINT32       ipAddress;     /**< Destination IP address         */
    UINT8        macAddress[6]; /**< Corresponding Mac Address      */
    int          age;           /**< In units of Counting Intervals */
} ActiveIpFlow;

/** Main structure to hold connection information */
typedef struct
{
    BOOL               inUse;
    VcNumber           vc;

    IxAtmLogicalPort   port;
    unsigned int       vpi;
    unsigned int       vci;

    EncapsulationType  encapsulationType; /** See IxFpathCodeletTemplates_p.h*/

    AtmRxCallback      atmRxHandler;
    IxAtmNpeRxVcId     npeVcId; /**
                                 * N.B. Obtained from IxAtmdAcc at
                                 * ixAtmdAccRxVcConnect().
                                 */

    /** IP Flow Learning and Aging Information */
    LearningIpFlow          learningIpFlow[NUM_LEARNING_FLOWS];
    ActiveIpFlow            activeIpFlow[MAX_FLOWS_PER_VC];
    BOOL                    ipFlowRequest;
    IpFlowRequestData       ipFlowRequestData;

    /** Statistics */
    Aal5Statistics          aal5Stats;
    EtherBridgeStatistics   etherBridgeStats;
    RoutedIPStatistics      routedIpStats;
} IxFpCodeletFastPathConnection;





/*************************************************************
**
**	Function Prototype
**
*************************************************************/


/**
 * @brief Start the Single VC Ethernet Bridge Fast Path codelet
 */
void
ixFpCodeletEthBStart( void );


/**
 * @brief Stop the Single VC Ethernet Bridge Fast Path codelet
 *
 * This function shows the correct procedure for shutting down a
 * fast path VC in conjunction with disconnecting the VC from IxAtmdAcc.
 *
 */
void
ixFpCodeletEthBStop( void );

/**
 * @brief Start the Single VC Routed Ip Fast Path codelet
 */
void
ixFpCodeletRoutedIpStart( void );

/**
 * @brief Stop the Single VC Routed IP Fast Path codelet
 *
 * This function shows the correct procedure for shutting down a
 * fast path VC in conjunction with disconnecting the VC from IxAtmdAcc.
 *
 */
void
ixFpCodeletRoutedIpStop( void );


/**
 * @brief Initialise this sub-component. Call this before using any other functions.
 */
IX_STATUS
ixFpCodeletTemplatesInit(void);

/**
 * @brief Create the basic template for the VC based on its encapsulation type
 */
void
ixFpCodeletTemplatesBaseTemplatePairCreate(
    VcNumber vc,
    EncapsulationType type );

/* ------------------------------------------------------------------------
 * @brief Set/Clears the Wan learn command in the xscale copy of a classifier
 *         template.
 * Set firstinstruction to FALSE if learning opcode to be last in the
 * template.
 */
IX_STATUS
ixFpCodeletTemplatesWanMacLearningSet(
    VcNumber vc,
    BOOL enabled,
    BOOL firstInstuction );

/**
 * @brief Set the src or destination ip address for an Ip flow on
 *         an Ip encapsulated VC.
 */
IX_STATUS
ixFpCodeletTemplatesIpFlowSet(
    VcNumber vc,        /* 0 - 7*/
    FlowId   flow,      /* 0 - 3 */
    UINT32 destIpAddr,  /* Classifier matches this addr */
    UINT8  *destMac );  /* Modifier inserts this in Ethernet header */

/**
 * @brief Return the template pair that has been built for the current Vc
 */
IX_STATUS
ixFpCodeletTemplatesTemplatePairGet(
    VcNumber vc,
    IxFpCodeletTemplate *classifier,
    IxFpCodeletTemplate *modifier );

/**
 * @brief Print out the classifier template for this VC
 */
IX_STATUS
ixFpCodeletTemplatesClsPrint(VcNumber vc);

/**
 * @brief Print out the modifier template for this VC
 */
IX_STATUS
ixFpCodeletTemplatesMdfPrint(VcNumber vc);

/**
 * @brief Get the encapsulation name.
 */
char *
ixFpCodeletTemplatesEncapuslationName( EncapsulationType t );


/** Do the main intialisation for the codelet */
IX_STATUS
ixFpCodeletInit( void );

/**
 * Creates a FP connection and invokes the templating mechanism
 * to create a basic fast path template for the chosen VC encapsulation
 * type.
 */
IX_STATUS
ixFpCodeletFastPathVcCreate( VcNumber vc );

/**
 * Delete the FP and ATM connection
 */
IX_STATUS
ixFpCodeletFastPathVcDelete( VcNumber vc );

/**
 * Show codelet information
 */
void
ixFpCodeletShow(void);

/** Gets a pointer to the connection for the specified VC */
IxFpCodeletFastPathConnection *
ixFpCodeletFastPathConnectionGet( VcNumber vc );


/* IxFpathAccCodeletUtils.c header */

/**
 * @brief Replenish an Atm Rx Free queue
 *
 * @param id - the Id by which the codelet knows the VC
 */
void
ixFpCodeletUtilsAtmReplenish (IxAtmdAccUserId id);

/**
 * @brief Atm Rx connect and register all callbacks
 *
 * @param port (in)       - The UTOPIA port number @sa IxAtmTypes.h
 * @param vpi (in)        - The virtual path index for the ATM VC
 * @param vci (in)        - The virtual channel index for the ATM VC
 * @param rxQueueId (in)  - the ATM Rx Q to be used for this VC @sa IxAtmdAcc.h
 * @param rxCallback (in) - the ATM receive routine.
 * @param userId (in)     - The ID by which the codelet knows the VC.           					    (i.e. VcNumber)
 * @param npeVcId (out)   - IxAtmdAcc fills out the npe VC id.
 */
IX_STATUS
ixFpCodeletUtilsAtmRxVcRegisterConnect(IxAtmLogicalPort port,
		    unsigned vpi,
		    unsigned vci,
		    IxAtmRxQueueId rxQueueId,
		    IxAtmdAccRxVcRxCallback rxCallback,
		    int userId,
                    IxAtmNpeRxVcId *npeVcId);

/**
 * @brief Atm Rx Disconnect
 *
 * @param userId - the Id by which the codelet knows the VC.
 */
IX_STATUS
ixFpCodeletUtilsAtmRxDisconnect( IxAtmdAccUserId userId );

/**
 * @brief Initialise the Utility functions.
 */
IX_STATUS
ixFpCodeletUtilsInit( void );


/* IXFpathAccCodeletBufMan.c header */

/**
 * @brief Initialise the buffer pool.
 */
IX_STATUS
ixFpCodeletBufManMbufPoolInit(void);

/**
 * @brief Get an mbuf from the pool and initialise it.
 */
IX_MBUF*
ixFpCodeletBufManMbufGet(unsigned int size);

/**
 * @brief Free an mbuf to the pool
 */
void
ixFpCodeletBufManMbufFree(IX_MBUF* buf);

BOOL
ixFpCodeletUtopiaFpgaStimulusAsMasterSet(void);



#endif /* IXFPATHACCCODELETDEFINES_P_H */





