/*
 * -- Intel Copyright Notice --
 * 
 * Copyright 2002 Intel Corporation All Rights Reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.
 * 
 * The Material is protected by worldwide copyright and trade secret laws
 * and treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way except in accordance with the
 * applicable license agreement .
 * 
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel, except in accordance with the
 * applicable license agreement.
 * 
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 * 
 * For further details, please see the file README.TXT distributed with
 * this software.
 * 
 * -- End Intel Copyright Notice --
*/
#ifndef _MBUF_H_
#define _MBUF_H_

#include <vx_types.h>

#include <linux/skbuff.h>

typedef struct mHdr
{
    struct mBlk *mNext;		/* Next buffer on the chain */
    struct mBlk *mNextPkt;	/* Next chain */
    char *mData;		/* Pointer to the data */
    int	mLen;			/* How much data */
    /* Strange, but differs from BSD */
    UCHAR mType;		/* mBlk data type */
    UCHAR mFlags;
    USHORT reserved; 
} M_BLK_HDR;

typedef struct	pktHdr
{
    void *rcvif;		/* Interface on which received*/
    int	len;			/* Length of the entire packet */
} M_PKT_HDR;

typedef struct mBlk
{
    M_BLK_HDR mBlkHdr  __attribute__ ((aligned(32)));		/* Header */
    M_PKT_HDR mBlkPktHdr;	/* pkthdr */
#if 0 /* not suposed to be used */
    CL_BLK *pClBlk;		/* Cluster blk pointer */
#endif
    void *priv;
} M_BLK;

#define	m_next mBlkHdr.mNext
#define	m_len mBlkHdr.mLen
#define	m_data mBlkHdr.mData
#define	m_type mBlkHdr.mType
#define	m_flags	mBlkHdr.mFlags
#define	m_nextpkt mBlkHdr.mNextPkt
#define m_act m_nextpkt
#define	m_pkthdr mBlkPktHdr
#define m_net_pool m_pkthdr.rcvif

#define SIZEOF_IP_HEADER sizeof(struct iphdr)

/* This function sets the skb field in mbuf with the given skb pointer. 
 * If not NULL, mbuf data and length fields are set from the skb values.
 * This function returns the previous value of mbuf->skb.
 */
static inline struct sk_buff *mbuf_swap_skb(M_BLK *mbuf, struct sk_buff *skb)
{
    struct sk_buff *res = mbuf->priv;
    mbuf->priv = skb;

    if (!skb)
	return res;

    mbuf->m_data = skb->data;
    mbuf->m_pkthdr.len = mbuf->m_len = skb->len;

    return res;
}


#endif
