/*
 * -- Intel Copyright Notice --
 * 
 * Copyright 2002 Intel Corporation All Rights Reserved.
 * 
 * The source code contained or described herein and all documents
 * related to the source code ("Material") are owned by Intel Corporation
 * or its suppliers or licensors.  Title to the Material remains with
 * Intel Corporation or its suppliers and licensors.
 * 
 * The Material is protected by worldwide copyright and trade secret laws
 * and treaty provisions. No part of the Material may be used, copied,
 * reproduced, modified, published, uploaded, posted, transmitted,
 * distributed, or disclosed in any way except in accordance with the
 * applicable license agreement .
 * 
 * No license under any patent, copyright, trade secret or other
 * intellectual property right is granted to or conferred upon you by
 * disclosure or delivery of the Materials, either expressly, by
 * implication, inducement, estoppel, except in accordance with the
 * applicable license agreement.
 * 
 * Unless otherwise agreed by Intel in writing, you may not remove or
 * alter this notice or any other notice embedded in Materials by Intel
 * or Intel's suppliers or licensors in any way.
 * 
 * For further details, please see the file README.TXT distributed with
 * this software.
 * 
 * -- End Intel Copyright Notice --
*/
#ifndef _SIMPLE_POOL_H_
#define _SIMPLE_POOL_H_

#include <mbuf.h>

#define mbuf_t M_BLK

typedef struct simple_netpool_t
{
    mbuf_t *free;
    spinlock_t lock;

#ifdef DEBUG_POOL
    mbuf_t *first;
    int count;
#endif
} simple_netpool_t;

#define SIMPLE_POOL_DECLARE(name, count) \
    mbuf_t name##_buf[count]; \
    simple_netpool_t name = {free:name##_buf}

#define SIMPLE_POOL_DATA_DECLARE(name, count, size) __u8 name[count * size]

/* Initialize the pool */ 
int simple_pool_init(simple_netpool_t *pool, int count, int size,
    __u8 *data);

/* Return free mbuf if exists.
 * If has_more != NULL, then *has_more will
 * be set to TRUE if there is at least one more free mbuf in the pool.
 */
mbuf_t *simple_pool_get(simple_netpool_t *pool, int *has_more);

/* Return the mbuf to the pool */
void simple_pool_put(mbuf_t *buf);

#endif
