#include <stdio.h>
#include <stdlib.h>

/* Jack ++ */
#include "IxCryptoAccJack.h" 
#include "IxCryptoAccCCDMgmt_p.h"
#include "IxCryptoAccUtilities_p.h"
/* Jack ++ 2003/09/26 */
#include <linux/interrupt.h>
#include <linux/spinlock.h>

#define J_IPH_LEN 20
#define J_AH_HEAD_ICV_OFFSET 12 
/* 
*  include 	(next header 	-> 1)
*		(payload length	-> 1)
*		(reserved	-> 2)
*		(SPI		-> 4)
*		(Sequence num	-> 4)
*
*/
#define J_ESP_HEAD_IV_OFFSET 8
/*
*  include	(SPI		-> 4)
*		(Sequence num	-> 4)
*
*/
/* if HMAC-96 is working, they will be 12 */ 
#define J_MD5_HASH_LEN	12 /* 16 */
#define J_SHA_HASH_LEN	12 /* 20 */


static spinlock_t ixp425_lock; 

/* Global variables */
/* PRIVATE */ static BOOL J_RegisterCBCalled = FALSE;
/* Jack ++ */

struct ipsec_xmit_info
{
       struct sk_buff *skb;
       struct net_device_stats *stats;
       caddr_t iv;
/* #ifdef NET_21 */
       struct rtable *rt;
/* #endif */ /* NET_21 */
};

struct ipsec_rcv_info
{
       struct sk_buff *skb;
       unsigned short xlen;
       unsigned char *authenticator;
};

/*
 * User include files.
 */
#include "IxQMgr.h"
#include "IxNpeDl.h"
#include "IxOsServices.h"
#include "ixp425.h"
#include "IxOsCacheMMU.h"
#include "ix_ossl.h"
#include "IxCryptoAccCodelet.h"
#include "IxCryptoAccCodelet_p.h"

/** Private functions prototype */
PRIVATE void ixCryptoAccCodeletRegisterCB (
            UINT32 cryptoCtxId,
            IX_MBUF *pMbuf, 
            IxCryptoAccStatus status);

PRIVATE void ixCryptoAccCodeletPerformCB (
            UINT32 cryptoCtxId,
            IX_MBUF *pSrcMbuf, 
            IX_MBUF *pDestMbuf,
            IxCryptoAccStatus status);

extern void ipsec_rcv_esp_done(struct crypto_request *req);
extern void ipsec_xmit_done(struct crypto_request *req,struct sk_buff *skb,struct rtable *rt);
extern struct sk_buff *skb_replace(struct sk_buff *skb,int pri, char *o_buf, int o_len, char *n_buf, int n_len);

/** Mbuf Pool */
static IX_MBUF *ixCryptoAccCodeletCryptoBufPool;
static IX_MBUF *ixCryptoAccCodeletCryptoFreeBufQHead = NULL;
static IX_MBUF *ixCryptoAccCodeletCryptoFreeBufQTail = NULL;
static INT8 *ixCryptoAccCodeletCryptoBufData;
static UINT32 ixCryptoAccCodeletCryptoFreeBufQSize = 0;

/** Callback function */

IxCryptoAccRegisterCompleteCallback registerCB;
IxCryptoAccPerformCompleteCallback performCB;


/** Guard for once-only init code, when initialised this variable is set to 
 * TRUE 
 */
PRIVATE BOOL ixCryptoAccCodeletInitialised = FALSE;


/**
 * @fn ixCryptoAccCodeletBufIsQEmpty (void)
 *
 * @brief This inline function is used to check whether the buffer
 *        queue is empty.
 *
 */

INLINE BOOL
ixCryptoAccCodeletBufIsQEmpty (void)
{
    BOOL status = FALSE;
    if (ixCryptoAccCodeletCryptoFreeBufQSize <= 0)
    { 
        status = TRUE;
    }
    return status;
}


/**
 * @fn ixCryptoAccCodeletBufIsQFull (void)
 *
 * @brief This inline function is used to check whether the buffer
 *        queue is full.
 *
 */

INLINE BOOL
ixCryptoAccCodeletBufIsQFull(void)
{
    BOOL status = FALSE;
    if (IX_CRYPTOACC_CODELET_CRYPTO_MBUF_POOL_SIZE 
        == ixCryptoAccCodeletCryptoFreeBufQSize)
    { 
        status = TRUE;
    }
    return status;
}

/**
 * @fn ixCryptoAccCodeletCryptoMbufToQTailAdd (IX_MBUF *pMbufToAdd)
 *
 * @brief This function adds a mbuf to the tail of the queue.
 *
 */
void ixCryptoAccCodeletCryptoMbufToQTailAdd (IX_MBUF *pMbufToAdd)
{
    IX_MBUF *pMbuf;
    
    if (!ixCryptoAccCodeletBufIsQFull())
    {
        pMbuf = ixCryptoAccCodeletCryptoFreeBufQTail;
        if (NULL != ixCryptoAccCodeletCryptoFreeBufQTail) 
        {
            /* Add pMbufToAdd in the last mbuf in the chain */
            IX_MBUF_NEXT_PKT_IN_CHAIN_PTR(pMbuf) = pMbufToAdd;    
        } /* End of if ixCryptoAccCodeletCryptoFreeBufQTail */
    
        /* Update the ixCryptoAccCodeletFreeBufQ with the last mbuf */
        ixCryptoAccCodeletCryptoFreeBufQTail = pMbufToAdd;
        IX_MBUF_NEXT_PKT_IN_CHAIN_PTR(pMbufToAdd) = NULL;
        ixCryptoAccCodeletCryptoFreeBufQSize++;
    } 
    else /* Buffer queue is full */
    {
        printf ("\nBuffer Queue is full\n");
    } /* End of if-else ixCryptoAccCodeletBufIsQFull() */
} /* End of ixCryptoAccCodeletCryptoMbufToQTailAdd () function */


/**
 * @fn ixCryptoAccCodeletCryptoMbufFromQHeadRemove (IX_MBUF **pMbufToRem)
 *
 * @brief This function removes a mbuf from the head of the queue.
 */
void ixCryptoAccCodeletCryptoMbufFromQHeadRemove(IX_MBUF **pMbufToRem)
{
    if (!ixCryptoAccCodeletBufIsQEmpty())
    {
        *pMbufToRem = ixCryptoAccCodeletCryptoFreeBufQHead;

        ixCryptoAccCodeletCryptoFreeBufQHead 
            = IX_MBUF_NEXT_PKT_IN_CHAIN_PTR((*pMbufToRem));
        ixCryptoAccCodeletCryptoFreeBufQSize--;
    } 
    else
    {
        *pMbufToRem = NULL;
        printf ("\nBuffer Queue is empty %d\n",
		ixCryptoAccCodeletCryptoFreeBufQSize);   
    } /* end of if-else ixCryptoAccCodeletBufIsQEmpty */
} /* End of ixCryptoAccCodeletCryptoMbufFromQHeadRemove () function */


/**
 * @fn IX_STATUS ixCryptoAccCodeletCryptoMemPoolInit (UINT32 pktLen)
 *
 * @brief Initialise Crypto Access Codelet MBUF pool
 */
IX_STATUS ixCryptoAccCodeletCryptoMemPoolInit (UINT32 pktLen /*, UINT32 authlen */)
{
    UINT32 bufNo;
    IX_MBUF *pMbuf;
    UINT32 mBlkSize = IX_XSCALE_CACHE_LINE_SIZE * ((sizeof (IX_MBUF) + 
                        (IX_XSCALE_CACHE_LINE_SIZE - 1)) /
                        IX_XSCALE_CACHE_LINE_SIZE);    
/* Jack ++ 2004/06/16 */
#if 0
    UINT32 pktSize = pktLen /*+ authlen*/ /* IX_CRYPTOACC_CODELET_AUTH_DIGEST_LEN */;
#else
    UINT32 pktSize = pktLen >= 64 ? pktLen : 64  ;
#endif
/* Jack -- */
/* Jack ++ */
/*
    UINT32 mDataSize = IX_XSCALE_CACHE_LINE_SIZE * ((pktSize + 
                        (IX_XSCALE_CACHE_LINE_SIZE - 1)) /
                        IX_XSCALE_CACHE_LINE_SIZE);
*/
    UINT32 mDataSize = pktSize ;

    ixCryptoAccCodeletCryptoBufPool = NULL;
    ixCryptoAccCodeletCryptoBufData = NULL;

    if (NULL == (ixCryptoAccCodeletCryptoBufPool 
                    = IX_ACC_DRV_DMA_MALLOC (
                        IX_CRYPTOACC_CODELET_CRYPTO_MBUF_POOL_SIZE
                        * mBlkSize)))
    {
        printf ("Error allocating mBuf pool for Crypto Codelet!\n");
        return IX_FAIL;
    }

    if (NULL == (ixCryptoAccCodeletCryptoBufData 
                    = IX_ACC_DRV_DMA_MALLOC(
                        IX_CRYPTOACC_CODELET_CRYPTO_MBUF_POOL_SIZE *
                        mDataSize)))
    {
	printk("because buf data alloc fail, jack will free buf pool\n");
        IX_CRYPTO_ACC_CODELET_DRV_DMA_FREE(
            ixCryptoAccCodeletCryptoBufPool,
            IX_CRYPTOACC_CODELET_CRYPTO_MBUF_POOL_SIZE * mBlkSize);

        printf ("Error allocating mBuf data pool for Crypto Codelet!\n");
        return IX_FAIL;
    }

    pMbuf = ixCryptoAccCodeletCryptoBufPool;
    /* Initilise mbuf pool head pointer */
	ixCryptoAccCodeletCryptoFreeBufQHead  = pMbuf;
    
    /* Format our mBufs */
    for (bufNo = 0;
        bufNo < IX_CRYPTOACC_CODELET_CRYPTO_MBUF_POOL_SIZE ;
        bufNo++)
    {
        /* Assign memory block to mData pointer in mbuf */ 
        IX_MBUF_MDATA (pMbuf) 
            = &ixCryptoAccCodeletCryptoBufData[bufNo * mDataSize];
    
        /* Initialise mbuf length */
        IX_MBUF_MLEN(pMbuf) = mDataSize;
        
        /* No chains */
        IX_MBUF_NEXT_BUFFER_IN_PKT_PTR(pMbuf)= NULL;
    
        /* Add newly formated buffer to our free queue */
        ixCryptoAccCodeletCryptoMbufToQTailAdd (pMbuf);
        pMbuf = (IX_MBUF *)((UINT8 *) pMbuf + mBlkSize);
    }

    return IX_SUCCESS;
} /* End of ixCryptoAccCodeletCryptoMemPoolInit () function */


/**
 * @fn IX_STATUS ixCryptoAccCodeletCryptoMemPoolFree (UINT32 pktLen)
 *
 * @brief Free all resources in Crypto Access Codelet MBUF pool
 *
 */
void ixCryptoAccCodeletCryptoMemPoolFree (UINT32 pktLen)
{
    UINT32 mBlkSize = IX_XSCALE_CACHE_LINE_SIZE * ((sizeof (IX_MBUF) + 
                        (IX_XSCALE_CACHE_LINE_SIZE - 1)) /
                        IX_XSCALE_CACHE_LINE_SIZE);
/* Jack ++ 2004/06/16 */
#if 0
    UINT32 pktSize = pktLen /*+ authlen*/ /* IX_CRYPTOACC_CODELET_AUTH_DIGEST_LEN */;
#else
    UINT32 pktSize = pktLen >= 64 ? pktLen : 64  ;
#endif
/* Jack -- */
/* to prevent redundant garbage */
/*
    UINT32 mDataSize = IX_XSCALE_CACHE_LINE_SIZE * ((pktSize + 
                        (IX_XSCALE_CACHE_LINE_SIZE - 1)) /
                        IX_XSCALE_CACHE_LINE_SIZE);          
*/
    UINT32 mDataSize = pktSize ;

    IX_CRYPTO_ACC_CODELET_DRV_DMA_FREE(
        ixCryptoAccCodeletCryptoBufPool,
        IX_CRYPTOACC_CODELET_CRYPTO_MBUF_POOL_SIZE * mBlkSize);

    IX_CRYPTO_ACC_CODELET_DRV_DMA_FREE(
        ixCryptoAccCodeletCryptoBufData,
        IX_CRYPTOACC_CODELET_CRYPTO_MBUF_POOL_SIZE * mDataSize);
    
    ixCryptoAccCodeletCryptoFreeBufQHead = NULL;
    ixCryptoAccCodeletCryptoFreeBufQTail = NULL;
    ixCryptoAccCodeletCryptoFreeBufQSize = 0;
} /* End of ixCryptoAccCodeletMemPoolFree () function */

#if 0
/**
 * @fn void ixCryptoAccCodeletDispatcherPoll (void)
 *
 * @brief This function polls the queues when poll mode set in the 
 *        function ixCryptoAccCodeletDispatcherStart. 
 */
void ixCryptoAccCodeletDispatcherPoll (void)
{
    /* Set dispatcher stop flag to FALSE */
    ixCryptoAccCodeletDispatcherPollStop = FALSE;
    
    while (1)
    {
        /* Exit thread if stop flag is set to TRUE */
        if (ixCryptoAccCodeletDispatcherPollStop)
        {
            break;  /* Exit the thread */
        }

#ifndef __vxworks
        /* In linux platform, the QMgr dispatcher thread needs to be put into 
         * sleep mode for 1 milisecond after each loop, in order to allow  
         * others thread to run.
         */
        ixOsServTaskSleep (1);
#endif /* ndef __vxworks */

        /* The crypto NPE uses queues 29 and 30. */
        (*ixCryptoAccCodeletDispatcherFunc) (IX_QMGR_QUELOW_GROUP);
    }
    
} /* End of function ixCryptoAccCodeletDispatcherPoll() */
#endif
#if 0
/**
 * @fn ixCryptoAccCodeletDispatcherStart (BOOL useInterrupt)
 *
 * @brief  This function starts the Queue manager dispatcher.
 *
 * @return IX_STATUS 
 */
IX_STATUS ixCryptoAccCodeletDispatcherStart (BOOL useInterrupt)
{
    UINT32 dispatchtid;

    /* Get QMgr dispatcher function pointer */
    ixQMgrDispatcherLoopGet (&ixCryptoAccCodeletDispatcherFunc);
    
    if (useInterrupt)   /* Interrupt mode */
    {
        printf ("\nStarting Interrupt mode for cryptoAcc!\n");
        
        /* Hook the QM QLOW dispatcher to the interrupt controller. 
         * The crypto NPE uses queues 29 and 30.
         */
        if (IX_SUCCESS != ixOsServIntBind (IXP425_INT_LVL_QM1,
                              (IxVoidFnVoidPtr)ixCryptoAccCodeletDispatcherFunc,
                              (void *)IX_QMGR_QUELOW_GROUP))
        {
            printf ("Failed to bind to QM1 interrupt\n");
            return IX_FAIL;
        }
    }
    else  /* Poll mode */
    {
        printf ("\nStarting Polling mode for cryptoAcc!\n");
        if (IX_OSSL_ERROR_SUCCESS != ix_ossl_thread_create (
                                        (ix_ossl_thread_entry_point_t)
                                        ixCryptoAccCodeletDispatcherPoll,
                                        NULL,
                                        &dispatchtid))
        {
            printf ("Error spawning dispatch task\n");
            return IX_FAIL;
        }

        ix_ossl_thread_set_priority (
            dispatchtid, 
            IX_CRYPTOACC_CODELET_QMR_DISPATCHER_PRIORITY);
    }

    return IX_SUCCESS;
} /* End of ixCryptoAccCodeletDispatcherStart () function */
#endif

/**
 * @fn ixCryptoAccCodeletRegisterCtx(
 *         UINT32 opType,
 *         IxCryptoAccCtx *forwardCtx,
 *         IxCryptoAccCtx *reverseCtx,
 *         UINT32 *forwardCtxId,
 *         UINT32 *reverseCtxId)
 *
 * @brief  This function registers to the Crypto Access Component for
 *         three different operation which demonstrates the capability
 *         of the security hardware accelerator.
 *         The types of operation are:
 *          1 - Encryption and Decryption
 *          2 - Authentication: Calculation and Check
 *          3 - Encryption/Decryption with authentication
 */
IX_STATUS ixCryptoAccCodeletRegisterCtx (
	struct crypto_request *req,
	UINT32 *J_CtxId,
	IxCryptoAccCtx *J_Ctx )
{
    IxCryptoAccStatus status;
    IX_MBUF *pPrimaryMbuf = NULL;
    IX_MBUF *pSecondaryMbuf = NULL;
    UINT16 i = 0;
    
    /* Primary Mbuf and Secondary Mbuf is required for authentication 
     * and combined service for forward context
     */

	if((J_Ctx->operation == IX_CRYPTO_ACC_OP_ENCRYPT_AUTH) ||
	   (J_Ctx->operation == IX_CRYPTO_ACC_OP_AUTH_CALC)  ||
	   (J_Ctx->operation == IX_CRYPTO_ACC_OP_AUTH_DECRYPT) ||
	   (J_Ctx->operation == IX_CRYPTO_ACC_OP_AUTH_CHECK) ) {
	      ixCryptoAccCodeletCryptoMbufFromQHeadRemove (&pPrimaryMbuf);
	      ixCryptoAccCodeletCryptoMbufFromQHeadRemove (&pSecondaryMbuf);

		/* Check if the mbufs are NULL */
		if ((NULL == pPrimaryMbuf) || (NULL == pSecondaryMbuf))	{
			printf ("\nInvalid mbufs!\n");
			return IX_FAIL;
		}
	}

    /* Register the forward context */
/* Jack ++ 2004/01/07 */
    spin_lock_bh(&ixp425_lock);

    status = ixCryptoAccCtxRegister (
                 J_Ctx,
                 pPrimaryMbuf,
                 pSecondaryMbuf,
		 /* ixCryptoAccCodeletRegisterCB */ registerCB ,
		 /* ixCryptoAccCodeletPerformCB */ performCB ,
                 J_CtxId );


    if (IX_CRYPTO_ACC_STATUS_SUCCESS != status)
    {
        printf ("Forward registration Failed. Status = %d\n", status);
	/* Jack ++ 2004/01/07 */
	spin_unlock_bh(&ixp425_lock);
        return IX_FAIL;
    }

    i = 0;

    /* Wait for the forward register callback to return */

    while (! J_RegisterCBCalled )
    {

	{ /* replace sleep function */
	    int j = 0, k = 0;
		for(j=0;j < 8;j++) k++;
	}        

        /* Wait for IX_CRYPTOACC_CODELET_MAX_TIME_WAIT times before return 
         * fail status 
         */
        if (IX_CRYPTOACC_CODELET_MAX_TIME_WAIT < i)
        {
            printf ("\nRegistration incomplete, timeout for forward callback\n");
	    /* Jack ++ 2004/01/07 */
	    spin_unlock_bh(&ixp425_lock);
            return IX_FAIL;
        }
        i++;
    } /* end of while (! J_RegisterCBCalled) */

/* Now, Register is Success, point to req */
	J_RegisterCBCalled = FALSE;

/* Jack ++ 2003/01/07 */
    spin_unlock_bh(&ixp425_lock);

	memcpy(&ixCryptoCtx[*J_CtxId].Jack_req, req, sizeof(IxCryptoAccCryptoCtx));
	/* Jack ++ 2004/06/16 */
        {
                struct ipsec_rcv_info *pp;
		pp = (struct ipsec_rcv_info *) ixCryptoCtx[*J_CtxId].Jack_req.priv;

                if (req->packetLen < 64) {
                        pp->skb->less_64_length = 64 - req->packetLen;
                } else {
                        pp->skb->less_64_length = 0;
                }
        }
	/* Jack -- */

    return IX_SUCCESS;
} /* End of ixCryptoAccCodeletRegisterCtx () function */


/**
 * @fn ixCryptoAccCodeletUnregisterCtx (void)
 *
 * @brief  This function unregisters all the contexts registered.
 */
/* void ixCryptoAccCodeletUnregisterCtx (void) */
void Unregister_Ctx(UINT32 CtxId)
{
    if (IX_CRYPTO_ACC_STATUS_SUCCESS != 
        ixCryptoAccCtxUnregister (CtxId))
    {
        printf ("\nFailed to unregister context %d\n",CtxId);
    }
/*
    J_RegisterCBCalled = FALSE;
*/
    /* Initialised to an invalid number, zero is a valid ID */
    CtxId = IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS;  
return ;
}

/**
 * @fn  ixCryptoAccCodeletInit (void)
 * @brief This function initialises the neccessary components.
 *
 */

/* PRIVATE */ IX_STATUS ixCryptoAccJackInit (void)
{
    if (ixCryptoAccCodeletInitialised) 
    {
        printf ("Crypto codelet already initialised\n");
        return IX_SUCCESS;
    }

    /* Initialise stop flag to TRUE */
/*
    ixCryptoAccCodeletDispatcherPollStop = TRUE;
*/

    /* Initialise cryptoAcc component */
    if (IX_CRYPTO_ACC_STATUS_SUCCESS != ixCryptoAccInit ())
    {
        printf ("Error initialising Crypto access component!\n");
        return IX_FAIL;
    }

    /* set flag to TRUE to indicate codelet has been initialised successfully */
    ixCryptoAccCodeletInitialised = TRUE;
    printf ("cryptoAcc Codelet Initialiasation complete!\n\n");

    return IX_SUCCESS;
} /* End of ixCryptoAccCodeletInit () function */


/** 
 * @fn ixCryptoAccCodeletRegisterCB(
 *         UINT32 cryptoCtxId,
 *         IX_MBUF *pMbuf, 
 *         IxCryptoAccStatus status)
 *
 * @brief  Cryptographic Context registration callback function to be called
 *         in QMgr Dispatcher context when the registration completed.
 *
 * @param "UINT32 [in] cryptoCtxId" - Crypto Context ID
 * @param "IX_MBUF* [in] pMbuf" - mbuf pointer
 * @param "IxCryptoAccStatus [in] status" - status of callback
 *
 * @return None
 */
PRIVATE void ixCryptoAccCodeletRegisterCB (
        UINT32 cryptoCtxId,
        IX_MBUF *pMbuf, 
        IxCryptoAccStatus status)
{

    /* We are expecting two callbacks for authentication and combined service 
     * The pMbuf could be null for encrypt/decrypt ONLY operation.
     */
    if (NULL != pMbuf)
    {
        /* Must return the buffer to the pool irrespective of the status */
        ixCryptoAccCodeletCryptoMbufToQTailAdd (pMbuf);
    } 

    if (IX_CRYPTO_ACC_STATUS_SUCCESS == status)
    {
	J_RegisterCBCalled = TRUE;
/*
        ixOsServLog(LOG_MESSAGE, 
            "RegistrationCB is successful. Context %d\n", 
            cryptoCtxId, 
            0, 0, 0, 0, 0);
*/
    } /* End of if (IX_CRYPTO_ACC_STATUS_SUCCESS == status) */
    else 
    {
/*
        if (IX_CRYPTO_ACC_STATUS_WAIT == status)
        {
            ixOsServLog(LOG_MESSAGE, 
                "Register not completed yet; Context %d\n", 
                cryptoCtxId, 
                0, 0, 0, 0, 0);
        }
        else
        {
            ixOsServLog(LOG_WARNING, "Registration failed; Context %d\n", 
                cryptoCtxId, 0, 0, 0, 0, 0);
        }
*/
    } /* End of if else (IX_CRYPTO_ACC_STATUS_SUCCESS == status) */
} /* End of ixCryptoAccCodeletRegisterCB () function */

/* Jack ++ */
__inline__ UINT32 ixOsServWordSwap(UINT32 wordIn)
{
    /* Storage for the swapped word */
    UINT32 wordOut;

    /* wordIn = A, B, C, D */
    __asm__ (" eor r1, %1, %1, ror #16;"  /* R1 =      A^C, B^D, C^A, D^B */
             " bic r1, r1, #0x00ff0000;"  /* R1 =      A^C, 0  , C^A, D^B */
             " mov %0, %1, ror #8;"       /* wordOut = D,   A,   B,   C   */
             " eor %0, %0, r1, lsr #8;"   /* wordOut = D,   C,   B,   A   */
             : "=r" (wordOut) : "r" (wordIn) : "r1");

    return wordOut;
}



/**
 * @fn ixCryptoAccCodeletPerformCB (
 *        UINT32 cryptoCtxId,
 *        IX_MBUF *pSrcMbuf, 
 *        IX_MBUF *pDestMbuf,
 *        IxCryptoAccStatus status)
 *
 * @brief  Crypto services request callback fuction to be called when the 
 *         crypto service s request completed. Performance rate is calculated 
 *         in this function.
 *
 * @param "UINT32 [in] cryptoCtxId" - Crypto Context ID
 * @param "IX_MBUF* [in] pSrcMbuf" - source mbuf pointer
 * @param "IX_MBUF* [in] pDestMbuf" - destination mbuf pointer
 * @param "IxCryptoAccStatus [in] status" - status of callback
 *
 * @return None
 */
PRIVATE void ixCryptoAccCodeletPerformCB (
        UINT32 cryptoCtxId,
        IX_MBUF *pSrcMbuf, 
        IX_MBUF *pDestMbuf,
        IxCryptoAccStatus status)
{
//////////////////////////
/*
		{
			UINT32 *P;
			int P_len = pSrcMbuf->m_len;
			P = (UINT32 *)(pSrcMbuf->m_data);

			while(P_len > 0) {
				*P = (UINT32)ixOsServWordSwap((UINT32)(*P));
				P_len -= sizeof(UINT32);
				P ++;
			}
		}
*/
///////////////////////////
    if(ixCryptoCtx[cryptoCtxId].Jack_req.op.cmd & CRYPT_DECODE) {
	/* Decrypt Callback function is ipsec_rcv_esp_done */
      switch (status)
      {
        case IX_CRYPTO_ACC_STATUS_SUCCESS:
	{
	  struct ipsec_rcv_info *xmit;
	  
	  xmit = (struct ipsec_rcv_info *)ixCryptoCtx[cryptoCtxId].Jack_req.priv ;
		{
		  struct iphdr *iph = xmit->skb->nh.iph;
		  char *data = (void *)iph + 
			ixCryptoCtx[cryptoCtxId].Jack_req.temp_iphlen;

		  xmit->skb = skb_replace(xmit->skb,GFP_ATOMIC,data,
			xmit->skb->len  - 
				ixCryptoCtx[cryptoCtxId].Jack_req.temp_iphlen,
			pSrcMbuf->m_data,pSrcMbuf->m_len);
		}
	ipsec_rcv_esp_done(&ixCryptoCtx[cryptoCtxId].Jack_req);
	break;
	} /* end of case IX_CRYPTO_ACC_STATUS_SUCCESS: */
	default :
      } /* end of switch */

    } else {  /* Encrypt Callback function is ipsec_xmit_done */
      switch (status)
      {
        case IX_CRYPTO_ACC_STATUS_SUCCESS:
	{
	  struct ipsec_xmit_info *xmit;
	  xmit = (struct ipsec_xmit_info *)ixCryptoCtx[cryptoCtxId].Jack_req.priv ;
/*
		{
		  struct iphdr *iph = xmit->skb->nh.iph;
		  char *data = (void *)iph + iph->ihl*4;

		  xmit->skb = skb_replace(xmit->skb,GFP_ATOMIC,data,
			xmit->skb->len - iph->ihl*4 ,
			pSrcMbuf->m_data,pSrcMbuf->m_len);
		}
*/
		{
		  struct iphdr *iph = xmit->skb->nh.iph;
		  char *data = (void *)iph + 
			ixCryptoCtx[cryptoCtxId].Jack_req.temp_iphlen;

		  xmit->skb = skb_replace(xmit->skb,GFP_ATOMIC,data,
			xmit->skb->len - 
				ixCryptoCtx[cryptoCtxId].Jack_req.temp_iphlen,
			pSrcMbuf->m_data,pSrcMbuf->m_len);
		}

	ipsec_xmit_done(&ixCryptoCtx[cryptoCtxId].Jack_req,xmit->skb,xmit->rt);

	break;
	} /* end of case IX_CRYPTO_ACC_STATUS_SUCCESS: */
	default :
      } /* end of switch */
    } /* end of if CRYPT_DECODE */

return ;

} /* End of ixCryptoAccCodeletPerformCB () function */


/**
 * @fn ixCryptoAccCodeletMain (
 *          IxCryptAccCodeletOperation operationType,
 *          UINT32 packetLen)
 *
 * @brief  This is the entry point function to the codelet to choose the 
 *         operation for the codelet and packet length to be used. 
 *         This is the main function of the codelet where crypto contexts
 *         registration and crypto perform services are done. Based on the  
 *         selected operation and selected packet length, packets are sent to 
 *         cryptoAcc for processing. 
 */

int change_format(
struct crypto_request *req)
{
	struct crypto_operation *oper_ptr;

	if(req == NULL) {
		printk("change_format req == NULL\n");
		return -1;
	}

	if(req->J_Ctx == NULL) {
		printk("change_format J_Ctx == NULL\n");
		return -1;
	}


	oper_ptr = &req->op ;

  { /* Start of A */

    int With_Auth = 0;
    int With_Enc = 0;

    while(oper_ptr->cmd) {

        if(oper_ptr->cmd & CRYPT_3DES) {
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherAlgo = 
				IX_CRYPTO_ACC_CIPHER_3DES;
		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherKeyLen = 
				IX_CRYPTO_ACC_3DES_KEY_192;
		memcpy(((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.key.cipherKey,
				&oper_ptr->u.des.key[0],
                                IX_CRYPTO_ACC_3DES_KEY_192);

                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherBlockLen = 
				IX_CRYPTO_ACC_DES_BLOCK_64;
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherInitialVectorLen =
				IX_CRYPTO_ACC_DES_IV_64;
		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherMode = 
				IX_CRYPTO_ACC_MODE_CBC;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptStartOffset =
				oper_ptr->offset ;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptDataLen =
				oper_ptr->len;

		{
		  struct ipsec_rcv_info *pp;
		  pp = (struct ipsec_rcv_info *) req->priv;

		  memcpy(&((IxCryptoAccCtx *)(req->J_Ctx))->ivec[0],
			((pp->skb->data + J_IPH_LEN) + J_ESP_HEAD_IV_OFFSET),
			IX_CRYPTO_ACC_MAX_CIPHER_IV_LENGTH
			/*IX_CRYPTO_ACC_DES_IV_64*/);
		}

		With_Enc = 1;
        }

	else if(oper_ptr->cmd & CRYPT_DES) {
//////////////////////
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherAlgo = 
			IX_CRYPTO_ACC_CIPHER_DES;

		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherKeyLen = 
			IX_CRYPTO_ACC_DES_KEY_64;

		memcpy(((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.key.cipherKey,
				oper_ptr->u.des.key,
                                IX_CRYPTO_ACC_DES_KEY_64);

                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherBlockLen = 
				IX_CRYPTO_ACC_DES_BLOCK_64;
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherInitialVectorLen =
				IX_CRYPTO_ACC_DES_IV_64;
		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherMode = 
				IX_CRYPTO_ACC_MODE_CBC;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptStartOffset =
				oper_ptr->offset ;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptDataLen =
				oper_ptr->len;

		{
		  struct ipsec_rcv_info *pp;
		  pp = (struct ipsec_rcv_info *) req->priv;
		  memcpy(&((IxCryptoAccCtx *)(req->J_Ctx))->ivec[0],
			((pp->skb->data + J_IPH_LEN) + J_ESP_HEAD_IV_OFFSET),
			IX_CRYPTO_ACC_DES_IV_64);
		}

		With_Enc = 1;
	}

//////////////////////
	else if(oper_ptr->cmd & CRYPT_AES_128) {
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherAlgo = 
			IX_CRYPTO_ACC_CIPHER_AES;

		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherKeyLen = 
			IX_CRYPTO_ACC_AES_KEY_128;

		memcpy(((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.key.cipherKey,
				oper_ptr->u.aes.key,
                                IX_CRYPTO_ACC_AES_KEY_128);

                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherBlockLen = 
				IX_CRYPTO_ACC_AES_BLOCK_128;
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherInitialVectorLen =
				IX_CRYPTO_ACC_AES_CBC_IV_128;
		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherMode = 
				IX_CRYPTO_ACC_MODE_CBC;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptStartOffset =
				oper_ptr->offset ;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptDataLen =
				oper_ptr->len;

		{
		  struct ipsec_rcv_info *pp;
		  pp = (struct ipsec_rcv_info *) req->priv;
		  memcpy(&((IxCryptoAccCtx *)(req->J_Ctx))->ivec[0],
			((pp->skb->data + J_IPH_LEN) + J_ESP_HEAD_IV_OFFSET),
			IX_CRYPTO_ACC_AES_CBC_IV_128);
		}

		With_Enc = 1;
	}
//////////////////////
	else if(oper_ptr->cmd & CRYPT_AES_192) {
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherAlgo = 
			IX_CRYPTO_ACC_CIPHER_AES;

		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherKeyLen = 
			IX_CRYPTO_ACC_AES_KEY_192;

		memcpy(((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.key.cipherKey,
				oper_ptr->u.aes.key,
                                IX_CRYPTO_ACC_AES_KEY_192);

                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherBlockLen = 
				IX_CRYPTO_ACC_AES_BLOCK_128;
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherInitialVectorLen =
				IX_CRYPTO_ACC_AES_CBC_IV_128;
		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherMode = 
				IX_CRYPTO_ACC_MODE_CBC;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptStartOffset =
				oper_ptr->offset ;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptDataLen =
				oper_ptr->len;

		{
		  struct ipsec_rcv_info *pp;
		  pp = (struct ipsec_rcv_info *) req->priv;
		  memcpy(&((IxCryptoAccCtx *)(req->J_Ctx))->ivec[0],
			((pp->skb->data + J_IPH_LEN) + J_ESP_HEAD_IV_OFFSET),
			IX_CRYPTO_ACC_AES_CBC_IV_128);
		}

		With_Enc = 1;
	}
//////////////////////
	else if(oper_ptr->cmd & CRYPT_AES_256) {
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherAlgo = 
			IX_CRYPTO_ACC_CIPHER_AES;

		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherKeyLen = 
			IX_CRYPTO_ACC_AES_KEY_256;

		memcpy(((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.key.cipherKey,
				oper_ptr->u.aes.key,
                                IX_CRYPTO_ACC_AES_KEY_256);

                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherBlockLen = 
				IX_CRYPTO_ACC_AES_BLOCK_128;
                ((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherInitialVectorLen =
				IX_CRYPTO_ACC_AES_CBC_IV_128;
		((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.cipherMode = 
				IX_CRYPTO_ACC_MODE_CBC;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptStartOffset =
				oper_ptr->offset ;
		((IxCryptoAccCtx *)(req->J_Ctx))->cryptDataLen =
				oper_ptr->len;

		{
		  struct ipsec_rcv_info *pp;
		  pp = (struct ipsec_rcv_info *) req->priv;
		  memcpy(&((IxCryptoAccCtx *)(req->J_Ctx))->ivec[0],
			((pp->skb->data + J_IPH_LEN) + J_ESP_HEAD_IV_OFFSET),
			IX_CRYPTO_ACC_AES_CBC_IV_128);
		}

		With_Enc = 1;
	}
//////////////////////

	else if (oper_ptr->cmd & CRYPT_MD5 ) 
	{
                ((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authAlgo = 
			IX_CRYPTO_ACC_AUTH_MD5;
                ((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authKeyLen = 
			IX_CRYPTO_ACC_MD5_KEY_128;
                ((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authDigestLen = 
			J_MD5_HASH_LEN /* IX_CRYPTO_ACC_MD5_DIGEST_128 */;

                memcpy(((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.key.authKey,
                        (oper_ptr->u.hash.hmac_key),
                        IX_CRYPTO_ACC_MD5_KEY_128);

		((IxCryptoAccCtx *)(req->J_Ctx))->authStartOffset =
				oper_ptr->offset;
		((IxCryptoAccCtx *)(req->J_Ctx))->authDataLen =
				oper_ptr->len;

		With_Auth = 1;
        }

	else if (oper_ptr->cmd & CRYPT_SHA ) {

                ((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authAlgo = 
			IX_CRYPTO_ACC_AUTH_SHA1;
                ((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authKeyLen = 
			IX_CRYPTO_ACC_SHA1_KEY_160;
                ((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authDigestLen = 
			J_SHA_HASH_LEN /* IX_CRYPTO_ACC_SHA1_DIGEST_160 */;

                memcpy(((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.key.authKey,
                        (oper_ptr->u.hash.hmac_key),
                        IX_CRYPTO_ACC_SHA1_KEY_160);

		((IxCryptoAccCtx *)(req->J_Ctx))->authStartOffset =
				oper_ptr->offset;
		((IxCryptoAccCtx *)(req->J_Ctx))->authDataLen =
				oper_ptr->len;

		With_Auth = 1;
        }
	else {
		printk("Not 3DES, DES, MD5, SHA1\n");
		break;
	}

	if(oper_ptr->next) oper_ptr = oper_ptr->next;
	else break;
    } /* end of while loop */

	if( (!With_Auth) && (!With_Enc) ) {
	//	printk("no auth and no enc\n");
		return -1;
	}

	if(oper_ptr->cmd & CRYPT_DECODE) { /* Decode */

		if(With_Auth && With_Enc) {
		    {
			struct ipsec_rcv_info *pp;
			pp = (struct ipsec_rcv_info *) req->priv;
			req->packetLen =  pp->skb->len - J_IPH_LEN ;
		    }        

			((IxCryptoAccCtx *)(req->J_Ctx))->icvOffset 
			= req->inlen - 
			((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authDigestLen;

			((IxCryptoAccCtx *)(req->J_Ctx))->operation = IX_CRYPTO_ACC_OP_AUTH_DECRYPT;

			req->temp_iphlen = J_IPH_LEN;

		} else if(With_Auth) {
		    {
			struct ipsec_rcv_info *pp;
			pp = (struct ipsec_rcv_info *) req->priv;
			req->packetLen =  pp->skb->len ;
		    }        

			((IxCryptoAccCtx *)(req->J_Ctx))->icvOffset 
			= J_IPH_LEN + J_AH_HEAD_ICV_OFFSET; 

			((IxCryptoAccCtx *)(req->J_Ctx))->operation = IX_CRYPTO_ACC_OP_AUTH_CHECK;

			req->temp_iphlen = 0;

		} else if(With_Enc) {
		    {
			struct ipsec_rcv_info *pp;
			pp = (struct ipsec_rcv_info *) req->priv;
			req->packetLen =  pp->skb->len - J_IPH_LEN ;
		    }        
			((IxCryptoAccCtx *)(req->J_Ctx))->icvOffset 
			= req->inlen - 
			((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authDigestLen;

			((IxCryptoAccCtx *)(req->J_Ctx))->operation = IX_CRYPTO_ACC_OP_DECRYPT;

			req->temp_iphlen = J_IPH_LEN;

		} else
			printk("Why do you come here ? Decode error\n");
	} else { /* Encode */

		if(With_Auth && With_Enc) {
		    {
			struct ipsec_rcv_info *pp;
			pp = (struct ipsec_rcv_info *) req->priv;
			req->packetLen =  pp->skb->len - J_IPH_LEN ;
		    }        

			((IxCryptoAccCtx *)(req->J_Ctx))->icvOffset 
			= req->inlen; 

			((IxCryptoAccCtx *)(req->J_Ctx))->operation = IX_CRYPTO_ACC_OP_ENCRYPT_AUTH;

			req->temp_iphlen = J_IPH_LEN;

		} else if(With_Auth) {
		    {
			struct ipsec_rcv_info *pp;
			pp = (struct ipsec_rcv_info *) req->priv;
			req->packetLen =  pp->skb->len ;
		    }        

			((IxCryptoAccCtx *)(req->J_Ctx))->icvOffset 
			= J_IPH_LEN + J_AH_HEAD_ICV_OFFSET; 

			((IxCryptoAccCtx *)(req->J_Ctx))->operation = IX_CRYPTO_ACC_OP_AUTH_CALC;

			req->temp_iphlen = 0;

		} else if(With_Enc) {
		    {
			struct ipsec_rcv_info *pp;
			pp = (struct ipsec_rcv_info *) req->priv;
			req->packetLen =  pp->skb->len - J_IPH_LEN ;
		    }        

			((IxCryptoAccCtx *)(req->J_Ctx))->icvOffset 
			= req->inlen; 

			((IxCryptoAccCtx *)(req->J_Ctx))->operation = IX_CRYPTO_ACC_OP_ENCRYPT;

			req->temp_iphlen = J_IPH_LEN;

		} else
			printk("Why do you come here ? Encode error\n");
	}
  } /* End of A */

	((IxCryptoAccCtx *)(req->J_Ctx))->cipherCtx.securityParameterIndex = 0;
	((IxCryptoAccCtx *)(req->J_Ctx))->useDifferentSrcAndDestMbufs = FALSE;
	req->J_CtxId = IX_CRYPTO_ACC_MAX_ACTIVE_SA_TUNNELS;

return 0;
} /* end of change_format */

/*
static int j_limit_count = 0;
#define J_LIMIT_COUNT_NUM 6
*/

IX_STATUS ixCryptoAccCodeletMain ( struct crypto_request *req )
{

    IxCryptoAccStatus status;
    IX_MBUF *pMbuf;

    registerCB = ixCryptoAccCodeletRegisterCB;
    performCB = ixCryptoAccCodeletPerformCB;

	(IxCryptoAccCtx *)(req->J_Ctx) = kmalloc(sizeof(IxCryptoAccCtx), GFP_ATOMIC);

	if(! req->J_Ctx) {
		return IX_FAIL;
	} 

	if(change_format(req) < 0) {
		return IX_FAIL;
	}

/**************************** Jack ****************************/

    if (IX_SUCCESS 
        != ixCryptoAccCodeletCryptoMemPoolInit (req->packetLen /* ,
		((IxCryptoAccCtx *)(req->J_Ctx))->authCtx.authDigestLen */))
    {
        printf ("Mbuf pool init failed\n");
	kfree(((IxCryptoAccCtx *)(req->J_Ctx)));
        return IX_FAIL;
    }

    if (IX_SUCCESS != ixCryptoAccCodeletRegisterCtx(
			req,
			&(req->J_CtxId),
			((IxCryptoAccCtx *)(req->J_Ctx))))
    {
        printf ("Registration Failed.\n");
	kfree(((IxCryptoAccCtx *)(req->J_Ctx)));
        return IX_FAIL;
    }

        /* Get an mbuf */
        ixCryptoAccCodeletCryptoMbufFromQHeadRemove (&pMbuf);

        /* Check if the mbuf is NULL */
        if (NULL == pMbuf)
        {
            printf ("\nInvalid mbuf!\n");
	    kfree(((IxCryptoAccCtx *)(req->J_Ctx)));
            return IX_FAIL;
        }

	/* copy data into pMbuf */
	{
		struct ipsec_rcv_info *pp;
		pp = (struct ipsec_rcv_info *) req->priv;
		memcpy(pMbuf->mBlkHdr.mData,pp->skb->data + req->temp_iphlen , pp->skb->len - req->temp_iphlen );
	}

/////////////////////////
	        
        /* This while loop will always call perform until it is  not 
         * successful but will continue to retry perform if the status 
         * is queue_full. Start with forwardCtxId and the reverseCtxId
         * will be used in the perform callback.
         */
            status = ixCryptoAccAuthCryptPerform (
                        req->J_CtxId, /* CtxId */
                        pMbuf, /* pSrcMbuf */
                        NULL, /* pDestMbuf */
                        ((IxCryptoAccCtx *)(req->J_Ctx))->authStartOffset , /* authStartOffset */
                        ((IxCryptoAccCtx *)(req->J_Ctx))->authDataLen , /* authDataLen */
                        ((IxCryptoAccCtx *)(req->J_Ctx))->cryptStartOffset , /* cryptStartOffset */
                        ((IxCryptoAccCtx *)(req->J_Ctx))->cryptDataLen , /* cryptDataLen */
                        ((IxCryptoAccCtx *)(req->J_Ctx))->icvOffset  , /* ICV offset */
                        &((IxCryptoAccCtx *)(req->J_Ctx))->ivec[0] /* IV */);

	kfree(((IxCryptoAccCtx *)(req->J_Ctx)));

	while(ixCryptoCtx[req->J_CtxId].reqCount != 
		ixCryptoCtx[req->J_CtxId].reqDoneCount)
	{
		int ii,ik=0;
		for(ii=0;ii< 1; ii++)
			ik++;
	}

	Unregister_Ctx(req->J_CtxId);  
	ixCryptoAccCodeletCryptoMemPoolFree(req->packetLen); 

	/* Free Structure */
	{
	  struct crypto_operation *op;

	  kfree(req->priv);

	  op = req->op.next;
	  while(op != NULL) {
		struct crypto_operation *del;

		del = op;
		op = op->next;

		kfree(del);

	  }

	  if(req->saved_header) {
		 kfree(req->saved_header);
	  }

	  if(req->oskb) {
		 kfree_skb(req->oskb);
	  }

	  if(req->ident_s) {

		kfree_skb(req->ident_s);

	  }

	  if(req->ident_d) {

		kfree_skb(req->ident_d);

	  }

	  crypto_free_request(req);
	} /* end of Free Structure */

 return IX_SUCCESS ; 

} /* end of function ixCryptoAccCodeletMain () */

