/*
 * File: ixp425_eth.c
 *
 * Author: Intel Corporation
 *
 * IXP425 Ethernet Driver for Linux
 *
 * Design Notes:
 *
 * -- Intel/GPL Copyright Notice --
 *
 * This software program is available to you under a choice of one of two
 * licenses. You may choose to be licensed under either the GNU General Public 
 * License (GPL) Version 2, June 1991, available at
 * http://www.fsf.org/copyleft/gpl.html, 
 * or the Intel BSD + Patent License, the text of which follows:
 *
 * "Recipient has requested a license and Intel Corporation ("Intel") is
 * willing to grant a license for the software entitled ixp425 NPE Ethernet
 * driver (the "Software") being provided by Intel Corporation.
 *
 * The following definitions apply to this License: 
 *
 * "Licensed Patents" means patent claims licensable by Intel Corporation which
 * are necessarily infringed by the use or sale of the Software alone or when
 * combined with the operating system referred to below.
 *
 * "Recipient" means the party to whom Intel delivers this Software.
 * "Licensee" means Recipient and those third parties that receive a license to
 * any operating system available under the GNU Public License version 2.0 or
 * later.
 *
 * Copyright (c) 2002-2003 Intel Corporation. All rights reserved. 
 *
 * The license is provided to Recipient and Recipients Licensees under the
 * following terms.  
 *
 * Redistribution and use in source and binary forms of the Software, with or
 * without modification, are permitted provided that the following conditions
 * are met: 
 * Redistributions of source code of the Software may retain the above
 * copyright notice, this list of conditions and the following disclaimer. 
 *
 * Redistributions in binary form of the Software may reproduce the above
 * copyright notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the distribution. 
 *
 * Neither the name of Intel Corporation nor the names of its contributors
 * shall be used to endorse or promote products derived from this Software 
 * without specific prior written permission.
 *
 * Intel hereby grants Recipient and Licensees a non-exclusive, worldwide,
 * royalty-free patent license under Licensed Patents to make, use, sell, offer
 * to sell, import and otherwise transfer the Software, if any, in source code
 * and object code form. This license shall include changes to the Software 
 * that are error corrections or other minor changes to the Software that do
 * not add functionality or features when the Software is incorporated in any
 * version of a operating system that has been distributed under the GNU
 * General Public License 2.0 or later.  This patent license shall apply to the
 * combination of the Software and any operating system licensed under the GNU 
 * Public License version 2.0 or later if, at the time Intel provides the
 * Software to Recipient, such addition of the Software to the then publicly
 * available versions of such operating system available under the GNU Public
 * License version 2.0 or later (whether in gold, beta or alpha form) causes
 * such combination to be covered by the Licensed Patents. The patent license
 * shall not apply to any other combinations which include the Software. No
 * hardware per se is licensed hereunder. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE."
 *
 * -- End Intel/GPL Copyright Notice --
 */


/* NOTE:
 * By default, this driver uses predefined MAC addresses.
 * These are set in global var 'default_mac_addr' in this file.
 * If required, these can be changed at run-time using
 * the 'ifconfig' tool.
 *
 * Example - to set ixp0 MAC address to 00:22:44:66:88:AA, 
 * run ifconfig with the following arguments:
 *   `ifconfig ixp0 hw ether 0022446688AA`
 */

/*
 * System-defined header files
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/delay.h>
#include <linux/mii.h>
#include <asm/io.h>
#include <asm/errno.h>
/* 2004/3/9 Steve ++ */
#include <linux/proc_fs.h>
#include <asm/uaccess.h>
/* Steve -- */

/*
 * IXP400 Software specific header files
 */
#include <ixp425.h>
#include <IxEthAcc.h>
#include <IxEthDB.h>
#include <IxQMgr.h>
#include <IxNpeDl.h>
#include <IxNpeMh.h>
#include <IxOsServices.h>
#include <IxOsBuffPoolMgt.h>
#include <IxEthNpe.h>
#include <IxNpeMicrocode.h>

/* memset buffer contents to CC [debug] */
#ifdef DEBUG
#define MEMSET_BUFF_CC
#endif


/* Hardcoded parameters */

/*
 * Module version information
 */
MODULE_DESCRIPTION("Ethernet driver for IXP425");
MODULE_LICENSE("Copyright (c) Intel Corporation. 2002");
MODULE_AUTHOR("Intel Corporation");
#define MODULE_NAME "ixp425_eth"
#define MODULE_VERSION "0.0.9"

/* Module parameters */
static int log_level = 0;
static int no_csr_init = 0;
static int no_phy_scan = 0;
MODULE_PARM(log_level, "i");
MODULE_PARM_DESC(log_level, "Set log level: 0 - None, 1 - Verbose, 2 - Debug");
MODULE_PARM(no_csr_init, "i");
MODULE_PARM_DESC(no_csr_init, "If non-zero, do not initialise Intel IXP400 Software Release core components");
MODULE_PARM(no_phy_scan, "i");
MODULE_PARM_DESC(no_phy_scan, "If non-zero, use hard-coded phy addresses");

/* devices will be called ixp0 and ixp1 */
/* Jack ++ */
/* #define DEVICE_NAME "ixp" */
#define DEVICE_NAME "eth" 

/* 2005/7/27 Steve ++ */
#if 0
static struct timer_list eth_rx_timer;
#endif
/* Steve -- */

static struct net_device ixp425_devices[IX_ETH_ACC_NUMBER_OF_PORTS];

/* boolean values for PHY link speed, duplex, and autonegotiation */
#define PHY_SPEED_10    0
#define PHY_SPEED_100   1
#define PHY_DUPLEX_HALF 0
#define PHY_DUPLEX_FULL 1
#define PHY_AUTONEG_OFF 0
#define PHY_AUTONEG_ON  1

/* Time before kernel will decide that the driver had stuck in transmit */
#define DEV_WATCHDOG_TIMEO (10*HZ)

/* Interval between media-sense/duplex checks */
#define MEDIA_CHECK_INTERVAL 3
/* Interval in milliseconds */
#define MEDIA_CHECK_INTERVAL_MS (MEDIA_CHECK_INTERVAL * 1000)

/* number of packets to prealloc for the Tx pool */
#define RX_QUEUE_PREALLOC 128

/* Maximum number of packets in Tx queue before netif_stop_queue() */
#define TX_QUEUE_MAX_LEN 128

/* Maximum number of addresses the EthAcc multicast filter can hold */
#define IX_ETH_ACC_MAX_MULTICAST_ADDRESSES 256

/* size of the SKBs to allocate */
#define PKT_SIZE 2048

/* NPE-B Functionality: Ethernet with FastPath, No Crypto */
#define IX_ETH_NPE_B_IMAGE_ID IX_NPEDL_NPEIMAGE_NPEB_ETH_FPATH
/* NPE-C Functionality: Ethernet without FastPath, No Crypto  */
/* Jack ++ */
#if 0
#define IX_ETH_NPE_C_IMAGE_ID IX_NPEDL_NPEIMAGE_NPEC_ETH
#else
/* #define IX_ETH_NPE_C_IMAGE_ID IX_NPEDL_NPEIMAGE_NPEC_CRYPTO_ETH */
#define IX_ETH_NPE_C_IMAGE_ID IX_NPEDL_NPEIMAGE_NPEC_CRYPTO_AES_ETH
#endif

/*
 * Macros to turn on/off debug messages
 */
/* Print kernel error */
#define P_ERROR(args...) \
    printk(KERN_ERR MODULE_NAME ": " args)
/* Print kernel warning */
#define P_WARN(args...) \
    printk(KERN_WARNING MODULE_NAME ": " args)
/* Print kernel notice */
#define P_NOTICE(args...) \
    printk(KERN_NOTICE MODULE_NAME ": " args)
/* Print kernel info */
#define P_INFO(args...) \
    printk(KERN_INFO MODULE_NAME ": " args)
/* Print verbose message. Enabled/disabled by 'log_level' param */
#define P_VERBOSE(args...) \
    if (log_level >= 1) printk(MODULE_NAME ": " args)
/* Print debug message. Enabled/disabled by 'log_level' param  */
#define P_DEBUG(args...) \
    if (log_level >= 2) { \
        printk("%s: %s()\n", MODULE_NAME, __FUNCTION__); \
        printk(args); }
/* Print trace message */
#define TRACE \
    if (log_level >= 2) printk("%s: %s(): line %d\n", MODULE_NAME, __FUNCTION__, __LINE__)

/* Private device data */
typedef struct {
    spinlock_t lock;

    IxEthAccPortId port_id; /* MAC port_id */

    struct net_device_stats stats; /* device statistics */

    /* preallocated RX pool */
    IX_MBUF rx_pool_buf[RX_QUEUE_PREALLOC];
    IX_MBUF_POOL *rx_pool;

    /* preallocated TX pool */
    IX_MBUF tx_pool_buf[TX_QUEUE_MAX_LEN];
    IX_MBUF_POOL *tx_pool;

    /* Set to 1 in dev->open and to 0 in dev->stop.
     * Checked by rx_cb to decide if to pass packet to the kernel [1] or
     * back to the pool [0].
     * Also used to stop the kernel thread for link monitoring.
     */
    int running;

} priv_data_t;

/* Collection of boolean PHY configuration parameters */
typedef struct {
    int speed100;
    int duplexFull;
    int autoNegEnabled;
} phy_cfg_t;
	

/*
 * STATIC VARIABLES
 *
 * This section sets several default values for each port.
 * These may be edited if required.
 */


/* The PHY addresses mapped to Intel IXP400 Software EthAcc ports.
 *
 * These are hardcoded here but typically overwritten
 * by a PHY scan. A PHY scan is enabled by default but
 * can optionally be disabled if required in 2 ways:
 *  - by setting CONFIG_IXP425_CSR_ETH_SKIP_PHY_SCAN
 *  - by passing module param no_phy_scan with non-zero value
 *
 * Up to 32 PHYs may be discovered by a phy scan. Addresses
 * of all PHYs found will be stored here, but only the first
 * 2 will be used with the Intel IXP400 Software EthAcc ports.
 *
 * See also the function phy_init() in this file.
 *
 * NOTE: The hardcoded PHY addresses have been verified on
 * the IXDP425 and Coyote (IXP4XX RG) Development platforms.
 * However, they may differ on other platforms.
 */
static int phyAddresses[IXP425_ETH_ACC_MII_MAX_ADDR];

/* The default configuration of the phy on each Intel IXP400 Software EthAcc port.
 *
 * This configuration is loaded when the phy's are discovered.
 * More PHYs can optionally be configured here by adding more
 * configuration entries in the array below.  The PHYs will be
 * configured in the order that they are found, starting with the
 * lowest PHY address.
 *
 * See also function phy_init() in this file
 */
static phy_cfg_t default_phy_cfg[] =
{
    {PHY_SPEED_100, PHY_DUPLEX_FULL, PHY_AUTONEG_ON}, /* Port 1 */
    {PHY_SPEED_100, PHY_DUPLEX_FULL, PHY_AUTONEG_ON}  /* Port 2 */
};

/* Mutex lock used to coordinate access to IxEthAcc functions
 * which manipulate the MII registers on the PHYs
 */
static struct semaphore *miiAccessMutex;

/* Flags which is set when corresponding NPE is running,
 * cleared when NPE is stopped
 */
static int npeRunning[IX_ETH_ACC_NUMBER_OF_PORTS];


/*
 * Intel IXP400 Software ERROR NUMBER FUNCTIONS
 */
/* Convert IxEthAcc return codes to suitable Linux return codes */
int convert_error_ethAcc (IxEthAccStatus error)
{
    switch (error)
    {
	case IX_ETH_ACC_SUCCESS:            return  0;
	case IX_ETH_ACC_FAIL:               return -1;
	case IX_ETH_ACC_INVALID_PORT:       return -ENODEV;
	case IX_ETH_ACC_PORT_UNINITIALIZED: return -EPERM;
	case IX_ETH_ACC_MAC_UNINITIALIZED:  return -EPERM;
	case IX_ETH_ACC_INVALID_ARG:        return -EINVAL;
	case IX_ETH_TX_Q_FULL:              return -EAGAIN;
	case IX_ETH_ACC_NO_SUCH_ADDR:       return -EFAULT;
	default:                            return -1;
    };
}


/*
 * DEBUG UTILITY FUNCTIONS
 */
#ifdef DEBUG_DUMP

static void hex_dump(void *buf, int len)
{
    int i;

    for (i = 0 ; i < len; i++)
    {
	printk("%02x", ((u8*)buf)[i]);
	if (i%2)
	    printk(" ");
	if (15 == i%16)
	    printk("\n");
    }
    printk("\n");
}

static void mbuf_dump(char *name, IX_MBUF *mbuf)
{
    printk("+++++++++++++++++++++++++++\n"
	"%s MBUF dump mbuf=%p, m_data=%p, m_len=%d, len=%d\n",
	name, mbuf, mbuf->m_data, mbuf->m_len, mbuf->m_pkthdr.len);
    printk(">> mbuf:\n");
    hex_dump(mbuf, sizeof(*mbuf));
    printk(">> m_data:\n");
    hex_dump(__va(mbuf->m_data), mbuf->m_pkthdr.len);
    printk("\n-------------------------\n");
}

static void skb_dump(char *name, struct sk_buff *skb)
{
    printk("+++++++++++++++++++++++++++\n"
	"%s SKB dump skb=%p, data=%p, len=%d\n",
	name, skb, skb->data, skb->len);
    printk(">> data:\n");
    hex_dump(skb->data, skb->len);
    printk("\n-------------------------\n");
}
#endif


/*
 * BUFFER MANAGEMENT
 */

/* Return mbuf+skb pair to the Intel IXP400 Software.
 * In the case of an error, free skb and return mbuf to the pool
 */
static int rx_buff_replenish(int port_id, IX_MBUF *mbuf)
{
    int res;

    if ((res = ixEthAccPortRxFreeReplenish(port_id, mbuf)))
    {
	P_WARN("ixEthAccPortRxFreeReplenish failed for port %d, res = %d",
		port_id, res);

	dev_kfree_skb_any(mbuf_swap_skb(mbuf, NULL));
	IX_MBUF_POOL_PUT(mbuf);

	return convert_error_ethAcc(res);
    }

    return 0;
}

/* Allocate new skb and replenish the pair.
 * in the case of an error, mbuf will be returned to the pool.
 * Note: in any case, [old] skb will be extracted from the mbuf and
 * returned in skbret(if not NULL)
 */
static int rx_buff_recycle(int port_id, IX_MBUF *mbuf, struct sk_buff **skbret)
{
    struct sk_buff *skb = dev_alloc_skb(PKT_SIZE);

    /* eth header is 14 bytes, and we need ip header to be alligned
     * by 16 bytes
     */
    if (skb)
	skb_reserve(skb, 2);

    /* Our USB driver needs 48 byte reserved to avoid memcpy */
#ifdef CONFIG_USB_RNDIS
    skb_reserve(skb, 48);
#endif

    /* extract skb from mbuf */
    if (skbret)
	*skbret = mbuf_swap_skb(mbuf, skb);
    else
	mbuf_swap_skb(mbuf, skb);

    if (!skb)
    {
	/* failed to alloc skb -> return mbuf to the pool, it'll be
	 * picked up later
	 */
	IX_MBUF_POOL_PUT(mbuf);
	return -ENOMEM;
    }

#ifdef MEMSET_BUFF_CC
    memset(skb->head, 0xCC, skb->end - skb->head);
#endif

    return rx_buff_replenish(port_id, mbuf);
}

/* Allocate an skb for every mbuf in the rx_pool
 * and pass the pair to the Intel IXP400 Software
 */
static int rx_buff_prealloc(struct net_device *dev)
{
    int res;
    priv_data_t *priv = dev->priv;
    IX_MBUF *mbuf;

    while ((IX_MBUF_POOL_GET(priv->rx_pool, &mbuf) == IX_SUCCESS))
    {
	/* we pass NULL as skbret, becouse an mbuf from the pool
	 * does not have an skb attached
	 */
	if ((res = rx_buff_recycle(priv->port_id, mbuf, NULL)))
	    return res;
    }

    return 0;
}


/* 
 * KERNEL THREADS
 */

/* This timer will check the PHY for the link duplex and
 * update the MAC accordingly
 */
struct ixp425_eth_thread
{
        struct task_struct *thread;
        wait_queue_head_t queue;
        struct completion thr_exited;
        int time_to_die;
} ;

static struct ixp425_eth_thread eth_thread;

static int
media_check_thread (void* arg)
{
        int linkUp, speed100, fullDuplex, autonegotiate;
        int res, timeout, i = 0;

        daemonize ();
        reparent_to_init();

        init_completion (&eth_thread.thr_exited);
        spin_lock_irq(&current->sigmask_lock);
        sigemptyset(&current->blocked);
        recalc_sigpending(current);
        init_waitqueue_head(&eth_thread.queue);
        spin_unlock_irq(&current->sigmask_lock);

        eth_thread.thread = current ;
        strcpy (current->comm, "ixp425_eth");
        while (1) {
                do {
			timeout = interruptible_sleep_on_timeout(&eth_thread.queue, 3*HZ);
                } while (!signal_pending (current) && (timeout > 0));

                if (signal_pending (current)) {
                        spin_lock_irq(&current->sigmask_lock);
                        flush_signals(current);
                        spin_unlock_irq(&current->sigmask_lock);
                }

                if (eth_thread.time_to_die)
                        break;

		down(miiAccessMutex);
                res = ixEthAccMiiLinkStatus(phyAddresses[0],
                                            &linkUp,
                                            &speed100,
                                            &fullDuplex,
                                            &autonegotiate);
		up(miiAccessMutex);

                if (res != IX_ETH_ACC_SUCCESS)
                        break;

                if (linkUp == TRUE) {
                        if (fullDuplex == TRUE)
				ixEthAccPortDuplexModeSet(IX_ETH_PORT_2, IX_ETH_ACC_FULL_DUPLEX);
                        else
				ixEthAccPortDuplexModeSet(IX_ETH_PORT_2, IX_ETH_ACC_HALF_DUPLEX);
                }
        }
        complete_and_exit(&eth_thread.thr_exited, 0);
	return 0;
}


/*
 * TIMERS
 */

/* This timer will call ixEthDBDatabaseMaintenance every
 * IX_ETH_DB_MAINTENANCE_TIME seconds
 */
static void maintenance_timer_cb(unsigned long data);

static struct timer_list maintenance_timer = {
    function:&maintenance_timer_cb
};

static void maintenance_timer_set(void)
{
    maintenance_timer.expires = jiffies + IX_ETH_DB_MAINTENANCE_TIME * HZ;
    add_timer(&maintenance_timer);
}

static void maintenance_timer_clear(void)
{
    del_timer_sync(&maintenance_timer);
}

static void maintenance_timer_cb(unsigned long data)
{
    ixEthDBDatabaseMaintenance();

    maintenance_timer_set();
}

/* This callback is called when transmission of the packed is done, and
 * IxEthAcc does not need the buffer anymore.
 */
static void tx_done_cb(u32 callbackTag, IX_MBUF *mbuf)
{
    struct net_device *dev = (struct net_device *)callbackTag;
    priv_data_t *priv = dev->priv;

    TRACE;

    priv->stats.tx_packets++; /* total packets transmitted */
    priv->stats.tx_bytes += mbuf->m_len; /* total bytes transmitted */

    /* extract skb from the mbuf, free skb and return the mbuf to the pool */
    dev_kfree_skb_any(mbuf_swap_skb(mbuf, NULL));
    IX_MBUF_POOL_PUT(mbuf);

    /* Start the queue in case it was stopped in dev_hard_start_xmit
     * (when queue-full detected)
     */
    if (priv->running && netif_queue_stopped(dev))
    {
	TRACE;
	netif_wake_queue(dev);
    }
}

/* This callback is called when new packet received from MAC
 * and ready to be transfered up-stack.
 *
 * If this is a valid packet, then new skb is allocated and switched
 * with the one in mbuf, which is pushed upstack.
 *
 * In the case of en empty/error packet, the mbuf just pushed
 * back to the Intel IXP400 Software [unless device is down, in which case it's returned to
 * the pool].
 */
static void rx_cb(u32 callbackTag, IX_MBUF *mbuf, IxEthAccPortId portId)
{
    struct net_device *dev = (struct net_device *)callbackTag;
    priv_data_t *priv = dev->priv;
    struct sk_buff *skb;
    int len;

    if (mbuf->m_len)
    {
	TRACE;

	/* valid packet */

	len = mbuf->m_len; /* preserve the mbuf len */

	/* allocate new skb and "swap" it with the skb that is tied to the
	 * mbuf. then return the mbuf + new skb to the Intel IXP400 Software.
	 */
	rx_buff_recycle(priv->port_id, mbuf, &skb);

	skb_put(skb, len);
	skb->dev = dev;
	skb->protocol = eth_type_trans(skb, dev);

	/* push upstack */
	TRACE;

	priv->stats.rx_packets++; /* total packets received */
	priv->stats.rx_bytes += len; /* total bytes received */

	netif_rx(skb);
    }
    else if (!priv->running)
    {
	TRACE;

	/* this is a "junk" buffer, returned by the
	 * call to PortDisable
	 */

	/* free the skb & return the mbuf to the pool */
	dev_kfree_skb_any(mbuf_swap_skb(mbuf, NULL));
	IX_MBUF_POOL_PUT(mbuf);

	return;
    }
    else
    {
	TRACE;
	/* Should not get here!
	 * error buf!!! -> push the mbuf back to the Intel IXP400 Software */
	rx_buff_replenish(priv->port_id, mbuf);
    }

    TRACE;
}

/* Set promiscuous/multicast mode for the MAC */
static void dev_set_multicast_list(struct net_device *dev)
{
    int res;
    priv_data_t *priv = dev->priv;
    IxEthAccMacAddr addr1 = {};

/* 4 possible scenarios here
 *
 * scenarios:
 * #1 - promiscuous mode ON
 * #2 - promiscuous mode OFF, accept NO multicast addresses
 * #3 - promiscuous mode OFF, accept ALL multicast addresses
 * #4 - promiscuous mode OFF, accept LIST of multicast addresses 
 */

    TRACE;

    /* if called from irq handler, lock already acquired */
    if (!in_irq())
	spin_lock_irq(&priv->lock);

    /* clear multicast addresses that were set the last time (if exist) */
    ixEthAccPortMulticastAddressLeaveAll (priv->port_id);

/**** SCENARIO #1 ****/
    /* Set promiscuous mode */
    if (dev->flags & IFF_PROMISC)
    {
	if ((res = ixEthAccPortPromiscuousModeSet(priv->port_id)))
	{
	    P_ERROR("%s: ixEthAccPortPromiscuousModeSet failed on port %d\n",
		    dev->name, priv->port_id);
	}
	else
	{
	    P_NOTICE("%s: Entering promiscuous mode\n", dev->name);
	}

	goto Exit;
    }


/**** SCENARIO #2 ****/

    /* Clear promiscuous mode */
    if ((res = ixEthAccPortPromiscuousModeClear(priv->port_id)))
    {
	/* should not get here */
	P_ERROR("%s: ixEthAccPortPromiscuousModeClear failed for port %d\n",
		dev->name, priv->port_id);
    }
    else
    {
	P_NOTICE("%s: Leaving promiscuous mode\n", dev->name);
    }


/**** SCENARIO #3 ****/
    /* If there's more addresses than we can handle, get all multicast
     * packets and sort the out in software
     */
    /* Set multicast mode */
    if ((dev->flags & IFF_ALLMULTI) || 
	(dev->mc_count > IX_ETH_ACC_MAX_MULTICAST_ADDRESSES))
    {
	/* Get ALL multicasts:
	 * This will result in
         *     MASK == 01:00:00:00:00:00
         *     ADDR == 01:00:00:00:00:00
	 * Meaning ALL MULTICAST addresses will be accepted
	 */
        ixEthAccPortMulticastAddressJoinAll(priv->port_id);

	P_VERBOSE("%s: Accepting ALL multicast packets\n", dev->name);
	goto Exit;
    }


/**** SCENARIO #4 ****/
    /* Store all of the multicast addresses in the hardware filter */
    if ((dev->mc_count))
    {
	/* now join the current address list */
	/* Get only multicasts from the list */
	struct dev_mc_list *mc_ptr;

	for (mc_ptr = dev->mc_list; mc_ptr; mc_ptr = mc_ptr->next)
	{
	    memcpy (&addr1.macAddress[0], &mc_ptr->dmi_addr[0],
		    IX_IEEE803_MAC_ADDRESS_SIZE);
	    ixEthAccPortMulticastAddressJoin (priv->port_id, &addr1);
	}
    }


Exit:
    if (!in_irq())
	spin_unlock_irq(&priv->lock);
}


static int npe_start(IxEthAccPortId port_id)
{
    int res;
    UINT32 npeImageId;

    switch (port_id)
    {
        case IX_ETH_PORT_1:
	    npeImageId = IX_ETH_NPE_B_IMAGE_ID;
	    break;
        case IX_ETH_PORT_2:
	    npeImageId = IX_ETH_NPE_C_IMAGE_ID;
	    break;
	default:
	    P_ERROR("Invalid port specified. IXP Ethernet NPE not started\n");
	    return -ENODEV;
    }

    /* Initialise and Start NPEs */
    if ((res = ixNpeDlNpeInitAndStart(npeImageId)))
    {
	P_ERROR("Error starting NPE for Ethernet port %d!\n", port_id);
	return -1;
    }

    /* set this flag to indicate that NPE is running */
    npeRunning[port_id] = 1;

    return 0;
}

/* Enable the MAC port.
 * Called on do_dev_open and dev_tx_timeout
 */
static int port_enable(struct net_device *dev)
{
    int res;
    priv_data_t *priv = dev->priv;

    P_DEBUG("port_enable(%s)\n", dev->name);

    /* we need to restart the NPE here if it has been
     * stopped by port_disable()
     */
    if (!npeRunning[priv->port_id])
    {
        IxEthAccMacAddr npeMacAddr;

        if ((res = npe_start(priv->port_id)))
        {
            TRACE;
            return res;
        }

        /* when we restart the NPEs we must also set the unicast MAC address
         * again, before ixEthAccPortEnable is called
         */
        memcpy(&npeMacAddr.macAddress, &dev->dev_addr, IX_IEEE803_MAC_ADDRESS_SIZE);
        if ((res = ixEthAccPortUnicastMacAddressSet(priv->port_id, &npeMacAddr)))
        {
            P_ERROR("Failed to set MAC address for port %d, res = %d\n",
                    priv->port_id, res);
            return convert_error_ethAcc(res);
        }
    }

    if ((res = ixEthAccPortTxFrameAppendFCSEnable(priv->port_id)))
    {
        TRACE;
        return convert_error_ethAcc(res);
    }

    if ((res = ixEthAccPortRxFrameAppendFCSDisable(priv->port_id)))
    {
	TRACE;
	return convert_error_ethAcc(res);
    }

    if((res = ixEthAccTxSchedulingDisciplineSet(priv->port_id,
						FIFO_NO_PRIORITY)))
    {
        TRACE;
	return convert_error_ethAcc(res);
    }

    if ((res = ixEthAccPortEnable(priv->port_id)))
    {
	P_ERROR("%s: ixEthAccPortEnable failed for port %d, res = %d\n",
		dev->name, priv->port_id, res);
	return convert_error_ethAcc(res);
    }

    if ((res = ixEthDBPortAgingEnable(priv->port_id)))
    {
        P_ERROR("%s: ixEthDBPortAgingEnable failed for port %d, res = %d\n",
                dev->name, priv->port_id, res);
        return -1;
    }

    TRACE;

    priv->running = 1;

    netif_start_queue(dev);

    TRACE;

    return 0;
}

/* Disable the MAC port.
 * Called on do_dev_stop and dev_tx_timeout
 */
static void port_disable(struct net_device *dev)
{
    priv_data_t *priv = dev->priv;
    int res;

    TRACE;

    netif_stop_queue(dev);

    priv->running = 0;

    if ((res = ixEthAccPortDisable(priv->port_id)))
    {
	/* should not get here */
	P_ERROR("%s: BUG: ixEthAccPortDisable(%d) failed with res=%d\n",
		dev->name, priv->port_id, res);
    }

    /* ixEthAccPortDisable will stop the NPE */
    /* Clear this flag to show that NPE is stopped */
    npeRunning[priv->port_id] = 0;

    TRACE;
}

/* Open the device.
 * Request resources and start interrupts
 */
static int do_dev_open(struct net_device *dev)
{
    int res;
    priv_data_t *priv = dev->priv;

/* 2003/11/7 Steve ++ */
        if (priv->running)
                return 0;
/* Steve -- */

    /* Preallocate RX pool */
    P_DEBUG("%s: Preallocating buffers\n", dev->name);
    if ((res = rx_buff_prealloc(dev)))
	return res;

    /* Set callbacks */
    if ((res = ixEthAccPortTxDoneCallbackRegister(priv->port_id, tx_done_cb,
	(u32)dev)))
    {
	P_ERROR("%s: ixEthAccPortTxDoneCallbackRegister for port %d failed, "
		"res = %d\n", dev->name, priv->port_id, res);
	return convert_error_ethAcc(res);
    }

    if ((res = ixEthAccPortRxCallbackRegister(priv->port_id, rx_cb,
	(u32)dev)))
    {
	P_ERROR("%s: ixEthAccPortRxCallbackRegister for port %d failed, "
		"res = %d\n", dev->name, priv->port_id, res);
	return convert_error_ethAcc(res);
    }

    if ((res = port_enable(dev)))
    {
        return res;
    }
    
    return res;
}

/* Close the device.
 * Free resources acquired in dev_start
 */
static int do_dev_stop(struct net_device *dev)
{
/* 2003/11/7 Steve ++ */
#if 0
	priv_data_t *priv = dev->priv;
	if (priv->port_id == IX_ETH_PORT_1) {
	    TRACE;
	    netif_stop_queue(dev);

	    port_disable(dev);
	}
#endif
/* Steve -- */
    return 0;
}

/* this is called by kernel to transmit packet */
static int dev_hard_start_xmit(struct sk_buff *skb, struct net_device *dev)
{
    int res;
    priv_data_t *priv = dev->priv;
    IX_MBUF *mbuf;

    TRACE;

    /* get mbuf struct from tx_pool */
    IX_MBUF_POOL_GET(priv->tx_pool, &mbuf);
    if (!mbuf)
	BUG(); /* Shouldn't happen */
    
    /* if pool has no more buffers */
    if (IX_MBUF_POOL_FREE_COUNT(priv->tx_pool) == 0)
    {
	TRACE;
	netif_stop_queue(dev);
    }

    mbuf_swap_skb(mbuf, skb); /* this should return NULL, as mbufs in the pool
    			       * have no skb attached
			       */

    /* flush the mbuf data from the cache */
    IX_ACC_DATA_CACHE_FLUSH(IX_MBUF_MDATA(mbuf), IX_MBUF_MLEN(mbuf));

    if ((res = ixEthAccPortTxFrameSubmit(priv->port_id, mbuf,
	IX_ETH_ACC_TX_DEFAULT_PRIORITY)))
    {
	P_ERROR("%s: ixEthAccPortTxFrameSubmit failed for port %d, res = %d\n",
		dev->name, priv->port_id, res);
	netif_stop_queue(dev);
	return convert_error_ethAcc(res);
    }

    return 0;
}

/* This function is called when kernel thinks that TX is stuck */
static void dev_tx_timeout(struct net_device *dev)
{
    TRACE;

    /* just restart the port */
    port_disable(dev);
    port_enable(dev);
}

static int dev_do_ioctl(struct net_device *dev, struct ifreq *req, int cmd)
{
    priv_data_t *priv = dev->priv;
    struct mii_ioctl_data *data = (struct mii_ioctl_data *) & req->ifr_data;
    int phy;
    int res = 0;

    TRACE;
/* 2003/11/11 Steve ++ */
	if (priv->port_id == IX_ETH_PORT_2) {
		phy = phyAddresses[0];
		data->phy_id = phy;
	} else
		goto nodev;
/* Steve -- */

    switch (cmd)
    {
	/*
	 * IOCTL's for mii-tool support
	 */

	/* Get address of MII PHY in use */
	case SIOCGMIIPHY:
	case SIOCDEVPRIVATE:
	    data->phy_id = phy;

        /* Read MII PHY register */
	case SIOCGMIIREG:		
	case SIOCDEVPRIVATE+1:
	    down (miiAccessMutex);     /* lock the MII register access mutex */
	    if ((res = ixEthAccMiiReadRtn (data->phy_id, data->reg_num, &data->val_out)))
	    {
		P_ERROR("Error reading MII reg %d on phy %d\n",
		       data->phy_id, data->reg_num);
		res = -1;
	    }
	    up (miiAccessMutex);	/* release the MII register access mutex */
	    return res;

	/* Write MII PHY register */
	case SIOCSMIIREG:
	case SIOCDEVPRIVATE+2:
	    down (miiAccessMutex);     /* lock the MII register access mutex */
	    if ((res = ixEthAccMiiWriteRtn (data->phy_id, data->reg_num, data->val_in)))
	    {
		P_ERROR("Error reading MII reg %d on phy %d\n",
			data->phy_id, data->reg_num);
		res = -1;
	    }
	    up (miiAccessMutex);	/* release the MII register access mutex */
	    return res;

	default:
	    return -EOPNOTSUPP;
    }

nodev:
    return -ENODEV;
}

static struct net_device_stats *dev_get_stats(struct net_device *dev)
{
    int res;
    /* "stats" should be cache-safe.
     * we alligne "stats" and "priv" by 32 bytes, so that the cache
     * operations will not affect "res" and "priv"
     */
    IxEthEthObjStats stats __attribute__ ((aligned(32))) = {};
    priv_data_t *priv __attribute__ ((aligned(32)));

    TRACE;

    priv = dev->priv;

    /* Get HW stats and translate to the net_device_stats */
    if (!netif_running(dev))
    {
	TRACE;
	return &priv->stats;
    }

    TRACE;
#if 0
    clean_dcache_range((u32)&stats, (u32)((&stats) + 1) - 1);
    if ((res = ixEthAccMibIIStatsGetClear(priv->port_id, &stats)))
    {
	P_ERROR("%s: ixEthAccMibIIStatsGet failed for port %d, res = %d\n",
		dev->name, priv->port_id, res);
	return &priv->stats;
    }

    TRACE;

    invalidate_dcache_range((u32)&stats, (u32)((&stats) + 1) - 1);

    /* bad packets received */
    priv->stats.rx_errors += 
	stats.dot3StatsAlignmentErrors +
	stats.dot3StatsFCSErrors +
	stats.dot3StatsFrameTooLongs +
	stats.dot3StatsInternalMacReceiveErrors;

    /* packet transmit problems */
    priv->stats.tx_errors += 
	stats.dot3StatsLateCollisions +
	stats.dot3StatsExcessiveCollsions +
	stats.dot3StatsInternalMacTransmitErrors +
	stats.dot3StatsCarrierSenseErrors;

    priv->stats.collisions +=
	stats.dot3StatsSingleCollisionFrames +
	stats.dot3StatsMultipleCollisionFrames;

    /* recved pkt with crc error */
    priv->stats.rx_crc_errors +=
	stats.dot3StatsFCSErrors;

    /* recv'd frame alignment error */
    priv->stats.rx_frame_errors +=
	stats.dot3StatsFrameTooLongs;

    /* detailed tx_errors */
    priv->stats.tx_carrier_errors +=
	stats.dot3StatsCarrierSenseErrors;
#endif
    return &priv->stats;
}

/* Initialize QMgr and bint it's interrupts */
static int qmgr_init(void)
{
    int res;

    /* Initialise Queue Manager */
    P_VERBOSE("Initialising Queue Manager...\n");
    if ((res = ixQMgrInit()))
    {
	P_ERROR("Error initialising queue manager!\n");
	return -1;
    }

    /*
     * Hook the QM QLOW dispatcher to the interrupt controller.
     * The ethernet NPEs use queues 24 through 27.
     */
    if ((res = ixOsServIntBind(IXP425_INT_LVL_QM1,
	(void(*)(void*))ixQMgrDispatcherLoopRun, (void *)IX_QMGR_QUELOW_GROUP)))
    {
	P_ERROR("Failed to bind to QM1 interrupt!\n");
	return -1;
    }

    return 0;
}

static int ethacc_init(void)
{
    int res;

    if ((res = ixEthAccInit()))
    {
	P_ERROR("ixEthAccInit failed with res=%d\n", res);
	return convert_error_ethAcc(res);
    }

    if ((res = ixEthAccPortInit(IX_ETH_PORT_1)))
    {
	P_ERROR("Failed to initialize Eth port 1 res = %d\n", res);
	return convert_error_ethAcc(res);
    }

    if ((res = ixEthAccPortInit(IX_ETH_PORT_2)))
    {
	P_ERROR("Failed to initialize Eth port 2 res = %d\n", res);
	return convert_error_ethAcc(res);
    }

    return 0;
}

#ifndef CONFIG_IXP425_CSR_ETH_SKIP_PHY_SCAN

static int phy_init(void)
{
    int res;
    int physcan[IXP425_ETH_ACC_MII_MAX_ADDR];
    int i, phy_found, num_phys_to_set;

    /* Use hardcoded phy addresses */
    if (no_phy_scan) /* Module parameter */
    {
	num_phys_to_set = (sizeof(default_phy_cfg) / sizeof(phy_cfg_t));
    }
    /* Do a PHY scan */
    else
    {
	if ((res = ixEthAccMiiPhyScan(physcan)))
	{
	    P_ERROR("PHY scan failed\n");
	    return convert_error_ethAcc(res);
	}
	
	for (i=0, phy_found=0; i < 2 /*IXP425_ETH_ACC_MII_MAX_ADDR*/; i++)
	{
	    if (physcan[i])
	    {
		P_INFO("Found PHY %d at %d\n", phy_found, i);
		phyAddresses[phy_found] = i;
		
		if (++phy_found == IXP425_ETH_ACC_MII_MAX_ADDR)
		    break;
	    }
	}

	num_phys_to_set = phy_found;
    }

    /* Reset and Set each phy properties */
    for (i=0; i < num_phys_to_set; i++) 
    {
	P_VERBOSE("Configuring PHY %d\n", i);
	P_VERBOSE("\tSpeed %s\tDuplex %s\tAutonegotiation %s\n",
		  (default_phy_cfg[i].speed100) ? "100" : "10",   
		  (default_phy_cfg[i].duplexFull) ? "FULL" : "HALF",  
		  (default_phy_cfg[i].autoNegEnabled) ? "ON" : "OFF");

	ixEthAccMiiPhyReset(phyAddresses[i]);

	ixEthAccMiiPhyConfig(phyAddresses[i],
	    default_phy_cfg[i].speed100,   
	    default_phy_cfg[i].duplexFull,  
	    default_phy_cfg[i].autoNegEnabled);
    }

    for (i=0; i < IX_ETH_ACC_NUMBER_OF_PORTS; i++) 
    {
	P_INFO("%s%d is using the PHY at address %d\n",
	       DEVICE_NAME, i, phyAddresses[i]);

	/* Set the MAC to the same duplex mode as the phy */
	ixEthAccPortDuplexModeSet(i,
            (default_phy_cfg[i].duplexFull) ?
                 IX_ETH_ACC_FULL_DUPLEX : IX_ETH_ACC_HALF_DUPLEX);
    }

    return 0;
}
#endif

/* zhangzh 2003/07/25 */
typedef struct tagBootParm {
        char version[16];
        unsigned char my_mac[6];
        unsigned char my_mac2[6];
        unsigned int my_ip;
        unsigned int server_ip;
        unsigned int timeout;
} tBootParm;

#define CONFIG_POSITION_IN_ROM (0x50040000-0x100)
#define     EBOOT_VERSION       "0.5.0"
#define     EBOOT_VERSION_LEN   5

/* zhangzh */


/* This function set's default MAC addresses for the ports.
 * They will be used until dev->set_mac_address hook is called
 */
static int set_def_mac_addr(IxEthAccPortId port)
{
    int res;
    IxEthAccMacAddr npeMacAddr;

#if 1
    /* zhangzh 2003/07/25 */
    unsigned long map;
    tBootParm BootParm;

    map = (unsigned long)ioremap(CONFIG_POSITION_IN_ROM, sizeof(tBootParm));
    memcpy((void *)&BootParm, (void *)map, sizeof(tBootParm));
    iounmap((void *)map);

    if(!strncmp(BootParm.version, EBOOT_VERSION, EBOOT_VERSION_LEN)) {
        if(port) {
                memcpy(npeMacAddr.macAddress, BootParm.my_mac2, 6);
        } else {
                memcpy(npeMacAddr.macAddress, BootParm.my_mac, 6);
        }
    } else {
    /* zhangzh */
    npeMacAddr.macAddress[0] =
    npeMacAddr.macAddress[1] =
    npeMacAddr.macAddress[2] =
    npeMacAddr.macAddress[3] =
    npeMacAddr.macAddress[4] =
    npeMacAddr.macAddress[5] = 0x04 + port * 2;

    /* zhangzh 2003/07/25 */
    }
    /* zhangzh */

    if ((res = ixEthAccPortUnicastMacAddressSet(port, &npeMacAddr)))
    {
        printk("Failed to set MAC address for port %d, res = %d\n", port, res);
        return res;
    }

    printk("PHY %d MAC address is:\t", port);
    ixEthAccPortUnicastAddressShow(port);

    return 0;

#else
    if ((res = ixEthAccPortUnicastMacAddressSet(port, &default_mac_addr[port])))
    {
	P_ERROR("Failed to set MAC address for port %d, res = %d\n", port, res);
	return convert_error_ethAcc(res);
    }

    P_INFO("PHY %d MAC address is:\t", port);
    ixEthAccPortUnicastAddressShow(port);

    return 0;
#endif
}

/* set port MAC addr and update the dev struct if successfull */
int dev_set_mac_address(struct net_device *dev, void *addr)
{
    int res;
    priv_data_t *priv = dev->priv;
    IxEthAccMacAddr npeMacAddr;

    /* Set MAC addr in h/w */
    memcpy(&npeMacAddr.macAddress, &addr, IX_IEEE803_MAC_ADDRESS_SIZE);
    if ((res = ixEthAccPortUnicastMacAddressSet(priv->port_id, &npeMacAddr)))
    {
	P_ERROR("Failed to set MAC address for port %d, res = %d\n",
		priv->port_id, res);
	return convert_error_ethAcc(res);
    }

    /* update dev struct */
    memcpy(dev->dev_addr, &addr, IX_IEEE803_MAC_ADDRESS_SIZE);

    return 0;
}

/* Initialize device structs.
 * Resource allocation is deffered until do_dev_open
 */
static int __devinit ixp425_eth_probe(struct net_device *dev)
{
    int res;
    static int found_devices = 0;
    priv_data_t *priv;
    IxEthAccMacAddr mac_addr;

    TRACE;

    /* there are only 2 available ports */
    if (found_devices >= IX_ETH_ACC_NUMBER_OF_PORTS)
	return -ENODEV;

    SET_MODULE_OWNER(dev);

    /* set device name */
/* Jack ++ */
    strcpy(dev->name, found_devices ? DEVICE_NAME "1" : DEVICE_NAME "0"); 

    /* fill in dev struct callbacks */
    dev->open = do_dev_open;
    dev->stop = do_dev_stop;

    dev->hard_start_xmit = dev_hard_start_xmit;

    dev->watchdog_timeo = DEV_WATCHDOG_TIMEO;
    dev->tx_timeout = dev_tx_timeout;

    dev->do_ioctl = dev_do_ioctl;
    dev->get_stats = dev_get_stats;
    dev->set_multicast_list = dev_set_multicast_list;
    dev->flags |= IFF_MULTICAST;

    dev->set_mac_address = dev_set_mac_address;

    /* allocate and initialize priv struct */
    priv = dev->priv = kmalloc(sizeof(priv_data_t), GFP_KERNEL);
    if (dev->priv == NULL)
	return -ENOMEM;

    memset(dev->priv, 0, sizeof(priv_data_t));

    /* initialize RX pool */
    IX_MBUF_POOL_INIT_NO_ALLOC(&priv->rx_pool, priv->rx_pool_buf, NULL,
	RX_QUEUE_PREALLOC, 0, "IXP425 Ethernet driver Rx Pool");

    /* initialize TX pool */
    IX_MBUF_POOL_INIT_NO_ALLOC(&priv->tx_pool, priv->tx_pool_buf, NULL,
	TX_QUEUE_MAX_LEN, 0, "IXP425 Ethernet driver Tx Pool");

    priv->lock = SPIN_LOCK_UNLOCKED;
    priv->port_id = found_devices ? IX_ETH_PORT_2 : IX_ETH_PORT_1;

    /* initialize ethernet device */
    ether_setup(dev);

    /* mac address */
    if ((res = ixEthAccPortUnicastMacAddressGet(priv->port_id, &mac_addr)))
	return convert_error_ethAcc(res);
    memcpy(dev->dev_addr, &mac_addr, IX_IEEE803_MAC_ADDRESS_SIZE);

    found_devices++;

    return 0;
}

/* 2004/3/9 Steve ++ */
static int
get_phy_status(char *buffer,
	       char **start,
	       off_t offset,
	       int length,
	       int *eof,
	       void *data)
{
	int linkUp, speed100, fullDuplex, autonegotiate, res;
	int len = 0;
	off_t pos = 0;
	off_t begin = 0;
	int *address = (int *)(data);

	down(miiAccessMutex);
	res = ixEthAccMiiLinkStatus(*address,
				    &linkUp,
				    &speed100,
				    &fullDuplex,
				    &autonegotiate);
	up(miiAccessMutex);
	if (res != IX_ETH_ACC_SUCCESS)
		return len;

	len += sprintf(buffer+len, "Link is %s\n"
				   "Autoneg is %s\n"
				   "Speed is %sMb/s\n"
				   "Duplex is %s\n", 
		       (linkUp ? "Up" : "Down"),
		       (autonegotiate ? "On" : "Off"),
		       (speed100 ? "100" : "10"),
		       (fullDuplex ? "Full" : "Half"));

	pos = begin + len;
	if(pos < offset) { len = 0; begin = pos; }
	if(pos > offset + length) { len = 0; }

	*start = buffer + (offset - begin);
	len -= (offset - begin);
	if(len > length) len = length;
	return len;
}

static int
set_phy_status(struct file *file,
	       const char *input,
	       unsigned long size,
	       void *data)
{
	int *address = (int *)(data);
	int speed100 = 0, fullDuplex = 0, autonegotiate = 0, len;
	unsigned int mark;
	char buffer[1024], *end;

	if (size > (sizeof(buffer) - 1))
		return size;
	len = size;
        if (copy_from_user(buffer, input, len))
                return -EFAULT;
	buffer[len - 1] = '\0';
	mark = simple_strtoul(buffer, &end, 10);
	if (mark & BMCR_ANENABLE) {
		autonegotiate = 1;		
	} else {
		if (mark & BMCR_SPEED100)
			speed100 = 1;

		if (mark & BMCR_FULLDPLX)
			fullDuplex = 1;
	}

	down(miiAccessMutex);
	ixEthAccMiiPhyReset(*address);
	ixEthAccMiiPhyConfig(*address,
			     speed100,
			     fullDuplex,
			     autonegotiate);
	up(miiAccessMutex);
	return size;
}
/* Steve -- */

/* 2005/7/27 Steve ++ */
static int
get_eth_status(char *buffer,
	       char **start,
	       off_t offset,
	       int length,
	       int *eof,
	       void *data)
{
	ixEthELTShow(0);
	ixEthELTShow(1);
	return 0;
}

static int
set_eth_status(struct file *file,
	       const char *input,
	       unsigned long size,
	       void *data)
{
	char buffer[1024], *end;
	int len, id = 0;

	if (size > (sizeof(buffer) - 1))
		return size;
	len = size;
        if (copy_from_user(buffer, input, len))
                return -EFAULT;
	buffer[len - 1] = '\0';
	id = simple_strtoul(buffer, &end, 10);
	if (id < IX_ETH_ACC_NUMBER_OF_PORTS && id >= 0) {
		rx_buff_prealloc(&ixp425_devices[id]);
#if 0
	        netif_stop_queue(&ixp425_devices[id]);
		port_disable(&ixp425_devices[id]);
		do_dev_open(&ixp425_devices[id]);
#endif
	}
	return size;
}

#if 0
static void
rx_timer_action(unsigned long timeout)
{
	int i = 0;
	static unsigned long rx_packet[IX_ETH_ACC_NUMBER_OF_PORTS];
#if 0
	for (i = 0; i < IX_ETH_ACC_NUMBER_OF_PORTS; i++) {
		struct net_device *dev = &ixp425_devices[i];
		priv_data_t *priv = dev->priv;

		if (rx_packet[i] != priv->stats.rx_packets)
			rx_packet[i] = priv->stats.rx_packets;
		else {
			printk(KERN_EMERG "restart %s\n", dev->name);
#if 0
			spin_lock(&dev->xmit_lock);
			printk(KERN_EMERG "port_disable");
			port_disable(dev);
			printk(KERN_EMERG "port_enable");
			port_enable(dev);
			spin_unlock(&dev->xmit_lock);
#endif
		}
	}

	eth_rx_timer.expires = (30 * HZ) + jiffies;
	add_timer(&eth_rx_timer);
#endif
}
#endif
/* Steve -- */

/* Module initialization and cleanup */

/* #ifdef MODULE */

int ixp425_init_module(void)
{
    int res, i;
    struct net_device *dev;
/* 2004/3/9 Steve ++ */
	struct proc_dir_entry *proc_phy;
/* Steve -- */
/* 2005/7/27 Steve ++ */
	struct proc_dir_entry *proc_eth;
/* Steve -- */

    TRACE;

printk("init ixp425_init_module\n");

    P_INFO("Initializing " MODULE_NAME " driver " MODULE_VERSION "\n");

    TRACE;

    /* Do not initialise Intel IXP400 Software core components if no_csr_init is set */
    if (no_csr_init) /* module parameter */
    {
	P_WARN("init_csr==0, no initialisation of Intel IXP400 Software "
	       "core components performed\n");
    }
    else
    {
	if ((res = qmgr_init()))
	    return res;

	if ((res = ixNpeMhInitialize(IX_NPEMH_NPEINTERRUPTS_YES)))
	    return -1;
    }

    TRACE;

    for (i = 0; i < IX_ETH_ACC_NUMBER_OF_PORTS; i++)
    {
	if ((res = npe_start(i)))
	    return res;
    }

    if ((res = ethacc_init()))
	return res;

#ifndef CONFIG_IXP425_CSR_ETH_SKIP_PHY_SCAN
    if ((res = phy_init()))
	return res;

/* 2003/11/7 Steve ++ */
        res = kernel_thread(media_check_thread, 0, CLONE_FS | CLONE_FILES);
        if (res < 0)
                printk("unable create kernel thread\n");
/* Steve -- */
#endif

    if ((res = set_def_mac_addr(IX_ETH_PORT_1)))
	return res;

    if ((res = set_def_mac_addr(IX_ETH_PORT_2)))
	return res;

    /* Set scheduling discipline */
    ixEthAccTxSchedulingDisciplineSet(IX_ETH_PORT_1, FIFO_NO_PRIORITY);
    ixEthAccTxSchedulingDisciplineSet(IX_ETH_PORT_2, FIFO_NO_PRIORITY);

    for (i = 0; i < IX_ETH_ACC_NUMBER_OF_PORTS; i++)
    {
	dev = &ixp425_devices[i];

	dev->init = ixp425_eth_probe;

	if ((res = register_netdev(dev)))
	    P_ERROR("Failed to register netdev. res = %d\n", res);
    }

    maintenance_timer_set();

    /* initialise the MII register access mutex */
    miiAccessMutex = (struct semaphore *) kmalloc(sizeof(struct semaphore), GFP_KERNEL);
    if (!miiAccessMutex)
	return -ENOMEM;
    init_MUTEX(miiAccessMutex);

/* 2004/3/9 Steve ++ */
	proc_phy = create_proc_entry("driver/PHY", 600, 0);
	if (proc_phy) {
		proc_phy->read_proc = get_phy_status;
		proc_phy->write_proc = set_phy_status;
		proc_phy->data = &phyAddresses[0];
	}
/* Steve -- */
/* 2005/7/27 Steve ++ */
	proc_eth = create_proc_entry("driver/ETH", 600, 0);
	if (proc_phy) {
		proc_eth->read_proc = get_eth_status;
		proc_eth->write_proc = set_eth_status;
	}
/* Steve -- */

/* 2005/7/27 Steve ++ */
#if 0
	init_timer(&eth_rx_timer);
	eth_rx_timer.function = rx_timer_action;
	eth_rx_timer.expires = (60 * HZ) + jiffies;
	add_timer(&eth_rx_timer);
#endif
/* Steve -- */

/* Jack ++ */
/*        ixCryptoAccJackInit();  */
/*	ixCryptoAccCodeletCryptoMemPoolInit (64); */

    return 0;
}

void ixp425_cleanup_module(void)
{
    int i;

    TRACE;

/* 2004/3/9 Steve ++ */
	remove_proc_entry("driver/PHY", NULL);
/* Steve -- */

    /* We can only get here when the module use count is 0,
     * so there's no need to stop devices.
     */

    maintenance_timer_clear();

    for (i = 0; i < IX_ETH_ACC_NUMBER_OF_PORTS; i++)
    {
	struct net_device *dev = &ixp425_devices[i];
	if (dev->priv != NULL)
	{
	    unregister_netdev(dev);
	    kfree(dev->priv);
	    dev->priv = NULL;
	}
    }
}

/* #endif */

module_init(ixp425_init_module);
module_exit(ixp425_cleanup_module);


