/*
 *  ebtables
 *
 *	Authors:
 *	Bart De Schuymer		<bart.de.schuymer@pandora.be>
 *
 *	ebtables.h,v 1.0prerelease 2001/08/01
 *
 *  This code is stongly inspired on the iptables code which is
 *  Copyright (C) 1999 Paul `Rusty' Russell & Michael J. Neuling
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 */

#ifndef __LINUX_BRIDGE_EFF_H
#define __LINUX_BRIDGE_EFF_H
#include <linux/if_bridge.h>
#include <linux/if.h> /* IFNAMSIZ */
#include <linux/netfilter_bridge.h>
#include <linux/if_ether.h> /* ETH_ALEN */
/* [gs]etsockopt numbers */
#define EPF_BASE_CTL            128

#define EPF_SO_SET_ENTRIES      (EPF_BASE_CTL)
#define EPF_SO_SET_COUNTERS     (EPF_SO_SET_ENTRIES+1)
#define EPF_SO_SET_ALLOWC       (EPF_SO_SET_COUNTERS+1)
#define EPF_SO_SET_MAX          (EPF_SO_SET_ALLOWC+1)

#define EPF_SO_GET_INFO         (EPF_BASE_CTL)
#define EPF_SO_GET_ENTRIES      (EPF_SO_GET_INFO+1)
#define EPF_SO_GET_ADDITIONAL   (EPF_SO_GET_ENTRIES+1)
#define EPF_SO_GET_IN_KERNEL    (EPF_SO_GET_ADDITIONAL+1)
#define EPF_SO_GET_MAX          (EPF_SO_GET_IN_KERNEL+1)

#define EPF_ACCEPT   0
#define EPF_DROP     1
#define EPF_CONTINUE 2 // for rules that just want to see the counters
#define EPF_LOG      3
#define EPF_SNAT     4
#define EPF_DNAT     5
#define NUMTARGETS   6

/* counters allowed or not */
#define EPF_NOCOUNTERS 0
#define EPF_COUNTERS 1

// if a table has to be added, search for EPF_NUMCHAINS0 in br_epf.c and do
// the appropriate. Also change EPF_NUMTABLES and EPF_POSSIBLE_T (obviously)
#define EPF_NUMCHAINS0 3 // filter
#define EPF_NUMCHAINS1 3 // nat
#define EPF_MAXCHAINS 3 // used in struct epf_info and userprogram

struct epf_entries {
	char policy;/* one standard (accept or drop) per hook */
	unsigned int n;/* nr. of entries */
	unsigned char data[0];/* entry list */
};

// used for the bitmask of epf_entry
#define EPF_NOPROTO 0x01
#define EPF_802_3 0x02
#define EPF_SOURCEMAC 0x04
#define EPF_DESTMAC 0x08
#define EPF_IP 0x10
// inversion flags
#define EPF_ISOURCE 0x01
#define EPF_IDEST 0x02
#define EPF_IIN 0x04
#define EPF_IOUT 0x08
#define EPF_IPROTO 0x10

/* one entry */
struct epf_entry {
	unsigned short	ethproto;		/* packet type ID field	*/
	char in[IFNAMSIZ];
	char out[IFNAMSIZ];
	unsigned char sourcemac[ETH_ALEN];
	unsigned char destmac[ETH_ALEN];
	char target;
	unsigned int bitmask;
	unsigned int invflags;
	char *additional_info;
};

#define EPF_FILTERTABLE 0
#define EPF_NATTABLE 1
#define EPF_NUMTABLES 2

struct epf_info {
	unsigned int nentries[EPF_MAXCHAINS];
	unsigned short allow_counters;
	unsigned char table;
};

static struct epf_entries start[] = {
	{0,0},
	{0,0},
	{0,0}
};

struct epf_counter
{
	u_int64_t pcnt;
};

#define EPF_LOG_IP 0x01 // if the frame is made by ip, log the ip information
struct epf_log_info{
	unsigned char loglevel;
	char prefix[30];
	unsigned char bitmask;
};

struct epf_nat_info{
	unsigned char mac[ETH_ALEN];
};

#define EPF_IP_SOURCE 0x01
#define EPF_IP_DEST 0x02
#define EPF_IP_TOS 0x04
#define EPF_IP_PROTO 0x08
// the same values are used for the invflags
struct epf_ip_info{
	__u32	saddr;
	__u32	daddr;
	__u32	smsk;
	__u32	dmsk;
	__u8	tos;
	__u8	protocol;
	unsigned char bitmask;
	unsigned char invflags;
};

//values used to identify what functionality was compiled into the kernel
#define EPF_IN_KERNEL_FILTER 0x01
#define EPF_IN_KERNEL_NAT    0x02
#define EPF_IN_KERNEL_LOG    0x04
#define EPF_IN_KERNEL_IPF    0x08
#endif
