#ifndef __IF_CRYPTO_H
#define __IF_CRYPTO_H
#ifdef USE_RC4
#include <openssl/rc4.h>
#endif /* USE_RC4 */
#include <linux/des_locl.h>
#include <linux/slab.h>

typedef struct RC4_KEY_HW {
	unsigned char raw[16];		/* Raw key */		
	int	len;				/* Key len */
} RC4_KEY_HW;

typedef unsigned short	crypto_cmd_t;
struct crypto_engine_def;

struct crypto_request
{
	char 	*inbuf;								/* Input buffer */
	char 	*outbuf;								/* Output buffer */
	int 	inlen;								/* Length of input buffer */
	int 	outlen;								/* On input, max size. On Output, real size of output */
	unsigned int 	flags;					
	struct crypto_engine_def *engine;	/* Engine to use. If NULL, use best one */
	struct crypto_operation
	{
		crypto_cmd_t cmd;						/* Operation to perform */
		int	offset;				/* Offset, from inbuf, to start operation */
		int 	len;					/* Length of operation */
		union
		{
			struct CRYPTO_RC4
			{
				struct RC4_KEY_HW 	*key;
			} rc4;
			struct CRYPTO_DES
			{
				des_cblock	key[3];		/* For DES, key[0] only is used */
				des_cblock	*ivec;		/* NULL if no ivec */
			} des;
			/* Jack ++ 2004/01/19 for AES */
			struct CRYPTO_AES
			{
				unsigned char key[32];	/* aes key */
				int	len; 		/* key len */
				unsigned char *ivec;
			} aes;
			/* Jack -- 2004/01/19 */
			struct CRYPTO_HASH
			{
				unsigned char hmac_key[64];
				unsigned char hmac_key_len;
				unsigned char result[20];	/* On output, contains the hash result */
				unsigned char reslen;		/* Contains the result len */
			} hash;
		} u;
		struct crypto_operation *next;			/* If non-NULL, pointer to next operation to perform on buffer 
												* Users are responsable for memory allocated for *next 
												*/
	} op;
	void 	(*callback)(struct crypto_request *);	/* function to call when done request. This struct * is passed as argument. */
	void 	*priv;						/* Private to requester */
/* Jack ++ */
/*
	void	*ixCryptoAccCodeletCryptoBufPool ;
	void	*ixCryptoAccCodeletCryptoBufData ;
*/
	unsigned int packetLen;
	void *pPrimaryMbuf ;
	void *pSecondaryMbuf ;
	void *pMbuf;
	unsigned int J_CtxId;
	void *J_Ctx;
	des_cblock J_iv;
	void *saved_header;
	void *oskb;
	void *ident_s;
	void *ident_d;
	int temp_iphlen;
/* Jack ++ 2004/01/09 */
        uint8_t natt_type ;
        uint8_t natt_head ;
        uint16_t natt_sport ;
        uint16_t natt_dport ;
};


/* Commands */
#define CRYPT_NO_OP		0x00
#define CRYPT_ENCODE 	0x00
#define CRYPT_DECODE 	0x80 
#define CRYPT_RC4 		0x01
#define CRYPT_DES			0x02		/* Should not be used... insecure (key space too small) */
#define CRYPT_3DES		0x04
# define CRYPT_ECB		0x08		/* Meaningful only for [3]DES */
# define CRYPT_CBC		0x10		/* Meaningful only for [3]DES */
#define CRYPT_MD5			0x20
#define CRYPT_SHA			0x40
# define CRYPT_HMAC		0x08		/* Meaningful only for MD5|SHA */
# define CRYPT_96			0x10		/* Meaningful only for MD5|SHA */
# define CRYPT_APPEND	0x100		/* Meaningful only for MD5|SHA, append hash to output */
#define CRYPT_RANDOM	0x200		/* Get some random bytes in outbuf */
/* Jack ++ 2004/01/19 for AES-128, AES-192 and AES-256 */
#define CRYPT_AES_128	0x400
#define CRYPT_AES_192	0x800
#define CRYPT_AES_256	0x1000

/* Flags */
#define CRYPT_SYNC				0x1	/* Must wait for completion */
#define CRYPT_HIGH_PRIORITY 0x2	/* Process in priority (insert at end of queue) */
#define CRYPT_DATA			0x4		/* Use to filter data from other stuff */
#define CRYPT_SOFTWARE	0x8		/* Force software, or in output, was done in Software automagically */
#define CRYPT_HARDWARE	0x10		/* Force hardware, or in output, was done in Hardware */

/* Return codes */
#define CRYPT_READY	1
#define CRYPT_OK		0
#define CRYPT_ERROR	-1
#define CRYPT_NO_RESSOURCE 	-2

enum crypto_engine_attributes
{
	IS_SOFTWARE	=0x01,		
	IS_HARDWARE	=0x02,
	IS_ASYNC		=0x04,	/* Can return CRYPT_OK and calls back when done */
	IS_SYNC		=0x08		/* Can be instructed to wait until completion (BUSY LOOPING?!) */
};


/* Prototypes */

struct crypto_request *crypto_alloc_request(int flags);
int crypto_execute(struct crypto_request *req);	
int crypto_query(crypto_cmd_t cap,unsigned int attrib);	/* Returns TRUE when there is an engine that matches */ 
/* Optionnaly, open the engine to use and put it in req->engine */
struct crypto_engine_def *crypto_open_engine(crypto_cmd_t cap,unsigned int attrib);
void crypto_close_engine(struct crypto_engine_def *eng);

static inline void crypto_free_request(struct crypto_request *r)
{
	kfree(r);
}

/**************************************************
*
* For crypto providers 
*
***************************************************/

struct crypto_engine_def
{
	char *name;
	crypto_cmd_t	capability;
	unsigned int	attributes;
	int (*request_handler)(struct crypto_request *req);
	void (*open)(void);
	void (*close)(void);
};


int crypto_register_engine(struct crypto_engine_def *eng);
int crypto_unregister_engine(char *name);


#endif
