#ifndef _IP_CONNTRACK_H
#define _IP_CONNTRACK_H
/* Connection state tracking for netfilter.  This is separated from,
   but required by, the NAT layer; it can also be used by an iptables
   extension. */

#include <linux/config.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <asm/atomic.h>

enum ip_conntrack_info
{
	/* Part of an established connection (either direction). */
	IP_CT_ESTABLISHED,

	/* Like NEW, but related to an existing connection, or ICMP error
	   (in either direction). */
	IP_CT_RELATED,

	/* Started a new connection to track (only
           IP_CT_DIR_ORIGINAL); may be a retransmission. */
	IP_CT_NEW,

	/* >= this indicates reply direction */
	IP_CT_IS_REPLY,

	/* Number of distinct IP_CT types (no NEW in reply dirn). */
	IP_CT_NUMBER = IP_CT_IS_REPLY * 2 - 1
};

/* 2002/4/10 Steve ++ 
 * The total len of conntrack mark is 8 bytes.
 * byte 0 - byte 3 for policy chain name.
 * byte 4 :
 *  bit 0 - bit 3 : for policy rule chain (outgoing, incoming .....) && VPN
 *  bit 4 : Log mark
 *  bit 5 : Statistics mark
 *  bit 6 : Alarm mark
 *  bit 7 : Bandwidth management mark
 * byte 5 :
 *  bit 0 : Action ( ACCEPT ) mark
 *  bit 1 : Interanl auth mark
 *  bit 2 : External auth mark
 *  bit 3 : DMZ auth mark
 *  bit 4 : Urlblocking mark
 *  bit 5 : Reenter mark
 *  bit 6 - bit 7 : reserved
 * byte 6 - byte 7 : reserved
 */
#define		CHAIN_NAME_MASK		0x00000000ffffffff
#define		POLICY_CHAIN_MASK	0x0000000f00000000
#define		OUTGOING_MASK		0x0000000100000000
#define		INCOMING_MASK		0x0000000200000000
#define		EXTODMZ_MASK		0x0000000300000000
#define		INTODMZ_MASK		0x0000000400000000
#define		DMZTOEX_MASK		0x0000000500000000
#define		DMZTOIN_MASK		0x0000000600000000
#define		VPNTOIN_MASK		0x0000000700000000
#define		INTOVPN_MASK		0x0000000800000000
#define		VPNTODMZ_MASK		0x0000000900000000
#define		DMZTOVPN_MASK		0x0000000a00000000
/* SuKent++, 2003/05/05 18:50 */
#define		PPTPTOIN_MASK		0x0000000b00000000
#define		INTOPPTP_MASK		0x0000000c00000000
#define		PPTPTODMZ_MASK		0x0000000d00000000
#define		DMZTOPPTP_MASK		0x0000000e00000000
/* ******** */

#define		POLICY_LOG_MARK		0x0000001000000000
#define		POLICY_STATISTICS_MARK	0x0000002000000000
#define		POLICY_ALARM_MARK	0x0000004000000000
#define		POLICY_BM_MARK		0x0000008000000000
#define		POLICY_ACCEPT_MARK	0x0000010000000000
#define		INTERNAL_AUTH_MARK	0x0000020000000000
#define		EXTERNAL_AUTH_MARK	0x0000040000000000
#define		DMZ_AUTH_MARK		0x0000080000000000
#define		URLBLOCK_MARK		0x0000100000000000
#define		ENTER_FORWARD_MARK	0x0000200000000000
#define		REENTER_MARK		0x0000400000000000
/* zhangzh 2003/04/18 */
#define		QUOTA_MARK		0x0000800000000000
#define		DNSRELAY_MARK		0x0000000000000006

/* zhangzh 2003/07/31 */
#define         POLICY_URLBLOCK_MARK    0x0010000000000000
/* zhangzh */

/* 2004/1/16 Steve ++ */
#define     REQUIRE_AUTH_MARK       0x0020000000000000
#define     REQUIRE_AUTH_MASK       0xffdfffffffffffff
/* Steve -- */

/* 2005/3/16 Justin Modify */
#define	    CONTENTBLOCK_MARK	    0x0100000000000000
#define     VIRUSFTP_MARK           0x0200000000000000
/* Justin-- */
/* 2005/4/25 Steve ++ */
#define	    POLICY_IPS_MARK	    0x0400000000000000
/* Steve -- */
/* 2005/10/7 Hanson ++ */
#define	    STATI_SESSION_MARK	    0x8000000000000000
/* Hanson +. */

/* 2003/1/3 Steve ++
 * For log WAN port connection information
 */ 
#define		WAN1_MARK		0x0001000000000000
#define		WAN2_MARK		0x0002000000000000
#define		WAN3_MARK		0x0003000000000000
#define		WAN4_MARK		0x0004000000000000
#define		WAN5_MARK		0x0005000000000000
#define		WAN6_MARK		0x0006000000000000
#define		WAN7_MARK		0x0007000000000000
#define		WAN8_MARK		0x0008000000000000
#define		MAX_WAN_PORT		8
#define		WAN_PORT_OFFSET		48
extern unsigned long wan_connection_num[MAX_WAN_PORT];
/* Steve -- */
/* Steve -- */

#ifdef __KERNEL__

#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/netfilter_ipv4/ip_conntrack_tcp.h>
#include <linux/netfilter_ipv4/ip_conntrack_icmp.h>

#ifdef CONFIG_NF_DEBUG
#define IP_NF_ASSERT(x)							\
do {									\
	if (!(x))							\
		/* Wooah!  I'm tripping my conntrack in a frenzy of	\
		   netplay... */					\
		printk("NF_IP_ASSERT: %s:%i(%s)\n",			\
		       __FILE__, __LINE__, __FUNCTION__);		\
} while(0)
#else
#define IP_NF_ASSERT(x)
#endif

/* Bitset representing status of connection. */
enum ip_conntrack_status {
	/* It's an expected connection: bit 0 set.  This bit never changed */
	IPS_EXPECTED_BIT = 0,
	IPS_EXPECTED = (1 << IPS_EXPECTED_BIT),

	/* We've seen packets both ways: bit 1 set.  Can be set, not unset. */
	IPS_SEEN_REPLY_BIT = 1,
	IPS_SEEN_REPLY = (1 << IPS_SEEN_REPLY_BIT),

	/* Conntrack should never be early-expired. */
	IPS_ASSURED_BIT = 2,
	IPS_ASSURED = (1 << IPS_ASSURED_BIT),
};

#ifdef CONFIG_IP_NF_NAT_NEEDED
#include <linux/netfilter_ipv4/ip_nat.h>
#endif

/* Add protocol helper include file here */
#if defined(CONFIG_IP_NF_TALK) || defined(CONFIG_IP_NF_TALK_MODULE)
#include <linux/netfilter_ipv4/ip_conntrack_talk.h>
#endif

#if defined(CONFIG_IP_NF_IRC) || defined(CONFIG_IP_NF_IRC_MODULE)
#include <linux/netfilter_ipv4/ip_conntrack_irc.h>
#endif

#if defined(CONFIG_IP_NF_H323) || defined(CONFIG_IP_NF_H323_MODULE)
#include <linux/netfilter_ipv4/ip_conntrack_h323.h>
#endif

#if defined(CONFIG_IP_NF_FTP) || defined(CONFIG_IP_NF_FTP_MODULE)
#include <linux/netfilter_ipv4/ip_conntrack_ftp.h>
#include <linux/netfilter_ipv4/ip_conntrack_pptp.h>
#endif

/* 2002/8/2 Steve ++ */
#if defined(CONFIG_IP_NF_MSN) || defined(CONFIG_IP_NF_MSN_MODULE)
#include <linux/netfilter_ipv4/ip_conntrack_msn.h>
#include <linux/netfilter_ipv4/ip_conntrack_pbx.h>
#endif
/* Steve -- */
/* 2005/4/16 Steve ++ */
#include <linux/netfilter_ipv4/ip_conntrack_sip.h>
/* Steve -- */

struct ip_conntrack_expect
{
	/* Internal linked list (global expectation list) */
	struct list_head list;

	/* expectation list for this master */
	struct list_head expected_list;

	/* The conntrack of the master connection */
	struct ip_conntrack *expectant;

	/* The conntrack of the sibling connection, set after
	 * expectation arrived */
	struct ip_conntrack *sibling;

	/* Tuple saved for conntrack */
	struct ip_conntrack_tuple ct_tuple;

	/* Data filled out by the conntrack helpers follow: */
	
	/* We expect this tuple, with the following mask */
	struct ip_conntrack_tuple tuple, mask;

	/* Function to call after setup and insertion */
	int (*expectfn)(struct ip_conntrack *new);

	/* At which sequence number did this expectation occur */
	u_int32_t seq;

	union {
		/* insert conntrack helper private data (expect) here */
#if defined(CONFIG_IP_NF_TALK) || defined(CONFIG_IP_NF_TALK_MODULE)
		struct ip_ct_talk_expect exp_talk_info;
#endif
#if defined(CONFIG_IP_NF_IRC) || defined(CONFIG_IP_NF_IRC_MODULE)
		struct ip_ct_irc_expect exp_irc_info;
#endif
#if defined(CONFIG_IP_NF_H323) || defined(CONFIG_IP_NF_H323_MODULE)
		struct ip_ct_h225_expect exp_h225_info;
#endif
#if defined(CONFIG_IP_NF_FTP) || defined(CONFIG_IP_NF_FTP_MODULE)
		struct ip_ct_ftp_expect exp_ftp_info;
		struct ip_ct_pptp_expect exp_pptp_info;
#endif
/* 2002/8/2 Steve ++ */
#if defined(CONFIG_IP_NF_MSN) || defined(CONFIG_IP_NF_MSN_MODULE)
		struct ip_ct_msn_expect exp_msn_info;
		struct ip_ct_pbx_expect exp_pbx_info;
#endif
/* Steve -- */

#ifdef CONFIG_IP_NF_NAT_NEEDED
		union {
			/* insert nat helper private data here */
		} nat;
#endif
	} help;
};

/* 2004/7/22 Steve ++ */
struct skype_state {
        unsigned int seq;
        unsigned int ack;
        int state;
};
/* Steve -- */

/* 2004/9/22 Steve ++ */
#define	ACCOUNTING_LEN 60
#define	ACCOUNTING_REPORT_TIMER		(30 * HZ)
#include <linux/ip.h>
struct accounting_report {
	unsigned long lastseen;
	unsigned long upstream;
	unsigned long downstream;
	int policy;
	unsigned char sip[ACCOUNTING_LEN];
	unsigned char rip[ACCOUNTING_LEN];
};
/* Steve -- */

#define IP_CONNTRACK_EXPECT_SIZEOF_HEAD \
	(2*sizeof(struct list_head) \
	+ 2*sizeof(struct ip_conntrack *) \
	+ 3*sizeof(struct ip_conntrack_tuple))
#define IP_CONNTRACK_EXPECT_SIZEOF_DATA \
	(sizeof(struct ip_conntrack_expect) \
	- IP_CONNTRACK_EXPECT_SIZEOF_HEAD)

struct ip_conntrack
{
	/* Usage count in here is 1 for hash table/destruct timer, 1 per skb,
           plus 1 for any connection(s) we are `master' for */
	struct nf_conntrack ct_general;

	/* These are my tuples; original and reply */
	struct ip_conntrack_tuple_hash tuplehash[IP_CT_DIR_MAX];

	/* Have we seen traffic both ways yet? (bitset) */
	volatile unsigned long status;

	/* Timer function; drops refcnt when it goes off. */
	struct timer_list timeout;

	/* If we're expecting another related connection, this will be
           in expected linked list */
	struct list_head sibling_list;
	
	/* Current number of expected connections */
	unsigned int expecting;

	/* If we were expected by an expectation, this will be it */
	struct ip_conntrack_expect *master;

	/* Helper, if any. */
	struct ip_conntrack_helper *helper;

	/* Our various nf_ct_info structs specify *what* relation this
           packet has to the conntrack */
	struct nf_ct_info infos[IP_CT_NUMBER];

	/* Storage reserved for other modules: */

	union {
		struct ip_ct_tcp tcp;
		struct ip_ct_icmp icmp;
	} proto;

	union {
		/* insert conntrack helper private data (master) here */
#if defined(CONFIG_IP_NF_TALK) || defined(CONFIG_IP_NF_TALK_MODULE)
		struct ip_ct_talk_master ct_talk_info;
#endif
#if defined(CONFIG_IP_NF_IRC) || defined(CONFIG_IP_NF_IRC_MODULE)
		struct ip_ct_irc_master ct_irc_info;
#endif
#if defined(CONFIG_IP_NF_H323) || defined(CONFIG_IP_NF_H323_MODULE)
		struct ip_ct_h225_master ct_h225_info;
#endif
#if defined(CONFIG_IP_NF_FTP) || defined(CONFIG_IP_NF_FTP_MODULE)
		struct ip_ct_ftp_master ct_ftp_info;
		struct ip_ct_pptp_master ct_pptp_info;
#endif
/* 2002/8/2 Steve ++ */
#if defined(CONFIG_IP_NF_MSN) || defined(CONFIG_IP_NF_MSN_MODULE)
		struct ip_ct_msn_master ct_msn_info;
		struct ip_ct_pbx_master ct_pbx_info;
#endif
/* Steve -- */
/* 2005/4/16 Steve ++ */
		struct ip_ct_sip_master ct_sip_info;
/* Steve -- */
	} help;

#ifdef CONFIG_IP_NF_NAT_NEEDED
	struct {
		struct ip_nat_info info;
		union {
			/* insert nat helper private data here */
		} help;
#if defined(CONFIG_IP_NF_TARGET_MASQUERADE) || \
	defined(CONFIG_IP_NF_TARGET_MASQUERADE_MODULE)
		int masq_index;
#endif
	} nat;
#endif /* CONFIG_IP_NF_NAT_NEEDED */

#if defined(CONFIG_IP_NF_CONNTRACK_MARK)
/* 2002/3/14 Steve ++ */
//	unsigned long mark;
	unsigned long long mark;
	unsigned long offset;
	unsigned long target;
/* 2002/5/14 Steve ++ */
	unsigned long skbmark;
	unsigned long routemark;
	unsigned long long vipmark;
/* Steve -- */
#endif
/* 2002/12/11 Steve ++ */
	unsigned long pmtu;
/* Steve -- */
/* 2003/1/6 Steve ++ */
	int indexwan;
/* Steve -- */
/* 2003/4/17 Steve ++ */
	unsigned long long total_bytes;
/* Steve -- */
/* 2004/4/21 Steve ++ */
#if 0
        struct {
                unsigned int numpackets;
                char * app_proto;
                char * app_data;
                unsigned int app_data_len;
        } layer7;
#endif
/* Steve -- */
/* 2004/7/22 Steve ++ */
        struct skype_state skype;
/* Steve -- */
/* 2004/9/22 Steve ++ */
#ifdef ACCOUNTING_REPORT
	struct accounting_report report;
#endif
/* Steve -- */
/* 2004/5/5 Steve ++ */
        unsigned int rulemark;
/* Steve -- */
/* 2005/10/7 Steve ++ */
	struct sk_buff *oskb;
/* Steve -- */
};

/* get master conntrack via master expectation */
#define master_ct(conntr) (conntr->master ? conntr->master->expectant : NULL)

/* Alter reply tuple (maybe alter helper).  If it's already taken,
   return 0 and don't do alteration. */
extern int
ip_conntrack_alter_reply(struct ip_conntrack *conntrack,
			 const struct ip_conntrack_tuple *newreply);

/* Is this tuple taken? (ignoring any belonging to the given
   conntrack). */
extern int
ip_conntrack_tuple_taken(const struct ip_conntrack_tuple *tuple,
			 const struct ip_conntrack *ignored_conntrack);

/* Return conntrack_info and tuple hash for given skb. */
extern struct ip_conntrack *
ip_conntrack_get(struct sk_buff *skb, enum ip_conntrack_info *ctinfo);

extern struct module *ip_conntrack_module;

extern int invert_tuplepr(struct ip_conntrack_tuple *inverse,
			  const struct ip_conntrack_tuple *orig);

/* Refresh conntrack for this many jiffies */
extern void ip_ct_refresh(struct ip_conntrack *ct,
			  unsigned long extra_jiffies);

/* These are for NAT.  Icky. */
/* Call me when a conntrack is destroyed. */
extern void (*ip_conntrack_destroyed)(struct ip_conntrack *conntrack);

/* Returns new sk_buff, or NULL */
struct sk_buff *
ip_ct_gather_frags(struct sk_buff *skb);

/* Delete all conntracks which match. */
extern void
ip_ct_selective_cleanup(int (*kill)(const struct ip_conntrack *i, void *data),
			void *data);

/* It's confirmed if it is, or has been in the hash table. */
static inline int is_confirmed(struct ip_conntrack *ct)
{
	return ct->tuplehash[IP_CT_DIR_ORIGINAL].list.next != NULL;
}

extern unsigned int ip_conntrack_htable_size;

#endif /* __KERNEL__ */
#endif /* _IP_CONNTRACK_H */
