#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/udp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>

DECLARE_LOCK(ip_amit_lock);
struct module *ip_conntrack_amit = THIS_MODULE;


#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static int amit_help_msg(const struct iphdr *iph, size_t len,
		         struct ip_conntrack *ct,
		         enum ip_conntrack_info ctinfo)
{
	int dir = CTINFO2DIR(ctinfo);
	struct ip_conntrack_expect expect, *exp = &expect;

DEBUGP("ip_ct_amit_help: 1\n");
#if 0
	if (dir != IP_CT_DIR_ORIGINAL)
		return NF_ACCEPT;
#endif

DEBUGP("ip_ct_amit_help: 2\n");
	LOCK_BH(&ip_amit_lock);
	exp->tuple = ((struct ip_conntrack_tuple)
		{ { ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip,
		    { 0 } },
		  { ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip,
		    { 0 },
		    IPPROTO_TCP }});
	exp->mask = ((struct ip_conntrack_tuple)
		{ { 0xFFFFFFFF, { 0 } },
		  { 0xFFFFFFFF, { 0 }, 0xFFFF }});

	exp->expectfn = NULL;

	ip_conntrack_expect_related(ct, exp);
	UNLOCK_BH(&ip_amit_lock);

DEBUGP("ip_ct_amit_help: 3\n");
	return NF_ACCEPT;
}

static int help(const struct iphdr *iph,
		 size_t len,
		 struct ip_conntrack *ct,
		 enum ip_conntrack_info ctinfo)
{
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	size_t udplen;

	if (ctinfo != IP_CT_ESTABLISHED
	    && ctinfo != IP_CT_ESTABLISHED + IP_CT_IS_REPLY) {
		DEBUGP("ip_ct_amit_help: Conntrackinfo = %u\n", ctinfo);
		return NF_ACCEPT;
	}

	/* Not whole UDP header? */
	udplen = len - iph->ihl * 4;
	if (udplen < sizeof(struct udphdr)) {
		return NF_ACCEPT;
	}

	/* Checksum invalid?  Ignore. */
	/* FIXME: Source route IP option packets --RR */
	if (csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP,
			      csum_partial((char *)udph, udplen, 0))) {
		DEBUGP("ip_ct_amit_help: bad csum: %p %u %u.%u.%u.%u %u.%u.%u.%u\n",
		       udph, udplen, NIPQUAD(iph->saddr),
		       NIPQUAD(iph->daddr));
		return NF_ACCEPT;
	}

	amit_help_msg(iph, len, ct, ctinfo);
	return NF_ACCEPT;
}

static struct ip_conntrack_helper amit_helpers = { { NULL, NULL },
						    { { 0, { __constant_htons(407) } },
						    { 0, { 0 }, IPPROTO_UDP } },
						    { { 0, { 0xFFFF } },
						    { 0, { 0 }, 0xFFFF } },
						    8,
						    help };

static int __init init(void)
{
	ip_conntrack_helper_register(&amit_helpers);
	return 0;
}

static void __exit fini(void)
{
	ip_conntrack_helper_unregister(&amit_helpers);
}

module_init(init);
module_exit(fini);
