#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/udp.h>
#include <net/tcp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_msn.h>

MODULE_DESCRIPTION("MSN 'brute force' connection tracking module");

DECLARE_LOCK(ip_msn_lock);
struct module *ip_conntrack_msn = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static int
str_to_u16(char *pbuf, __u16 *pval) 
{
	int n = 0;

	*pval = 0;
	while (pbuf[n] >= '0' && pbuf[n] <= '9') {
		*pval = (*pval * 10) + (pbuf[n] - '0');
		n++;
	}

	return n;
}

static int
str_to_strip_u16(char *pbuf, __u16 *pval) 
{
	int n = 0;

	*pval = 0;
	while ((pbuf[n] >= '0' && pbuf[n] <= '9') || pbuf[n] == '\0') {
		if (pbuf[n] != '\0')
			*pval = (*pval * 10) + (pbuf[n] - '0');
		n++;
	}

	return n;
}

static int 
try_strip_number(const unsigned char *data, size_t dlen, u_int32_t array[],
		 int array_size, char sep)
{
	u_int32_t i, len;

	memset(array, 0, sizeof(array[0])*array_size);

	/* Keep data pointing at next char. */
	for (i = 0, len = 0; len < dlen && i < array_size; len++, data++) {
		if (*data >= '0' && *data <= '9')
			array[i] = array[i]*10 + *data - '0';
		else if (*data == '\0')
			continue;
		else if (*data == sep)
			i++;
		else {
			/* Unexpected character; true if it's the
			   terminator and we're finished. */
			if (i == array_size - 1)
				return len;

			DEBUGP("Char %u (got %u nums) `%u' unexpected\n",
				len, i, *data);
			return 0;
		}
	}
	DEBUGP("Failed to fill %u numbers separated by %c\n", array_size, sep);

	return 0;
}

static int 
try_number(const unsigned char *data, size_t dlen, u_int32_t array[],
	   int array_size, char sep)
{
	u_int32_t i, len;

	memset(array, 0, sizeof(array[0])*array_size);

	/* Keep data pointing at next char. */
	for (i = 0, len = 0; len < dlen && i < array_size; len++, data++) {
		if (*data >= '0' && *data <= '9')
			array[i] = array[i]*10 + *data - '0';
		else if (*data == sep)
			i++;
		else {
			/* Unexpected character; true if it's the
			   terminator and we're finished. */
			if (i == array_size - 1)
				return len;

			DEBUGP("Char %u (got %u nums) `%u' unexpected\n",
				len, i, *data);
			return 0;
		}
	}
	DEBUGP("Failed to fill %u numbers separated by %c\n", array_size, sep);

	return 0;
}

static int msn_expect(struct ip_conntrack *ct);

static int 
msn_control_voice_help(const struct iphdr *iph, size_t len,
			struct ip_conntrack *ct,
			enum ip_conntrack_info ctinfo)
{
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	unsigned char *data = (unsigned char *)((const char *)udph + sizeof(struct udphdr));
	unsigned char *data_limit;
	u_int32_t udplen = len - iph->ihl * 4;
	u_int32_t datalen = udplen - sizeof(struct udphdr);
	int dir = CTINFO2DIR(ctinfo);
	struct ip_ct_msn_master *info = &ct->help.ct_msn_info;
	struct ip_conntrack_expect expect, *exp = &expect;
	struct ip_conntrack_expect expect_voice, *exp_voice = &expect_voice;
	struct ip_conntrack_expect expect_voice_2, *exp_voice_2 = &expect_voice_2;
	struct ip_ct_msn_expect *exp_info = &exp->help.exp_msn_info;
	u_int16_t data_port, data_voice_port, data_voice_port_2;
	u_int32_t data_ip = 0;
	unsigned int i, addrlen = 0, portlen = 0, voiceportlen = 0;
	u_int32_t array[4];
	int invite = 0, sipok = 0, ack = 0;

	DEBUGP("ct_msn_control_voice_help: help entered %u.%u.%u.%u:%u->%u.%u.%u.%u:%u\n",
		NIPQUAD(iph->saddr), ntohs(udph->source),
		NIPQUAD(iph->daddr), ntohs(udph->dest));

	/* Can't track connections formed before we registered */
	if (!info)
		return NF_ACCEPT;
		
	/* Until there's been traffic both ways, don't look in packets. */
	if (ctinfo != IP_CT_ESTABLISHED && ctinfo != IP_CT_RELATED
	    && ctinfo != IP_CT_ESTABLISHED + IP_CT_IS_REPLY) {
		DEBUGP("ct_msn_help: Conntrackinfo = %u\n", ctinfo);
		return NF_ACCEPT;
	}

        if (udplen < sizeof(struct udphdr)) {
		DEBUGP("ct_msn_voice_control_help: too short for udph, udplen = %u\n", (unsigned)udplen);
                return NF_ACCEPT;
        }

        /* Checksum invalid?  Ignore. */
        /* FIXME: Source route IP option packets --RR */
        if (csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP,
                              csum_partial((char *)udph, udplen, 0))) {
	        DEBUGP("ct_msn_control_voice_help: bad csum: %p %u %u.%u.%u.%u %u.%u.%u.%u\n",
                       udph, udplen, NIPQUAD(iph->saddr),
                       NIPQUAD(iph->daddr));
                return NF_ACCEPT;
        }

	data_limit = (unsigned char *) data + datalen;

	if (!memcmp(data, "INVITE ", 7)) {
		printk("ip_conntrack_msn: INVITE\n");
		invite = 1;
	} else if (!memcmp(data, "SIP/2.0 200 OK", 14)) {
		printk("ip_conntrack_msn: SIP/2.0 200 OK\n");
		sipok = 1;
	} else
		return NF_ACCEPT;

	for (i = 0; data < (data_limit - 14); data++, i++) {
		if (!memcmp(data, "Contact: <sip:", 14)) {
			data += 14;
			i += 14;
			addrlen = try_number(data, datalen - i, array, 4, '.');
			if (!addrlen)
				continue;
			data += addrlen;
			data_ip = htonl((array[0] << 24) | (array[1] << 16) | (array[2] << 8) | array[3]);
			if (*data != ':')
				continue;
			data++;
                	portlen = str_to_u16(data, &data_port);
			if (!portlen)
				continue;
			data_port = htons(data_port);
	                data += portlen;
		}

		if (!memcmp(data, "m=audio ", 8)) {
			data += 8;
			voiceportlen = str_to_u16(data, &data_voice_port);
			if (!voiceportlen)
				continue;
			data_voice_port_2 = data_voice_port + 1;
			data_voice_port = htons(data_voice_port);
			data_voice_port_2 = htons(data_voice_port_2);
		}
	}

	if (!addrlen || !portlen || !voiceportlen)
		return NF_ACCEPT;

	LOCK_BH(&ip_msn_lock);
	info->is_msn = IP_CT_VOICE_TRANS;
	exp_info->port = data_port;
	exp_info->dir = dir;

	exp->tuple = ((struct ip_conntrack_tuple)
			{ { 0,
			  { 0 } },
			{ data_ip,
			  { data_port },
			  IPPROTO_UDP }});
	exp->mask = ((struct ip_conntrack_tuple)
			{ { 0, { 0 } },
			  { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});

	if (invite)
		exp->expectfn = msn_expect;
	else
		exp->expectfn = NULL;
			
	/* Ignore failure; should only happen with NAT */
	if (!ip_conntrack_expect_checked(ct, exp))
		ip_conntrack_expect_related(ct, exp);

	exp_voice->tuple = ((struct ip_conntrack_tuple)
			   { { 0,
			     { 0 } },
			   { data_ip,
			     { data_voice_port },
			   IPPROTO_UDP }});
	exp_voice->mask = ((struct ip_conntrack_tuple)
			   { { 0, { 0 } },
			     { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});
	
	exp_voice->expectfn = NULL;
			
	/* Ignore failure; should only happen with NAT */
	ip_conntrack_expect_related(ct, exp_voice);

	exp_voice_2->tuple = ((struct ip_conntrack_tuple)
			   { { 0,
			     { 0 } },
			   { data_ip,
			     { data_voice_port_2 },
			   IPPROTO_UDP }});
	exp_voice_2->mask = ((struct ip_conntrack_tuple)
			   { { 0, { 0 } },
			     { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});
	
	exp_voice_2->expectfn = NULL;
			
	/* Ignore failure; should only happen with NAT */
	ip_conntrack_expect_related(ct, exp_voice_2);

	UNLOCK_BH(&ip_msn_lock);
	return NF_ACCEPT;
}

/* MSN helper is not registered! 
   Only the helper function and the max number of expectations count */
static struct ip_conntrack_helper msn_control_voice = { { NULL, NULL },
				           		{ { 0, { 0 } },
				           		  { 0, { 0 }, IPPROTO_UDP } },
				           		{ { 0, { 0xFFFF } },
				           		  { 0, { 0 }, 0xFFFF } },
				          		 8,
				          		 msn_control_voice_help };

static int msn_expect(struct ip_conntrack *ct)
{
	WRITE_LOCK(&ip_conntrack_lock);
	ct->helper = &msn_control_voice;
	DEBUGP("msn_expect: helper for %p added\n", ct);
	WRITE_UNLOCK(&ip_conntrack_lock);
	
	return NF_ACCEPT;	/* unused */
}

/* FIXME: This should be in userspace.  Later. */
static int 
msn_help(const struct iphdr *iph, size_t len,
	 struct ip_conntrack *ct,
	 enum ip_conntrack_info ctinfo)
{
	struct tcphdr *tcph = (void *)iph + iph->ihl * 4;
	unsigned char *data = (unsigned char *) tcph + tcph->doff * 4;
	unsigned char *data_limit, *data_beg;
	unsigned char *__data = data;
	u_int32_t tcplen = len - iph->ihl * 4;
	u_int32_t datalen = tcplen - tcph->doff * 4;
	int dir = CTINFO2DIR(ctinfo);
	struct ip_ct_msn_master *info = &ct->help.ct_msn_info;
	struct ip_conntrack_expect expect, *exp = &expect;
	struct ip_ct_msn_expect *exp_info = &exp->help.exp_msn_info;
	u_int32_t data_ip = 0;
	u_int16_t data_port = 0;
	unsigned int i, addrlen, portlen;
	u_int32_t array[4];
	int clientip = 0;

	/* Can't track connections formed before we registered */
	if (!info)
		return NF_ACCEPT;

	/* Until there's been traffic both ways, don't look in packets. */
	if (ctinfo != IP_CT_ESTABLISHED
	    && ctinfo != IP_CT_ESTABLISHED + IP_CT_IS_REPLY) {
		DEBUGP("ct_msn_help: Conntrackinfo = %u\n", ctinfo);
		return NF_ACCEPT;
	}

	/* Not whole TCP header or too short packet? */
	if (tcplen < sizeof(struct tcphdr) || tcplen < tcph->doff * 4 + 5) {
		DEBUGP("ct_msn_help: tcplen = %u\n", (unsigned)tcplen);
		return NF_ACCEPT;
	}

	/* Checksum invalid?  Ignore. */
	/* FIXME: Source route IP option packets --RR */
	if (tcp_v4_check(tcph, tcplen, iph->saddr, iph->daddr,
			      csum_partial((char *)tcph, tcplen, 0))) {
		DEBUGP("ct_msn_help: bad csum: %p %u %u.%u.%u.%u %u.%u.%u.%u\n",
		       tcph, tcplen, NIPQUAD(iph->saddr),
		       NIPQUAD(iph->daddr));
		return NF_ACCEPT;
	}
	
	data_limit = (unsigned char *) data + datalen;
	for (i = 0; data < (data_limit - 31); data++, i++) {
		if (!memcmp(data, "\0<\0t\0c\0p\0p\0o\0r\0t\0>\0", 19)) {
			data += 19;
			data_beg = data;
			info->is_msn = IP_CT_VIDEO_SESSION;
			portlen = str_to_strip_u16(data, &data_port);
			if (!portlen)
				continue;
			data_port = htons(data_port);
			data += portlen;
		}

		if (!memcmp(data, "\0<\0t\0c\0p\0i\0p\0a\0d\0d\0r\0e\0s\0s\0001\0>\0", 31)) {
			data += 31;
			data_beg = data;
			i += 31;
			addrlen = try_strip_number(data, datalen - i, array, 4, '.');
			if (!addrlen)
				continue;
			data += addrlen;
			data_ip = htonl((array[0] << 24) | (array[1] << 16) | (array[2] << 8) | array[3]);
		}
		if (!memcmp(data, "\r\nClientIP: ", 12)) {
			data += 12;
			data_beg = data;
			clientip = 1;
		}
 
		if (!memcmp(data, "\r\nIP-Address: ", 14)) {
			data += 14;
			data_beg = data;
			i += 14;
			addrlen = try_number(data, datalen - i, array, 4, '.');
			if (!addrlen)
				continue;
			data += addrlen;
			data_ip = htonl((array[0] << 24) | (array[1] << 16) | (array[2] << 8) | array[3]);
			if (*data == ':') {
				data += 1;
				info->is_msn = IP_CT_VOICE_SESSION_INIT;
				portlen = str_to_u16(data, &data_port);
				if (!portlen)
					continue;

				data_port = htons(data_port);
				data += portlen;
			}
		}

		if (!memcmp(data, "\r\nIPv4Internal-Addrs: ", 22)) {
			data += 22;
			data_beg = data;
			i += 22;
			addrlen = try_number(data, datalen - i, array, 4, '.');
			if (!addrlen)
				continue;
			data += addrlen;
			data_ip = htonl((array[0] << 24) | (array[1] << 16) | (array[2] << 8) | array[3]);
		}

		if (!memcmp(data, "\r\nPort: ", 8)) {
			data += 8;
			info->is_msn = IP_CT_FILE_TRANS;
			portlen = str_to_u16(data, &data_port);
			if (!portlen)
				continue;
			data_port = htons(data_port);
			data += portlen;
		}

		if (!memcmp(data, "\r\nIPv4Internal-Port: ", 21)) {
			data += 21;
			info->is_msn = IP_CT_FILE_TRANS;
			portlen = str_to_u16(data, &data_port);
			if (!portlen)
				continue;
			data_port = htons(data_port);
			data += portlen;
		}
	}

	if (data_ip && data_port) {
		/* update the MSN info */
		LOCK_BH(&ip_msn_lock);
		exp_info->port = data_port;
		exp_info->dir = dir;
		exp_info->len = data - data_beg;
		DEBUGP("ct_msn_help: len = %u\n", exp_info->len);
		exp->seq = ntohl(tcph->seq) + (data_beg - __data);

		if (info->is_msn == IP_CT_FILE_TRANS ||
		    info->is_msn == IP_CT_VIDEO_SESSION) {
			exp->tuple = ((struct ip_conntrack_tuple)
					{ { 0,
					{ 0 } },
					{ data_ip,
					{ data_port },
					IPPROTO_TCP }});
			exp->mask = ((struct ip_conntrack_tuple)
					{ { 0, { 0 } },
					  { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});
			exp->expectfn = NULL;
		} else {
			exp->tuple = ((struct ip_conntrack_tuple)
					{ { 0,
					{ 0 } },
					{ data_ip,
					{ data_port },
					IPPROTO_UDP }});
			exp->mask = ((struct ip_conntrack_tuple)
					{ { 0, { 0 } },
					  { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});
			exp->expectfn = msn_expect;
		}
				
		/* Ignore failure */
		ip_conntrack_expect_related(ct, exp);
		DEBUGP("ct_msn_help: new MSN requested %u.%u.%u.%u->%u.%u.%u.%u:%u\n",
			NIPQUAD(ct->tuplehash[!dir].tuple.src.ip),
			NIPQUAD(iph->saddr), ntohs(data_port));
		UNLOCK_BH(&ip_msn_lock);
	} else if (clientip) {
		LOCK_BH(&ip_msn_lock);
		info->dir = dir;
		info->offset[IP_CT_DIR_REPLY] = data - data_beg;
		info->seq[IP_CT_DIR_REPLY] = ntohl(tcph->seq) + (data_beg - __data);
		UNLOCK_BH(&ip_msn_lock);
	}

	return NF_ACCEPT;
}

static int 
msn_rx_help(const struct iphdr *iph, size_t len,
	 struct ip_conntrack *ct,
	 enum ip_conntrack_info ctinfo)
{
	return NF_ACCEPT;
}

static struct ip_conntrack_helper msn = { { NULL, NULL },
					   { { 0, { __constant_htons(IP_CT_IS_MSN) } },
					     { 0, { 0 }, IPPROTO_TCP } },
					   { { 0, { 0xFFFF } },
					     { 0, { 0 }, 0xFFFF } },
					   8,
					   msn_help };

static struct ip_conntrack_helper msn_rx = { { NULL, NULL },
					   { { 0, { __constant_htons(IP_CT_MSN_RX) } },
					     { 0, { 0 }, IPPROTO_UDP } },
					   { { 0, { 0xFFFF } },
					     { 0, { 0 }, 0xFFFF } },
					   1,
					   msn_rx_help };

				
static int __init init(void)
{
	int ret = 0;

	ret = ip_conntrack_helper_register(&msn);
	if (ret) {
		printk("ip_conntrack_msn: MSN conntrack fails\n");
		return ret;
	}

	ret = ip_conntrack_helper_register(&msn_rx);
	if (ret) {
		ip_conntrack_helper_unregister(&msn);
		printk("ip_conntrack_msn: MSN RX protocol conntrack fails\n");
	}

	return ret;
}

static void __exit fini(void)
{
	/* Unregister MSN helper */	
	ip_conntrack_helper_unregister(&msn);
	ip_conntrack_helper_unregister(&msn_rx);
}

module_init(init);
module_exit(fini);
