#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/udp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_pbx.h>

DECLARE_LOCK(ip_pbx_lock);
struct module *ip_conntrack_pbx = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static int help(const struct iphdr *iph, size_t len,
		struct ip_conntrack *ct,
		enum ip_conntrack_info ctinfo)
{
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	int dir = CTINFO2DIR(ctinfo);
	struct ip_ct_pbx_master *info = &ct->help.ct_pbx_info;
	u_int32_t udplen = len - iph->ihl * 4;
	
	/* Can't track connections formed before we registered */
	if (!info)
		return NF_ACCEPT;

	if (udplen < sizeof(struct udphdr))
		return NF_ACCEPT;

	if (csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP,
			      csum_partial((char *)udph, udplen, 0)))
		return NF_ACCEPT;

	LOCK_BH(&ip_pbx_lock);
	info->dir = dir;
	UNLOCK_BH(&ip_pbx_lock);
	return NF_ACCEPT;

}

static struct ip_conntrack_helper pbx[2] = {
					{ { NULL, NULL },
					  { { 0, { __constant_htons(PBX_PORT) } },
					{ 0, { 0 }, IPPROTO_UDP } },
					  { { 0, { 0xFFFF } },
					{ 0, { 0 }, 0xFFFF } },
					  1,
					  help
					},
					{ { NULL, NULL },
					  { { 0, { __constant_htons(PBX_DATA_PORT) } },
					{ 0, { 0 }, IPPROTO_UDP } },
					  { { 0, { 0xFFFF } },
					{ 0, { 0 }, 0xFFFF } },
					  1,
					  help
					}} ;

static int __init init(void)
{
	int ret;
	ret = ip_conntrack_helper_register(&pbx[0]);
	if (ret)
		return ret;
	ret = ip_conntrack_helper_register(&pbx[1]);
	if (ret)
		ip_conntrack_helper_unregister(&pbx[0]);

	return ret;
}

static void __exit fini(void)
{
	ip_conntrack_helper_unregister(&pbx[0]);
	ip_conntrack_helper_unregister(&pbx[1]);
}

module_init(init);
module_exit(fini);
