#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/udp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_sip.h>

MODULE_DESCRIPTION("SIP 'brute force' connection tracking module");

DECLARE_LOCK(ip_sip_lock);
struct module *ip_conntrack_sip = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static int
str_to_u16(char *pbuf, __u16 *pval) 
{
	int n = 0;

	*pval = 0;
	while (pbuf[n] >= '0' && pbuf[n] <= '9') {
		*pval = (*pval * 10) + (pbuf[n] - '0');
		n++;
	}

	return n;
}

static int 
try_number(const unsigned char *data, size_t dlen, u_int32_t array[],
	   int array_size, char sep)
{
	u_int32_t i, len;

	memset(array, 0, sizeof(array[0])*array_size);

	/* Keep data pointing at next char. */
	for (i = 0, len = 0; len < dlen && i < array_size; len++, data++) {
		if (*data >= '0' && *data <= '9')
			array[i] = array[i]*10 + *data - '0';
		else if (*data == sep)
			i++;
		else {
			/* Unexpected character; true if it's the
			   terminator and we're finished. */
			if (i == array_size - 1)
				return len;

			DEBUGP("Char %u (got %u nums) `%u' unexpected\n",
				len, i, *data);
			return 0;
		}
	}
	DEBUGP("Failed to fill %u numbers separated by %c\n", array_size, sep);

	return 0;
}

/* FIXME: This should be in userspace.  Later. */
static int 
sip_help(const struct iphdr *iph, size_t len,
	 struct ip_conntrack *ct,
	 enum ip_conntrack_info ctinfo)
{
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	unsigned char *data = (unsigned char *)((const char *)udph + sizeof(struct udphdr));
	unsigned char *__data = data;
	unsigned char *data_limit;
	u_int32_t udplen = len - iph->ihl * 4;
	u_int32_t datalen = udplen - sizeof(struct udphdr);
	int dir = CTINFO2DIR(ctinfo);
	u_int32_t data_ip = 0;
	u_int16_t data_port = 0, data_rtsp_port = 0, data_video_port = 0;
	u_int32_t array[4];
	unsigned int i, addrlen, portlen;
	struct ip_conntrack_expect expect, *exp = &expect;
	struct ip_conntrack_expect expect_rtsp, *exp_rtsp = &expect_rtsp;
	struct ip_conntrack_expect expect_video, *exp_video = &exp_video;

	if (udplen < sizeof(struct udphdr))
                return NF_ACCEPT;

        /* Checksum invalid?  Ignore. */
        /* FIXME: Source route IP option packets --RR */
        if (csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP,
                              csum_partial((char *)udph, udplen, 0))) {
	        DEBUGP("ip_conntrack_sip: bad csum: %p %u %u.%u.%u.%u %u.%u.%u.%u\n",
                       udph, udplen, NIPQUAD(iph->saddr),
                       NIPQUAD(iph->daddr));
                return NF_ACCEPT;
        }

	DEBUGP("---ip_conntrack_sip---");
	data_limit = (unsigned char *) data + datalen;
	if (datalen < 8 || memcmp(data, "INVITE ", 7))
		return NF_ACCEPT;

	for (i = 0; data < (data_limit - 8); data++, i++) {
                if (!memcmp(data, "IN IP4 ", 7)) {
                        data += 7;
                        addrlen = try_number(data, datalen, array, 4, '.');
			if (!addrlen)
				continue;
			data += addrlen;
			data_ip = htonl((array[0] << 24) | (array[1] << 16) | (array[2] << 8) | array[3]);
                }
#if 0
		if (!memcmp(data, "From: ", 6)) {
			data += 6;
			for (; data < (data_limit - 8); data++) {
				if (*data == '@') {
					data++;
					addrlen = try_number(data, datalen, array, 4, '.');
					if (!addrlen)
						continue;
					data += addrlen;
					data_ip = htonl((array[0] << 24) | (array[1] << 16) | (array[2] << 8) | array[3]);
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
					break;
                        }
		}
#endif
		if (!memcmp(data, "m=audio ", 8)) {
			data += 8;
			portlen = str_to_u16(data, &data_port);
			if (!portlen)
				continue;

			data_rtsp_port = data_port + 1;
			data_port = htons(data_port);
			DEBUGP("data_port: %d", data_port);
			data_rtsp_port = htons(data_rtsp_port);
			data += portlen;
		}

		if (!memcmp(data, "m=video ", 8)) {
			data += 8;
			portlen = str_to_u16(data, &data_video_port);
			if (!portlen)
				continue;

			data_video_port = htons(data_video_port);
			DEBUGP("data_video_port: %d", data_video_port);
			data += portlen;
		}
	}

	if (data_ip && data_port) {
		/* update the MSN info */
		LOCK_BH(&ip_sip_lock);
		DEBUGP("ip_conntrack_sip: create expect");
		exp->tuple = ((struct ip_conntrack_tuple)
				{ { 0,
				{ 0 } },
				{ data_ip,
				{ data_port },
				IPPROTO_UDP }});
		exp->mask = ((struct ip_conntrack_tuple)
				{ { 0, { 0 } },
				  { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});
		exp->expectfn = NULL;
				
		/* Ignore failure */
		ip_conntrack_expect_related(ct, exp);
		exp_rtsp->tuple = ((struct ip_conntrack_tuple)
				{ { 0,
				{ 0 } },
				{ data_ip,
				{ data_rtsp_port },
				IPPROTO_UDP }});
		exp_rtsp->mask = ((struct ip_conntrack_tuple)
				{ { 0, { 0 } },
				  { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});
		exp_rtsp->expectfn = NULL;
				
		/* Ignore failure */
		ip_conntrack_expect_related(ct, exp_rtsp);
		UNLOCK_BH(&ip_sip_lock);
	}

	if (data_ip && data_video_port) {
		/* update the MSN info */
		LOCK_BH(&ip_sip_lock);
		DEBUGP("ip_conntrack_sip: create expect");
		exp_video->tuple = ((struct ip_conntrack_tuple)
				{ { 0,
				{ 0 } },
				{ data_ip,
				{ data_video_port },
				IPPROTO_UDP }});
		exp_video->mask = ((struct ip_conntrack_tuple)
				{ { 0, { 0 } },
				  { 0xFFFFFFFF, { 0xFFFF }, 0xFFFF }});
		exp_video->expectfn = NULL;
				
		/* Ignore failure */
		ip_conntrack_expect_related(ct, exp);
		UNLOCK_BH(&ip_sip_lock);
	}

	return NF_ACCEPT;
}

static struct ip_conntrack_helper sip_rx = { { NULL, NULL },
					   { { 0, { __constant_htons(IP_CT_SIP) } },
					     { 0, { 0 }, IPPROTO_UDP } },
					   { { 0, { 0xFFFF } },
					     { 0, { 0 }, 0xFFFF } },
					   8,
					   sip_help };

				
static int __init init(void)
{
	int ret = 0;

	ret = ip_conntrack_helper_register(&sip_rx);
	if (ret) {
		printk("ip_conntrack_msn: SIP conntrack fails\n");
		return ret;
	}

	return ret;
}

static void __exit fini(void)
{
	/* Unregister SIP helper */	
	ip_conntrack_helper_unregister(&sip_rx);
}

module_init(init);
module_exit(fini);
