// $Id: ip_conntrack_standalone.c,v 1.3 2002/03/14 03:40:29 steve Exp $
// $Log: ip_conntrack_standalone.c,v $
// Revision 1.3  2002/03/14 03:40:29  steve
// *** empty log message ***
//
// Revision 1.2  2002/01/08 09:13:22  steve
// Remove tcp window tracking.
//
/* This file contains all the functions required for the standalone
   ip_conntrack module.

   These are not required by the compatibility layer.
*/

/* (c) 1999 Paul `Rusty' Russell.  Licenced under the GNU General
   Public Licence. */

#include <linux/types.h>
#include <linux/ip.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/proc_fs.h>
#include <linux/version.h>
#include <net/checksum.h>

#define ASSERT_READ_LOCK(x) MUST_BE_READ_LOCKED(&ip_conntrack_lock)
#define ASSERT_WRITE_LOCK(x) MUST_BE_WRITE_LOCKED(&ip_conntrack_lock)

#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_protocol.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/listhelp.h>

/* 2002/4/18 Steve ++ */
#include <linux/timer.h>
static struct timer_list conntrack_timer;
/* Steve -- */

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

struct module *ip_conntrack_module = THIS_MODULE;

/* 2005/9/23 Hanson ++ */
static int
destory_conntrack_by_session(struct file *file, const char *input, unsigned long size, void *data)
{
	unsigned int i, len;
	unsigned long long mark;
	char buffer[1024], *end;
	int hour = (30 * 60);
	int conntrack_delay_timer = (1 * 60 + 10);
	struct timeval currenttvsec;
	
	if (size > 1024)
		len = 1024;
	else
		len = size;

	READ_LOCK(&ip_conntrack_lock);
	if (copy_from_user(buffer, input, len)) {
		READ_UNLOCK(&ip_conntrack_lock);
		return -EFAULT;
	}
	buffer[len - 1] = '\0';
//	mark = simple_strtoull(buffer, &end, 10);
	for (i = 0; i < ip_conntrack_htable_size; i++) {
		if (LIST_FIND(&ip_conntrack_hash[i], destory_conntrack_iterate_session,
			      struct ip_conntrack_tuple_hash *, &buffer))
			break;
	}

	READ_UNLOCK(&ip_conntrack_lock);
	do_gettimeofday(&currenttvsec);
	del_timer(&conntrack_timer);
	conntrack_timer.expires = jiffies + ((hour - (currenttvsec.tv_sec % hour) + conntrack_delay_timer) * HZ);
	add_timer(&conntrack_timer);
	return size;
}

static inline int
destory_conntrack_iterate_session(const struct ip_conntrack_tuple_hash *hash,
			  const char *buffer)
{
	char src_ip[20], dst_ip[20], match_src_ip[20], match_dst_ip[20], match_type[8], type[8];
	char sport[8], dport[8], match_sport[8], match_dport[8];
	int  urlblock_flag=0, policy_mark=0;
        unsigned long premark = (unsigned long)((hash->ctrack->mark >> 32) & 0xffffffff);

	memset(src_ip, '\0', sizeof(src_ip));
	memset(dst_ip, '\0', sizeof(dst_ip));
	memset(match_src_ip, '\0', sizeof(match_src_ip));
	memset(match_dst_ip, '\0', sizeof(match_dst_ip));
	memset(sport, '\0', sizeof(sport));
	memset(dport, '\0', sizeof(dport));
	memset(match_sport, '\0', sizeof(match_sport));
	memset(match_dport, '\0', sizeof(match_dport));
	memset(type, '\0', sizeof(type));
	memset(match_type, '\0', sizeof(match_type));

	if(buffer[0] != '+' && buffer[0] != '-')
		return 0;

        if((hash->ctrack->mark & 0xffffffff) > 0) {
            policy_mark = premark & 0xf;
            if(policy_mark < 1 || policy_mark > 6) {
                return 0;
            }
        } else {
                return 0;
        }
        urlblock_flag = (premark >> 12) & 0x1;

	sscanf(buffer+1, "%s %s %s %s %s", 
		match_src_ip, match_dst_ip, match_sport, match_dport, match_type);

        sprintf(src_ip, "%u.%u.%u.%u",
                   NIPQUAD(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip));
        if(urlblock_flag == 1) {
            sprintf(dst_ip, "%u.%u.%u.%u",
                   NIPQUAD(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip));
        } else {
            sprintf(dst_ip, "%u.%u.%u.%u",
                   NIPQUAD(hash->ctrack->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip));

        }
	if(strcmp(match_src_ip, src_ip))
		return 0;
	if(strcmp(match_dst_ip, dst_ip))
		return 0;

        if(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum == 17) {
            sprintf(sport, "%hu", ntohs(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port));
	    if(urlblock_flag == 1)
		sprintf(dport, "%hu", ntohs(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.udp.port));
	    else
		sprintf(dport, "%hu", ntohs(hash->ctrack->tuplehash[IP_CT_DIR_REPLY].tuple.src.u.udp.port));
        } else if(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum == 6) {
            sprintf(sport, "%hu", ntohs(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.tcp.port));
	    if(urlblock_flag == 1)
		sprintf(dport, "%hu", ntohs(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.tcp.port));
	    else
		sprintf(dport, "%hu", ntohs(hash->ctrack->tuplehash[IP_CT_DIR_REPLY].tuple.src.u.tcp.port));
	} else if(hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum == 1) {
	        sprintf(type, "%u", hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.icmp.type);
		if(strcmp(match_type, type))
			return 0;
		sprintf(sport, "0");
		sprintf(dport, "0");
	} else {
		sprintf(sport, "0");
		sprintf(dport, "0");
	}

	if(strcmp(match_sport, sport))
		return 0;
	if(strcmp(match_dport, dport))
		return 0;

	if(buffer[0] == '+') {
//		hash->ctrack->mark |= 0x8000000000000000;
		hash->ctrack->mark |= STATI_SESSION_MARK;
	} else if(buffer[0] == '-') {
		hash->ctrack->mark &= ~STATI_SESSION_MARK;
	}

	return 0;
}

static int
list_conntracks_session(char *buffer, char **start, off_t offset, int length)
{
	unsigned int i=0;
	unsigned int len = 0;
	off_t upto = 0;
	struct list_head *e;

	READ_LOCK(&ip_conntrack_lock);

	/* Traverse hash; print originals then reply. */
	for (i = 0; i < ip_conntrack_htable_size; i++) {
		if (LIST_FIND(&ip_conntrack_hash[i], conntrack_iterate_session,
			      struct ip_conntrack_tuple_hash *,
			      buffer, offset, &upto, &len, length))
			goto finished;
	}

 finished:
	READ_UNLOCK(&ip_conntrack_lock);

	/* `start' hack - see fs/proc/generic.c line ~165 */
	*start = (char *)((unsigned int)upto - offset);
	return len;

}

static inline int
conntrack_iterate_session(const struct ip_conntrack_tuple_hash *hash,
		  char *buffer, off_t offset, off_t *upto,
		  unsigned int *len, unsigned int maxlen)
{
	unsigned int newlen;
	IP_NF_ASSERT(hash->ctrack);

	MUST_BE_READ_LOCKED(&ip_conntrack_lock);

	/* Only count originals */
	if (DIRECTION(hash))
		return 0;

	if ((*upto)++ < offset)
		return 0;

	newlen = print_conntrack_session(buffer + *len, hash->ctrack);
	if (*len + newlen > maxlen)
		return 1;
	else *len += newlen;

	return 0;
}

static unsigned int
print_conntrack_session(char *buffer, const struct ip_conntrack *conntrack)
{
	unsigned int len=0;
	char	ip_tmp[20];
	int	i, policy_mark=0, urlblock_flag=0;
	struct ip_conntrack_protocol *proto
		= __find_proto(conntrack->tuplehash[IP_CT_DIR_ORIGINAL]
			       .tuple.dst.protonum);
/* 2004/5/27 Steve ++ */
        unsigned long premark = (unsigned long)((conntrack->mark >> 32) & 0xffffffff);
/* Steve -- */
#if 0
	memset(ip_tmp, '\0', sizeof(ip_tmp));
	sprintf(ip_tmp, "%u.%u.%u.%u", NIPQUAD(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip));
	if(!strcmp(ip_tmp, "127.0.0.1")) {
		return len;
	}
	sprintf(ip_tmp, "%u.%u.%u.%u", NIPQUAD(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip));
	if(!strcmp(ip_tmp, "127.0.0.1")) {
		return len;
	}
#endif
	if((conntrack->mark & 0xffffffff) > 0) {
	    policy_mark = premark & 0xf;
	    if(policy_mark < 1 || policy_mark > 6) {
		return len;
	    }
	} else {
		return len;
	}
	urlblock_flag = (premark >> 12) & 0x1;

	len = sprintf(buffer, "%s %u 0 0 ",
		      proto->name,
		      conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum);

#if defined(CONFIG_IP_NF_CONNTRACK_MARK)
	len += sprintf(buffer + len, "%lx%llx ", premark, conntrack->mark);
#else
	len += sprintf(buffer + len, "0 ");
#endif

	len += sprintf(buffer + len, "%u.%u.%u.%u ",
		NIPQUAD(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip)); 
        if(urlblock_flag == 1) {
	    len += sprintf(buffer + len, "%u.%u.%u.%u ",
		   NIPQUAD(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip));
	} else {
	    len += sprintf(buffer + len, "%u.%u.%u.%u ",
		   NIPQUAD(conntrack->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip));

	}

        if(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum == 17) {
                len += sprintf(buffer + len, "sport=%hu dport=%hu ",
                       ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port),
                       (urlblock_flag == 1) ? ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.udp.port) :
					      ntohs(conntrack->tuplehash[IP_CT_DIR_REPLY].tuple.src.u.udp.port));
        } else if(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum == 6) {
                len += sprintf(buffer + len, "sport=%hu dport=%hu ",
                       ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.tcp.port),
                       (urlblock_flag == 1) ? ntohs(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.tcp.port) :
					      ntohs(conntrack->tuplehash[IP_CT_DIR_REPLY].tuple.src.u.tcp.port));
	} else if(conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.protonum == 1) {
	        len += sprintf(buffer + len, "type=%u ",
                       conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.icmp.type);
	}

	len += sprintf(buffer + len, "policy=%d, urlblock=%d ", policy_mark, urlblock_flag);
/* 2004/4/21 Steve ++ */
#if 0
        if(conntrack->layer7.app_proto)
                len += sprintf(buffer + len, "l7proto=%s ",
                                conntrack->layer7.app_proto);
#endif
/* Steve -- */

	len += sprintf(buffer + len, "\n");

	return len;
}

static unsigned int
print_tuple_session(char *buffer, const struct ip_conntrack_tuple *tuple,
	    struct ip_conntrack_protocol *proto)
{
	int len;

	len = sprintf(buffer, "%u.%u.%u.%u %u.%u.%u.%u ",
		      NIPQUAD(tuple->src.ip), NIPQUAD(tuple->dst.ip));

	len += proto->print_tuple(buffer + len, tuple);

	return len;
}

char *my_index(char *str, char ch) {
        char *p;
        p=str;
        while(*p != ch) {
                p++;
                if(*p == '\0') return NULL;
        }

return p;
}
/* Hanson +. */

static unsigned int
print_tuple(char *buffer, const struct ip_conntrack_tuple *tuple,
	    struct ip_conntrack_protocol *proto)
{
	int len;

	len = sprintf(buffer, "src=%u.%u.%u.%u dst=%u.%u.%u.%u ",
		      NIPQUAD(tuple->src.ip), NIPQUAD(tuple->dst.ip));

	len += proto->print_tuple(buffer + len, tuple);

	return len;
}

/* FIXME: Don't print source proto part. --RR */
static unsigned int
print_expect(char *buffer, const struct ip_conntrack_expect *expect)
{
	unsigned int len;

	len = sprintf(buffer, "EXPECTING: proto=%u ",
		      expect->tuple.dst.protonum);
	len += print_tuple(buffer + len, &expect->tuple,
			   __find_proto(expect->tuple.dst.protonum));
	len += sprintf(buffer + len, "\n");
	return len;
}

static unsigned int
print_conntrack(char *buffer, const struct ip_conntrack *conntrack)
{
	unsigned int len;
	struct ip_conntrack_protocol *proto
		= __find_proto(conntrack->tuplehash[IP_CT_DIR_ORIGINAL]
			       .tuple.dst.protonum);
/* 2004/5/27 Steve ++ */
        unsigned long premark = (unsigned long)((conntrack->mark >> 32) & 0xffffffff);
/* Steve -- */


	len = sprintf(buffer, "%-8s %u %lu ",
		      proto->name,
		      conntrack->tuplehash[IP_CT_DIR_ORIGINAL]
		      .tuple.dst.protonum,
		      timer_pending(&conntrack->timeout)
		      ? (conntrack->timeout.expires - jiffies)/HZ : 0);

	len += proto->print_conntrack(buffer + len, conntrack);
	len += print_tuple(buffer + len,
			   &conntrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple,
			   proto);
	if (!(conntrack->status & IPS_SEEN_REPLY))
		len += sprintf(buffer + len, "[UNREPLIED] ");
	len += print_tuple(buffer + len,
			   &conntrack->tuplehash[IP_CT_DIR_REPLY].tuple,
			   proto);
	if (conntrack->status & IPS_ASSURED)
		len += sprintf(buffer + len, "[ASSURED] ");
	len += sprintf(buffer + len, "use=%u ",
		       atomic_read(&conntrack->ct_general.use));
#if defined(CONFIG_IP_NF_CONNTRACK_MARK)
/* 2002/3/14 Steve ++ */
//	len += sprintf(buffer + len, "mark=%d ", conntrack->mark);
/* 2004/5/27 Steve ++ */
//	len += sprintf(buffer + len, "mark=%llx ", conntrack->mark);
	len += sprintf(buffer + len, "mark=%lx%llx ", premark, conntrack->mark);
/* Steve -- */
/* Steve -- */
#endif

/* 2004/4/21 Steve ++ */
#if 0
        if(conntrack->layer7.app_proto)
                len += sprintf(buffer + len, "l7proto=%s ",
                                conntrack->layer7.app_proto);
#endif
/* Steve -- */

	len += sprintf(buffer + len, "\n");

	return len;
}

/* Returns true when finished. */
static inline int
conntrack_iterate(const struct ip_conntrack_tuple_hash *hash,
		  char *buffer, off_t offset, off_t *upto,
		  unsigned int *len, unsigned int maxlen)
{
	unsigned int newlen;
	IP_NF_ASSERT(hash->ctrack);

	MUST_BE_READ_LOCKED(&ip_conntrack_lock);

	/* Only count originals */
	if (DIRECTION(hash))
		return 0;

	if ((*upto)++ < offset)
		return 0;

	newlen = print_conntrack(buffer + *len, hash->ctrack);
	if (*len + newlen > maxlen)
		return 1;
	else *len += newlen;

	return 0;
}

/* 2002/4/4 Steve ++ */
static inline int
destory_conntrack_iterate(const struct ip_conntrack_tuple_hash *hash,
			  unsigned long long mark)
{
	unsigned long long match = hash->ctrack->mark;

	if (mark != 0) {
		if ((match & CHAIN_NAME_MASK) == mark)
			hash->ctrack->mark = (hash->ctrack->mark | REENTER_MARK) & (~POLICY_ACCEPT_MARK);
	} else
		hash->ctrack->mark = (hash->ctrack->mark | REENTER_MARK) & (~POLICY_ACCEPT_MARK);

	return 0;
}

static int
destory_conntrack_by_mark(struct file *file, const char *input, unsigned long size, void *data)
{
	unsigned int i, len;
	unsigned long long mark;
	char buffer[1024], *end;
	int hour = (30 * 60);
	int conntrack_delay_timer = (1 * 60 + 10);
	struct timeval currenttvsec;
	
	if (size > 1024)
		len = 1024;
	else
		len = size;

	READ_LOCK(&ip_conntrack_lock);
	if (copy_from_user(buffer, input, len)) {
		READ_UNLOCK(&ip_conntrack_lock);
		return -EFAULT;
	}
	buffer[len - 1] = '\0';
	mark = simple_strtoull(buffer, &end, 10);
	for (i = 0; i < ip_conntrack_htable_size; i++) {
		if (LIST_FIND(&ip_conntrack_hash[i], destory_conntrack_iterate,
			      struct ip_conntrack_tuple_hash *, mark))
			break;
	}

	READ_UNLOCK(&ip_conntrack_lock);
	do_gettimeofday(&currenttvsec);
	del_timer(&conntrack_timer);
	conntrack_timer.expires = jiffies + ((hour - (currenttvsec.tv_sec % hour) + conntrack_delay_timer) * HZ);
	add_timer(&conntrack_timer);
	return size;
}

#include <asm/io.h>

static void
destory_conntrack_by_timeout(unsigned long timeout)
{
	int i = 0;
	int hour = (30 * 60);
	int conntrack_delay_timer = (1 * 60 + 10);
	struct timeval currenttvsec;

	READ_LOCK(&ip_conntrack_lock);
	for (i = 0; i < ip_conntrack_htable_size; i++) {
		if (LIST_FIND(&ip_conntrack_hash[i], destory_conntrack_iterate,
			      struct ip_conntrack_tuple_hash *, 0))
                        break;
        }
        READ_UNLOCK(&ip_conntrack_lock);
	do_gettimeofday(&currenttvsec);
	conntrack_timer.expires = jiffies + ((hour - (currenttvsec.tv_sec % hour) + conntrack_delay_timer) * HZ);
	add_timer(&conntrack_timer);
}
/* Steve -- */
/* 2004/3/5 Steve ++ */
static inline int
count_conntrack_iterate(const struct ip_conntrack_tuple_hash *hash,
			unsigned long long mark, 
			int *count)
{
	unsigned long long match = hash->ctrack->mark;
#if 0
	__u32 osaddr;
#endif
	/* Only count originals */
	if (DIRECTION(hash))
		return 0;
#if 0
	osaddr = hash->ctrack->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
	if ((match & CHAIN_NAME_MASK) == mark &&
	    saddr == osaddr)
#endif
	if ((match & CHAIN_NAME_MASK) == mark)
		(*count)++ ;
	
	return 0;
}

#if 0
int
count_same_conntracks(unsigned long long mark, __u32 saddr)
#endif
int
count_same_conntracks(unsigned long long mark)
{
	int count = 0;
	int i = 0;
	
	READ_LOCK(&ip_conntrack_lock);
	for (i = 0; i < ip_conntrack_htable_size ; i++)
	{
		if (LIST_FIND(&ip_conntrack_hash[i],
			      count_conntrack_iterate, 
			      struct ip_conntrack_tuple_hash *, 
			      mark, 
			      &count))
			break;
	}
	READ_UNLOCK(&ip_conntrack_lock);
	return count;
}
/* Steve -- */
static int
list_conntracks(char *buffer, char **start, off_t offset, int length)
{
	unsigned int i;
	unsigned int len = 0;
	off_t upto = 0;
	struct list_head *e;

	READ_LOCK(&ip_conntrack_lock);
	/* Traverse hash; print originals then reply. */
	for (i = 0; i < ip_conntrack_htable_size; i++) {
		if (LIST_FIND(&ip_conntrack_hash[i], conntrack_iterate,
			      struct ip_conntrack_tuple_hash *,
			      buffer, offset, &upto, &len, length))
			goto finished;
	}

	/* Now iterate through expecteds. */
	for (e = expect_list.next; e != &expect_list; e = e->next) {
		unsigned int last_len;
		struct ip_conntrack_expect *expect
			= (struct ip_conntrack_expect *)e;
		if (upto++ < offset) continue;

		last_len = len;
		len += print_expect(buffer + len, expect);
		if (len > length) {
			len = last_len;
			goto finished;
		}
	}

 finished:
	READ_UNLOCK(&ip_conntrack_lock);

	/* `start' hack - see fs/proc/generic.c line ~165 */
	*start = (char *)((unsigned int)upto - offset);
	return len;
}

/* 2002/9/1 Steve ++ */
static int
destory_conntrack_by_status(struct file *file, const char *input, unsigned long size, void *data)
{
	unsigned int status, len, i;
	char buffer[1024], *end;
	
	if (size > 1024)
		len = 1024;
	else
		len = size;

	READ_LOCK(&ip_conntrack_lock);
	if (copy_from_user(buffer, input, len)) {
		READ_UNLOCK(&ip_conntrack_lock);
		return -EFAULT;
	}
	buffer[len - 1] = '\0';
	status = simple_strtoul(buffer, &end, 10);
	if (status)
		goto finish;

	for (i = 0; i < ip_conntrack_htable_size; i++) {
		while (1) {
			if (!early_drop(&ip_conntrack_hash[i]))
				break;
		}
	}

finish:
	READ_UNLOCK(&ip_conntrack_lock);
	return len;
}

static int
ip_conntrack_del_noreply(char *buffer, char **start, off_t offset, int length)
{
	unsigned int len = 0;
	return len;
}
/* Steve -- */

static unsigned int ip_confirm(unsigned int hooknum,
			       struct sk_buff **pskb,
			       const struct net_device *in,
			       const struct net_device *out,
			       int (*okfn)(struct sk_buff *))
{
	/* We've seen it coming out the other side: confirm it */
	return ip_conntrack_confirm(*pskb);
}

static unsigned int ip_refrag(unsigned int hooknum,
			      struct sk_buff **pskb,
			      const struct net_device *in,
			      const struct net_device *out,
			      int (*okfn)(struct sk_buff *))
{
	struct rtable *rt = (struct rtable *)(*pskb)->dst;

	/* We've seen it coming out the other side: confirm */
	if (ip_confirm(hooknum, pskb, in, out, okfn) != NF_ACCEPT)
		return NF_DROP;

	/* Local packets are never produced too large for their
	   interface.  We degfragment them at LOCAL_OUT, however,
	   so we have to refragment them here. */
	if (rt != NULL && (*pskb)->len > rt->u.dst.pmtu) {
		/* No hook can be after us, so this should be OK. */
		ip_fragment(*pskb, okfn);
		return NF_STOLEN;
	}
	return NF_ACCEPT;
}

static unsigned int ip_conntrack_local(unsigned int hooknum,
				       struct sk_buff **pskb,
				       const struct net_device *in,
				       const struct net_device *out,
				       int (*okfn)(struct sk_buff *))
{
	/* root is playing with raw sockets. */
	if ((*pskb)->len < sizeof(struct iphdr)
	    || (*pskb)->nh.iph->ihl * 4 < sizeof(struct iphdr)) {
		if (net_ratelimit())
			printk("ipt_hook: happy cracking.\n");
		return NF_ACCEPT;
	}
	return ip_conntrack_in(hooknum, pskb, in, out, okfn);
}

/* Connection tracking may drop packets, but never alters them, so
   make it the first hook. */
static struct nf_hook_ops ip_conntrack_in_ops
= { { NULL, NULL }, ip_conntrack_in, PF_INET, NF_IP_PRE_ROUTING,
	NF_IP_PRI_CONNTRACK };
static struct nf_hook_ops ip_conntrack_local_out_ops
= { { NULL, NULL }, ip_conntrack_local, PF_INET, NF_IP_LOCAL_OUT,
	NF_IP_PRI_CONNTRACK };
/* Refragmenter; last chance. */
static struct nf_hook_ops ip_conntrack_out_ops
= { { NULL, NULL }, ip_refrag, PF_INET, NF_IP_POST_ROUTING, NF_IP_PRI_LAST };
static struct nf_hook_ops ip_conntrack_local_in_ops
= { { NULL, NULL }, ip_confirm, PF_INET, NF_IP_LOCAL_IN, NF_IP_PRI_LAST-1 };

static int init_or_cleanup(int init)
{
	struct proc_dir_entry *proc;
/* 2002/9/1 Steve ++ */
	struct proc_dir_entry *proc_status;
/* Steve -- */
/* 2005/9/23 Hanson ++ */
        struct proc_dir_entry *proc_session;
/* Hanson +. */

	int ret = 0;
	unsigned long hour = (30 * 60);
	int conntrack_delay_timer = (1 * 60 + 10);
	struct timeval currenttvsec;

	if (!init) goto cleanup;

	ret = ip_conntrack_init();
	if (ret < 0)
		goto cleanup_nothing;

/* 2002/9/1 Steve ++ */
	proc_status = proc_net_create("ip_conntrack_del_noreply", 0,
				     ip_conntrack_del_noreply);
	if (!proc_status) goto cleanup_init;
	proc_status->owner = THIS_MODULE;
	proc_status->write_proc = destory_conntrack_by_status;
/* Steve -- */

	proc = proc_net_create("ip_conntrack",0,list_conntracks);
	if (!proc) goto cleanup_proc_status;
	proc->owner = THIS_MODULE;
/* 2002/4/4 Steve ++ */
	proc->write_proc = destory_conntrack_by_mark;
/* Steve -- */

/* 2005/9/23 Hanson ++ */
        proc_session = proc_net_create("ip_conntrack_session", 0,
                                     list_conntracks_session);
        if (!proc_session) goto cleanup_proc_session;
        proc_session->owner = THIS_MODULE;
        proc_session->write_proc = destory_conntrack_by_session;
/* Hanson +. */

/* 2002/4/18 Steve ++ */
	init_timer(&conntrack_timer);
	do_gettimeofday(&currenttvsec);
	conntrack_timer.function = destory_conntrack_by_timeout;
	conntrack_timer.expires = jiffies + ((hour - (currenttvsec.tv_sec % hour) + conntrack_delay_timer) * HZ);
	add_timer(&conntrack_timer);
/* Steve -- */

	ret = nf_register_hook(&ip_conntrack_in_ops);
	if (ret < 0) {
		printk("ip_conntrack: can't register in hook.\n");
		goto cleanup_proc;
	}
	ret = nf_register_hook(&ip_conntrack_local_out_ops);
	if (ret < 0) {
		printk("ip_conntrack: can't register local out hook.\n");
		goto cleanup_inops;
	}
	ret = nf_register_hook(&ip_conntrack_out_ops);
	if (ret < 0) {
		printk("ip_conntrack: can't register post-routing hook.\n");
		goto cleanup_inandlocalops;
	}
	ret = nf_register_hook(&ip_conntrack_local_in_ops);
	if (ret < 0) {
		printk("ip_conntrack: can't register local in hook.\n");
		goto cleanup_inoutandlocalops;
	}

	return ret;

 cleanup:
	nf_unregister_hook(&ip_conntrack_local_in_ops);
 cleanup_inoutandlocalops:
	nf_unregister_hook(&ip_conntrack_out_ops);
 cleanup_inandlocalops:
	nf_unregister_hook(&ip_conntrack_local_out_ops);
 cleanup_inops:
	nf_unregister_hook(&ip_conntrack_in_ops);
 cleanup_proc:
	proc_net_remove("ip_conntrack");
/* 2002/9/1 Steve ++ */
 cleanup_proc_status:
	proc_net_remove("ip_conntrack_del_noreply");
/* Steve -- */
/* 2005/9/23 Hanson ++ */
 cleanup_proc_session:
        proc_net_remove("ip_conntrack_session");
/* Hanson +. */

 cleanup_init:
	ip_conntrack_cleanup();
 cleanup_nothing:
	return ret;
}

/* FIXME: Allow NULL functions and sub in pointers to generic for
   them. --RR */
int ip_conntrack_protocol_register(struct ip_conntrack_protocol *proto)
{
	int ret = 0;
	struct list_head *i;

	WRITE_LOCK(&ip_conntrack_lock);
	for (i = protocol_list.next; i != &protocol_list; i = i->next) {
		if (((struct ip_conntrack_protocol *)i)->proto
		    == proto->proto) {
			ret = -EBUSY;
			goto out;
		}
	}

	list_prepend(&protocol_list, proto);
	MOD_INC_USE_COUNT;

 out:
	WRITE_UNLOCK(&ip_conntrack_lock);
	return ret;
}

/* FIXME: Implement this --RR */
#if 0
void ip_conntrack_protocol_unregister(struct ip_conntrack_protocol *proto)
{
}
#endif

static int __init init(void)
{
	return init_or_cleanup(1);
}

static void __exit fini(void)
{
	init_or_cleanup(0);
}

module_init(init);
module_exit(fini);

EXPORT_SYMBOL(ip_conntrack_protocol_register);
EXPORT_SYMBOL(invert_tuplepr);
EXPORT_SYMBOL(ip_conntrack_alter_reply);
EXPORT_SYMBOL(ip_conntrack_destroyed);
EXPORT_SYMBOL(ip_conntrack_get);
EXPORT_SYMBOL(ip_conntrack_module);
EXPORT_SYMBOL(ip_conntrack_helper_register);
EXPORT_SYMBOL(ip_conntrack_helper_unregister);
EXPORT_SYMBOL(ip_ct_selective_cleanup);
EXPORT_SYMBOL(ip_ct_refresh);
EXPORT_SYMBOL(ip_conntrack_expect_related);
EXPORT_SYMBOL(ip_conntrack_change_expect);
EXPORT_SYMBOL(ip_conntrack_unexpect_related);
EXPORT_SYMBOL(ip_conntrack_find_get);
EXPORT_SYMBOL(ip_conntrack_tuple_taken);
EXPORT_SYMBOL(ip_ct_gather_frags);
EXPORT_SYMBOL(ip_conntrack_htable_size);
EXPORT_SYMBOL(ip_conntrack_lock);
