#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/udp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>

MODULE_DESCRIPTION("AMIT 'brute force' connection tracking module");

DECLARE_LOCK_EXTERN(ip_amit_lock);
struct module *ip_nat_amit = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static unsigned int
amit_nat_expected(struct sk_buff **pskb,
		  unsigned int hooknum,
		  struct ip_conntrack *ct,
		  struct ip_nat_info *info)
{
	struct ip_nat_multi_range mr;
	u_int32_t newip, newsrcip, newdstip;
	u_int16_t port;
	struct ip_conntrack *master = master_ct(ct);
	unsigned int ret;

	IP_NF_ASSERT(info);
	IP_NF_ASSERT(master);

	IP_NF_ASSERT(!(info->initialized & (1<<HOOK2MANIP(hooknum))));

	DEBUGP("amit_nat_expected: We have a connection!\n");
	LOCK_BH(&ip_amit_lock);
	DEBUGP("master: ");
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_REPLY].tuple);
	DEBUGP("conntrack: ");
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);

        newdstip = master->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip;
        newsrcip = master->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;

	port = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.all;
	UNLOCK_BH(&ip_amit_lock);

        if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_SRC)
                newip = newsrcip;
        else
                newip = newdstip;

	DEBUGP("amit_nat_expected: IP to %u.%u.%u.%u\n", NIPQUAD(newip));
	DEBUGP("amit_nat_expected: Port to %u\n", ntohs(port));
	
	mr.rangesize = 1;
	/* We don't want to manip the per-protocol, just the IPs... */
	mr.range[0].flags = IP_NAT_RANGE_MAP_IPS;
	mr.range[0].min_ip = mr.range[0].max_ip = newip;

	/* ... unless we're doing a MANIP_DST, in which case, make
	   sure we map to the correct port */
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_DST) {
		mr.range[0].flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		mr.range[0].min = mr.range[0].max
			= ((union ip_conntrack_manip_proto)
				{ port });
	}

	ret = ip_nat_setup_info(ct, &mr, hooknum);
	return ret;
}

static unsigned int 
amit_nat_help(struct ip_conntrack *ct,
	      struct ip_conntrack_expect *exp,
	      struct ip_nat_info *info,
	      enum ip_conntrack_info ctinfo,
	      unsigned int hooknum,
	      struct sk_buff **pskb)
{
	return NF_ACCEPT;
}

static struct ip_nat_helper amit_help = { { NULL, NULL },
				     { { 0, { __constant_htons(407) } },
				       { 0, { 0 }, IPPROTO_UDP } },
				     { { 0, { 0xFFFF } },
				       { 0, { 0 }, 0xFFFF } },
				     amit_nat_help, 
				     amit_nat_expected,
				     NULL, 
				     "AMIT" };

static int __init init(void)
{
	int ret;
	
	ret = ip_nat_helper_register(&amit_help);
	if (ret != 0) {
		printk("ip_nat_amit: cannot initialize the module!\n");
		return ret;
	}
	return ret;
}

static void __exit fini(void)
{
	ip_nat_helper_unregister(&amit_help);
}

module_init(init);
module_exit(fini);
