/* 
 * H.323 'brute force' extension for NAT alteration. 
 * Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>
 *
 * Based on ip_masq_h323.c for 2.2 kernels from CoRiTel, Sofia project.
 * (http://www.coritel.it/projects/sofia/nat.html)
 * Uses Sampsa Ranta's excellent idea on using expectfn to 'bind'
 * the unregistered helpers to the conntrack entries.
 */


#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/tcp.h>
/* 2003/10/16 Steve ++ */
#include <net/udp.h>
/* Steve -- */

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_h323.h>

MODULE_AUTHOR("Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>");
MODULE_DESCRIPTION("H.323 'brute force' connection tracking module");

DECLARE_LOCK_EXTERN(ip_h323_lock);
struct module *ip_nat_h323 = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

/* 2003/10/15 Steve ++ */
static int udp_h225_nat_mangle(struct ip_conntrack *ct,
                               enum ip_conntrack_info ctinfo,
                               struct sk_buff **pskb)
{
        return NF_ACCEPT;
}

static int udp_h225_address_fixup(struct ip_conntrack *ct,
                                  struct sk_buff **pskb,
                                  enum ip_conntrack_info ctinfo)
{
        struct iphdr *iph = (*pskb)->nh.iph;
        struct udphdr *udph = (void *)iph + iph->ihl*4;
        char *data = (char *) udph + sizeof(struct udphdr);
        u_int32_t udplen = (*pskb)->len - iph->ihl*4;
        int datalen = udplen - sizeof(struct udphdr);
        char *data_limit = data + datalen;
        struct ip_ct_h225_master *info = &ct->help.ct_h225_info;
        u_int32_t newip, oldip, data_ip;
        int mangle = 0;

        MUST_BE_LOCKED(&ip_h323_lock);
        if (info->dir == IP_CT_DIR_REPLY) {
                /* Must be where client thinks server is */
                newip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
                oldip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip;
        } else {
                /* Must be where server thinks client is */
                newip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
                oldip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
        }

        for (; data < (data_limit - 5); data++) {
                data_ip = *((u_int32_t *)data);
                if (data_ip == oldip) {
                        mangle = 1;
                        *(u_int32_t *)data = newip;
                }
        }

        if (mangle) {
                (*pskb)->csum = csum_partial((char *)udph + sizeof(struct udphdr), udplen - sizeof(struct udphdr), 0);
                udph->check = 0;
                udph->check = csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP, csum_partial((char *)udph, sizeof(struct udphdr), (*pskb)->csum));
        }
        return 1;
}

static unsigned int udp_h225_nat_help(struct ip_conntrack *ct,
                                      struct ip_conntrack_expect *exp,
                                      struct ip_nat_info *info,
                                      enum ip_conntrack_info ctinfo,
                                      unsigned int hooknum,
                                      struct sk_buff **pskb)
{
        int dir;

        dir = CTINFO2DIR(ctinfo);
        if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
              || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
                DEBUGP("talk_help: Not touching dir %s at hook %s\n",
                       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
                       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
                       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
                       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
                return NF_ACCEPT;
        }

        LOCK_BH(&ip_h323_lock);
        if (ct)
                udp_h225_address_fixup(ct, pskb, ctinfo);
        UNLOCK_BH(&ip_h323_lock);
        return NF_ACCEPT;
}
static struct ip_nat_helper udp_h245 = { { NULL, NULL },
                                        { { 0, { 0 } },
                                          { 0, { 0 }, IPPROTO_UDP } },
                                        { { 0, { 0xFFFF } },
                                          { 0, { 0 }, 0xFFFF } },
                                        udp_h225_nat_help,
                                        NULL,
                                        udp_h225_nat_mangle,
                                        "H.245_UDP" };
/* Steve -- */
/* FIXME: Time out? --RR */

static unsigned int 
h225_nat_expected(struct sk_buff **pskb,
		  unsigned int hooknum,
		  struct ip_conntrack *ct,
		  struct ip_nat_info *info);

static unsigned int h225_nat_help(struct ip_conntrack *ct,
				  struct ip_conntrack_expect *exp,
			 	  struct ip_nat_info *info,
			 	  enum ip_conntrack_info ctinfo,
			 	  unsigned int hooknum,
			 	  struct sk_buff **pskb);
		  
static struct ip_nat_helper h245 = { { NULL, NULL },
				     { { 0, { 0 } },
				       { 0, { 0 }, IPPROTO_TCP } },
				     { { 0, { 0xFFFF } },
				       { 0, { 0 }, 0xFFFF } },
				     h225_nat_help,
				     h225_nat_expected,
				     NULL,
				     "H.245" };

static unsigned int
h225_nat_expected(struct sk_buff **pskb,
		  unsigned int hooknum,
		  struct ip_conntrack *ct,
		  struct ip_nat_info *info)
{
	struct ip_nat_multi_range mr;
	u_int32_t newdstip, newsrcip, newip;
	u_int16_t port;
	struct ip_ct_h225_expect *exp_info;
	struct ip_ct_h225_master *master_info;
	struct ip_conntrack *master = master_ct(ct);
	unsigned int is_h225, ret;
/* 2003/10/15 Steve ++ */
        struct iphdr *iph = (*pskb)->nh.iph;
/* Steve -- */
	
	IP_NF_ASSERT(info);
	IP_NF_ASSERT(master);

	IP_NF_ASSERT(!(info->initialized & (1<<HOOK2MANIP(hooknum))));

	DEBUGP("h225_nat_expected: We have a connection!\n");
	master_info = &ct->master->expectant->help.ct_h225_info;
	exp_info = &ct->master->help.exp_h225_info;

	LOCK_BH(&ip_h323_lock);

	DEBUGP("master: ");
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_REPLY].tuple);
	DEBUGP("conntrack: ");
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	if (exp_info->dir == IP_CT_DIR_ORIGINAL) {
		/* Make connection go to the client. */
		newdstip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
		DEBUGP("h225_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to client)\n",
		       NIPQUAD(newsrcip), NIPQUAD(newdstip));
	} else {
		/* Make the connection go to the server */
		newdstip = master->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip;
		newsrcip = master->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
		DEBUGP("h225_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to server)\n",
		       NIPQUAD(newsrcip), NIPQUAD(newdstip));
	}
	port = exp_info->port;
	is_h225 = master_info->is_h225 == IP_CT_IS_H225;
	UNLOCK_BH(&ip_h323_lock);
	
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_SRC)
		newip = newsrcip;
	else
		newip = newdstip;

	DEBUGP("h225_nat_expected: IP to %u.%u.%u.%u\n", NIPQUAD(newip));

	mr.rangesize = 1;
	/* We don't want to manip the per-protocol, just the IPs... */
	mr.range[0].flags = IP_NAT_RANGE_MAP_IPS;
	mr.range[0].min_ip = mr.range[0].max_ip = newip;

	/* ... unless we're doing a MANIP_DST, in which case, make
	   sure we map to the correct port */
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_DST) {
		mr.range[0].flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		mr.range[0].min = mr.range[0].max
			= ((union ip_conntrack_manip_proto)
				{ port });
	}

	ret = ip_nat_setup_info(ct, &mr, hooknum);
	
	if (is_h225) {
		DEBUGP("h225_nat_expected: H.225, setting NAT helper for %p\n", ct);
		/* NAT expectfn called with ip_nat_lock write-locked */
/* 2003/10/15 Steve ++ */
                if (iph->protocol == IPPROTO_UDP)
                        info->helper = &udp_h245;
                else
                        info->helper = &h245;
/* Steve -- */
	}
	return ret;
}

static int h323_signal_address_fixup(struct ip_conntrack *ct,
			      	     struct sk_buff **pskb,
			      	     enum ip_conntrack_info ctinfo)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct tcphdr *tcph = (void *)iph + iph->ihl*4;
	char *data = (char *) tcph + tcph->doff * 4;
	u_int32_t tcplen = (*pskb)->len - iph->ihl*4;
	struct ip_ct_h225_master *info = &ct->help.ct_h225_info; 
	u_int32_t newip;
	u_int16_t port;
	int i;

	MUST_BE_LOCKED(&ip_h323_lock);
	DEBUGP("h323_signal_address_fixup: offsets %u + 6  and %u + 6 in %u\n", 
		info->offset[IP_CT_DIR_ORIGINAL], 
		info->offset[IP_CT_DIR_REPLY],
		tcplen);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_REPLY].tuple);

	for (i = 0; i < IP_CT_DIR_MAX; i++) {
		DEBUGP("h323_signal_address_fixup: %s %s\n",
			info->dir == IP_CT_DIR_ORIGINAL ? "original" : "reply",
			i == IP_CT_DIR_ORIGINAL ? "caller" : "callee");
		if (!between(info->seq[i], ntohl(tcph->seq), 
			     ntohl(tcph->seq) + tcplen - tcph->doff * 4))
			continue;
		if (!between(info->seq[i] + 6, ntohl(tcph->seq),
			     ntohl(tcph->seq) + tcplen - tcph->doff * 4)) {
			/* Partial retransmisison. It's a cracker being funky. */
			if (net_ratelimit()) {
				printk("H.323_NAT: partial packet %u/6 in %u/%u\n",
				     info->seq[i],
				     ntohl(tcph->seq),
				     ntohl(tcph->seq) + tcplen - tcph->doff * 4);
			}
			return NF_DROP;
		}

		/* Change address inside packet to match way we're mapping
		   this connection. */
		if (i == IP_CT_DIR_ORIGINAL) {
			newip = ct->tuplehash[!info->dir].tuple.dst.ip;
			port = ct->tuplehash[!info->dir].tuple.dst.u.tcp.port;
		} else {
			newip = ct->tuplehash[!info->dir].tuple.src.ip;
			port = ct->tuplehash[!info->dir].tuple.src.u.tcp.port;
		}

		DEBUGP("h323_signal_address_fixup: orig %s IP:port %u.%u.%u.%u:%u\n", 
			i == IP_CT_DIR_ORIGINAL ? "source" : "dest  ", 
			NIPQUAD(*((u_int32_t *)(data + info->offset[i]))), 
			ntohs(*((u_int16_t *)(data + info->offset[i] + 4))));

		/* Modify the packet */
		*(u_int32_t *)(data + info->offset[i]) = newip;
		*(u_int16_t *)(data + info->offset[i] + 4) = port;
	
		DEBUGP("h323_signal_address_fixup:  new %s IP:port %u.%u.%u.%u:%u\n", 
			i == IP_CT_DIR_ORIGINAL ? "source" : "dest  ", 
			NIPQUAD(*((u_int32_t *)(data + info->offset[i]))), 
			ntohs(*((u_int16_t *)(data + info->offset[i] + 4))));
	}

	/* fix checksum information */

	(*pskb)->csum = csum_partial((char *)tcph + tcph->doff*4,
				     tcplen - tcph->doff*4, 0);

	tcph->check = 0;
	tcph->check = tcp_v4_check(tcph, tcplen, iph->saddr, iph->daddr,
				   csum_partial((char *)tcph, tcph->doff*4,
					   (*pskb)->csum));
	ip_send_check(iph);

	return 1;
}

static int h323_data_fixup(struct ip_ct_h225_expect *info,
			   struct ip_conntrack *ct,
			   struct sk_buff **pskb,
			   enum ip_conntrack_info ctinfo,
			   struct ip_conntrack_expect *expect)
{
	u_int32_t newip;
	struct ip_conntrack_tuple newtuple;
	struct iphdr *iph = (*pskb)->nh.iph;
	struct tcphdr *tcph = (void *)iph + iph->ihl*4;
	char *data = (char *) tcph + tcph->doff * 4;
	u_int32_t tcplen = (*pskb)->len - iph->ihl*4;
	struct ip_ct_h225_master *master_info = &ct->help.ct_h225_info;
	int is_h225;
/* 2003/10/15 Steve ++ */
        u_int32_t datalen = tcplen - tcph->doff * 4;
        char *data_limit, *dataptr;
        u_int32_t data_ip, oldip;
/* Steve -- */

	MUST_BE_LOCKED(&ip_h323_lock);
	DEBUGP("h323_data_fixup: offset %u + 6 in %u\n", info->offset, tcplen);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_REPLY].tuple);

	if (!between(expect->seq + 6, ntohl(tcph->seq),
		    ntohl(tcph->seq) + tcplen - tcph->doff * 4)) {
		/* Partial retransmisison. It's a cracker being funky. */
		if (net_ratelimit()) {
			DEBUGP("H.323_NAT: partial packet %u/6 in %u/%u\n",
			     expect->seq,
			     ntohl(tcph->seq),
			     ntohl(tcph->seq) + tcplen - tcph->doff * 4);
		}
		return NF_DROP;
	}

	/* Change address inside packet to match way we're mapping
	   this connection. */
	if (info->dir == IP_CT_DIR_REPLY) {
		/* Must be where client thinks server is */
		newip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
/* 2003/10/15 Steve ++ */
                oldip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip;
/* Steve -- */
		/* Expect something from client->server */
		newtuple.src.ip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		newtuple.dst.ip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
	} else {
		/* Must be where server thinks client is */
		newip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
/* 2003/10/15 Steve ++ */
                oldip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
/* Steve -- */
		/* Expect something from server->client */
		newtuple.src.ip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip;
		newtuple.dst.ip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
	}

	is_h225 = (master_info->is_h225 == IP_CT_IS_H225);

	if (is_h225) {
/* 2003/10/15 Steve ++ */
//              newtuple.dst.protonum = IPPROTO_TCP;
                newtuple.dst.protonum = expect->tuple.dst.protonum;
/* Steve -- */
		newtuple.src.u.tcp.port = expect->tuple.src.u.tcp.port;
/* 2003/10/16 Steve ++ */
		if (newtuple.dst.protonum == IPPROTO_TCP)
			newtuple.dst.u.tcp.port = expect->tuple.dst.u.tcp.port;
		else
			newtuple.dst.u.udp.port = expect->tuple.dst.u.udp.port;
/* Steve -- */
	} else {
		newtuple.dst.protonum = IPPROTO_UDP;
		newtuple.src.u.udp.port = expect->tuple.src.u.udp.port;
/* 2003/10/16 Steve ++ */
		newtuple.dst.u.udp.port = expect->tuple.dst.u.udp.port;
/* Steve -- */
	}
#if 0	
	/* Try to get same port: if not, try to change it. */
	for (port = ntohs(info->port); port != 0; port++) {
		if (is_h225)
			newtuple.dst.u.tcp.port = htons(port);
		else
			newtuple.dst.u.udp.port = htons(port);

		if (ip_conntrack_change_expect(expect, &newtuple) == 0)
			break;
	}
	if (port == 0) {
		DEBUGP("h323_data_fixup: no free port found!\n");
		return 0;
	}

	port = htons(port);
#endif
	DEBUGP("h323_data_fixup: orig IP:port %u.%u.%u.%u:%u\n", 
		NIPQUAD(*((u_int32_t *)(data + info->offset))), 
		ntohs(*((u_int16_t *)(data + info->offset + 4))));

/* 2003/10/15 Steve ++ */
	ip_conntrack_change_expect(expect, &newtuple);
        data_limit = data + datalen;
        dataptr = data;
        for (; dataptr < (data_limit - 5); dataptr++) {
                data_ip = *((u_int32_t *)dataptr);
                if (data_ip == oldip)
                        *(u_int32_t *)dataptr = newip;
        }
#if 0
        /* Modify the packet */
        *(u_int32_t *)(data + info->offset) = newip;
        *(u_int16_t *)(data + info->offset + 4) = port;
#endif
/* Steve -- */
	
	DEBUGP("h323_data_fixup: new IP:port %u.%u.%u.%u:%u\n", 
		NIPQUAD(*((u_int32_t *)(data + info->offset))), 
		ntohs(*((u_int16_t *)(data + info->offset + 4))));

	/* fix checksum information  */
	/* FIXME: usually repeated multiple times in the case of H.245! */

	(*pskb)->csum = csum_partial((char *)tcph + tcph->doff*4,
				     tcplen - tcph->doff*4, 0);

	tcph->check = 0;
	tcph->check = tcp_v4_check(tcph, tcplen, iph->saddr, iph->daddr,
				   csum_partial((char *)tcph, tcph->doff*4,
					   (*pskb)->csum));
	ip_send_check(iph);

	return 1;
}

static int h225_nat_mangle(struct ip_conntrack *ct,
			   enum ip_conntrack_info ctinfo,
			   struct sk_buff **pskb)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct tcphdr *tcph = (struct tcphdr *)((u_int32_t *)iph + iph->ihl);
	unsigned int datalen;
	struct ip_ct_h225_master *info = &ct->help.ct_h225_info;

	datalen = (*pskb)->len - iph->ihl*4 - tcph->doff*4;

	return (between(info->seq[IP_CT_DIR_ORIGINAL], ntohl(tcph->seq), ntohl(tcph->seq) + datalen)
		|| between(info->seq[IP_CT_DIR_REPLY], ntohl(tcph->seq), ntohl(tcph->seq) + datalen));
}


static unsigned int h225_nat_help(struct ip_conntrack *ct,
				  struct ip_conntrack_expect *exp,
			 	  struct ip_nat_info *info,
			 	  enum ip_conntrack_info ctinfo,
			 	  unsigned int hooknum,
			 	  struct sk_buff **pskb)
{
	int dir;
	struct ip_ct_h225_expect *exp_info;
	
	/* Only mangle things once: original direction in POST_ROUTING
	   and reply direction on PRE_ROUTING. */
	dir = CTINFO2DIR(ctinfo);
	if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
	      || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
		DEBUGP("nat_h323: Not touching dir %s at hook %s\n",
		       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
		       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
		       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
		       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
		return NF_ACCEPT;
	}

	if (!exp) {
		LOCK_BH(&ip_h323_lock);
		if (!h323_signal_address_fixup(ct, pskb, ctinfo)) {
			UNLOCK_BH(&ip_h323_lock);
			return NF_DROP;
		}
		UNLOCK_BH(&ip_h323_lock);
		return NF_ACCEPT;
	}
		
	exp_info = &exp->help.exp_h225_info;

	LOCK_BH(&ip_h323_lock);
	if (!h323_data_fixup(exp_info, ct, pskb, ctinfo, exp)) {
		UNLOCK_BH(&ip_h323_lock);
		return NF_DROP;
	}
	UNLOCK_BH(&ip_h323_lock);

	return NF_ACCEPT;
}

static struct ip_nat_helper h225 = { { NULL, NULL },
				     { { 0, { __constant_htons(IP_CT_IS_H225) } },
				       { 0, { 0 }, IPPROTO_TCP } },
				     { { 0, { 0xFFFF } },
				       { 0, { 0 }, 0xFFFF } },
				     h225_nat_help, 
				     h225_nat_expected,
				     h225_nat_mangle,
				     "H.225" };

/* 2003/10/16 Steve ++ */
static unsigned int
gatekeeper_nat_expected(struct sk_buff **pskb,
		  	unsigned int hooknum,
		  	struct ip_conntrack *ct,
		  	struct ip_nat_info *info)
{
	struct ip_nat_multi_range mr;
	u_int32_t newdstip, newsrcip, newip;
	u_int16_t port;
	struct ip_ct_h225_expect *exp_info;
	struct ip_ct_h225_master *master_info;
	struct ip_conntrack *master = master_ct(ct);
	unsigned int ret;
	
	IP_NF_ASSERT(info);
	IP_NF_ASSERT(master);

	IP_NF_ASSERT(!(info->initialized & (1<<HOOK2MANIP(hooknum))));

	DEBUGP("h225_nat_expected: We have a connection!\n");
	master_info = &ct->master->expectant->help.ct_h225_info;
	exp_info = &ct->master->help.exp_h225_info;

	LOCK_BH(&ip_h323_lock);

	DEBUGP("master: ");
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_REPLY].tuple);
	DEBUGP("conntrack: ");
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	if (exp_info->dir == IP_CT_DIR_ORIGINAL) {
		/* Make connection go to the client. */
		newdstip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
		DEBUGP("h225_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to client)\n",
		       NIPQUAD(newsrcip), NIPQUAD(newdstip));
	} else {
		/* Make the connection go to the server */
		newdstip = master->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip;
		newsrcip = master->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
		DEBUGP("h225_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to server)\n",
		       NIPQUAD(newsrcip), NIPQUAD(newdstip));
	}
	port = exp_info->port;
	UNLOCK_BH(&ip_h323_lock);
	
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_SRC)
		newip = newsrcip;
	else
		newip = newdstip;

	mr.rangesize = 1;
	/* We don't want to manip the per-protocol, just the IPs... */
	mr.range[0].flags = IP_NAT_RANGE_MAP_IPS;
	mr.range[0].min_ip = mr.range[0].max_ip = newip;

	/* ... unless we're doing a MANIP_DST, in which case, make
	   sure we map to the correct port */
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_DST) {
		mr.range[0].flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		mr.range[0].min = mr.range[0].max
			= ((union ip_conntrack_manip_proto)
				{ port });
	}

	ret = ip_nat_setup_info(ct, &mr, hooknum);
	info->helper = &h225;
	return ret;
}

static int gatekeeper_nat_mangle(struct ip_conntrack *ct,
			         enum ip_conntrack_info ctinfo,
			         struct sk_buff **pskb)
{
	return NF_ACCEPT;
}

static int gatekeeper_data_fixup(struct ip_ct_h225_expect *info,
			   	 struct ip_conntrack *ct,
			  	 struct sk_buff **pskb,
			 	 enum ip_conntrack_info ctinfo,
			 	 struct ip_conntrack_expect *expect)
{
	struct ip_conntrack_tuple newtuple;
	struct iphdr *iph = (*pskb)->nh.iph;
        struct udphdr *udph = (void *)iph + iph->ihl*4;
        char *data = (char *) udph + sizeof(struct udphdr);
        u_int32_t udplen = (*pskb)->len - iph->ihl*4;
        int datalen = udplen - sizeof(struct udphdr);
        char *data_limit, *dataptr;
        u_int32_t newsrcip, oldsrcip, data_ip;
	int mangle = 0;

	MUST_BE_LOCKED(&ip_h323_lock);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_REPLY].tuple);

	if (info->dir == IP_CT_DIR_REPLY) {
                oldsrcip = ct->tuplehash[info->dir].tuple.src.ip;
                newsrcip = ct->tuplehash[!info->dir].tuple.dst.ip;
	}  else
		return 1;

	newtuple.src.ip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
	newtuple.dst.ip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
	newtuple.dst.protonum = expect->tuple.dst.protonum;
	newtuple.src.u.tcp.port = expect->tuple.src.u.tcp.port;
	newtuple.dst.u.tcp.port = expect->tuple.dst.u.tcp.port;
	ip_conntrack_change_expect(expect, &newtuple);

	if (datalen < 5)
		return 1;

        data_limit = data + datalen;
	dataptr = data;
        for (; dataptr < (data_limit - 5); dataptr++) {
                data_ip = *((u_int32_t *)dataptr);
                if (data_ip == oldsrcip) {
                        mangle = 1;
                        *(u_int32_t *)dataptr = newsrcip;
                }
        }

        if (mangle) {
		(*pskb)->csum = csum_partial((char *)udph + sizeof(struct udphdr), udplen - sizeof(struct udphdr), 0);
		udph->check = 0;
		udph->check = csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP, csum_partial((char *)udph, sizeof(struct udphdr), (*pskb)->csum));
	}

	return 1;
}

static int gatekeeper_address_fixup(struct ip_conntrack *ct,
                                    struct sk_buff **pskb,
                                    enum ip_conntrack_info ctinfo)
{
        struct iphdr *iph = (*pskb)->nh.iph;
        struct udphdr *udph = (void *)iph + iph->ihl*4;
        u_int32_t udplen = (*pskb)->len - iph->ihl*4;
	int datalen = udplen - sizeof(struct udphdr);
        char *data = (char *) udph + sizeof(struct udphdr);
	struct ip_ct_h225_master *info = &ct->help.ct_h225_info;
        char *data_limit = data + datalen;
        u_int32_t newsrcip, oldsrcip, data_ip;
        int mangle = 0;
	char *dataptr;
	int dir;

        MUST_BE_LOCKED(&ip_h323_lock);
        dir = CTINFO2DIR(ctinfo);
        if (dir != IP_CT_DIR_REPLY || datalen < 5)
		return 1;

	oldsrcip = ct->tuplehash[info->dir].tuple.src.ip;
	newsrcip = ct->tuplehash[!info->dir].tuple.dst.ip;

	dataptr = data;
        for (; dataptr < (data_limit - 5); dataptr++) {
                data_ip = *((u_int32_t *)dataptr);
                if (data_ip == oldsrcip) {
                        mangle = 1;
                        *(u_int32_t *)dataptr = newsrcip;
                }
        }

        if (mangle) {
		(*pskb)->csum = csum_partial((char *)udph + sizeof(struct udphdr), udplen - sizeof(struct udphdr), 0);
		udph->check = 0;
		udph->check = csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP, csum_partial((char *)udph, sizeof(struct udphdr), (*pskb)->csum));
        }
        return 1;
}

static unsigned int gatekeeper_nat_help(struct ip_conntrack *ct,
				        struct ip_conntrack_expect *exp,
			 	        struct ip_nat_info *info,
			 	        enum ip_conntrack_info ctinfo,
			 	        unsigned int hooknum,
			 	        struct sk_buff **pskb)
{
        int dir;
	struct ip_ct_h225_expect *exp_info;

        dir = CTINFO2DIR(ctinfo);
        if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
              || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
                DEBUGP("talk_help: Not touching dir %s at hook %s\n",
                       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
                       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
                       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
                       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
                return NF_ACCEPT;
        }

	if (!exp) {	
		LOCK_BH(&ip_h323_lock);
		if (!gatekeeper_address_fixup(ct, pskb, ctinfo)) {
			UNLOCK_BH(&ip_h323_lock);
			return NF_DROP;
		}
		UNLOCK_BH(&ip_h323_lock);
		return NF_ACCEPT;
	}
		
	exp_info = &exp->help.exp_h225_info;

	LOCK_BH(&ip_h323_lock);
	if (!gatekeeper_data_fixup(exp_info, ct, pskb, ctinfo, exp)) {
		UNLOCK_BH(&ip_h323_lock);
		return NF_DROP;
	}
	UNLOCK_BH(&ip_h323_lock);
	return NF_ACCEPT;
}

static struct ip_nat_helper gatekeeper = { { NULL, NULL },
				           { { 0, { __constant_htons(IP_CT_IS_GATEKEEPER) } },
				             { 0, { 0 }, IPPROTO_UDP } },
				           { { 0, { 0xFFFF } },
				             { 0, { 0 }, 0xFFFF } },
				           gatekeeper_nat_help, 
				     	   gatekeeper_nat_expected,
				           gatekeeper_nat_mangle,
				           "GateKeeper" };
/* Steve -- */

static int __init init(void)
{
	int ret;
	
	ret = ip_nat_helper_register(&h225);

	if (ret != 0) {
		printk("ip_nat_h323: cannot initialize the module!\n");
		return ret;
	}

	ret = ip_nat_helper_register(&gatekeeper);
	return ret;
}

static void __exit fini(void)
{
	ip_nat_helper_unregister(&h225);
	ip_nat_helper_unregister(&gatekeeper);
}

module_init(init);
module_exit(fini);
