#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/tcp.h>
#include <net/udp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_msn.h>

MODULE_DESCRIPTION("MSN 'brute force' connection tracking module");

DECLARE_LOCK_EXTERN(ip_msn_lock);
struct module *ip_nat_msn = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

#define IS_DIGIT(c)    (c >= '0' && c <= '9')
#define IS_DIGITDOT(c) ((c >= '0' && c <= '9') || c == '.')
#define IS_ALPHA(c)    ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z'))
#define IS_WSPACE(c)   (c == ' ' || c == '\t' || c == '\r' || c == '\n')
#define SKIP_WSPACE(p,n) while( n > 0 && IS_WSPACE(*p) ) { n--; p++; }

const char alfabetoBase64[]= "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

static char
*encbase64(int *b, int n, char *c)
{
	if(n==2)
		b[2]=0;
	if(n==1)
		b[1]=0;

	c[0]=alfabetoBase64[b[0]>>2];
	c[1]=alfabetoBase64[((b[0] & 0x03) << 4) | ((b[1] & 0xf0) >> 4)];
	c[2]=alfabetoBase64[((b[1] & 0x0f) << 2) | ((b[2] & 0xc0) >> 6)];
	c[3]=alfabetoBase64[b[2] & 0x3f];

	if(n<=2)
		c[3]='=';
	if(n<=1)
		c[2]='=';

	return c;
}

static int
base64crypt(char *pw, char *encrypt)
{
	char c[4];
	int b[3];
	char *buf = pw, *enc64 = encrypt;
	int n = 0;

	buf = pw;
	while (1) {
		b[n] = *buf;
		if (*buf == '\0') {
			if (n) {
				encbase64((int *)b, n, c);
				strncpy(enc64, c, sizeof(c));
				enc64 += 4;
				*enc64 = '\0';
			}
			break;
		}
		n++;
		if (n == 3) {
			encbase64((int *)b, n, c);
			strncpy(enc64, c, sizeof(c));
			enc64 += 4;
			*enc64 = '\0';
			n = 0;
		}
		buf++;
	}

	return 0;
}


static unsigned char
bytetocrypt_1(unsigned char data)
{
	return ((data & 0x80) + ((data & 0x40) ^ 0x40) + (data & 0x3e) + ((data & 0x01) ^ 0x01));
}

static unsigned char
bytetocrypt_2(unsigned char data)
{
	return ((data & 0xc0) + ((data & 0x30) ^ 0x30) + (data & 0x0e) + ((data & 0x01) ^ 0x01));
}

static u_int32_t
iptocrypt(u_int32_t ip)
{
	u_int32_t ipcrypt = 0;
	u_int32_t hip = __constant_ntohl(ip);
	unsigned char *data = (unsigned char *)&hip;

	ipcrypt = (bytetocrypt_1(*(data)) << 24) +
		  (bytetocrypt_2(*(data+1)) << 16) +
		  (bytetocrypt_1(*(data+2)) << 8) +
		  (bytetocrypt_2(*(data+3)));
	return __constant_htonl(ipcrypt);
}

/* FIXME: Time out? --RR */
static unsigned int 
msn_nat_expected(struct sk_buff **pskb,
		  unsigned int hooknum,
		  struct ip_conntrack *ct,
		  struct ip_nat_info *info);

static unsigned int 
msn_nat_voice_expected(struct sk_buff **pskb,
		       unsigned int hooknum,
		       struct ip_conntrack *ct,
		       struct ip_nat_info *info);

static unsigned int msn_nat_help(struct ip_conntrack *ct,
				  struct ip_conntrack_expect *exp,
			 	  struct ip_nat_info *info,
			 	  enum ip_conntrack_info ctinfo,
			 	  unsigned int hooknum,
			 	  struct sk_buff **pskb);

static unsigned int msn_nat_control_voice_help(struct ip_conntrack *ct,
				  	       struct ip_conntrack_expect *exp,
			 	  	       struct ip_nat_info *info,
			 	  	       enum ip_conntrack_info ctinfo,
			 	  	       unsigned int hooknum,
			 	  	       struct sk_buff **pskb);

static unsigned int msn_nat_voice_help(struct ip_conntrack *ct,
			  	       struct ip_conntrack_expect *exp,
			  	       struct ip_nat_info *info,
			  	       enum ip_conntrack_info ctinfo,
			  	       unsigned int hooknum,
			  	       struct sk_buff **pskb);
		  
static struct ip_nat_helper msn_control_voice = { { NULL, NULL },
				     		{ { 0, { 0 } },
				     		  { 0, { 0 }, IPPROTO_UDP } },
				     		{ { 0, { 0xFFFF } },
				     		  { 0, { 0 }, 0xFFFF } },
				     		msn_nat_control_voice_help,
				     		msn_nat_voice_expected,
				     		NULL,
				     		"MSN_CONTROL_VOICE" };

static int
try_number(const unsigned char *data, size_t dlen, u_int32_t array[],
           int array_size, char sep)
{
        u_int32_t i, len;

        memset(array, 0, sizeof(array[0])*array_size);

        /* Keep data pointing at next char. */
        for (i = 0, len = 0; len < dlen && i < array_size; len++, data++) {
                if (*data >= '0' && *data <= '9')
                        array[i] = array[i]*10 + *data - '0';
                else if (*data == sep)
                        i++;
                else {
                        /* Unexpected character; true if it's the
                           terminator and we're finished. */
                        if (i == array_size - 1)
                                return len;

                        DEBUGP("Char %u (got %u nums) `%u' unexpected\n",
                                len, i, *data);
                        return 0;
                }
        }
        DEBUGP("Failed to fill %u numbers separated by %c\n", array_size, sep);

        return 0;
}

static int
try_strip_number(const unsigned char *data, size_t dlen, u_int32_t array[],
		 int array_size, char sep)
{
        u_int32_t i, len;

        memset(array, 0, sizeof(array[0])*array_size);

        /* Keep data pointing at next char. */
        for (i = 0, len = 0; len < dlen && i < array_size; len++, data++) {
                if (*data >= '0' && *data <= '9')
                        array[i] = array[i]*10 + *data - '0';
		else if (*data == '\0')
			continue;
                else if (*data == sep)
                        i++;
                else {
                        /* Unexpected character; true if it's the
                           terminator and we're finished. */
                        if (i == array_size - 1)
                                return len;

                        DEBUGP("Char %u (got %u nums) `%u' unexpected\n",
                                len, i, *data);
                        return 0;
                }
        }
        DEBUGP("Failed to fill %u numbers separated by %c\n", array_size, sep);

        return 0;
}

static int
str_to_u32(char* pbuf, __u32* pval)
{
	int n = 0;

	*pval = 0;
	while (pbuf[n] >= '0' && pbuf[n] <= '9') {
		*pval = (*pval * 10) + (pbuf[n] - '0');
		n++;
	}

	return n;
}

static int
str_to_u16(char *pbuf, __u16 *pval)
{
        int n = 0;

        *pval = 0;
        while (pbuf[n] >= '0' && pbuf[n] <= '9') {
                *pval = (*pval * 10) + (pbuf[n] - '0');
                n++;
        }

        return n;
}

static unsigned int
msn_nat_voice_expected(struct sk_buff **pskb,
		       unsigned int hooknum,
		       struct ip_conntrack *ct,
		       struct ip_nat_info *info)
{
	struct ip_nat_multi_range mr;
	u_int32_t newdstip, newsrcip, newip;
	u_int16_t port;
	struct ip_ct_msn_expect *exp_info;
	struct ip_ct_msn_master *master_info;
	struct ip_conntrack *master = master_ct(ct);
	unsigned int ret;
	struct iphdr *iph = (*pskb)->nh.iph;

	IP_NF_ASSERT(info);
	IP_NF_ASSERT(master);

	IP_NF_ASSERT(!(info->initialized & (1<<HOOK2MANIP(hooknum))));

	DEBUGP("msn_nat_expected: We have a connection!\n");
	master_info = &ct->master->expectant->help.ct_msn_info;
	exp_info = &ct->master->help.exp_msn_info;

	LOCK_BH(&ip_msn_lock);

	DEBUGP("master: ");
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_REPLY].tuple);
	DEBUGP("conntrack: ");
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);

	if (exp_info->dir == IP_CT_DIR_ORIGINAL) {
		/* Make connection go to the client. */
		newdstip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
		DEBUGP("msn_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to client)\n",
		       NIPQUAD(newsrcip), NIPQUAD(newdstip));
	} else {
		/* Make the connection go to the server */
		newdstip = iph->daddr;
		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
		DEBUGP("msn_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to server)\n",
		       NIPQUAD(newsrcip), NIPQUAD(newdstip));
	}

	port = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.all;
	UNLOCK_BH(&ip_msn_lock);
	
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_SRC)
		newip = newsrcip;
	else
		newip = newdstip;

	DEBUGP("msn_nat_expected: IP to %u.%u.%u.%u\n", NIPQUAD(newip));
	DEBUGP("msn_nat_expected: Port to %u\n", port);
	
	mr.rangesize = 1;
	/* We don't want to manip the per-protocol, just the IPs... */
	mr.range[0].flags = IP_NAT_RANGE_MAP_IPS;
	mr.range[0].min_ip = mr.range[0].max_ip = newip;

	/* ... unless we're doing a MANIP_DST, in which case, make
	   sure we map to the correct port */
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_DST) {
		mr.range[0].flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		mr.range[0].min = mr.range[0].max
			= ((union ip_conntrack_manip_proto)
				{ port });
	}

	ret = ip_nat_setup_info(ct, &mr, hooknum);
	info->helper = &msn_control_voice;
	return ret;
}

static int 
msn_control_voice_fixup(struct ip_ct_msn_expect *info,
		        struct ip_conntrack *ct,
		        struct sk_buff **pskb,
		        enum ip_conntrack_info ctinfo,
		        struct ip_conntrack_expect *expect)
{
	u_int32_t newip;
	struct ip_conntrack_tuple newtuple;
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl*4;
	char *data = (char *) udph + sizeof(struct udphdr);
	char *data_limit = NULL;
	char *__data = data;
	u_int32_t udplen = (*pskb)->len - iph->ihl*4;
	u_int32_t datalen = udplen - sizeof(struct udphdr);
	int i = 0;
	char data_ip[20], sumbuf[20];
	u_int32_t viapos, vialen = 0;
	u_int32_t contactpos, contactlen = 0;
	u_int32_t contentpos, contentlen = 0, contentsum = 0;
	u_int32_t inip4pos, inip4len = 0;
	u_int32_t cinip4pos, cinip4len = 0;
	u_int32_t array[4];
	int invite = 0, sipok = 0, ack = 0;


	MUST_BE_LOCKED(&ip_msn_lock);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_REPLY].tuple);

	data_limit = (char *) data + datalen;
	if (!memcmp(data, "INVITE ", 7)) {
		invite = 1;
	} else if (!memcmp(data, "SIP/2.0 200 OK", 14)) {
		sipok = 1;
	} else if (!memcmp(data, "ACK ", 4))
		ack = 1;
	else
		return NF_ACCEPT;

	/* fixed data */
	for (; data < (data_limit - 17); data++) {
		if (!memcmp(data, "Via: SIP/2.0/UDP ", 17)) {
			data += 17;
			viapos = data - __data;
			vialen = try_number(data, datalen, array, 4, '.');
			if (!vialen)
				continue;
		}

		if (!memcmp(data, "Contact: <sip:", 14)) {
			data += 14;
			contactpos = data - __data;
                        contactlen = try_number(data, datalen, array, 4, '.');
                        if (!contactlen)
                                continue;
		}

		if (!memcmp(data, "Content-Length: ", 16)) {
			data += 16;
			contentpos = data - __data;
			contentlen = str_to_u32(data, &contentsum);
			if (!contentlen)
				continue;
		}

		if (!memcmp(data, "IN IP4 ", 7)) {
			data += 7;
			inip4pos = data - __data;
			inip4len = try_number(data, datalen, array, 4, '.');
			if (!inip4len)
				continue;
		}

		if (!memcmp(data, "c=IN IP4 ", 9)) {
			data += 9;
			cinip4pos = data - __data;
			cinip4len = try_number(data, datalen, array, 4, '.');
			if (!cinip4len)
				continue;
		}
	}


	if ((!vialen || !contactlen || !contentlen || !inip4len || !cinip4len) &&
	    (!vialen && ack))
		return NF_ACCEPT;

	newip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
	if (!ack) {
		newtuple.dst.protonum = IPPROTO_UDP;
		newtuple.src.u.udp.port = expect->tuple.src.u.udp.port;
		newtuple.dst.u.udp.port = expect->tuple.dst.u.udp.port;
		newtuple.dst.ip = newip;
		newtuple.src.ip = expect->tuple.src.ip;
		ip_conntrack_change_expect(expect, &newtuple);
	}

	sprintf(data_ip, "%u.%u.%u.%u", NIPQUAD(newip));
	if (contentlen) {
		contentsum += 2 * (strlen(data_ip) - inip4len);
		sprintf(sumbuf, "%u", contentsum);
	}

	if (cinip4len) {
		i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
					     cinip4pos, 
					     cinip4len,
					     data_ip, strlen(data_ip));
		if (!i)
			return NF_ACCEPT;
	}

	if (inip4len) {
		i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
					     inip4pos, inip4len,
					     data_ip, strlen(data_ip));
		if (!i)
			return NF_ACCEPT;
	}

	if (contentlen && !ack) {
		i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
					     contentpos, 
					     contentlen,
					     sumbuf, strlen(sumbuf));
		if (!i)
			return NF_ACCEPT;
	}

	if (contactlen) {
		i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
					     contactpos, contactlen,
					     data_ip, strlen(data_ip));
		if (!i)
			return NF_ACCEPT;
	}

	if (vialen && (invite | ack)) {
		i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
					     viapos, vialen,
					     data_ip, strlen(data_ip));
		if (!i)
			return NF_ACCEPT;
	}

	return NF_ACCEPT;
}

static unsigned int 
msn_nat_control_voice_help(struct ip_conntrack *ct,
			   struct ip_conntrack_expect *exp,
			   struct ip_nat_info *info,
			   enum ip_conntrack_info ctinfo,
			   unsigned int hooknum,
			   struct sk_buff **pskb)
{
	int dir;
	struct ip_ct_msn_expect *exp_info;

	/* Only mangle things once: original direction in POST_ROUTING
	   and reply direction on PRE_ROUTING. */
	dir = CTINFO2DIR(ctinfo);
	if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
	      || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
		DEBUGP("nat_msn_voice_help: Not touching dir %s at hook %s\n",
		       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
		       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
		       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
		       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
		return NF_ACCEPT;
	}

	exp_info = &exp->help.exp_msn_info;

	LOCK_BH(&ip_msn_lock);
	if (!msn_control_voice_fixup(exp_info, ct, pskb, ctinfo, exp)) {
		UNLOCK_BH(&ip_msn_lock);
		return NF_DROP;
	}
	UNLOCK_BH(&ip_msn_lock);
	return NF_ACCEPT;
}

static unsigned int
msn_nat_expected(struct sk_buff **pskb,
		  unsigned int hooknum,
		  struct ip_conntrack *ct,
		  struct ip_nat_info *info)
{
	struct ip_nat_multi_range mr;
	u_int32_t newdstip, newsrcip, newip;
	u_int16_t port;
	struct ip_ct_msn_expect *exp_info;
	struct ip_ct_msn_master *master_info;
	struct ip_conntrack *master = master_ct(ct);
	unsigned int is_msn, ret;
	struct iphdr *iph = (*pskb)->nh.iph;

	IP_NF_ASSERT(info);
	IP_NF_ASSERT(master);

	IP_NF_ASSERT(!(info->initialized & (1<<HOOK2MANIP(hooknum))));

	DEBUGP("msn_nat_expected: We have a connection!\n");
	master_info = &ct->master->expectant->help.ct_msn_info;
	exp_info = &ct->master->help.exp_msn_info;

	LOCK_BH(&ip_msn_lock);

	DEBUGP("master: ");
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_REPLY].tuple);
	DEBUGP("conntrack: ");
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);

        if (exp_info->dir == IP_CT_DIR_ORIGINAL) {
                /* Make connection go to the client. */
                newdstip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
                if (master_info->is_msn == IP_CT_VOICE_TRANS)
        		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
                else
                        newsrcip = iph->saddr;
		DEBUGP("msn_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to client)\n",
                       NIPQUAD(newsrcip), NIPQUAD(newdstip));
        } else {
                /* Make the connection go to the server */
                newdstip = iph->daddr;
                if (master_info->is_msn == IP_CT_VOICE_TRANS)
        		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
                else
			newsrcip = master->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
		DEBUGP("msn_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to server)\n",
                       NIPQUAD(newsrcip), NIPQUAD(newdstip));
        }

	port = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.all;
	is_msn = master_info->is_msn == IP_CT_FILE_TRANS;
	UNLOCK_BH(&ip_msn_lock);
	
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_SRC)
		newip = newsrcip;
	else
		newip = newdstip;

	DEBUGP("msn_nat_expected: IP to %u.%u.%u.%u\n", NIPQUAD(newip));
	DEBUGP("msn_nat_expected: Port to %u\n", port);
	
	mr.rangesize = 1;
	/* We don't want to manip the per-protocol, just the IPs... */
	mr.range[0].flags = IP_NAT_RANGE_MAP_IPS;
	mr.range[0].min_ip = mr.range[0].max_ip = newip;

	/* ... unless we're doing a MANIP_DST, in which case, make
	   sure we map to the correct port */
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_DST) {
		mr.range[0].flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		mr.range[0].min = mr.range[0].max
			= ((union ip_conntrack_manip_proto)
				{ port });
	}

	ret = ip_nat_setup_info(ct, &mr, hooknum);
	
	if (!is_msn) {
		DEBUGP("msn_nat_expected: MSN, setting NAT helper for %p\n", ct);
		if (master_info->is_msn == IP_CT_VOICE_SESSION_INIT)
			info->helper = &msn_control_voice;
	}

	return ret;
}

static int 
msn_data_fixup(struct ip_ct_msn_expect *info,
	       struct ip_conntrack *ct,
	       struct sk_buff **pskb,
	       enum ip_conntrack_info ctinfo,
	       struct ip_conntrack_expect *expect)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct tcphdr *tcph = (void *)iph + iph->ihl*4;
	char *data = (char *) tcph + tcph->doff * 4;
	u_int32_t tcplen = (*pskb)->len - iph->ihl*4;
	u_int32_t datalen = tcplen - tcph->doff * 4;
	int i = 0;
	char sumbuf[20], ipbuf[40], conbuf[20], ipenc64buf[30], ipportbuf[30];
	unsigned int mimelen, mimesum, mimepos;
	unsigned int conlen = 0, consum, conpos;
	unsigned int ippos = 0, iplen = 0;
	unsigned int ipenc64pos = 0, ipenc64len = 0;
	unsigned int p2pdest = 0, newiplen = 0;
	char *check1pos, *check2pos;
	char *__data = data;
	char *data_limit = (char *) data + datalen;
	u_int32_t newip;
	u_int16_t data_port;
	struct ip_conntrack_tuple newtuple;
	struct ip_ct_msn_master *master_info = &ct->help.ct_msn_info;
	u_int32_t array[4];

	MUST_BE_LOCKED(&ip_msn_lock);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_REPLY].tuple);

        /* Look for "MSG <seqno> <v> <mimelen>" */
        if (strncmp(data, "MSG ", 4) != 0) return NF_ACCEPT;
        data += 4;

        if (!IS_DIGIT(*data) && !IS_ALPHA(*data)) return NF_ACCEPT;
        while (IS_DIGITDOT(*data) || IS_ALPHA(*data)) data++;
        if (*data != ' ') return NF_ACCEPT;
        data++;

        if (!IS_DIGIT(*data) && !IS_ALPHA(*data)) return NF_ACCEPT;
        while (IS_DIGITDOT(*data) || IS_ALPHA(*data)) data++;
        if (*data != ' ') return NF_ACCEPT;
        data++;

        if (!IS_DIGIT(*data)) return NF_ACCEPT;
        mimepos = data - __data;
        mimelen = str_to_u32(data, &mimesum);

	for (; data < (data_limit - 31); data++) {
		if (!memcmp(data, "\0<\0t\0c\0p\0i\0p\0a\0d\0d\0r\0e\0s\0s\0001\0>\0", 31)) {
			data += 31;
			ippos = data - __data;
			iplen = try_strip_number(data, datalen, array, 4, '.');
			if (!iplen)
				continue;
			data += iplen;
                }

		if (!memcmp(data, "\r\nIP-Address: ", 14)) {
			data += 14;
			ippos = data - __data;
			iplen = try_number(data, datalen, array, 4, '.');
			if (!iplen)
				continue;

			data += iplen;
			if (*data == ':') {
				data += 1;
				str_to_u16(data, &data_port);
			}
		}

		if (!memcmp(data, "\r\nIP-Address-Enc64: ", 20)) {
			data += 20;
			ipenc64pos = data - __data;
			for (; data < (data_limit - 2); data++) {
				if (!memcmp(data, "\r\n", 2))
					break;

				ipenc64len++;
			}
		}

		if (!memcmp(data, "\r\nIPv4Internal-Addrs: ", 22)) {
			data += 22;
			ippos = data - __data;
			iplen = try_number(data, datalen, array, 4, '.');
			if (!iplen)
				continue;
		}

		if (!memcmp(data, "\r\nContent-Length: ", 18)) {
			data += 18;
			conpos = data - __data;
			conlen = str_to_u32(data, &consum);
		}

		if (!memcmp(data, "\r\nP2P-Dest: ", 12)) {
			p2pdest = 1;
			data += 12;
			for (; data < (data_limit - 4); data++) {
				if (!memcmp(data, "\r\n\r\n", 4)) {
					data += 4;
					check1pos = data + 16;
					check2pos = data + 24;
					break;
				}
			}
		}
	}

	if (!mimelen || !iplen)
		return NF_ACCEPT;

	if (master_info->is_msn == IP_CT_FILE_TRANS ||
	    master_info->is_msn == IP_CT_VIDEO_SESSION) {
		newtuple.dst.protonum = IPPROTO_TCP;
		newtuple.src.u.tcp.port = expect->tuple.src.u.tcp.port;
		newtuple.dst.u.tcp.port = expect->tuple.dst.u.tcp.port;
	} else {
		newtuple.dst.protonum = IPPROTO_UDP;
		newtuple.src.u.udp.port = expect->tuple.src.u.udp.port;
		newtuple.dst.u.udp.port = expect->tuple.dst.u.udp.port;
	}
	newip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
	newtuple.dst.ip = newip;
	newtuple.src.ip = expect->tuple.src.ip;
	ip_conntrack_change_expect(expect, &newtuple);

	if (master_info->is_msn == IP_CT_VIDEO_SESSION) {
		char newipbuf[20], *newipptr;
		int j = 0;

		newipptr = ipbuf;
		memset(ipbuf, '\0', sizeof(ipbuf));
		sprintf(newipbuf, "%u.%u.%u.%u", NIPQUAD(newip));
		while (newipbuf[j]) {
			*newipptr = newipbuf[j];
			j++;
			newipptr += 2;
			newiplen += 2;
		}
	} else
		sprintf(ipbuf, "%u.%u.%u.%u", NIPQUAD(newip));

	if (p2pdest) {
		if (master_info->is_msn == IP_CT_VIDEO_SESSION) {
			*check1pos += (newiplen - iplen);
			*check2pos += (newiplen - iplen);
		} else {
			*check1pos += (strlen(ipbuf) - iplen);
			*check2pos += (strlen(ipbuf) - iplen);
		}
	}

	if (conlen) {
		consum += (strlen(ipbuf) - iplen);
		sprintf(conbuf, "%u", consum);
		mimesum += (strlen(conbuf) - conlen);
	}

	if (ipenc64len) {
		sprintf(ipportbuf, "%s:%u", ipbuf, data_port);
		base64crypt(ipportbuf, ipenc64buf);
		mimesum += (strlen(ipenc64buf) - ipenc64len);
	}

	if (master_info->is_msn == IP_CT_VIDEO_SESSION)
		mimesum += (newiplen - iplen);
	else
		mimesum += (strlen(ipbuf) - iplen);

	sprintf(sumbuf, "%u", mimesum);

	if (ipenc64len) {
		i = ip_nat_mangle_tcp_packet(pskb, ct, ctinfo,
					     ipenc64pos, ipenc64len,
					     ipenc64buf, strlen(ipenc64buf));
		if (!i)
			return NF_ACCEPT;
	}

	if (master_info->is_msn == IP_CT_VIDEO_SESSION)
		i = ip_nat_mangle_tcp_packet(pskb, ct, ctinfo,
					     ippos, iplen,
					     ipbuf, newiplen);
	else
		i = ip_nat_mangle_tcp_packet(pskb, ct, ctinfo,
					     ippos, iplen,
					     ipbuf, strlen(ipbuf));
	if (!i)
		return NF_ACCEPT;

	if (conlen) {
		i = ip_nat_mangle_tcp_packet(pskb, ct, ctinfo,
					     conpos, conlen,
					     conbuf, strlen(conbuf));
		if (!i)
			return NF_ACCEPT;
	}

	i = ip_nat_mangle_tcp_packet(pskb, ct, ctinfo,
				     mimepos, mimelen,
				     sumbuf, strlen(sumbuf));
	if (!i)
		return NF_ACCEPT;

	return NF_ACCEPT;
}

static int 
msn_nat_mangle(struct ip_conntrack *ct,
	       enum ip_conntrack_info ctinfo,
	       struct sk_buff **pskb)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct tcphdr *tcph = (struct tcphdr *)((u_int32_t *)iph + iph->ihl);
	unsigned int datalen;
	struct ip_ct_msn_master *info = &ct->help.ct_msn_info;

	datalen = (*pskb)->len - iph->ihl*4 - tcph->doff*4;

	return (between(info->seq[IP_CT_DIR_ORIGINAL], ntohl(tcph->seq), ntohl(tcph->seq) + datalen)
		|| between(info->seq[IP_CT_DIR_REPLY], ntohl(tcph->seq), ntohl(tcph->seq) + datalen));
}

static unsigned int
msn_clientip_fixup(struct ip_conntrack *ct,
		   struct sk_buff **pskb,
		   enum ip_conntrack_info ctinfo)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct tcphdr *tcph = (void *)iph + iph->ihl*4;
	char *data = (char *) tcph + tcph->doff * 4;
	char *__data = data;
	u_int32_t tcplen = (*pskb)->len - iph->ihl*4;
	u_int32_t datalen = tcplen - tcph->doff * 4;
	char *data_limit = (char *) data + datalen;
	char sumbuf[20], cipbuf[20];
	unsigned int mimelen = 0, mimesum, mimepos;
	unsigned int ciplen = 0, cippos;
	int i = 0;
	u_int32_t array[4], newip;

        /* Look for "MSG <seqno> <v> <mimelen>" */
        if (strncmp(data, "MSG ", 4) != 0) return NF_ACCEPT;
        data += 4;

        if (!IS_DIGIT(*data) && !IS_ALPHA(*data)) return NF_ACCEPT;
        while (IS_DIGITDOT(*data) || IS_ALPHA(*data)) data++;
        if (*data != ' ') return NF_ACCEPT;
        data++;

        if (!IS_DIGIT(*data) && !IS_ALPHA(*data)) return NF_ACCEPT;
        while (IS_DIGITDOT(*data) || IS_ALPHA(*data)) data++;
        if (*data != ' ') return NF_ACCEPT;
        data++;

        if (!IS_DIGIT(*data)) return NF_ACCEPT;
        mimepos = data - __data;
        mimelen = str_to_u32(data, &mimesum);

	for (; data < (data_limit - 18); data++) {
		if (!memcmp(data, "\r\nClientIP: ", 12)) {
			data += 12;
			cippos = data - __data;
			ciplen = try_number(data, datalen, array, 4, '.');
			if (!ciplen)
				continue;
		}
	}

	if (!mimelen || !ciplen)
		return NF_ACCEPT;

	newip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
	sprintf(cipbuf, "%u.%u.%u.%u", NIPQUAD(newip));

        mimesum += (strlen(cipbuf) - ciplen);
        sprintf(sumbuf, "%u", mimesum);

	i = ip_nat_mangle_tcp_packet(pskb, ct, ctinfo,
				     cippos, ciplen,
				     cipbuf, strlen(cipbuf));
	if (!i)
		return NF_ACCEPT;

	i = ip_nat_mangle_tcp_packet(pskb, ct, ctinfo,
				     mimepos, mimelen,
				     sumbuf, strlen(sumbuf));
	if (!i)
		return NF_ACCEPT;

	return NF_ACCEPT;
}		   

static unsigned int 
msn_nat_help(struct ip_conntrack *ct,
	     struct ip_conntrack_expect *exp,
	     struct ip_nat_info *info,
	     enum ip_conntrack_info ctinfo,
	     unsigned int hooknum,
	     struct sk_buff **pskb)
{
	int dir;
	struct ip_ct_msn_expect *exp_info;

	/* Only mangle things once: original direction in POST_ROUTING
	   and reply direction on PRE_ROUTING. */
	dir = CTINFO2DIR(ctinfo);
	if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
	      || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
		DEBUGP("nat_msn: Not touching dir %s at hook %s\n",
		       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
		       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
		       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
		       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
		return NF_ACCEPT;
	}

	if (!exp) {
		LOCK_BH(&ip_msn_lock);
		msn_clientip_fixup(ct, pskb, ctinfo);
		UNLOCK_BH(&ip_msn_lock);
		return NF_ACCEPT;
	}

	exp_info = &exp->help.exp_msn_info;

	LOCK_BH(&ip_msn_lock);
	if (!msn_data_fixup(exp_info, ct, pskb, ctinfo, exp)) {
		UNLOCK_BH(&ip_msn_lock);
		return NF_DROP;
	}
	UNLOCK_BH(&ip_msn_lock);
	return NF_ACCEPT;
}

static int 
msn_rx_nat_mangle(struct ip_conntrack *ct,
		  enum ip_conntrack_info ctinfo,
		  struct sk_buff **pskb)
{
	return NF_ACCEPT;
}

static unsigned int 
msn_rx_nat_help(struct ip_conntrack *ct,
		struct ip_conntrack_expect *exp,
		struct ip_nat_info *info,
		enum ip_conntrack_info ctinfo,
		unsigned int hooknum,
		struct sk_buff **pskb)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	unsigned int udplen = (*pskb)->len - iph->ihl * 4;
	char *data = (char *)udph + sizeof(struct udphdr);
	u_int32_t datalen = udplen - sizeof(struct udphdr);
	u_int32_t cid, newip, oldip;
	int dir;

	if (datalen < 20)
		return NF_ACCEPT;

	dir = CTINFO2DIR(ctinfo);
	if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
	   || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
		DEBUGP("nat_msn_rx: Not touching dir %s at hook %s\n",
 		dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
 		       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
 		       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
 		       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
 		return NF_ACCEPT;
        }

	if (dir == IP_CT_DIR_ORIGINAL) {
		newip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
		oldip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
	} else {
		newip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		oldip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
	}

	memcpy(&cid, (data + 4), 4);
	if (cid == iptocrypt(oldip)) {
		cid = iptocrypt(newip);
		memcpy((data + 4), &cid, 4);
	}

	return NF_ACCEPT;
}

static struct ip_nat_helper msn = { { NULL, NULL },
				     { { 0, { __constant_htons(IP_CT_IS_MSN) } },
				       { 0, { 0 }, IPPROTO_TCP } },
				     { { 0, { 0xFFFF } },
				       { 0, { 0 }, 0xFFFF } },
				     msn_nat_help, 
				     msn_nat_expected,
				     msn_nat_mangle,
				     "MSN" };

static struct ip_nat_helper msn_rx = { { NULL, NULL },
				     { { 0, { __constant_htons(IP_CT_MSN_RX) } },
				       { 0, { 0 }, IPPROTO_UDP } },
				     { { 0, { 0xFFFF } },
				       { 0, { 0 }, 0xFFFF } },
				     msn_rx_nat_help, 
				     NULL,
				     msn_rx_nat_mangle, 
				     "MSN RX" };

static int __init init(void)
{
	int ret;
	
	ret = ip_nat_helper_register(&msn);
	if (ret != 0) {
		printk("ip_nat_msn: cannot initialize the module!\n");
		return ret;
	}

	ret = ip_nat_helper_register(&msn_rx);
	if (ret != 0) {
		ip_nat_helper_unregister(&msn);
		printk("ip_nat_msn: cannot initialize the module!\n");
	}

	return ret;
}

static void __exit fini(void)
{
	ip_nat_helper_unregister(&msn);
	ip_nat_helper_unregister(&msn_rx);
}

module_init(init);
module_exit(fini);
