#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/udp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_pbx.h>

DECLARE_LOCK_EXTERN(ip_pbx_lock);
struct module *ip_nat_pbx = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

/* FIXME: Time out? --RR */

static int pbx_address_fixup(struct ip_conntrack *ct,
			     struct sk_buff **pskb,
			     enum ip_conntrack_info ctinfo)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl*4;
	char *data = (char *) udph + sizeof(struct udphdr);
	u_int32_t udplen = (*pskb)->len - iph->ihl*4;
	int datalen = udplen - sizeof(struct udphdr);
	char *data_limit = data + datalen;
	struct ip_ct_pbx_master *info = &ct->help.ct_pbx_info; 
	u_int32_t newsrcip, newdstip, oldsrcip, olddstip, data_ip;
	int mangle = 0;

	MUST_BE_LOCKED(&ip_pbx_lock);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_REPLY].tuple);

	if (info->dir == IP_CT_DIR_ORIGINAL) {
		oldsrcip = ct->tuplehash[info->dir].tuple.src.ip;
		olddstip = ct->tuplehash[info->dir].tuple.dst.ip;
		newsrcip = ct->tuplehash[!info->dir].tuple.dst.ip;
		newdstip = ct->tuplehash[!info->dir].tuple.src.ip;
	} else {
		oldsrcip = ct->tuplehash[info->dir].tuple.src.ip;
		olddstip = ct->tuplehash[info->dir].tuple.dst.ip;
		newsrcip = ct->tuplehash[!info->dir].tuple.dst.ip;
		newdstip = ct->tuplehash[!info->dir].tuple.src.ip;
	}

	DEBUGP("newsrcip = %u.%u.%u.%u newdstip = %u.%u.%u.%u\n",
		NIPQUAD(newsrcip),
		NIPQUAD(newdstip));
	for (; data < (data_limit - 5); data++) {
		data_ip = *((u_int32_t *)data);
		if (data_ip == oldsrcip) {
			mangle = 1;
			*(u_int32_t *)data = newsrcip;
		} else if (data_ip == olddstip) {
			mangle = 1;
			*(u_int32_t *)data = newdstip;
		}
	}

	if (mangle) {
		(*pskb)->csum = csum_partial((char *)udph + sizeof(struct udphdr), udplen - sizeof(struct udphdr), 0);
		udph->check = 0;
		udph->check = csum_tcpudp_magic(iph->saddr, iph->daddr, udplen, IPPROTO_UDP,
                	                        csum_partial((char *)udph, sizeof(struct udphdr), (*pskb)->csum));
	}
	return 1;
}

static int
pbx_mangle(struct ip_conntrack *ct,
	   enum ip_conntrack_info ctinfo,
	   struct sk_buff **pskb)
{
	return NF_ACCEPT;
}

static unsigned int 
pbx_help(struct ip_conntrack *ct,
     struct ip_conntrack_expect *exp,
     struct ip_nat_info *info,
     enum ip_conntrack_info ctinfo,
     unsigned int hooknum,
     struct sk_buff **pskb)
{
	int dir;

        dir = CTINFO2DIR(ctinfo);
        if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
              || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
                DEBUGP("talk_help: Not touching dir %s at hook %s\n",
                       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
                       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
                       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
                       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
                return NF_ACCEPT;
        }

	LOCK_BH(&ip_pbx_lock);
	pbx_address_fixup(ct, pskb, ctinfo);
	UNLOCK_BH(&ip_pbx_lock);
	return NF_ACCEPT;
}

static struct ip_nat_helper pbx_helper[2] = {
					{ { NULL, NULL },
					  { { 0, { __constant_htons(PBX_PORT) } },
					{ 0, { 0 }, IPPROTO_UDP } },
					  { { 0, { 0xFFFF } },
					{ 0, { 0 }, 0xFFFF } },
					pbx_help, 
					NULL,
					pbx_mangle,
					"pbx_control" },
					{ { NULL, NULL },
					  { { 0, { __constant_htons(PBX_DATA_PORT) } },
					{ 0, { 0 }, IPPROTO_UDP } },
					  { { 0, { 0xFFFF } },
					{ 0, { 0 }, 0xFFFF } },
					pbx_help, 
					NULL,
					pbx_mangle,
					"pbx_data" }};

static int __init init(void)
{
	int ret;

	ret = ip_nat_helper_register(&pbx_helper[0]);
	if (ret) {
		printk("ip_nat_pbx: [0] fails\n");
		return ret;
	}

	ret = ip_nat_helper_register(&pbx_helper[1]);
	if (ret) {
		printk("ip_nat_pbx: [1] fails\n");
		ip_nat_helper_unregister(&pbx_helper[0]);
	}
	return ret;
}

static void __exit fini(void)
{
	ip_nat_helper_unregister(&pbx_helper[0]);
	ip_nat_helper_unregister(&pbx_helper[1]);
}

module_init(init);
module_exit(fini);
