#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/ip.h>
#include <net/checksum.h>
#include <net/udp.h>

#include <linux/netfilter_ipv4/lockhelp.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <linux/netfilter_ipv4/ip_conntrack_tuple.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_sip.h>

MODULE_DESCRIPTION("SIP 'brute force' connection tracking module");

DECLARE_LOCK_EXTERN(ip_sip_lock);
struct module *ip_nat_sip = THIS_MODULE;

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static struct ip_conntrack_expect
*expect_find(struct ip_conntrack *ct, u_int32_t dstip, u_int16_t dstport)
{
	const struct ip_conntrack_tuple tuple = { { 0, { 0 } },
			{ dstip, { htons(dstport) }, IPPROTO_UDP }};

	return ip_conntrack_expect_find_get(&tuple);
}

static int
try_number(const unsigned char *data, size_t dlen, u_int32_t array[],
           int array_size, char sep)
{
        u_int32_t i, len;

        memset(array, 0, sizeof(array[0])*array_size);

        /* Keep data pointing at next char. */
        for (i = 0, len = 0; len < dlen && i < array_size; len++, data++) {
                if (*data >= '0' && *data <= '9')
                        array[i] = array[i]*10 + *data - '0';
                else if (*data == sep)
                        i++;
                else {
                        /* Unexpected character; true if it's the
                           terminator and we're finished. */
                        if (i == array_size - 1)
                                return len;

                        DEBUGP("Char %u (got %u nums) `%u' unexpected\n",
                                len, i, *data);
                        return 0;
                }
        }
        DEBUGP("Failed to fill %u numbers separated by %c\n", array_size, sep);

        return 0;
}

static int
str_to_u32(char* pbuf, __u32* pval)
{
	int n = 0;

	*pval = 0;
	while (pbuf[n] >= '0' && pbuf[n] <= '9') {
		*pval = (*pval * 10) + (pbuf[n] - '0');
		n++;
	}

	return n;
}

static int
str_to_u16(char *pbuf, __u16 *pval)
{
        int n = 0;

        *pval = 0;
        while (pbuf[n] >= '0' && pbuf[n] <= '9') {
                *pval = (*pval * 10) + (pbuf[n] - '0');
                n++;
        }

        return n;
}

static unsigned int
sip_nat_expected(struct sk_buff **pskb,
		       unsigned int hooknum,
		       struct ip_conntrack *ct,
		       struct ip_nat_info *info)
{
	struct ip_nat_multi_range mr;
	u_int32_t newdstip, newsrcip, newip;
	u_int16_t port;
	struct ip_conntrack *master = master_ct(ct);
	unsigned int ret;
	struct iphdr *iph = (*pskb)->nh.iph;

	IP_NF_ASSERT(info);
	IP_NF_ASSERT(master);

	IP_NF_ASSERT(!(info->initialized & (1<<HOOK2MANIP(hooknum))));

	DEBUGP("sip_nat_expected: We have a connection!\n");
	LOCK_BH(&ip_sip_lock);
	DEBUGP("master: ");
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_ORIGINAL].tuple);
	DUMP_TUPLE(&master->tuplehash[IP_CT_DIR_REPLY].tuple);
	DEBUGP("conntrack: ");
	DUMP_TUPLE(&ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple);

/* 2005/7/5 Steve ++ */
	if ((master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip ==
	     master->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip) &&
	    (master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip ==
	     master->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip)) {
		newdstip = iph->daddr;
		newsrcip = iph->saddr;
	} else {
/* Steve -- */
		/* Make connection go to the client. */
		newdstip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
	}

	DEBUGP("sip_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to client)\n",
	       NIPQUAD(newsrcip), NIPQUAD(newdstip));
#if 0
	} else {
		/* Make the connection go to the server */
		newdstip = iph->daddr;
		newsrcip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
		DEBUGP("msn_nat_expected: %u.%u.%u.%u->%u.%u.%u.%u (to server)\n",
		       NIPQUAD(newsrcip), NIPQUAD(newdstip));
	}
#endif
	port = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.u.all;
	UNLOCK_BH(&ip_sip_lock);

	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_SRC)
		newip = newsrcip;
	else
		newip = newdstip;

	DEBUGP("sip_nat_expected: IP to %u.%u.%u.%u\n", NIPQUAD(newip));
	DEBUGP("sip_nat_expected: Port to %u\n", port);
	
	mr.rangesize = 1;
	/* We don't want to manip the per-protocol, just the IPs... */
	mr.range[0].flags = IP_NAT_RANGE_MAP_IPS;
	mr.range[0].min_ip = mr.range[0].max_ip = newip;

	/* ... unless we're doing a MANIP_DST, in which case, make
	   sure we map to the correct port */
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_DST) {
		mr.range[0].flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		mr.range[0].min = mr.range[0].max
			= ((union ip_conntrack_manip_proto)
				{ port });
	}

	ret = ip_nat_setup_info(ct, &mr, hooknum);
	return ret;
}

static int 
sip_data_fixup(struct ip_conntrack *ct,
	       struct sk_buff **pskb,
	       enum ip_conntrack_info ctinfo,
	       struct ip_conntrack_expect *expect)
{
	struct iphdr *iph = (*pskb)->nh.iph;
        struct udphdr *udph = (void *)iph + iph->ihl * 4;
        unsigned char *data = (unsigned char *)((const char *)udph + sizeof(struct udphdr));
        unsigned char *__data = data;
        unsigned char *data_limit;
	struct ip_conntrack_tuple newtuple, rcpt_newtuple, video_newtuple;
        u_int32_t udplen = (*pskb)->len - iph->ihl * 4;
        u_int32_t datalen = udplen - sizeof(struct udphdr);
	u_int32_t newsip, newdip, origsip;
	u_int32_t array[4];
	int i = 0, invite = 0, trying = 0, ack = 0, j = 0;
	char newsipbuf[20], origsipbuf[20], contentbuf[8];
	int dir;
        struct ip_conntrack_expect *rcpt_exp = NULL, *video_exp = NULL;
	u_int32_t rcpt_port, video_port, videoportlen = 0;
	u_int32_t ackippos, ackiplen = 0;
	u_int32_t fippos, fiplen = 0;
	u_int32_t tippos, tiplen = 0;
	u_int32_t viaippos, viaiplen = 0;
	u_int32_t conippos, coniplen = 0;
	u_int32_t contentpos, contentlen = 0, contentsum = 0;
	u_int32_t inip4pos, inip4len = 0;
	u_int32_t cinip4pos, cinip4len = 0;
	u_int32_t inviteippos, inviteiplen = 0;
	u_int32_t notifyippos, notifyiplen = 0;
	u_int32_t callidpos, callidlen = 0;
	int acknum = -1, fipnum = -1, tipnum = -1, vianum = -1, conipnum = -1, contentnum = -1;
	int inip4num = -1, cinip4num = -1, invitenum = -1, notifynum = -1, callidnum = -1;
	int number = 0;
	struct ip_ct_sip_master *info = &ct->help.ct_sip_info;

	MUST_BE_LOCKED(&ip_sip_lock);
	if (datalen >= 7 && !memcmp(data, "INVITE ", 7))
		invite = 1;
	else if (datalen >= 10 && !memcmp(data, "SUBSCRIBE ", 10))
		trying = 1;
	else if (datalen >= 4 && !memcmp(data, "BYE ", 4))
		trying = 1;
	else if (datalen >= 7 && !memcmp(data, "NOTIFY ", 7))
		trying = 1;
	else if (datalen >= 8 && !memcmp(data, "SIP/2.0 ", 8))
		trying = 1;
	else if (datalen >= 8 && !memcmp(data, "REGISTER", 8))
		trying = 1;
	else if (datalen >= 4 && !memcmp(data, "ACK ", 4))
		ack = 1;
	else
		return NF_ACCEPT;

	dir = CTINFO2DIR(ctinfo);
	origsip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
	if (dir == IP_CT_DIR_ORIGINAL) {
		newsip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
		newdip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.src.ip;
	} else {
		newsip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		newdip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.dst.ip;
//		origsip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;
	}

	data_limit = (unsigned char *) data + datalen;
	for (; data < (data_limit - 17); data++) {
		if (!memcmp(data, "INVITE ", 7)) {
			data += 7;
			for (; data < (data_limit - 17); data++) {
				if (*data == '@') {
					data++;
					inviteippos = data - __data;
					DEBUGP("ip_nat_sip: invite %x\n", inviteippos);
					inviteiplen = try_number(data, datalen, array, 4, '.');
					if (!inviteiplen)
						continue;
					data += inviteiplen;
					invitenum = number;
					number++;
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
                                        break;
			}
                }

		if (!memcmp(data, "NOTIFY ", 7)) {
			data += 7;
			for (; data < (data_limit - 17); data++) {
				if (*data == '@') {
					data++;
					notifyippos = data - __data;
					DEBUGP("ip_nat_sip: notify %x\n", notifyippos);
					notifyiplen = try_number(data, datalen, array, 4, '.');
					if (!notifyiplen)
						continue;
					data += notifyiplen;
					notifynum = number;
					number++;
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
                                        break;
			}
                }

		if (!memcmp(data, "ACK ", 4)) {
			data += 4;
			for (; data < (data_limit - 17); data++) {
				if (*data == '@') {
					data++;
					ackippos = data - __data;
					DEBUGP("ip_nat_sip: ACK %x\n", ackippos);
					ackiplen = try_number(data, datalen, array, 4, '.');
					if (!ackiplen)
						continue;
					data += ackiplen;
					acknum = number;
					number++;
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
                                        break;
			}
                }

		if (!memcmp(data, "Call-ID: ", 9)) {
			data += 9;
			for (; data < (data_limit - 17); data++) {
				if (*data == '@') {
					data++;
					callidpos = data - __data;
					DEBUGP("ip_nat_sip: callidpos %x\n", callidpos);
					callidlen = try_number(data, datalen, array, 4, '.');
					if (!callidlen)
						continue;
					data += callidlen;
					callidnum = number;
					number++;
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
                                        break;
			}
                }

		if (!memcmp(data, "From: ", 6)) {
			data += 6;
			for (; data < (data_limit - 17); data++) {
				if (*data == '@') {
					data++;
					fippos = data - __data;
					DEBUGP("ip_nat_sip: from %x\n", fippos);
					fiplen = try_number(data, datalen, array, 4, '.');
					if (!fiplen)
						continue;
					data += fiplen;
					fipnum = number;
					number++;
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
                                        break;
			}
                }

		if (!memcmp(data, "To: ", 4)) {
			data += 4;
			for (; data < (data_limit - 17); data++) {
				if (*data == '@') {
					data++;
					tippos = data - __data;
					DEBUGP("ip_nat_sip: to %x\n", tippos);
					tiplen = try_number(data, datalen, array, 4, '.');
					if (!tiplen)
						continue;
					data += tiplen;
					tipnum = number;
					number++;
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
                                        break;
			}
		}

		if (!memcmp(data, "Via: SIP/2.0/UDP ", 17)) {
			data += 17;
			viaippos = data - __data;
			DEBUGP("ip_nat_sip: via %x\n", viaippos);
			viaiplen = try_number(data, datalen, array, 4, '.');
			if (!viaiplen)
				continue;
			data += viaiplen;
			vianum = number;
			number++;
		}

		if (!memcmp(data, "Contact: ", 9)) {
			data += 9;
			for (; data < (data_limit - 17); data++) {
				if (*data == '@') {
					data++;
					conippos = data - __data;
					DEBUGP("ip_nat_sip: contact %x\n", conippos);
					coniplen = try_number(data, datalen, array, 4, '.');
					if (!coniplen)
						continue;
					data += coniplen;
					conipnum = number;
					number++;
					break;
				} else if (*data == 0xd && *(data+1) == 0xa)
                                        break;
			}
		}

		if (!memcmp(data, "Content-Length: ", 16)) {
			data += 16;
			contentpos = data - __data;
			DEBUGP("ip_nat_sip: content-length %x\n", contentpos);
			contentlen = str_to_u32(data, &contentsum);
			contentnum = number;
			number++;
                        if (!contentlen)
                                continue;
                }

		if (!memcmp(data, "IN IP4 ", 7)) {
			data += 7;
			inip4pos = data - __data;
			DEBUGP("ip_nat_sip: inip4 %x\n", inip4pos);
			inip4len = try_number(data, datalen, array, 4, '.');
			inip4num = number;
			number++;
			if (!inip4len)
				continue;
		}

		if (!memcmp(data, "c=IN IP4 ", 9)) {
			data += 9;
			cinip4pos = data - __data;
			DEBUGP("ip_nat_sip: cinip4 %x\n", cinip4pos);
			cinip4len = try_number(data, datalen, array, 4, '.');
			cinip4num = number;
			number++;
			if (!cinip4len)
				continue;
		}

		if (!memcmp(data, "m=video ", 8)) {
			data += 8;
			videoportlen = str_to_u16(data, &video_port);
			if (!videoportlen)
				continue;

			data += videoportlen;
		}
	}
	sprintf(newsipbuf, "%u.%u.%u.%u", NIPQUAD(newsip));
	sprintf(origsipbuf, "%u.%u.%u.%u", NIPQUAD(origsip));
	DEBUGP("ip_nat_sip: newsipbuf: %s origsipbuf: %s\n",
		newsipbuf, origsipbuf);

	if (invite && expect) {
		DEBUGP("ip_nat_sip: change expect 1\n");
		rcpt_port = ntohs(expect->tuple.dst.u.udp.port) + 1;
		DEBUGP("ip_nat_sip: change expect 2\n");
		rcpt_exp = expect_find(ct, expect->tuple.dst.ip, rcpt_port);
		DEBUGP("ip_nat_sip: change expect 3\n");
		if (videoportlen)
			video_exp = expect_find(ct, expect->tuple.dst.ip, video_port);

                newtuple.dst.protonum = IPPROTO_UDP;
                newtuple.src.u.udp.port = expect->tuple.src.u.udp.port;
                newtuple.dst.u.udp.port = expect->tuple.dst.u.udp.port;
		newtuple.dst.ip = newsip;
		newtuple.src.ip = newdip;
		DEBUGP("ip_nat_sip: change expect 4\n");
		ip_conntrack_change_expect(expect, &newtuple);
		if (rcpt_exp) {
	                rcpt_newtuple.dst.protonum = IPPROTO_UDP;
        	        rcpt_newtuple.src.u.udp.port = rcpt_exp->tuple.src.u.udp.port;
                	rcpt_newtuple.dst.u.udp.port = rcpt_exp->tuple.dst.u.udp.port;
			rcpt_newtuple.dst.ip = newsip;
			rcpt_newtuple.src.ip = newdip;
			ip_conntrack_change_expect(rcpt_exp, &rcpt_newtuple);
		}
		if (video_exp) {
	                video_newtuple.dst.protonum = IPPROTO_UDP;
        	        video_newtuple.src.u.udp.port = video_exp->tuple.src.u.udp.port;
                	video_newtuple.dst.u.udp.port = video_exp->tuple.dst.u.udp.port;
			video_newtuple.dst.ip = newsip;
			video_newtuple.src.ip = newdip;
			ip_conntrack_change_expect(video_exp, &video_newtuple);
		}
	}

	if (dir == IP_CT_DIR_REPLY && info && !info->nostun)
		return NF_ACCEPT;

	for (j = number; j >= 0; j--) {
		if (j == cinip4num) {
			DEBUGP("ip_nat_sip: mangle udp cinip4\n");
			if (cinip4len) {
				DEBUGP("ip_nat_sip: cinip4len: %d %x\n", cinip4len, cinip4pos);
				
				if (strlen(origsipbuf) == cinip4len &&
				    !memcmp(origsipbuf, (void *)(__data+cinip4pos), cinip4len)) { 	
					DEBUGP("ip_nat_sip: cinip4len %d\n", cinip4len);
					contentsum += (strlen(newsipbuf) - cinip4len);

					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
							     	     cinip4pos, cinip4len,
                		                             	     newsipbuf, strlen(newsipbuf));
					if (info)
						info->nostun = 1;
				}
			}
		}

		if (j == inip4num) {
			DEBUGP("ip_nat_sip: mangle udp inip4\n");
			if (inip4len) {
				DEBUGP("ip_nat_sip: inip4len: %d %x\n", inip4len, inip4pos);
				if (strlen(origsipbuf) == inip4len && 
				    !memcmp(origsipbuf, (void *)(__data+inip4pos), inip4len)) {
					contentsum += (strlen(newsipbuf) - inip4len);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     inip4pos, inip4len,
               				                             newsipbuf, strlen(newsipbuf));
					if (info)
						info->nostun = 1;
				} 
			}
		}

		if (j == contentnum) {
			DEBUGP("ip_nat_sip: mangle udp content-length\n");
			if (contentlen) {
				sprintf(contentbuf, "%u", contentsum);
				i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
							     contentpos, contentlen,
		                                             contentbuf, strlen(contentbuf));
			}
		}

		if (j == conipnum) {
			DEBUGP("ip_nat_sip: mangle udp contact ip\n");
			DEBUGP("ip_nat_sip: coniplen: %d %x\n", coniplen, conippos);
			if (coniplen) {
				if (strlen(origsipbuf) == coniplen && 
				    !memcmp(origsipbuf, (void *)(__data+conippos),coniplen)) {
					contentsum += (strlen(newsipbuf) - coniplen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     conippos, coniplen,
        				                             newsipbuf, strlen(newsipbuf));
					if (info)
						info->nostun = 1;
				}
			}
		}

		if (j == vianum) {
			DEBUGP("ip_nat_sip: mangle udp VIA\n");
			if (viaiplen) {
				DEBUGP("ip_nat_sip: viaiplen: %d %x\n", viaiplen, viaippos);
				if (strlen(origsipbuf) == viaiplen && 
				    !memcmp(origsipbuf, (void *)(__data+viaippos),viaiplen)) {
					contentsum += (strlen(newsipbuf) - viaiplen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     viaippos, viaiplen,
                       				                     newsipbuf, strlen(newsipbuf));
					if (info)
						info->nostun = 1;
				}
			}
		}

		if (j == tipnum) {
			if (tiplen) {
				if (strlen(origsipbuf) == tiplen &&
				    !memcmp(origsipbuf, (void *)(__data+tippos), tiplen)) { 	
					contentsum += (strlen(newsipbuf) - tiplen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     tippos, tiplen,
        			                                     newsipbuf, strlen(newsipbuf));
				}
			}
		}

		if (j == fipnum) {
			if (fiplen) {
				if (strlen(origsipbuf) == fiplen &&
				    !memcmp(origsipbuf, (void *)(__data+fippos), fiplen)) { 	
					contentsum += (strlen(newsipbuf) - fiplen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     fippos, fiplen,
                       		        		             newsipbuf, strlen(newsipbuf));
				}
			}
		}

#if 0
		if (j == callidnum) {
			if (callidlen) {
				if (strlen(origsipbuf) == callidlen &&
				    !memcmp(origsipbuf, (void *)(__data+callidpos), callidlen)) {
					contentsum += (strlen(newsipbuf) - callidlen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     callidpos, callidlen,
                       		        		             newsipbuf, strlen(newsipbuf));
				}
			}
		}
#endif
		if (j == acknum) {
			DEBUGP("ip_nat_sip: mangle udp ack\n");
			if (ackiplen) {
				DEBUGP("ip_nat_sip: ackiplen: %d %x\n", ackiplen, ackippos);
				if (strlen(origsipbuf) == ackiplen && 
				    !memcmp(origsipbuf, (void *)(__data+ackippos),ackiplen)) {
					contentsum += (strlen(newsipbuf) - ackiplen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     ackippos, ackiplen,
                       				                     newsipbuf, strlen(newsipbuf));
				}
			}
		}
#if 1
		if (j == invitenum) {
			if (inviteiplen) {
				if (strlen(origsipbuf) == inviteiplen && 
				    !memcmp(origsipbuf, (void *)(__data+inviteippos),inviteiplen)) {
					contentsum += (strlen(newsipbuf) - inviteiplen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     inviteippos, inviteiplen,
                       				                     newsipbuf, strlen(newsipbuf));
				}
			}
		}
#endif
		if (j == notifynum) {
			DEBUGP("ip_nat_sip: mangle udp notify\n");
			if (notifyiplen) {
				DEBUGP("ip_nat_sip: notifyiplen: %d %x\n", notifyiplen, notifyippos);
				if (strlen(origsipbuf) == notifyiplen && 
				    !memcmp(origsipbuf, (void *)(__data+notifyippos),notifyiplen)) {
					contentsum += (strlen(newsipbuf) - notifyiplen);
					i = ip_nat_mangle_udp_packet(pskb, ct, ctinfo,
								     notifyippos, notifyiplen,
                       				                     newsipbuf, strlen(newsipbuf));
				}
			}
		}
	}

	return NF_ACCEPT;
}

static unsigned int
sip_nat_mangle(struct ip_conntrack *ct,
	       enum ip_conntrack_info ctinfo,
	       struct sk_buff **pskb)
{
	return NF_ACCEPT;
}

static unsigned int 
sip_nat_help(struct ip_conntrack *ct,
	     struct ip_conntrack_expect *exp,
	     struct ip_nat_info *info,
	     enum ip_conntrack_info ctinfo,
	     unsigned int hooknum,
	     struct sk_buff **pskb)
{
	int dir;

	/* Only mangle things once: original direction in POST_ROUTING
	   and reply direction on PRE_ROUTING. */
	dir = CTINFO2DIR(ctinfo);
	if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
	      || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
		DEBUGP("nat_sip: Not touching dir %s at hook %s\n",
		       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
		       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
		       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
		       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
		return NF_ACCEPT;
	}

	LOCK_BH(&ip_sip_lock);
	if (!sip_data_fixup(ct, pskb, ctinfo, exp)) {
		UNLOCK_BH(&ip_msn_lock);
		return NF_DROP;
	}
	UNLOCK_BH(&ip_sip_lock);
	DEBUGP("---ip_nat_sip---\n");
	return NF_ACCEPT;
}

static struct ip_nat_helper sip_help = { { NULL, NULL },
				     { { 0, { __constant_htons(IP_CT_SIP) } },
				       { 0, { 0 }, IPPROTO_UDP } },
				     { { 0, { 0xFFFF } },
				       { 0, { 0 }, 0xFFFF } },
				     sip_nat_help, 
				     sip_nat_expected,
				     sip_nat_mangle, 
				     "SIP RX" };

static int __init init(void)
{
	int ret;

	printk("init ip_nat_sip\n");	
	ret = ip_nat_helper_register(&sip_help);
	if (ret != 0) {
		printk("ip_nat_sip: cannot initialize the module!\n");
		return ret;
	}

	return ret;
}

static void __exit fini(void)
{
	ip_nat_helper_unregister(&sip_help);
}

module_init(init);
module_exit(fini);
