/*
 * This is a module which is used for alarming packets.
 */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <linux/spinlock.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_ALARM.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

/* Use lock to serialize, so printks don't overlap */
static spinlock_t alarm_lock = SPIN_LOCK_UNLOCKED;

static unsigned int
ipt_alarm_target(struct sk_buff **pskb,
	       unsigned int hooknum,
	       const struct net_device *in,
	       const struct net_device *out,
	       const void *targinfo,
	       void *userinfo)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	const struct ipt_alarm_info *alarminfo = targinfo;
	char level_string[4] = "< >";
	enum ip_conntrack_info ctinfo;
	struct timeval tval;
	struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)*pskb,
				&ctinfo);
	if (!ct)
		return 0;

	level_string[1] = '0' + (alarminfo->level % 8);
	do_gettimeofday(&tval);
	spin_lock_bh(&alarm_lock);
	printk(level_string);
	printk("ALARM: %lu %llx %lu\n", 
		ntohs(iph->tot_len),
		(ct->mark & 0x00000000ffffffff),
		tval.tv_sec);
	spin_unlock_bh(&alarm_lock);

	return IPT_CONTINUE;
}

static int ipt_alarm_checkentry(const char *tablename,
			      const struct ipt_entry *e,
			      void *targinfo,
			      unsigned int targinfosize,
			      unsigned int hook_mask)
{
	const struct ipt_alarm_info *alarminfo = targinfo;

	if (targinfosize != IPT_ALIGN(sizeof(struct ipt_alarm_info))) {
		DEBUGP("ALARM: targinfosize %u != %u\n",
		       targinfosize, IPT_ALIGN(sizeof(struct ipt_alarm_info)));
		return 0;
	}

	if (alarminfo->level >= 8) {
		DEBUGP("ALARM: level %u >= 8\n", alarminfo->level);
		return 0;
	}

	return 1;
}

static struct ipt_target ipt_alarm_reg
= { { NULL, NULL }, "ALARM", ipt_alarm_target, ipt_alarm_checkentry, NULL, 
    THIS_MODULE };

static int __init init(void)
{
	if (ipt_register_target(&ipt_alarm_reg))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_alarm_reg);
}

module_init(init);
module_exit(fini);
