/* This is a module which is used for setting the NFMARK field of an skb. */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/checksum.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_MARK.h>

static unsigned int
target(struct sk_buff **pskb,
       unsigned int hooknum,
       const struct net_device *in,
       const struct net_device *out,
       const void *targinfo,
       void *userinfo)
{
	const struct ipt_mark_target_info *markinfo = targinfo;
/* 2002/5/14 Steve ++ */
        enum ip_conntrack_info ctinfo;
        struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)(*pskb), &ctinfo);

	if (!ct)
		return IPT_CONTINUE;
#if 0
	if((*pskb)->nfmark != markinfo->mark) {
		(*pskb)->nfmark = markinfo->mark;
		(*pskb)->nfcache |= NFC_ALTERED;
	}
#endif

/* 2003/1/2 Steve ++ */
        if (ct) {
            switch(markinfo->mode) {
            case IPT_MARK_SET:
                ct->skbmark = markinfo->mark;
                break;
            case IPT_MARK_OR:
                ct->skbmark |= markinfo->mark;
                break;
            case IPT_MARK_AND:
                ct->skbmark &= markinfo->mark;
                break;
	    }
	}

#if 0
	if(ct->skbmark != markinfo->mark)
		ct->skbmark = markinfo->mark;
#endif
/* Steve -- */

	return IPT_CONTINUE;
}

static int
checkentry(const char *tablename,
	   const struct ipt_entry *e,
           void *targinfo,
           unsigned int targinfosize,
           unsigned int hook_mask)
{
	if (targinfosize != IPT_ALIGN(sizeof(struct ipt_mark_target_info))) {
		printk(KERN_WARNING "MARK: targinfosize %u != %Zu\n",
		       targinfosize,
		       IPT_ALIGN(sizeof(struct ipt_mark_target_info)));
		return 0;
	}

/* 2002/5/14 Steve ++ */
#if 0
	if (strcmp(tablename, "mangle") != 0) {
		printk(KERN_WARNING "MARK: can only be called from \"mangle\" table, not \"%s\"\n", tablename);
		return 0;
	}
#endif
/* Steve -- */
	return 1;
}

static struct ipt_target ipt_mark_reg
= { { NULL, NULL }, "MARK", target, checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
	if (ipt_register_target(&ipt_mark_reg))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_mark_reg);
}

module_init(init);
module_exit(fini);
