#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/checksum.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_ROUTEMARK.h>
#include <linux/netdevice.h>
#include <linux/spinlock.h>
#include <linux/timer.h>
/* 2005/2/14 Steve ++ */
#include <linux/netfilter_ipv4/ipt_condition.h>
/* Steve -- */

static spinlock_t routemark_lock = SPIN_LOCK_UNLOCKED;
static unsigned long wan_traffic[MAX_WAN_PORT];
/* 2004/7/7 Steve ++ */
static unsigned long up_traffic[MAX_WAN_PORT], down_traffic[MAX_WAN_PORT];

#define		UPSTREAM_PACKET_SIZE	64
#define		DOWNSTREAM_PACKET_SIZE	3500
/* Steve -- */

#define AUTO_DETECT_INTERVAL 3
static struct timer_list automark;
/* 2003/4/14 Steve ++ */
#include<linux/proc_fs.h>

static struct ipt_routemark_target_info target_info;
static struct proc_dir_entry *proc_net_wan_status = NULL;
/* Steve -- */
extern struct condition_variable *head_condition;

static void
reset_dev_info(struct ipt_routemark_target_info *info)
{
	struct net_device *dev;

        for (dev = dev_base; dev != NULL; dev = dev->next) {
		struct net_device_stats *stats = (dev->get_stats ? dev->get_stats(dev): NULL);

		if (stats) {
                        stats->rx_bytes = 0;
			stats->last_rx_bytes = 0;
			stats->diff_rx_bytes = 0;
			stats->last_rx_packets = 0;
                        stats->rx_packets = 0;
                        stats->rx_errors = 0;
                        stats->tx_bytes = 0;
                        stats->tx_packets = 0;
                        stats->tx_errors = 0;
/* 2004/7/7 Steve ++ */
			stats->last_tx_bytes = 0;
			stats->diff_tx_bytes = 0;
			stats->last_tx_packets = 0;
/* Steve -- */
                }
        }
}

static int
get_dev_traffic_info(struct ipt_routemark_target_info *info)
{
	struct net_device *dev;
	int i = 0;

	for (i = 0; i < info->target_num; i++) {
		for (dev = dev_base; dev != NULL; dev = dev->next) {
			if (dev &&
			    strlen(info->element[i].name) == strlen(dev->name) &&
			    !memcmp(info->element[i].name,
				    dev->name,
				    strlen(info->element[i].name))) {
				struct net_device_stats *stats = (dev->get_stats ? dev->get_stats(dev): NULL);
				if (stats && stats->rx_bytes >= stats->last_rx_bytes) {
					wan_traffic[i] = stats->rx_bytes;
					stats->last_rx_bytes = stats->rx_bytes;
					break;
				} else {
					reset_dev_info(info);
					return -1;
				}
			}
		}
	}
	return 0;
}

static int
get_dev_auto_info(struct ipt_routemark_target_info *info)
{
	struct net_device *dev;
	int i = 0;

	for (i = 0; i < info->target_num; i++) {
		for (dev = dev_base; dev != NULL; dev = dev->next) {
			if (dev &&
			    strlen(info->element[i].name) == strlen(dev->name) &&
			    !memcmp(info->element[i].name,
				    dev->name,
				    strlen(info->element[i].name))) {
				struct net_device_stats *stats = (dev->get_stats ? dev->get_stats(dev): NULL);
/* 2004/7/7 Steve ++ */
#if 0
				if (stats && stats->rx_bytes >= stats->last_rx_bytes) {
#endif
				if (stats && stats->diff_rx_bytes >= 0 &&
				    stats->diff_tx_bytes >= 0) {
#if 0
					wan_traffic[i] = stats->diff_rx_bytes;
#endif
					up_traffic[i] = stats->diff_tx_bytes / AUTO_DETECT_INTERVAL;
					down_traffic[i] = stats->diff_rx_bytes / AUTO_DETECT_INTERVAL;
/* Steve -- */
					break;
				} else {
					reset_dev_info(info);
					return -1;
				}
			}
		}
	}
	return 0;
}

static int
get_dev_packet_info(struct ipt_routemark_target_info *info)
{
	struct net_device *dev;
	int i = 0;

	for (i = 0; i < info->target_num; i++) {
		for (dev = dev_base; dev != NULL; dev = dev->next) {
			if (dev &&
			    strlen(info->element[i].name) == strlen(dev->name) &&
			    !memcmp(info->element[i].name,
				    dev->name,
				    strlen(info->element[i].name))) {
				struct net_device_stats *stats = (dev->get_stats ? dev->get_stats(dev): NULL);
				if (stats && stats->rx_packets >= stats->last_rx_packets) {
					wan_traffic[i] = stats->rx_packets;
					stats->last_rx_packets = stats->rx_packets;
					break;
				} else {
					reset_dev_info(info);
					return -1;
				}
			}
		}
	}
	return 0;
}

static void
get_auto_info(unsigned long timeout)
{
	struct net_device *dev;

	spin_lock_bh(&routemark_lock);
	for (dev = dev_base; dev != NULL; dev = dev->next) {
		if (dev) {
			struct net_device_stats *stats = (dev->get_stats ? dev->get_stats(dev): NULL);
			if (stats && stats->rx_bytes >= stats->last_rx_bytes) {
				stats->diff_rx_bytes = stats->rx_bytes - stats->last_rx_bytes;
				stats->last_rx_bytes = stats->rx_bytes;
			} else {
				stats->last_rx_bytes = 0;
				stats->rx_bytes = 0;
				stats->diff_rx_bytes = 0;
			}
/* 2004/7/7 Steve ++ */
			if (stats && stats->tx_bytes >= stats->last_tx_bytes) {
				stats->diff_tx_bytes = stats->tx_bytes - stats->last_tx_bytes;
				stats->last_tx_bytes = stats->tx_bytes;
			} else {
				stats->last_tx_bytes = 0;
				stats->tx_bytes = 0;
				stats->diff_tx_bytes = 0;
			}
/* Steve -- */
		}
	}
	spin_unlock_bh(&routemark_lock);

	automark.expires = jiffies + (HZ * AUTO_DETECT_INTERVAL);
	add_timer(&automark);
}

static unsigned long
get_session_mark(struct ipt_routemark_target_info *info)
{
	unsigned long mark = 0;
	int i = 0;
	unsigned long s1, s2;

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto session_done;

		if (target_info.element[info->element[i].id-1].status) {
			s1 = wan_connection_num[info->element[i].id-1] / info->element[i].weight;
			mark = info->element[i].mark;
			break;
		}
	}

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto session_done;

		if (target_info.element[info->element[i].id-1].status) {
			s2 = wan_connection_num[info->element[i].id-1] / info->element[i].weight;

			if (s2 < s1) {
				s1 = s2;
				mark = info->element[i].mark;
			}
		}
	}

session_done:
	return mark;
}

static unsigned long
get_vpn_session_mark(struct ipt_routemark_target_info *info)
{
	unsigned long mark = 0;
	int i = 0;
	unsigned long s1, s2;

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto session_done;

		if (info->element[i].status) {
			s1 = wan_connection_num[info->element[i].id-1] / info->element[i].weight;
			mark = info->element[i].mark;
			break;
		}
	}

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto session_done;

		if (info->element[i].status) {
			s2 = wan_connection_num[info->element[i].id-1] / info->element[i].weight;

			if (s2 < s1) {
				s1 = s2;
				mark = info->element[i].mark;
			}
		}
	}

session_done:
	return mark;
}

static unsigned long
get_auto_mark(struct ipt_routemark_target_info *info)
{
        unsigned long mark = 0;
        int i = 0;
        unsigned long s1, s2, w1, w2, b1, b2, c1, c2;
/* 2004/7/7 Steve ++ */
	unsigned long w_up = 0, w_down = 0;
/* Steve -- */

	spin_lock_bh(&routemark_lock);
	if (get_dev_auto_info(info) < 0) {
	/* dev overflow */
		mark = get_session_mark(info);
		spin_unlock_bh(&routemark_lock);
		goto auto_done;
	}

	spin_unlock_bh(&routemark_lock);

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto auto_done;

		if (target_info.element[info->element[i].id-1].status) {
/* 2004/7/7 Steve ++ */
#if 0
			b1 = wan_traffic[i] / info->element[i].weight;
#endif
			w_up = target_info.element[info->element[i].id-1].upstream;
			w_down = target_info.element[info->element[i].id-1].downstream;
			c1 = ((up_traffic[i] + UPSTREAM_PACKET_SIZE) / 1024 * w_down) +
			     ((down_traffic[i] + DOWNSTREAM_PACKET_SIZE) / 1024 *  w_up);
			if (c1)
				b1 = (w_up * w_down) / c1;
			else
				b1 = w_up * w_down;
/* Steve -- */
        		s1 = wan_connection_num[info->element[i].id-1];
			w1 = s1 / info->element[i].weight;
       		        mark = info->element[i].mark;
		        if (!s1)
				goto auto_done;

			break;
        	}
	}

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto auto_done;

		if (target_info.element[info->element[i].id-1].status) {
/* 2004/7/7 Steve ++ */
			w_up = target_info.element[info->element[i].id-1].upstream;
			w_down = target_info.element[info->element[i].id-1].downstream;
			c2 = ((up_traffic[i] + UPSTREAM_PACKET_SIZE) / 1024 * w_down) +
			     ((down_traffic[i] + DOWNSTREAM_PACKET_SIZE) / 1024 * w_up);
			if (c2)
				b2 = w_up * w_down / c2;
			else
				b2 = w_up * w_down;
#if 0			
			b2 = wan_traffic[i] / info->element[i].weight;
#endif
/* Steve -- */
			s2 = wan_connection_num[info->element[i].id-1];
			w2 = s2 / info->element[i].weight;
			if (!s2) {
				mark = info->element[i].mark;
				goto auto_done;
			}

			if ((b1 < b2) || ((b2 == b1) && (w2 < w1))) {
				b1 = b2;
				s1 = s2;
				w1 = w2;
				mark = info->element[i].mark;
			}
		}
	}

auto_done:
	return mark;
}

static unsigned long
get_vpn_auto_mark(struct ipt_routemark_target_info *info)
{
        unsigned long mark = 0;
        int i = 0;
        unsigned long s1, s2, w1, w2, b1, b2, c1, c2;
/* 2004/7/7 Steve ++ */
	unsigned long w_up = 0, w_down = 0;
/* Steve -- */

	spin_lock_bh(&routemark_lock);
	if (get_dev_auto_info(info) < 0) {
	/* dev overflow */
		mark = get_vpn_session_mark(info);
		spin_unlock_bh(&routemark_lock);
		goto auto_done;
	}

	spin_unlock_bh(&routemark_lock);

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto auto_done;

		if (info->element[i].status) {
/* 2004/7/7 Steve ++ */
#if 0
			b1 = wan_traffic[i] / info->element[i].weight;
#endif
			w_up = target_info.element[info->element[i].id-1].upstream;
			w_down = target_info.element[info->element[i].id-1].downstream;
			c1 = ((up_traffic[i] + UPSTREAM_PACKET_SIZE) / 1024 * w_down) +
			     ((down_traffic[i] + DOWNSTREAM_PACKET_SIZE) / 1024 *  w_up);
			if (c1)
				b1 = (w_up * w_down) / c1;
			else
				b1 = w_up * w_down;
/* Steve -- */
        		s1 = wan_connection_num[info->element[i].id-1];
			w1 = s1 / info->element[i].weight;
       		        mark = info->element[i].mark;
		        if (!s1)
				goto auto_done;

			break;
        	}
	}

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto auto_done;

		if (info->element[i].status) {
/* 2004/7/7 Steve ++ */
			w_up = target_info.element[info->element[i].id-1].upstream;
			w_down = target_info.element[info->element[i].id-1].downstream;
			c2 = ((up_traffic[i] + UPSTREAM_PACKET_SIZE) / 1024 * w_down) +
			     ((down_traffic[i] + DOWNSTREAM_PACKET_SIZE) / 1024 * w_up);
			if (c2)
				b2 = w_up * w_down / c2;
			else
				b2 = w_up * w_down;
#if 0			
			b2 = wan_traffic[i] / info->element[i].weight;
#endif
/* Steve -- */
			s2 = wan_connection_num[info->element[i].id-1];
			w2 = s2 / info->element[i].weight;
			if (!s2) {
				mark = info->element[i].mark;
				goto auto_done;
			}

			if ((b1 < b2) || ((b2 == b1) && (w2 < w1))) {
				b1 = b2;
				s1 = s2;
				w1 = w2;
				mark = info->element[i].mark;
			}
		}
	}

auto_done:
	return mark;
}

static unsigned long
get_traffic_mark(struct ipt_routemark_target_info *info)
{
        unsigned long mark = 0;
        int i = 0;
        unsigned long b1, b2, s1, s2;

	spin_lock_bh(&routemark_lock);
	if (get_dev_traffic_info(info) < 0) {
	/* dev overflow */
		mark = get_session_mark(info);
		spin_unlock_bh(&routemark_lock);
		goto traffic_done;
	}

	spin_unlock_bh(&routemark_lock);

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto traffic_done;

		if (target_info.element[info->element[i].id-1].status) {
			b1 = wan_traffic[i] / info->element[i].weight;
        		s1 = wan_connection_num[info->element[i].id-1];
			mark = info->element[i].mark;
        		if (!s1)
		                goto traffic_done;

			break;
		}
        }

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto traffic_done;


		if (target_info.element[info->element[i].id-1].status) {
			b2 = wan_traffic[i] / info->element[i].weight;
			s2 = wan_connection_num[info->element[i].id-1];
			if (!s2) {
				mark = info->element[i].mark;
				goto traffic_done;
			}

			if (b2 < b1) {
				b1 = b2;
				mark = info->element[i].mark;
			}
		}
	}

traffic_done:
	return mark;
}

static unsigned long
get_vpn_traffic_mark(struct ipt_routemark_target_info *info)
{
        unsigned long mark = 0;
        int i = 0;
        unsigned long b1, b2, s1, s2;

	spin_lock_bh(&routemark_lock);
	if (get_dev_traffic_info(info) < 0) {
	/* dev overflow */
		mark = get_vpn_session_mark(info);
		spin_unlock_bh(&routemark_lock);
		goto traffic_done;
	}

	spin_unlock_bh(&routemark_lock);

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto traffic_done;

		if (info->element[i].status) {
			b1 = wan_traffic[i] / info->element[i].weight;
        		s1 = wan_connection_num[info->element[i].id-1];
			mark = info->element[i].mark;
        		if (!s1)
		                goto traffic_done;

			break;
		}
        }

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto traffic_done;


		if (info->element[i].status) {
			b2 = wan_traffic[i] / info->element[i].weight;
			s2 = wan_connection_num[info->element[i].id-1];
			if (!s2) {
				mark = info->element[i].mark;
				goto traffic_done;
			}

			if (b2 < b1) {
				b1 = b2;
				mark = info->element[i].mark;
			}
		}
	}

traffic_done:
	return mark;
}

static unsigned long
get_packet_mark(struct ipt_routemark_target_info *info)
{
        unsigned long mark = 0;
        int i = 0;
        unsigned long b1, b2, s1, s2;

	spin_lock_bh(&routemark_lock);
	if (get_dev_packet_info(info) < 0) {
	/* dev overflow */
		mark = get_session_mark(info);
		spin_unlock_bh(&routemark_lock);
		goto packet_done;
	}

	spin_unlock_bh(&routemark_lock);

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto packet_done;

		if (target_info.element[info->element[i].id-1].status) {
			b1 = wan_traffic[i] / info->element[i].weight;
			s1 = wan_connection_num[info->element[i].id-1];
			mark = info->element[i].mark;
			if (!s1)
				goto packet_done;

			break;
		}
	}

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto packet_done;

		if (target_info.element[info->element[i].id-1].status) {
			b2 = wan_traffic[i] / info->element[i].weight;
			s2 = wan_connection_num[info->element[i].id-1];
			if (!s2) {
				mark = info->element[i].mark;
				goto packet_done;
			}

			if (b2 < b1) {
				b1 = b2;
				mark = info->element[i].mark;
			}
		}
	}

packet_done:
	return mark;
}

static unsigned long
get_vpn_packet_mark(struct ipt_routemark_target_info *info)
{
        unsigned long mark = 0;
        int i = 0;
        unsigned long b1, b2, s1, s2;

	spin_lock_bh(&routemark_lock);
	if (get_dev_packet_info(info) < 0) {
	/* dev overflow */
		mark = get_vpn_session_mark(info);
		spin_unlock_bh(&routemark_lock);
		goto packet_done;
	}

	spin_unlock_bh(&routemark_lock);

	for (i = 0; i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto packet_done;

		if (info->element[i].status) {
			b1 = wan_traffic[i] / info->element[i].weight;
			s1 = wan_connection_num[info->element[i].id-1];
			mark = info->element[i].mark;
			if (!s1)
				goto packet_done;

			break;
		}
	}

	for (i = (i + 1); i < MAX_WAN_PORT; i++) {
		if (info->target_num == i)
			goto packet_done;

		if (info->element[i].status) {
			b2 = wan_traffic[i] / info->element[i].weight;
			s2 = wan_connection_num[info->element[i].id-1];
			if (!s2) {
				mark = info->element[i].mark;
				goto packet_done;
			}

			if (b2 < b1) {
				b1 = b2;
				mark = info->element[i].mark;
			}
		}
	}

packet_done:
	return mark;
}

static unsigned int
target(struct sk_buff **pskb,
       unsigned int hooknum,
       const struct net_device *in,
       const struct net_device *out,
       const void *targinfo,
       void *userinfo)
{
	const struct ipt_routemark_target_info *markinfo = targinfo;
	struct ipt_routemark_target_info realinfo;
	enum ip_conntrack_info ctinfo;
	struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)(*pskb), &ctinfo);
	unsigned long routemark = 0;
	int i = 0, j = 0, k = 0;
/* 2005/2/14 Steve ++ */
	struct condition_variable *var;
/* Steve -- */

	if (!ct)
		return IPT_CONTINUE;

/* 2004/5/12 Steve ++ */
	if (markinfo->how == IPT_ROUTEMARK_SET_MARK) {
		ct->routemark = markinfo->mark;
		return NF_ACCEPT;
	}
/* Steve -- */

	memset(&realinfo, '\0', sizeof(realinfo));
	realinfo.mode = target_info.mode;
	realinfo.target_num = markinfo->target_num;

/* 2005/2/14 Steve ++ */
	if (markinfo->how == IPT_ROUTEMARK_SET_WAN) {
		for (i = 0; i < markinfo->target_num; i++) {
			k = 0;
			for (j = 0; j < markinfo->target_num; j++) {
				if (i == j)
					continue;

				if ((target_info.element[markinfo->element[i].id-1].prio > 
				     target_info.element[markinfo->element[j].id-1].prio) ||
				   ((target_info.element[markinfo->element[i].id-1].prio == target_info.element[markinfo->element[j].id-1].prio) && i > j))
					k++;
			}
			strcpy(realinfo.element[k].name, target_info.element[markinfo->element[i].id-1].name);
			realinfo.element[k].id = target_info.element[markinfo->element[i].id-1].id;
			realinfo.element[k].mark = target_info.element[markinfo->element[i].id-1].mark;
			realinfo.element[k].weight = target_info.element[markinfo->element[i].id-1].weight;
			realinfo.element[k].prio = target_info.element[markinfo->element[i].id-1].prio;
			realinfo.element[k].status = target_info.element[markinfo->element[i].id-1].status;
/* 2004/7/7 Steve ++ */
			realinfo.element[k].upstream = target_info.element[markinfo->element[i].id-1].upstream;
			realinfo.element[k].downstream = target_info.element[markinfo->element[i].id-1].downstream;
/* Steve -- */		
		}
	} else if (markinfo->how == IPT_ROUTEMARK_SET_VPN) {
		for (i = 0; i < markinfo->target_num; i++) {
			k = 0;
			for (j = 0; j < markinfo->target_num; j++) {
				if (i == j)
					continue;

				if ((target_info.element[markinfo->element[i].id-1].prio > 
				     target_info.element[markinfo->element[j].id-1].prio) ||
				   ((target_info.element[markinfo->element[i].id-1].prio == target_info.element[markinfo->element[j].id-1].prio) && i > j))
					k++;
			}
			strcpy(realinfo.element[k].name, target_info.element[markinfo->element[i].id-1].name);
			realinfo.element[k].id = target_info.element[markinfo->element[i].id-1].id;
			realinfo.element[k].mark = (target_info.element[markinfo->element[i].id-1].mark - 1) | 0x100;
			realinfo.element[k].weight = target_info.element[markinfo->element[i].id-1].weight;
			realinfo.element[k].prio = target_info.element[markinfo->element[i].id-1].prio;

			var = head_condition;
			while (var) {
				if (!strcmp(markinfo->element[i].vpnname, var->name)) {
					realinfo.element[k].status = var->boolean;
#if 0
printk("vpnname: %s varname: %s status: %d k: %d\n",
	markinfo->element[i].vpnname,
	var->name,
	realinfo.element[k].status,
	k);
#endif
					break;
				}
				var = var->next;
			}
/* 2004/7/7 Steve ++ */
			realinfo.element[k].upstream = target_info.element[markinfo->element[i].id-1].upstream;
			realinfo.element[k].downstream = target_info.element[markinfo->element[i].id-1].downstream;
/* Steve -- */		
		}
	} else
		return IPT_CONTINUE;
/* Steve -- */

	if (markinfo->how == IPT_ROUTEMARK_SET_WAN) {
		switch(realinfo.mode) {
		case IPT_ROUTEMARK_SESSION:
			if (!(routemark = get_session_mark(&realinfo)))
				return IPT_CONTINUE;

				ct->routemark = routemark;
				break;
		case IPT_ROUTEMARK_TRAFFIC:
			if (!(routemark = get_traffic_mark(&realinfo)))
				return IPT_CONTINUE;

			ct->routemark = routemark;
			break;
		case IPT_ROUTEMARK_PACKET:
			if (!(routemark = get_packet_mark(&realinfo)))
				return IPT_CONTINUE;
	
			ct->routemark = routemark;
			break;
		case IPT_ROUTEMARK_AUTO:
			if (!(routemark = get_auto_mark(&realinfo)))
				return IPT_CONTINUE;

			ct->routemark = routemark;
			break;
		}
	} else if (markinfo->how == IPT_ROUTEMARK_SET_VPN) {
		switch(realinfo.mode) {
		case IPT_ROUTEMARK_SESSION:
			if (!(routemark = get_vpn_session_mark(&realinfo)))
				return IPT_CONTINUE;

				ct->routemark = routemark;
				break;
		case IPT_ROUTEMARK_TRAFFIC:
			if (!(routemark = get_vpn_traffic_mark(&realinfo)))
				return IPT_CONTINUE;

			ct->routemark = routemark;
			break;
		case IPT_ROUTEMARK_PACKET:
			if (!(routemark = get_vpn_packet_mark(&realinfo)))
				return IPT_CONTINUE;
	
			ct->routemark = routemark;
			break;
		case IPT_ROUTEMARK_AUTO:
			if (!(routemark = get_vpn_auto_mark(&realinfo)))
				return IPT_CONTINUE;

			ct->routemark = routemark;
			break;
		}
	}
	return NF_ACCEPT;
}

static int
checkentry(const char *tablename,
	   const struct ipt_entry *e,
           void *targinfo,
           unsigned int targinfosize,
           unsigned int hook_mask)
{
	if (targinfosize != IPT_ALIGN(sizeof(struct ipt_routemark_target_info))) {
		printk(KERN_WARNING "MARK: targinfosize %u != %Zu\n",
		       targinfosize,
		       IPT_ALIGN(sizeof(struct ipt_routemark_target_info)));
		return 0;
	}

	return 1;
}

static int
ipt_ROUTEMARK_read_info(char *buffer, char **start,	off_t offset, int length, int *eof,	void *data)
{
	off_t pos = 0;
	off_t begin = 0;
	int len = 0;
	struct ipt_routemark_element_info *wan_info = (struct ipt_routemark_element_info *)data;

	spin_lock_bh(&routemark_lock);
	len += sprintf(buffer, "name: %s status: %d prio: %d weight: %d mark: %d upstream: %lu downstream: %lu\n",
		       wan_info->name,
		       wan_info->status,
		       wan_info->prio,
		       wan_info->weight,
		       wan_info->mark,
		       wan_info->upstream,
		       wan_info->downstream);

	pos = begin + len;
	if(pos < offset) { len = 0; begin = pos; }
	*start = buffer + (offset - begin);
	len -= (offset - begin);
	if(len > length) len = length;
	spin_unlock_bh(&routemark_lock);
	return len;
}

static int
ipt_ROUTEMARK_write_info(struct file *file, const char *input, unsigned long size, void *data)
{
	char buffer[1024];
	int len = 0;
	char *cp, *c, *end;
	struct ipt_routemark_element_info *wan_info = (struct ipt_routemark_element_info *)data;

	if(size > 1024) len = 1024; else len = size;
	memset(buffer, '\0', sizeof(buffer));
	spin_lock_bh(&routemark_lock);
	if(copy_from_user(buffer,input,len)) return -EFAULT;
	buffer[len-1]= '\0';
	cp = buffer;

	while (*cp) {
		if (!strncmp(cp, "prio=", 5)) {
			cp += 5;
			c = cp;
			while (*cp) {
				if (*cp == ' ') {
					*cp = '\0';
					break;
				}
				cp++;
			}
			wan_info->prio = simple_strtoul(c, &end, 10);
		} else if (!strncmp(cp, "weight=", 7)) {
			cp += 7;
			c = cp;
			while (*cp) {
				if (*cp == ' ') {
					*cp = '\0';
					break;
				}

				cp++;
			}
			wan_info->weight = simple_strtoul(c, &end, 10);
			if (!wan_info->weight)
				wan_info->weight = 1;
		} else if (!strncmp(cp, "name=", 5)) {
			cp += 5;
			c = cp;
			while (*cp) {
				if (*cp == ' ') {
					*cp = '\0';
					break;
				}
				cp++;
			}
			strcpy(wan_info->name, c);
		} else if (!strncmp(cp, "mark=", 5)) {
			cp += 5;
			c = cp;
			while (*cp) {
				if (*cp == ' ') {
					*cp = '\0';
					break;
				}

				cp++;
			}
			wan_info->mark = simple_strtoul(c, &end, 10);
		} else if (!strncmp(cp, "status=", 7)) {
			cp += 7;
			c = cp;
			while (*cp) {
				if (*cp == ' ') {
					*cp = '\0';
					break;
				}

				cp++;
			}
			if (*c == '0')
				wan_info->status = 0;
			else
				wan_info->status = 1;
		}
/* 2004/7/7 Steve ++ */
		else if (!strncmp(cp, "upstream=", 9)) {
			cp += 9;
			c = cp;
			while (*cp) {
				if (*cp == ' ') {
					*cp = '\0';
					break;
				}

				cp++;
			}
			wan_info->upstream = simple_strtoul(c, &end, 10) / 1024;
		}
		else if (!strncmp(cp, "downstream=", 11)) {
			cp += 11;
			c = cp;
			while (*cp) {
				if (*cp == ' ') {
					*cp = '\0';
					break;
				}

				cp++;
			}
			wan_info->downstream = simple_strtoul(c, &end, 10) / 1024;
		}
/* Steve -- */
		cp++;
	}

	spin_unlock_bh(&routemark_lock);
	return size;
}

static int
ipt_ROUTEMARK_read_mode_info(char *buffer,
			     char **start,
			     off_t offset,
			     int length,
			     int *eof,
			     void *data)
{
	off_t pos = 0;
	off_t begin = 0;
	int len = 0;

	spin_lock_bh(&routemark_lock);
	switch (target_info.mode) {
	case IPT_ROUTEMARK_SESSION:
		len += sprintf(buffer+len, "session\n");
		break;
	case IPT_ROUTEMARK_TRAFFIC:
		len += sprintf(buffer+len, "traffic\n");
		break;
	case IPT_ROUTEMARK_PACKET:
		len += sprintf(buffer+len, "packet\n");
		break;
	case IPT_ROUTEMARK_AUTO:
		len += sprintf(buffer+len, "auto\n");
		break;
	default:
		len += sprintf(buffer+len, "Unknown\n");
	}

	pos = begin + len;
	if(pos < offset) { len = 0; begin = pos; }
	*start = buffer + (offset - begin);
	len -= (offset - begin);
	if(len > length) len = length;

	spin_unlock_bh(&routemark_lock);
	return len;
}

static int
ipt_ROUTEMARK_write_mode_info(struct file *file, 
			      const char *input, 
			      unsigned long size,
			      void *data)
{
	char buffer[1024];
	int len = 0;

	if(size > 1024) len = 1024; else len = size;
	spin_lock_bh(&routemark_lock);
	if(copy_from_user(buffer,input,len)) return -EFAULT;
	buffer[len-1]= '\0';

	if (!strncmp(buffer, "session", 7))
		*((u_int8_t *)data) = IPT_ROUTEMARK_SESSION;
	else if (!strncmp(buffer, "traffic", 7))
		*((u_int8_t *)data) = IPT_ROUTEMARK_TRAFFIC;
	else if (!strncmp(buffer, "packet", 6))
		*((u_int8_t *)data) = IPT_ROUTEMARK_PACKET;
	else if (!strncmp(buffer, "auto", 4))
		*((u_int8_t *)data) = IPT_ROUTEMARK_AUTO;

	spin_unlock_bh(&routemark_lock);
	return len;
}

static struct ipt_target ipt_routemark_reg
= { { NULL, NULL }, "ROUTEMARK", target, checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
/* 2003/4/13 Steve ++ */
	int i = 0;
	struct proc_dir_entry *status_proc[MAX_WAN_PORT+1];
	char proc_name[20];

	proc_net_wan_status = proc_mkdir("ipt_wan_status", proc_net);
	if (!proc_net_wan_status)
		return -EACCES;

	status_proc[0] = create_proc_entry("MODE", 0644, proc_net_wan_status);
	if (!status_proc[0])
		return -ENOMEM;
	target_info.mode = IPT_ROUTEMARK_AUTO;
	target_info.target_num = MAX_WAN_PORT;
	status_proc[0]->owner = THIS_MODULE;
	status_proc[0]->read_proc = ipt_ROUTEMARK_read_mode_info;
	status_proc[0]->write_proc = ipt_ROUTEMARK_write_mode_info;
	status_proc[0]->data = &target_info.mode;

	for (i = 1; i <= MAX_WAN_PORT; i++) {
		sprintf(proc_name, "WAN%d", i);
		status_proc[i] = create_proc_entry(proc_name, 0644, proc_net_wan_status);
		if (!status_proc[i])
			return -ENOMEM;

		sprintf(target_info.element[i-1].name, "eth%d", i);
		target_info.element[i-1].id = i;
		target_info.element[i-1].mark = i;
		target_info.element[i-1].status = 0;
		target_info.element[i-1].weight = 1;
		target_info.element[i-1].prio = i;
/* 2004/7/7 Steve ++ */
		target_info.element[i-1].upstream = 512 * 1024 / 8;
		target_info.element[i-1].downstream = 512 * 1024 / 8;
/* Steve -- */
		status_proc[i]->owner = THIS_MODULE;
		status_proc[i]->read_proc = ipt_ROUTEMARK_read_info;
		status_proc[i]->write_proc = ipt_ROUTEMARK_write_info;
		status_proc[i]->data = &target_info.element[i-1];
	}
/* Steve -- */

	if (ipt_register_target(&ipt_routemark_reg))
		return -EINVAL;

	init_timer(&automark);
	automark.function = get_auto_info;
	automark.expires = jiffies + (HZ * AUTO_DETECT_INTERVAL);
	add_timer(&automark);
	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_routemark_reg);
}

module_init(init);
module_exit(fini);
