#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/checksum.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_RULEMARK.h>

static unsigned int
target(struct sk_buff **pskb,
       unsigned int hooknum,
       const struct net_device *in,
       const struct net_device *out,
       const void *targinfo,
       void *userinfo)
{
	const struct ipt_rulemark_target_info *markinfo = targinfo;
	enum ip_conntrack_info ctinfo;
	struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)(*pskb), &ctinfo);

	if (!ct)
		return IPT_CONTINUE;

	if (ct) {
		switch(markinfo->mode) {
		case IPT_MARK_SET:
			ct->rulemark = markinfo->mark;
			break;
		case IPT_MARK_OR:
			ct->rulemark |= markinfo->mark;
			break;
		case IPT_MARK_AND:
			ct->rulemark &= markinfo->mark;
			break;
		}
	}

	return IPT_CONTINUE;
}

static int
checkentry(const char *tablename,
	   const struct ipt_entry *e,
           void *targinfo,
           unsigned int targinfosize,
           unsigned int hook_mask)
{
	if (targinfosize != IPT_ALIGN(sizeof(struct ipt_rulemark_target_info))) {
		printk(KERN_WARNING "RULEMARK: targinfosize %u != %Zu\n",
		       targinfosize,
		       IPT_ALIGN(sizeof(struct ipt_rulemark_target_info)));
		return 0;
	}

	return 1;
}

static struct ipt_target ipt_rulemark_reg
= { { NULL, NULL }, "RULEMARK", target, checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
	if (ipt_register_target(&ipt_rulemark_reg))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_rulemark_reg);
}

module_init(init);
module_exit(fini);
