/*
 * This is a module which is used for setting the MSS option in TCP packets.
 *
 * Copyright (c) 2000 Marc Boucher
 */
#include <linux/module.h>
#include <linux/skbuff.h>

#include <linux/ip.h>
#include <net/tcp.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_TCPWINDOW.h>

#if 0
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

static u_int16_t
cheat_check(u_int32_t oldvalinv, u_int32_t newval, u_int16_t oldcheck)
{
	u_int32_t diffs[] = { oldvalinv, newval };
	return csum_fold(csum_partial((char *)diffs, sizeof(diffs),
                                      oldcheck^0xFFFF));
}

static unsigned int
ipt_tcpwindow_target(struct sk_buff **pskb,
		     unsigned int hooknum,
		     const struct net_device *in,
		     const struct net_device *out,
		     const void *targinfo,
		     void *userinfo)
{
	const struct ipt_tcpwindow_info *tcpwindowinfo = targinfo;
	struct tcphdr *tcph;
	struct iphdr *iph = (*pskb)->nh.iph;
	u_int16_t tcplen, newwindow, oldwindow;

	tcplen = (*pskb)->len - iph->ihl*4;
	tcph = (void *)iph + iph->ihl*4;

	if (tcplen < sizeof(*tcph)) {
		if (net_ratelimit())
			printk(KERN_ERR
			       "ipt_tcpwindow_target: bad length (%d bytes)\n",
			       (*pskb)->len);
		return NF_DROP;
	}

	newwindow = tcpwindowinfo->window;
	oldwindow = ntohs(tcph->window);

	if (oldwindow > newwindow) {
		tcph->window = htons(newwindow);
		tcph->check = cheat_check(htons(oldwindow)^0xFFFF,
					  htons(newwindow),
					  tcph->check);
	}

	(*pskb)->ip_summed = CHECKSUM_NONE;
	(*pskb)->nfcache |= NFC_UNKNOWN | NFC_ALTERED;
	return IPT_CONTINUE;
}

static int
ipt_tcpwindow_checkentry(const char *tablename,
		      const struct ipt_entry *e,
		      void *targinfo,
		      unsigned int targinfosize,
		      unsigned int hook_mask)
{
	const struct ipt_tcpwindow_info *tcpwindowinfo = targinfo;

	if (targinfosize != IPT_ALIGN(sizeof(struct ipt_tcpwindow_info)))
		return 0;

	if (e->ip.proto == IPPROTO_TCP)
		return 1;

	return 0;
}

static struct ipt_target ipt_tcpwindow_reg
= { { NULL, NULL }, "TCPWINDOW",
    ipt_tcpwindow_target, ipt_tcpwindow_checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
	return ipt_register_target(&ipt_tcpwindow_reg);
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_tcpwindow_reg);
}

module_init(init);
module_exit(fini);
