#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/checksum.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_VIPMARK.h>

static unsigned int
target(struct sk_buff **pskb,
       unsigned int hooknum,
       const struct net_device *in,
       const struct net_device *out,
       const void *targinfo,
       void *userinfo)
{
	const struct ipt_vipmark_target_info *markinfo = targinfo;
/* 2002/5/14 Steve ++ */
        enum ip_conntrack_info ctinfo;
        struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)(*pskb), &ctinfo);

	if (!ct)
		return IPT_CONTINUE;

/* 2003/1/2 Steve ++ */
        if (ct) {
            switch(markinfo->mode) {
            case IPT_MARK_SET:
                ct->vipmark = markinfo->mark;
                break;
            case IPT_MARK_OR:
                ct->vipmark |= markinfo->mark;
                break;
            case IPT_MARK_AND:
                ct->vipmark &= markinfo->mark;
                break;
	    }
	}
/* Steve -- */

	return IPT_CONTINUE;
}

static int
checkentry(const char *tablename,
	   const struct ipt_entry *e,
           void *targinfo,
           unsigned int targinfosize,
           unsigned int hook_mask)
{
	if (targinfosize != IPT_ALIGN(sizeof(struct ipt_vipmark_target_info))) {
		printk(KERN_WARNING "VIPMARK: targinfosize %u != %Zu\n",
		       targinfosize,
		       IPT_ALIGN(sizeof(struct ipt_vipmark_target_info)));
		return 0;
	}

	return 1;
}

static struct ipt_target ipt_vipmark_reg
= { { NULL, NULL }, "VIPMARK", target, checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
	if (ipt_register_target(&ipt_vipmark_reg))
		return -EINVAL;

	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_target(&ipt_vipmark_reg);
}

module_init(init);
module_exit(fini);
