#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/spinlock.h>
#include <linux/netfilter_ipv4/ipt_connlimit.h>
#include <linux/netfilter_ipv4/ip_tables.h>
/* 2004/3/16 Steve ++ */
#include <linux/ip.h>
/* Steve -- */

static spinlock_t connlimit_lock = SPIN_LOCK_UNLOCKED;

static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const void *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen,
      int *hotdrop)
{
	const struct ipt_connlimit_info *info = matchinfo;
	enum ip_conntrack_info ctinfo;
	struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)(skb), &ctinfo);
	int count = 0;
#if 0
/* 2004/3/16 Steve ++ */
	struct iphdr *iph = skb->nh.iph;
/* Steve -- */
#endif	
	spin_lock_bh(&connlimit_lock);
	if (ct && ctinfo == IP_CT_NEW)
	{
#if 0
		count = count_same_conntracks((unsigned long long)info->mark,
					      iph->saddr);
#endif
		count = count_same_conntracks((unsigned long long)info->mark);

		if (count >= info->limit)
		{
			spin_unlock_bh(&connlimit_lock);
			return 1;
		}
	}
	
	spin_unlock_bh(&connlimit_lock);
	return 0;
}

static int
checkentry(const char *tablename,
           const struct ipt_ip *ip,
           void *matchinfo,
           unsigned int matchsize,
           unsigned int hook_mask)
{
	if (matchsize != IPT_ALIGN(sizeof(struct ipt_connlimit_info)))
		return 0;
	
	return 1;
}

static struct ipt_match connlimit_match
= { { NULL, NULL }, "connlimit", &match, &checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
	return ipt_register_match(&connlimit_match);
}

static void __exit fini(void)
{
	ipt_unregister_match(&connlimit_match);
}

module_init(init);
module_exit(fini);
