#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/udp.h>
#include <linux/netfilter_ipv4/ipt_drelay.h>
#include <linux/netfilter_ipv4/ip_tables.h>

//#define DEBUG_DRELAY
#ifdef DEBUG_DRELAY
#define	dns_debug printk
#else
#define dns_debug
#endif

static u_int16_t val16;
static u_int32_t val32;

#define SET_UINT16(num, buff) memcpy(&val16, (u_int16_t*)*buff, 2); num = htons(val16); *buff += 2
#define SET_UINT32(num, buff) memcpy(&val32, (u_int32_t*)*buff, 4); num = htonl(val32); *buff += 4

static void
dns_decode_name(char *name, char **buf)
{
	int i, k, len, j;

	i = k = 0;
	while (**buf) {
		len = *(*buf)++;
		for (j = 0; j < len ; j++)
			name[k++] = *(*buf)++;

		name[k++] = '.';
	}
	name[k-1] = *(*buf)++;
}

static int
dns_decode_rr(struct dns_rr *rr, char **buf, char *header)
{
	if (**buf & 0xC0) {
		(*buf)++;
		header += *(*buf)++;
		dns_decode_name(rr->name, &header);
	} else
		dns_decode_name(rr->name, buf);

	SET_UINT16(rr->type, buf);

/* SuKent++, Thu 26 Jun 2003 08:13:15 AM GMT+8 */
#if 0	
	if (rr->type != A) {
		dns_debug("drelay: invalid dns type\n");
		return -1;
	}
#endif	

	SET_UINT16(rr->class, buf);

	return 0;
}

static
int dns_decode_message(struct dns_message *dns, char **buf)
{
	char *header_start = *buf;

	SET_UINT16(dns->header.id, buf);
	SET_UINT16(dns->header.flags.flags, buf);
	SET_UINT16(dns->header.qdcount, buf);
	SET_UINT16(dns->header.ancount, buf);
	SET_UINT16(dns->header.nscount, buf);
	SET_UINT16(dns->header.arcount, buf);

/* SuKent++, Thu 26 Jun 2003 08:14:24 AM GMT+8 */
	if (dns->header.ancount != 0 ||
	    dns->header.nscount != 0 ||
	    dns->header.arcount != 0) {
		dns_debug("drelay: dns not question\n");
		return -1;
	}
#if 0
	if (dns->header.qdcount != 1 ||
	    dns->header.ancount != 0 ||
	    dns->header.nscount != 0 ||
	    dns->header.arcount != 0) {
		dns_debug("drelay: dns not question\n");
		return -1;
	}
#endif	

	return dns_decode_rr(&dns->question, buf, header_start);
}

static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const void *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen,
      int *hotdrop)
{
	const struct ipt_drelay_info *info = matchinfo;
	const struct iphdr *iph = skb->nh.iph;
	const struct udphdr *udph = (void *)iph + iph->ihl*4;
	unsigned int udplen = skb->len - iph->ihl * 4;
	const char *data = (const char *)udph + sizeof(struct udphdr);
	struct dns_message dns;

	if (udph->dest != htons(DNS_PORT) ||
	    udplen <= (sizeof(struct udphdr) + sizeof(struct dns_header_s))) {
		dns_debug("drelay: udph->dest = %d\n", ntohs(udph->dest));
		return 0;
	}

	if (dns_decode_message(&dns, &data) < 0)
		return 0;

	dns_debug("drelay: dns.question.name = %s, info->string = %s,
			     info->len = %d\n",
		   dns.question.name,
		   info->string,
		   info->len);

/* SuKent++, Thu 26 Jun 2003 08:12:34 AM GMT+8 */
	if (dns.question.type == PTR)
		return	1;
	else
		return (strstr(dns.question.name, info->string));
#if 0
	return (strlen(dns.question.name) == info->len &&
		((memcmp(dns.question.name, info->string, info->len) == 0) ^ info->invert));
#endif		
}

static int
checkentry(const char *tablename,
           const struct ipt_ip *ip,
           void *matchinfo,
           unsigned int matchsize,
           unsigned int hook_mask)
{
	if (matchsize != IPT_ALIGN(sizeof(struct ipt_drelay_info)))
		return 0;

	if (ip->proto != IPPROTO_UDP) {
		printk("drelay: Only work in udp\n");
		return 0;
	}
	return 1;
}

static struct ipt_match drelay_match
= { { NULL, NULL }, "drelay", &match, &checkentry, NULL, THIS_MODULE };

static int __init init(void)
{
	return ipt_register_match(&drelay_match);
}

static void __exit fini(void)
{
	ipt_unregister_match(&drelay_match);
}

module_init(init);
module_exit(fini);
