#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <linux/ctype.h>
#include <linux/vmalloc.h>

#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_pElement.h>

static struct proc_dir_entry *proc_net_pElement;

struct pElement_list {
	char		name[PELEMENT_NAME_LEN];
	u_int16_t	refcount;
	u_int16_t	pause;
};

struct pElement_table {
	char name[24];
	int count;
	struct pElement_list *element;
	struct proc_dir_entry *status_proc;
	spinlock_t	plist_lock;
};

static struct pElement_table *pElement_inaddr, *pElement_exaddr;
static struct pElement_table *pElement_dmzaddr, *pElement_service;
static struct pElement_table *pElement_schedule;
static struct pElement_table *pElement_qos;
static struct pElement_table *pElement_authuser;
static struct pElement_table *pElement_vserver;
/* 2004/12/30 Hanson ++ VPN_TRUNK */
static struct pElement_table *pElement_vpn;
/* Hanson +. */


static spinlock_t list_lock = SPIN_LOCK_UNLOCKED;

static int 
ipt_pElement_write_info(struct file *file,
			const char *input,
			unsigned long size,
			void *data)
{
	char buffer[1024], *cp, *name;
	int len = 0, options = 0, first = 0, search = 0, count = 0;
	struct pElement_list *first_empty_list = NULL;
	struct pElement_table *curr_table = (struct pElement_table *)data;

	spin_lock_bh(&curr_table->plist_lock);
	if (size > (PELEMENT_NAME_LEN - 1))
		return size;
	else
		len = size;

	if (copy_from_user(buffer, input, len)) {
		spin_unlock_bh(&curr_table->plist_lock);
		return -EFAULT;
	}

	buffer[len - 1] = '\0';
	cp = buffer;
	if (*cp == '+') {
                cp++;
                if (*cp == '+') {
                        cp++;
                        options = 2;
                } else
                        options = 1;
	} else if (*cp == '-') {
                cp++;
                if (*cp == '-') {
                        cp++;
                        options = 3;
                } else
                        options = 0;
	} else {
		spin_unlock_bh(&curr_table->plist_lock);
		return len;
	}

	name = cp;
	for (count = 0; count < curr_table->count; count++) {
		if (!curr_table->element[count].name[0]) {
			if (!first) {
				first_empty_list = &curr_table->element[count];
				first = 1;
			}
			continue;
		}

                if (!strcmp(curr_table->element[count].name, name)) {
                        search = 1;
                        if (options == 1)
                                curr_table->element[count].refcount++;
                        else if (options == 2)
                                curr_table->element[count].pause++;
                        else if (options == 3)
                                curr_table->element[count].pause--;
                        else
                                curr_table->element[count].refcount--;

                        if (!curr_table->element[count].refcount &&
                            !curr_table->element[count].pause)
				memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));
                        break;
                }
	}

        if (!search && first_empty_list) {
                if (options == 1) {
                        strcpy(first_empty_list->name, name);
                        first_empty_list->refcount = 1;
                        first_empty_list->pause = 0;
                } else if (options == 2) {
                        strcpy(first_empty_list->name, name);
                        first_empty_list->refcount = 0;
                        first_empty_list->pause = 1;
                }
        }

	spin_unlock_bh(&curr_table->plist_lock);
	return size;
}

static int 
ipt_pElement_read_info(char *buffer,
		       char **start,
		       off_t offset,
		       int length,
		       int *eof,
		       void *data)
{
	int len = 0, count, last_len = 0;
	off_t pos = 0;
	off_t begin = 0;
	struct pElement_table *curr_table;

	curr_table = (struct pElement_table*)data;

	spin_lock_bh(&curr_table->plist_lock);
	for (count = 0; count < curr_table->count; count++) {
		if (!curr_table->element[count].name[0]) continue;

		last_len = len;
		len += sprintf(buffer+len, "%s %u %u\n",
			       curr_table->element[count].name,
			       curr_table->element[count].refcount,
			       curr_table->element[count].pause);

		pos = begin + len;
		if(pos < offset) { len = 0; begin = pos; }
		if(pos > offset + length) { len = last_len; break; }
	}

	*start = buffer + (offset - begin);
	len -= (offset - begin);
	if(len > length) len = length;
	spin_unlock_bh(&curr_table->plist_lock);
        return len;
}

static int 
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const void *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen,
      int *hotdrop)
{
	return 0;
}


static int
checkentry(const char *tablename,
	   const struct ipt_ip *ip,
	   void *matchinfo,
	   unsigned int matchsize,
	   unsigned int hook_mask)
{
	struct pElement_info *info = (struct pElement_info *)matchinfo;
	struct pElement_table *curr_table;
	int count = 0, first = 0, search = 0;
	struct pElement_list *first_empty_list = NULL;

	spin_lock_bh(&list_lock);

	first = 0;
	search = 0;
	first_empty_list = NULL;
	if (info->flag & INTERNAL_ADDRESS_FLAG) {
		curr_table = pElement_inaddr;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) {
				if (!first) {
					first_empty_list = &curr_table->element[count];
					first = 1;
				}
				continue;
			}

			if (!strcmp(curr_table->element[count].name,
				    info->inaddr_name)) {
				search = 1;
				curr_table->element[count].refcount++;
				break;
			}
		}

		if (!search && first_empty_list) {
			strcpy(first_empty_list->name, info->inaddr_name);
			first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
		}

		spin_unlock_bh(&curr_table->plist_lock);
	}


	first = 0;
	search = 0;
	first_empty_list = NULL;
	if (info->flag & EXTERNAL_ADDRESS_FLAG) {
		curr_table = pElement_exaddr;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) {
				if (!first) {
					first_empty_list = &curr_table->element[count];
					first = 1;
				}
				continue;
			}

			if (!strcmp(curr_table->element[count].name,
				    info->exaddr_name)) {
				search = 1;
				curr_table->element[count].refcount++;
				break;
			}
		}

		if (!search && first_empty_list) {
			strcpy(first_empty_list->name, info->exaddr_name);
			first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
		}

		spin_unlock_bh(&curr_table->plist_lock);
	}


	first = 0;
	search = 0;
	first_empty_list = NULL;
	if (info->flag & DMZ_ADDRESS_FLAG) {
		curr_table = pElement_dmzaddr;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) {
				if (!first) {
					first_empty_list = &curr_table->element[count];
					first = 1;
				}
				continue;
			}

			if (!strcmp(curr_table->element[count].name,
				    info->dmzaddr_name)) {
				search = 1;
				curr_table->element[count].refcount++;
				break;
			}
		}

		if (!search && first_empty_list) {
			strcpy(first_empty_list->name, info->dmzaddr_name);
			first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
		}

		spin_unlock_bh(&curr_table->plist_lock);
	}


	first = 0;
	search = 0;
	first_empty_list = NULL;
	if (info->flag & SERVICE_FLAG) {
		curr_table = pElement_service;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) {
				if (!first) {
					first_empty_list = &curr_table->element[count];
					first = 1;
				}
				continue;
			}

			if (!strcmp(curr_table->element[count].name,
				    info->service_name)) {
				search = 1;
				curr_table->element[count].refcount++;
				break;
			}
		}

		if (!search && first_empty_list) {
			strcpy(first_empty_list->name, info->service_name);
			first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
		}

		spin_unlock_bh(&curr_table->plist_lock);
	}


	first = 0;
	search = 0;
	first_empty_list = NULL;
	if (info->flag & SCHEDULE_FLAG) {
		curr_table = pElement_schedule;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) {
				if (!first) {
					first_empty_list = &curr_table->element[count];
					first = 1;
				}
				continue;
			}

			if (!strcmp(curr_table->element[count].name,
				    info->schedule_name)) {
				search = 1;
				curr_table->element[count].refcount++;
				break;
			}
		}

		if (!search && first_empty_list) {
			strcpy(first_empty_list->name, info->schedule_name);
			first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
		}

		spin_unlock_bh(&curr_table->plist_lock);
	}

	first = 0;
	search = 0;
	first_empty_list = NULL;
	if (info->flag & QOS_FLAG) {
		curr_table = pElement_qos;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) {
				if (!first) {
					first_empty_list = &curr_table->element[count];
					first = 1;
				}
				continue;
			}

			if (!strcmp(curr_table->element[count].name,
				    info->qos_name)) {
				search = 1;
				curr_table->element[count].refcount++;
				break;
			}
		}

		if (!search && first_empty_list) {
			strcpy(first_empty_list->name, info->qos_name);
			first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
		}

		spin_unlock_bh(&curr_table->plist_lock);
	}

	first = 0;
	search = 0;
	first_empty_list = NULL;
	if (info->flag & VSERVER_FLAG) {
		curr_table = pElement_vserver;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) {
				if (!first) {
					first_empty_list = &curr_table->element[count];
					first = 1;
				}
				continue;
			}

			if (!strcmp(curr_table->element[count].name,
				    info->vserver_name)) {
				search = 1;
				curr_table->element[count].refcount++;
				break;
			}
		}

		if (!search && first_empty_list) {
			strcpy(first_empty_list->name, info->vserver_name);
			first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
		}

		spin_unlock_bh(&curr_table->plist_lock);
	}

        first = 0;
        search = 0;
        first_empty_list = NULL;
        if (info->flag & AUTHUSER_FLAG) {
                curr_table = pElement_authuser;
                spin_lock_bh(&curr_table->plist_lock);
                for (count = 0; count < curr_table->count; count++) {
                        if (!curr_table->element[count].name[0]) {
                                if (!first) {
                first_empty_list = &curr_table->element[count];
                                        first = 1;
                                }
                                continue;
                        }

                        if (!strcmp(curr_table->element[count].name,
                                    info->authuser_name)) {
                                search = 1;
                                curr_table->element[count].refcount++;
                                break;
                        }
                }

                if (!search && first_empty_list) {
                        strcpy(first_empty_list->name, info->authuser_name);
                        first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
                }

                spin_unlock_bh(&curr_table->plist_lock);
        }
/* Steve -- */

/* 2004/12/30 Hanson ++ VPN_TRUNK */
        first = 0;
        search = 0;
        first_empty_list = NULL;
        if (info->flag & VPN_FLAG) {
                curr_table = pElement_vpn;
                spin_lock_bh(&curr_table->plist_lock);
                for (count = 0; count < curr_table->count; count++) {
                        if (!curr_table->element[count].name[0]) {
                                if (!first) {
                first_empty_list = &curr_table->element[count];
                                        first = 1;
                                }
                                continue;
                        }

                        if (!strcmp(curr_table->element[count].name,
                                    info->vpn_name)) {
                                search = 1;
                                curr_table->element[count].refcount++;
                                break;
                        }
                }

                if (!search && first_empty_list) {
                        strcpy(first_empty_list->name, info->vpn_name);
                        first_empty_list->refcount = 1;
			first_empty_list->pause = 0;
                }

                spin_unlock_bh(&curr_table->plist_lock);
        }
/* Hanson +. */

	spin_unlock_bh(&list_lock);
	return 1;
}

static void 
destroy(void *matchinfo, unsigned int matchsize)
{
	struct pElement_info *info = (struct pElement_info *)matchinfo;
	struct pElement_table *curr_table;
	int count = 0;

	spin_lock_bh(&list_lock);

	if (info->flag & INTERNAL_ADDRESS_FLAG) {
		curr_table = pElement_inaddr;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->inaddr_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}


	if (info->flag & EXTERNAL_ADDRESS_FLAG) {
		curr_table = pElement_exaddr;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->exaddr_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}


	if (info->flag & DMZ_ADDRESS_FLAG) {
		curr_table = pElement_dmzaddr;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->dmzaddr_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}


	if (info->flag & SERVICE_FLAG) {
		curr_table = pElement_service;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->service_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}


	if (info->flag & SCHEDULE_FLAG) {
		curr_table = pElement_schedule;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->schedule_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}

	if (info->flag & QOS_FLAG) {
		curr_table = pElement_qos;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->qos_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}

	if (info->flag & VSERVER_FLAG) {
		curr_table = pElement_vserver;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->vserver_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}

/* 2004/1/16 Steve ++ */
        if (info->flag & AUTHUSER_FLAG) {
                curr_table = pElement_authuser;
                spin_lock_bh(&curr_table->plist_lock);
                for (count = 0; count < curr_table->count; count++) {
                        if (!curr_table->element[count].name[0]) continue;

                        if (!strcmp(curr_table->element[count].name,
                            info->authuser_name)) {
                                curr_table->element[count].refcount--;
                                if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
				        memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

                                break;
                        }
                }
                spin_unlock_bh(&curr_table->plist_lock);
        }
/* Steve -- */

/* 2004/12/20 Hanson ++ VPN_TRUNK */
	if (info->flag & VPN_FLAG) {
		curr_table = pElement_vpn;
		spin_lock_bh(&curr_table->plist_lock);
		for (count = 0; count < curr_table->count; count++) {
			if (!curr_table->element[count].name[0]) continue;

			if (!strcmp(curr_table->element[count].name,
				    info->vpn_name)) {
				curr_table->element[count].refcount--;
				if (!curr_table->element[count].refcount &&
				    !curr_table->element[count].pause)
					memset(&curr_table->element[count], '\0', sizeof(struct pElement_list));

				break;
			}
		}
		spin_unlock_bh(&curr_table->plist_lock);
	}
/* Hanson +. */
	spin_unlock_bh(&list_lock);
}


static struct ipt_match pElement_match =
{ { NULL, NULL }, "pElement", &match, &checkentry, &destroy, THIS_MODULE };


static int
__init init(void)
{
//	spin_lock_bh(&list_lock);
	proc_net_pElement = proc_mkdir("ipt_pElement", proc_net);
	if (!proc_net_pElement)
		goto proc_mkdir_fails;

	/* Internal Address */
	pElement_inaddr = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
	if (!pElement_inaddr)
		goto inaddr_create_fails;

	memset(pElement_inaddr, '\0', sizeof(struct pElement_table));
	pElement_inaddr->plist_lock = SPIN_LOCK_UNLOCKED;
	pElement_inaddr->count = INTERNAL_ADDRESS_COUNT;
	strcpy(pElement_inaddr->name, "internal_address");
	pElement_inaddr->element = (struct pElement_list *)__get_free_pages(GFP_KERNEL, 1);
	if (!pElement_inaddr->element)
		goto inaddr_element;

	memset(pElement_inaddr->element, '\0', sizeof(struct pElement_list)*INTERNAL_ADDRESS_COUNT);
	pElement_inaddr->status_proc = create_proc_entry(pElement_inaddr->name, 0644, proc_net_pElement);
	if (!pElement_inaddr->status_proc)
		goto inaddr_proc;

	pElement_inaddr->status_proc->owner = THIS_MODULE;
	pElement_inaddr->status_proc->read_proc = ipt_pElement_read_info;
	pElement_inaddr->status_proc->write_proc = ipt_pElement_write_info;
	pElement_inaddr->status_proc->data = pElement_inaddr;

	/* External Address */
	pElement_exaddr = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
	if (!pElement_exaddr)
		goto exaddr_create_fails;

	memset(pElement_exaddr, '\0', sizeof(struct pElement_table));
	pElement_exaddr->plist_lock = SPIN_LOCK_UNLOCKED;
	pElement_exaddr->count = EXTERNAL_ADDRESS_COUNT;
	strcpy(pElement_exaddr->name, "external_address");
	pElement_exaddr->element = (struct pElement_list *)__get_free_pages(GFP_KERNEL, 1);
	if (!pElement_exaddr->element)
		goto exaddr_element;

	memset(pElement_exaddr->element, '\0', sizeof(struct pElement_list)*EXTERNAL_ADDRESS_COUNT);
	pElement_exaddr->status_proc = create_proc_entry(pElement_exaddr->name, 0644, proc_net_pElement);
	if (!pElement_exaddr->status_proc)
		goto exaddr_proc;

	pElement_exaddr->status_proc->owner = THIS_MODULE;
	pElement_exaddr->status_proc->read_proc = ipt_pElement_read_info;
	pElement_exaddr->status_proc->write_proc = ipt_pElement_write_info;
	pElement_exaddr->status_proc->data = pElement_exaddr;

	/* DMZ Address */
	pElement_dmzaddr = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
	if (!pElement_dmzaddr)
		goto dmzaddr_create_fails;

	memset(pElement_dmzaddr, '\0', sizeof(struct pElement_table));
	pElement_dmzaddr->plist_lock = SPIN_LOCK_UNLOCKED;
	pElement_dmzaddr->count = DMZ_ADDRESS_COUNT;
	strcpy(pElement_dmzaddr->name, "dmz_address");
	pElement_dmzaddr->element = (struct pElement_list *)__get_free_pages(GFP_KERNEL, 1);
	if (!pElement_dmzaddr->element)
		goto dmzaddr_element;

	memset(pElement_dmzaddr->element, '\0', sizeof(struct pElement_list)*DMZ_ADDRESS_COUNT);
	pElement_dmzaddr->status_proc = create_proc_entry(pElement_dmzaddr->name, 0644, proc_net_pElement);
	if (!pElement_dmzaddr->status_proc)
		goto dmzaddr_proc;

	pElement_dmzaddr->status_proc->owner = THIS_MODULE;
	pElement_dmzaddr->status_proc->read_proc = ipt_pElement_read_info;
	pElement_dmzaddr->status_proc->write_proc = ipt_pElement_write_info;
	pElement_dmzaddr->status_proc->data = pElement_dmzaddr;

	/* Service */
	pElement_service = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
	if (!pElement_service)
		goto service_create_fails;

	memset(pElement_service, '\0', sizeof(struct pElement_table));
	pElement_service->plist_lock = SPIN_LOCK_UNLOCKED;
	pElement_service->count = SERVICE_COUNT;
	strcpy(pElement_service->name, "service");
	pElement_service->element = (struct pElement_list *)__get_free_pages(GFP_KERNEL, 0);
	if (!pElement_service->element)
		goto service_element;

	memset(pElement_service->element, '\0', sizeof(struct pElement_list)*SERVICE_COUNT);
	pElement_service->status_proc = create_proc_entry(pElement_service->name, 0644, proc_net_pElement);
	if (!pElement_service->status_proc)
		goto service_proc;

	pElement_service->status_proc->owner = THIS_MODULE;
	pElement_service->status_proc->read_proc = ipt_pElement_read_info;
	pElement_service->status_proc->write_proc = ipt_pElement_write_info;
	pElement_service->status_proc->data = pElement_service;

	/* Schedule */
	pElement_schedule = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
	if (!pElement_schedule)
		goto schedule_create_fails;

	memset(pElement_schedule, '\0', sizeof(struct pElement_table));
	pElement_schedule->plist_lock = SPIN_LOCK_UNLOCKED;
	pElement_schedule->count = SCHEDULE_COUNT;
	strcpy(pElement_schedule->name, "schedule");
	pElement_schedule->element = (struct pElement_list *)__get_free_pages(GFP_KERNEL, 0);
	if (!pElement_schedule->element)
		goto schedule_element;

	memset(pElement_schedule->element, '\0', sizeof(struct pElement_list)*SCHEDULE_COUNT);
	pElement_schedule->status_proc = create_proc_entry(pElement_schedule->name, 0644, proc_net_pElement);
	if (!pElement_schedule->status_proc)
		goto schedule_proc;

	pElement_schedule->status_proc->owner = THIS_MODULE;
	pElement_schedule->status_proc->read_proc = ipt_pElement_read_info;
	pElement_schedule->status_proc->write_proc = ipt_pElement_write_info;
	pElement_schedule->status_proc->data = pElement_schedule;

	/* QoS */
	pElement_qos = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
	if (!pElement_qos)
		goto qos_create_fails;

	memset(pElement_qos, '\0', sizeof(struct pElement_table));
	pElement_qos->plist_lock = SPIN_LOCK_UNLOCKED;
	pElement_qos->count = QOS_COUNT;
	strcpy(pElement_qos->name, "qos");
	pElement_qos->element = (struct pElement_list *)__get_free_pages(GFP_KERNEL, 0);
	if (!pElement_qos->element)
		goto qos_element;

	memset(pElement_qos->element, '\0', sizeof(struct pElement_list)*QOS_COUNT);
	pElement_qos->status_proc = create_proc_entry(pElement_qos->name, 0644, proc_net_pElement);
	if (!pElement_qos->status_proc)
		goto qos_proc;

	pElement_qos->status_proc->owner = THIS_MODULE;
	pElement_qos->status_proc->read_proc = ipt_pElement_read_info;
	pElement_qos->status_proc->write_proc = ipt_pElement_write_info;
	pElement_qos->status_proc->data = pElement_qos;

	/* VSERVER */
	pElement_vserver = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
	if (!pElement_vserver)
		goto vserver_create_fails;

	memset(pElement_vserver, '\0', sizeof(struct pElement_table));
	pElement_vserver->plist_lock = SPIN_LOCK_UNLOCKED;
	pElement_vserver->count = VSERVER_COUNT;
	strcpy(pElement_vserver->name, "vserver");
	pElement_vserver->element = (struct pElement_list *)__get_free_pages(GFP_KERNEL, 0);
	if (!pElement_vserver->element)
		goto vserver_element;

	memset(pElement_vserver->element, '\0', sizeof(struct pElement_list)*VSERVER_COUNT);
	pElement_vserver->status_proc = create_proc_entry(pElement_vserver->name, 0644, proc_net_pElement);
	if (!pElement_vserver->status_proc)
		goto vserver_proc;

	pElement_vserver->status_proc->owner = THIS_MODULE;
	pElement_vserver->status_proc->read_proc = ipt_pElement_read_info;
	pElement_vserver->status_proc->write_proc = ipt_pElement_write_info;
	pElement_vserver->status_proc->data = pElement_vserver;

        pElement_authuser = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
        if (!pElement_authuser)
                goto authuser_create_fails;

        memset(pElement_authuser, '\0', sizeof(struct pElement_table));
        pElement_authuser->plist_lock = SPIN_LOCK_UNLOCKED;
        pElement_authuser->count = AUTHUSER_COUNT;
        strcpy(pElement_authuser->name, "authuser");
        pElement_authuser->element =  (struct pElement_list *)__get_free_pages(GFP_KERNEL, 1);
        if (!pElement_authuser->element)
                goto authuser_element;

        memset(pElement_authuser->element, '\0', sizeof(struct pElement_list)*AUTHUSER_COUNT);
        pElement_authuser->status_proc = create_proc_entry(pElement_authuser->name, 0644, proc_net_pElement);
        if (!pElement_authuser->status_proc)
                goto authuser_proc;

        pElement_authuser->status_proc->owner = THIS_MODULE;
        pElement_authuser->status_proc->read_proc = ipt_pElement_read_info;
        pElement_authuser->status_proc->write_proc = ipt_pElement_write_info;
        pElement_authuser->status_proc->data = pElement_authuser;

/* 2004/12/20 Hanson ++ VPN_TRUNK */
        pElement_vpn = kmalloc(sizeof(struct pElement_table), GFP_KERNEL);
        if (!pElement_vpn)
                goto vpn_create_fails;

        memset(pElement_vpn, '\0', sizeof(struct pElement_table));
        pElement_vpn->plist_lock = SPIN_LOCK_UNLOCKED;
        pElement_vpn->count = AUTHUSER_COUNT;
        strcpy(pElement_vpn->name, "vpn");
        pElement_vpn->element =  (struct pElement_list *)__get_free_pages(GFP_KERNEL, 1);
        if (!pElement_vpn->element)
                goto vpn_element;

        memset(pElement_vpn->element, '\0', sizeof(struct pElement_list)*VPN_COUNT);
        pElement_vpn->status_proc = create_proc_entry(pElement_vpn->name, 0644, proc_net_pElement);
        if (!pElement_vpn->status_proc)
                goto vpn_proc;

        pElement_vpn->status_proc->owner = THIS_MODULE;
        pElement_vpn->status_proc->read_proc = ipt_pElement_read_info;
        pElement_vpn->status_proc->write_proc = ipt_pElement_write_info;
        pElement_vpn->status_proc->data = pElement_vpn;
/* Hanson +. */

	if (ipt_register_match(&pElement_match))
		goto pElement_fails;

//	spin_unlock_bh(&list_lock);
	return 0;

pElement_fails:
/* 2004/12/30 Hanson +. */
	remove_proc_entry(pElement_vpn->name, proc_net_pElement);
vpn_proc:
        free_pages((unsigned long)(pElement_vpn->element), 1);
vpn_element:
	kfree(pElement_vpn);
vpn_create_fails:
/* Hanson +. */
	remove_proc_entry(pElement_authuser->name, proc_net_pElement);
authuser_proc:
        free_pages((unsigned long)(pElement_authuser->element), 1);
authuser_element:
	kfree(pElement_authuser);
authuser_create_fails:
	remove_proc_entry(pElement_vserver->name, proc_net_pElement);
vserver_proc:
        free_pages((unsigned long)(pElement_vserver->element), 0);
vserver_element:
        kfree(pElement_vserver);
vserver_create_fails:
	remove_proc_entry(pElement_qos->name, proc_net_pElement);
qos_proc:
        free_pages((unsigned long)(pElement_qos->element), 0);
qos_element:
        kfree(pElement_qos);
qos_create_fails:
	remove_proc_entry(pElement_schedule->name, proc_net_pElement);
schedule_proc:
	free_pages((unsigned long)(pElement_schedule->element), 0);
schedule_element:
	kfree(pElement_schedule);
schedule_create_fails:
	remove_proc_entry(pElement_service->name, proc_net_pElement);
service_proc:
	free_pages((unsigned long)(pElement_service->element), 0);
service_element:
	kfree(pElement_service);
service_create_fails:
	remove_proc_entry(pElement_dmzaddr->name, proc_net_pElement);
dmzaddr_proc:
	free_pages((unsigned long)(pElement_dmzaddr->element), 1);
dmzaddr_element:
	kfree(pElement_dmzaddr);
dmzaddr_create_fails:
	remove_proc_entry(pElement_exaddr->name, proc_net_pElement);
exaddr_proc:
	free_pages((unsigned long)(pElement_exaddr->element), 1);
exaddr_element:
	kfree(pElement_exaddr);
exaddr_create_fails:
	remove_proc_entry(pElement_inaddr->name, proc_net_pElement);
inaddr_proc:
	free_pages((unsigned long)(pElement_inaddr->element), 1);
inaddr_element:
	kfree(pElement_inaddr);
inaddr_create_fails:
	remove_proc_entry("ipt_pElement", proc_net);
proc_mkdir_fails:
//	spin_unlock_bh(&list_lock);
	return -ENOMEM;
}


static void
__exit fini(void)
{
//	spin_lock_bh(&list_lock);
	if (pElement_inaddr) {
		remove_proc_entry(pElement_inaddr->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_inaddr->element), 1);
		kfree(pElement_inaddr);
	}
	if (pElement_exaddr) {
		remove_proc_entry(pElement_exaddr->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_exaddr->element), 1);
		kfree(pElement_exaddr);
	}
	if (pElement_dmzaddr) {
		remove_proc_entry(pElement_dmzaddr->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_dmzaddr->element), 1);
		kfree(pElement_dmzaddr);
	}
	if (pElement_service) {
		remove_proc_entry(pElement_service->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_service->element), 0);
		kfree(pElement_service);
	}
	if (pElement_schedule) {
		remove_proc_entry(pElement_schedule->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_schedule->element), 0);
		kfree(pElement_schedule);
	}
	if (pElement_qos) {
		remove_proc_entry(pElement_qos->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_qos->element), 0);
		kfree(pElement_qos);
	}
	if (pElement_vserver) {
		remove_proc_entry(pElement_vserver->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_vserver->element), 0);
		kfree(pElement_vserver);
	}

	if (pElement_authuser) {
                remove_proc_entry(pElement_authuser->name, proc_net_pElement);
                free_pages((unsigned long)(pElement_authuser->element), 1);
                kfree(pElement_authuser);
	}

/* 2004/12/30 Hanson ++ VPN_TRUNK */
	if (pElement_vpn) {
		remove_proc_entry(pElement_vpn->name, proc_net_pElement);
		free_pages((unsigned long)(pElement_vpn->element), 0);
		kfree(pElement_vpn);
	}
/* Hanson +. */

	remove_proc_entry("ipt_pElement", proc_net);
	ipt_unregister_match(&pElement_match);
//	spin_unlock_bh(&list_lock);
}

module_init(init);
module_exit(fini);
