/*
 * This is the 1999 rewrite of IP Firewalling, aiming for kernel 2.3.x.
 *
 * Copyright (C) 1999 Paul `Rusty' Russell & Michael J. Neuling
 *
 * This table was expanded to use all five netfilter hooks for
 * IPv4 by Brad Chapman.
 */
#include <linux/config.h>
#include <linux/module.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netdevice.h>
#include <linux/skbuff.h>
#include <net/sock.h>
#include <net/route.h>
#include <linux/ip.h>
/* 2002/4/10 Steve ++ */
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <net/tcp.h>
#include <linux/proc_fs.h>
#include <linux/netfilter.h>
/* 2003/1/28 Steve ++ */
#include <linux/netfilter_ipv4/ipt_NETLINK.h>
#include <net/sock.h>
/* Steve -- */
/* 2004/12/29 Steve ++ */
#include <net/udp.h>
/* Steve -- */
/* 2005/1/23 Steve ++ */
#include <linux/socket.h>
/* Steve -- */

#define ASSERT_READ_LOCK(x) MUST_BE_READ_LOCKED(&ip_conntrack_lock)
#define ASSERT_WRITE_LOCK(x) MUST_BE_WRITE_LOCKED(&ip_conntrack_lock)
DECLARE_RWLOCK_EXTERN(ip_conntrack_lock);
/* Steve -- */

#define MANGLE_VALID_HOOKS ((1 << NF_IP_PRE_ROUTING) | \
			    (1 << NF_IP_LOCAL_IN) | \
			    (1 << NF_IP_FORWARD) | \
			    (1 << NF_IP_LOCAL_OUT) | \
			    (1 << NF_IP_POST_ROUTING))

/* Standard entry. */
struct ipt_standard
{
	struct ipt_entry entry;
	struct ipt_standard_target target;
};

struct ipt_error_target
{
	struct ipt_entry_target target;
	char errorname[IPT_FUNCTION_MAXNAMELEN];
};

struct ipt_error
{
	struct ipt_entry entry;
	struct ipt_error_target target;
};

static struct
{
	struct ipt_replace repl;
	struct ipt_standard entries[5];
	struct ipt_error term;
} initial_table __initdata
= { { "mangle", MANGLE_VALID_HOOKS, 6,
      sizeof(struct ipt_standard) * 5 + sizeof(struct ipt_error),
      { [NF_IP_PRE_ROUTING] 	0,
	[NF_IP_LOCAL_IN]	sizeof(struct ipt_standard),
	[NF_IP_FORWARD]		sizeof(struct ipt_standard) * 2,
	[NF_IP_LOCAL_OUT] 	sizeof(struct ipt_standard) * 3,
	[NF_IP_POST_ROUTING]	sizeof(struct ipt_standard) * 4 },
      { [NF_IP_PRE_ROUTING] 	0,
	[NF_IP_LOCAL_IN]	sizeof(struct ipt_standard),
	[NF_IP_FORWARD]		sizeof(struct ipt_standard) * 2,
	[NF_IP_LOCAL_OUT] 	sizeof(struct ipt_standard) * 3,
	[NF_IP_POST_ROUTING]	sizeof(struct ipt_standard) * 4 },
      0, NULL, { } },
    {
	    /* PRE_ROUTING */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* LOCAL_IN */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* FORWARD */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* LOCAL_OUT */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } },
	    /* POST_ROUTING */
	    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
		0,
		sizeof(struct ipt_entry),
		sizeof(struct ipt_standard),
		0, { 0, 0 }, { } },
	      { { { { IPT_ALIGN(sizeof(struct ipt_standard_target)), "" } }, { } },
		-NF_ACCEPT - 1 } }
    },
    /* ERROR */
    { { { { 0 }, { 0 }, { 0 }, { 0 }, "", "", { 0 }, { 0 }, 0, 0, 0 },
	0,
	sizeof(struct ipt_entry),
	sizeof(struct ipt_error),
	0, { 0, 0 }, { } },
      { { { { IPT_ALIGN(sizeof(struct ipt_error_target)), IPT_ERROR_TARGET } },
	  { } },
	"ERROR"
      }
    }
};

/* 2002/12/11 Steve ++ */
static unsigned long pmtu;
/* Steve -- */
/* 2003/1/28 Steve ++ */
static struct sock *ipfwsk;
/* Steve -- */
/* 2004/12/29 Steve ++ */
static struct sock *iplogsk;
/* Steve -- */

/* 2004/9/28 Steve ++ */
static char landev[IFNAMSIZ], dmzdev[IFNAMSIZ];
/* Steve -- */

static struct ipt_table packet_mangler
= { { NULL, NULL }, "mangle", &initial_table.repl,
    MANGLE_VALID_HOOKS, RW_LOCK_UNLOCKED, NULL };

/* 2002/7/2 Steve ++ */
static inline unsigned int
optlen(const u_int8_t *opt, unsigned int offset)
{
        /* Beware zero-length options: make finite progress */
        if (opt[offset] <= TCPOPT_NOP || opt[offset+1] == 0) return 1;
        else return opt[offset+1];
}

static u_int16_t
cheat_check(u_int32_t oldvalinv, u_int32_t newval, u_int16_t oldcheck)
{
        u_int32_t diffs[] = { oldvalinv, newval };
        return csum_fold(csum_partial((char *)diffs, sizeof(diffs),
                                      oldcheck^0xFFFF));
}

static void
update_tcpmss(struct sk_buff **pskb, struct ip_conntrack *ct)
{
	struct tcphdr *tcph;
	struct iphdr *iph = (*pskb)->nh.iph;
	u_int16_t tcplen;
	u_int16_t mss = pmtu - sizeof(struct iphdr) - sizeof(struct tcphdr);

	if (iph->protocol != IPPROTO_TCP)
		return;

        tcplen = (*pskb)->len - iph->ihl*4;
        tcph = (void *)iph + iph->ihl*4;

	if (tcplen < sizeof(*tcph)) {
		if (net_ratelimit())
			printk(KERN_ERR
				"update_tcpmss: bad length (%d bytes)\n",
				(*pskb)->len);
                return;
        }

	if (tcph->syn) {
		unsigned int i;
		u_int8_t *opt;
		u_int16_t newmss;

		READ_LOCK(&ip_conntrack_lock);
		if (ct && ct->pmtu && mss > ct->pmtu)
		        newmss = ct->pmtu;
		else
			newmss = mss;

		opt = (u_int8_t *)tcph;
		for (i = sizeof(struct tcphdr); i < tcph->doff*4; i += optlen(opt, i)) {
			if ((opt[i] == TCPOPT_MSS) &&
			   ((tcph->doff*4 - i) >= TCPOLEN_MSS) &&
			   (opt[i+1] == TCPOLEN_MSS)) {
                        	u_int16_t oldmss;

				oldmss = (opt[i+2] << 8) | opt[i+3];
				if (oldmss > newmss) {
                        		opt[i+2] = (newmss & 0xff00) >> 8;
                        		opt[i+3] = (newmss & 0x00ff);

                        		tcph->check = cheat_check(htons(oldmss)^0xFFFF,
                                        			  htons(newmss),
                                                  		  tcph->check);

					(*pskb)->ip_summed = CHECKSUM_NONE;
					(*pskb)->nfcache |= NFC_UNKNOWN | NFC_ALTERED;
				}
			}
		}
		READ_UNLOCK(&ip_conntrack_lock);
	} 
}
/* Steve -- */

/* 2004/9/22 Steve ++ */
void
do_accounting_report(struct ip_conntrack *ct)
{
#ifdef ACCOUNTING_REPORT
	struct sk_buff *outskb, *inskb;
	struct netlink_t nlhdr;
	int len = 0;

	READ_LOCK(&ip_conntrack_lock);
	len = sizeof(nlhdr) + ACCOUNTING_LEN;
	outskb = alloc_skb(len, GFP_ATOMIC);
	if (outskb) {
		if (ct->report.upstream) {
			nlhdr.mark = ct->mark;
			nlhdr.time = ct->report.lastseen / HZ;
			nlhdr.len = ct->report.upstream + sizeof(nlhdr);
			nlhdr.start = 0;

			nlhdr.reply = 0;
			skb_put(outskb, len);
			memcpy(outskb->data, &nlhdr, sizeof(nlhdr));
			memcpy((outskb->data) + sizeof(nlhdr), &ct->report.sip, len - sizeof(nlhdr));
			netlink_broadcast(ipfwsk, outskb, 0, ct->report.policy, GFP_KERNEL);
			ct->report.upstream = 0;
		} else
			kfree_skb(outskb);
	}

	inskb = alloc_skb(len, GFP_ATOMIC);
	if (inskb) {
		if (ct->report.downstream) {
			nlhdr.mark = ct->mark;
			nlhdr.time = ct->report.lastseen / HZ;
			nlhdr.len = ct->report.downstream + sizeof(nlhdr);
			nlhdr.start = 0;
			nlhdr.reply = 1;
			skb_put(inskb, len);
			memcpy(inskb->data, &nlhdr, sizeof(nlhdr));
			memcpy((inskb->data) + sizeof(nlhdr), &ct->report.rip, len - sizeof(nlhdr));
			netlink_broadcast(ipfwsk, inskb, 0, ct->report.policy, GFP_KERNEL);
			ct->report.downstream = 0;
		} else
			kfree_skb(inskb);
	}
	READ_UNLOCK(&ip_conntrack_lock);
#endif
}
/* Steve -- */

/* 2003/1/28 Steve ++ */
void
netlink_target(struct sk_buff **pskb, struct ip_conntrack *ct, enum ip_conntrack_info ctinfo, const struct net_device *in)
{
#ifdef ACCOUNTING_REPORT
	struct iphdr *ip = (*pskb)->nh.iph;
	struct sk_buff *outskb = NULL;
	struct netlink_t nlhdr;
	size_t len = 0;
	const char *indev, *physindev;
	static const char nulldevname[IFNAMSIZ] = { 0 };
	int direct = 1;
	u_int32_t policy;
	u_int32_t chainMark;

	indev = in ? in->name : nulldevname;
	physindev = (*pskb)->physindev ? (*pskb)->physindev->name : nulldevname;

	if (!strcmp(indev, landev) || !strcmp(physindev, landev) ||
	    !strcmp(indev, dmzdev) || !strcmp(physindev, dmzdev))
		direct = 0;
	if ((ct->mark & URLBLOCK_MARK) && ctinfo < IP_CT_IS_REPLY)
		direct = 0;

        len = sizeof(nlhdr) + 60;
	nlhdr.len = ntohs(ip->tot_len) + sizeof(nlhdr);
	nlhdr.mark = ct->mark;
	nlhdr.time = jiffies / HZ;
	chainMark = (nlhdr.mark & POLICY_CHAIN_MASK) >> 32;

	switch (chainMark) {
	case 1:
	case 5:
	case 8:
	case 10:
	case 0xc:
	case 0xe:
		policy = 1;
		if (direct)
			nlhdr.reply = 1;
		else
			nlhdr.reply = 0;
		break;
	case 2:
	case 3:
	case 7:
	case 9:
	case 0xb:
	case 0xd:
		policy = 2;
		if (direct)
			nlhdr.reply = 0;
		else
			nlhdr.reply = 1;
		break;
	default:
		return;
	}

	if (ctinfo == IP_CT_NEW || ctinfo == IP_CT_RELATED) {
		nlhdr.start = 1;
		outskb = alloc_skb(len, GFP_ATOMIC);
		if (outskb) {
			skb_put(outskb, len);
			memcpy(outskb->data, &nlhdr, sizeof(nlhdr));
			memcpy((outskb->data) + sizeof(nlhdr), ip, len - sizeof(nlhdr));
			netlink_broadcast(ipfwsk, outskb, 0, policy, GFP_KERNEL);
		} else {
			if (net_ratelimit())
				printk(KERN_WARNING "packet drop due to netlink failure\n");
		}

		ct->report.policy = policy;
	} else {
		if (nlhdr.reply) {
			ct->report.downstream += ntohs(ip->tot_len);
			memcpy(&ct->report.rip, ip, ACCOUNTING_LEN);
		} else {
			ct->report.upstream += ntohs(ip->tot_len);
			memcpy(&ct->report.sip, ip, ACCOUNTING_LEN);
		}

                if (jiffies >= (ct->report.lastseen + ACCOUNTING_REPORT_TIMER)) {
                        do_accounting_report(ct);
		        ct->report.lastseen = jiffies;
		}
        }
#endif
}
/* Steve -- */

/* 2004/12/29 Steve ++ */
static void
netlink_forward_packet(struct iphdr *iph,
		       unsigned int len,
		       int recurse,
		       char *hdrinfo)
{
	void *protoh = (u_int32_t *)iph + iph->ihl;
	unsigned int datalen = len - iph->ihl * 4;
	char protoinfo[12], portinfo[128];

	memset(protoinfo, '\0', sizeof(protoinfo));
	memset(portinfo, '\0', sizeof(portinfo));
	switch (iph->protocol) {
	case IPPROTO_TCP: {
		struct tcphdr *tcph = protoh;

		/* Max length: 10 "PROTO=TCP " */
		sprintf(protoinfo, "PROTO=TCP");
		if (ntohs(iph->frag_off) & IP_OFFSET)
                        break;

                if (datalen < sizeof (*tcph))
                        break;

                /* Max length: 20 "SPT=65535 DPT=65535 " */
                sprintf(portinfo, "SPT=%u DPT=%u",
                        ntohs(tcph->source), ntohs(tcph->dest));
                break;
        }
        case IPPROTO_UDP: {
                struct udphdr *udph = protoh;

                /* Max length: 10 "PROTO=UDP " */
                sprintf(protoinfo, "PROTO=UDP");

                if (ntohs(iph->frag_off) & IP_OFFSET)
                        break;
                /* Max length: 25 "INCOMPLETE [65535 bytes] " */
                if (datalen < sizeof (*udph))
                        break;

                sprintf(portinfo, "SPT=%u DPT=%u",
                        ntohs(udph->source), ntohs(udph->dest));
                break;
        }
        case IPPROTO_ICMP: {
                struct icmphdr *icmph = protoh;
                static size_t required_len[NR_ICMP_TYPES+1]
                        = { [ICMP_ECHOREPLY] = 4,
                            [ICMP_DEST_UNREACH]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_SOURCE_QUENCH]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_REDIRECT]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_ECHO] = 4,
                            [ICMP_TIME_EXCEEDED]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_PARAMETERPROB]
                            = 8 + sizeof(struct iphdr) + 8,
                            [ICMP_TIMESTAMP] = 20,
                            [ICMP_TIMESTAMPREPLY] = 20,
                            [ICMP_ADDRESS] = 12,
                            [ICMP_ADDRESSREPLY] = 12 };

                /* Max length: 11 "PROTO=ICMP " */
                sprintf(protoinfo, "PROTO=ICMP");

                if (ntohs(iph->frag_off) & IP_OFFSET)
                        break;

                /* Max length: 25 "INCOMPLETE [65535 bytes] " */
                if (datalen < 4)
                        break;

                sprintf(portinfo, "TYPE=%u", icmph->type);

                /* Max length: 25 "INCOMPLETE [65535 bytes] " */
                if (icmph->type <= NR_ICMP_TYPES
                    && required_len[icmph->type]
                    && datalen < required_len[icmph->type])
                        break;

                break;
        }
        /* Max length: 10 "PROTO 255 " */
        default:
                sprintf(protoinfo, "PROTO=%u", iph->protocol);
        }

        sprintf(hdrinfo, "SRC=%u.%u.%u.%u DST=%u.%u.%u.%u %s %s",
                NIPQUAD(iph->saddr),
                NIPQUAD(iph->daddr),
                protoinfo,
                portinfo);
}

void
netlink_printlog(struct sk_buff **pskb,
                 struct ip_conntrack *ct,
                 const struct net_device *in,
                 const struct net_device *out,
                 enum ip_conntrack_info ctinfo)
{
        unsigned long long mark = ct->mark;
/* 2004/5/27 Steve ++ */
        unsigned long premark = (unsigned long)((mark >> 32) & 0xffffffff);
/* Steve -- */
        struct sk_buff *skb = *pskb;
        struct iphdr *iph = (skb)->nh.iph;
        char level_string[4] = "<5>";
        int accept = 0;
        unsigned long chainMark = (unsigned long)(mark & CHAIN_NAME_MASK);
        char chainName[32];
        struct tm chaintime;
        char waninfo[12], dirinfo[12], hdrinfo[256];
        struct sk_buff *outskb = NULL;

        localtime(&chainMark, &chaintime);
        sprintf(chainName, "%.2d%.2d%.2d%.2d%.2d%.2d",
                            (chaintime.tm_year)%100,
                            (chaintime.tm_mon)+1,
                            chaintime.tm_mday,
                            chaintime.tm_hour,
                            chaintime.tm_min,
                            chaintime.tm_sec
        );

        if (mark & POLICY_ACCEPT_MARK)
                accept = 1;

        memset(hdrinfo, '\0', sizeof(hdrinfo));
        netlink_forward_packet(iph, (*pskb)->len, 1, hdrinfo);

        memset(waninfo, '\0', sizeof(waninfo));
        memset(dirinfo, '\0', sizeof(dirinfo));
        if (ct && (ct->mark & URLBLOCK_MARK)) {
                unsigned int wanindex = 0;

#if WAN_PORT >= 2
                wanindex = ((ct->mark >> WAN_PORT_OFFSET) & 0xf);
                if (wanindex > 0 && wanindex <= MAX_WAN_PORT)
                        sprintf(waninfo, "WAN=%u", wanindex);
#endif

                if (ctinfo < IP_CT_IS_REPLY)
                        sprintf(dirinfo, "DIR=out");
                else
                        sprintf(dirinfo, "DIR=in");
        } else {
/* 2003/3/11 Steve ++
 * Add WAN Port information
 */
                if (ct && ct->indexwan) {
                        unsigned int wanindex = 0;
/* 2004/12/28 Steve ++ */
#if 0
                        char dev_ethname[20], dev_pppname[20];
#endif
/* Steve -- */

                        wanindex = ((ct->mark >> WAN_PORT_OFFSET) & 0xf);
                        if (wanindex > 0 && wanindex <= MAX_WAN_PORT) {
#if WAN_PORT >= 2
                                sprintf(waninfo, "WAN=%u", wanindex);
#endif
/* 2004/12/28 Steve ++ */
#if 0
                                sprintf(dev_ethname, "eth%u", wanindex);
                                sprintf(dev_pppname, "ppp%u", (wanindex - 1));
                                if ((in &&
                                   (!strcmp(in->name, dev_ethname) ||
                                    !strcmp(in->name, dev_pppname))) ||
                                    ((*pskb)->physindev &&
                                    (!strcmp((*pskb)->physindev->name,dev_ethname) ||
                                     !strcmp((*pskb)->physindev->name, dev_pppname))))
                                        sprintf(dirinfo, "DIR=in");
                                else
                                        sprintf(dirinfo, "DIR=out");
#endif
                                if ((in &&
                                   (!strcmp(in->name, landev) ||
                                    !strcmp(in->name, dmzdev))) ||
                                    ((*pskb)->physindev &&
                                    (!strcmp((*pskb)->physindev->name, landev) ||
                                     !strcmp((*pskb)->physindev->name, dmzdev))))
                                        sprintf(dirinfo, "DIR=out");
                                else
                                        sprintf(dirinfo, "DIR=in");
/* Steve -- */
                        }
                }
        }
        outskb = alloc_skb(NETLINK_LOG_LEN, GFP_ATOMIC);
        if (outskb) {
                skb_put(outskb, NETLINK_LOG_LEN);
                sprintf(outskb->data, "'%s %s LOG ' %s LEN=%u %s %s MARK=%lx%llx\n",
                        chainName,
                        accept ? "ACCEPT" : "DROP",
                        hdrinfo,
                        (*pskb)->len,
                        waninfo,
                        dirinfo,
                        premark,
                        mark);
                netlink_broadcast(iplogsk, outskb, 0, 1, GFP_KERNEL);
        }
}
/* Steve -- */

/* The work comes in here from netfilter.c. */
static unsigned int
ipt_route_hook( unsigned int hook,
	 	struct sk_buff **pskb,
	 	const struct net_device *in,
	 	const struct net_device *out,
	 	int (*okfn)(struct sk_buff *))
{
        enum ip_conntrack_info ctinfo;
        struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)*pskb, &ctinfo);
	int ret = 0;

/* 2002/4/10 Steve ++ */
	if (hook == NF_IP_FORWARD) {
		unsigned long long match;

		if (!ct) {
			if (in && out && !strcmp(in->name, out->name))
				return NF_ACCEPT;
			else
				return NF_DROP;
		}

/* 2002/12/11 Steve ++ 
 * Update TCPMSS options
 */
		update_tcpmss(pskb, ct);
/* Steve -- */

/* 2003/1/6 Steve ++
 * Log WAN port connection information
 */
		if (ct && !ct->indexwan) {
			unsigned int wanindex = 0;

			wanindex = ((ct->mark >> WAN_PORT_OFFSET) & 0xf);
			if (wanindex > 0 && wanindex <= MAX_WAN_PORT) {
				ct->indexwan = 1;
                        	wan_connection_num[wanindex-1]++;
                        }
		}
/* Steve -- */

		match = ct->mark;
		ret = ipt_do_table(pskb, hook, in, out, &packet_mangler, NULL);
		if (ret == NF_DROP)
			match &= ~POLICY_ACCEPT_MARK;

/* 2004/12/29 Steve ++ */
#if 0
		if ((match & POLICY_LOG_MARK))
			ipt_do_printlog(pskb, ct, &packet_mangler, in, out, ctinfo);
#endif
		if ((match & POLICY_LOG_MARK))
			netlink_printlog(pskb, ct, in, out, ctinfo);
/* Steve -- */

		if (!(match & POLICY_ACCEPT_MARK))
			return NF_DROP;

/* 2003/7/17 Steve ++ */
		(*pskb)->forward = 1;
/* Steve -- */

		if ((match & POLICY_BM_MARK)) {
			if((*pskb)->nfmark != ct->skbmark) {
				(*pskb)->nfmark = ct->skbmark;
				(*pskb)->nfcache |= NFC_ALTERED;
			}
		}

/* 2003/1/28 Steve ++ */
#ifdef ACCOUNTING_REPORT
		netlink_target(pskb, ct, ctinfo, in);
#endif
/* Steve -- */

/* 2005/4/22 Steve ++ */
//		return NF_ACCEPT;
		return ret;
/* Steve -- */
	}
/* 2002/10/7 Steve ++ */
	else if (hook == NF_IP_PRE_ROUTING) {
		unsigned long long match;
/* zhangzh 2003/08/08 */
//#ifdef BANDWIDTH
		if (ct && (ct->mark & (QUOTA_MARK | INTERNAL_AUTH_MARK)) && (ct->mark & URLBLOCK_MARK)) {
			ret = ipt_do_table(pskb, NF_IP_FORWARD, in, out, &packet_mangler, NULL);
                        if (ret == NF_DROP)
                                return NF_DROP;
                }
//#endif

                if (ct) {
                        match = ct->mark;
                        if ((match & POLICY_BM_MARK)) {
                                if((*pskb)->nfmark != ct->skbmark) {
                                        (*pskb)->nfmark = ct->skbmark;
                                        (*pskb)->nfcache |= NFC_ALTERED;
                                }
                        }
                }

		return ipt_do_table(pskb, hook, in, out, &packet_mangler, NULL);
	}
/* Steve -- */
/* zhangzh 2003/08/08 */
//#ifdef BANDWIDTH
        else if (hook == NF_IP_POST_ROUTING) {
		unsigned long long match;

                if (ct && (ct->mark & (QUOTA_MARK | INTERNAL_AUTH_MARK)) && (ct->mark & URLBLOCK_MARK)) {
			ret = ipt_do_table(pskb, NF_IP_FORWARD, in, out, &packet_mangler, NULL);
                        if (ret == NF_DROP)
                                return NF_DROP;
                }

                if (ct) {
                        match = ct->mark;
                        if ((match & POLICY_BM_MARK)) {
                                if((*pskb)->nfmark != ct->skbmark) {
                                        (*pskb)->nfmark = ct->skbmark;
                                        (*pskb)->nfcache |= NFC_ALTERED;
                                }
                        }
                }

                return ipt_do_table(pskb, hook, in, out, &packet_mangler, NULL);
	}
//#endif
	return NF_ACCEPT;
}

/* FIXME: change in oif may mean change in hh_len.  Check and realloc
   --RR */
static int
route_me_harder(struct sk_buff *skb)
{
	struct iphdr *iph = skb->nh.iph;
	struct rtable *rt;
/* 2002/8/29 Steve ++ */
        enum ip_conntrack_info ctinfo;
        struct ip_conntrack *ct = ip_conntrack_get(skb, &ctinfo);
/* Steve -- */
	struct rt_key key = { dst:iph->daddr,
			      src:iph->saddr,
			      oif:skb->sk ? skb->sk->bound_dev_if : 0,
			      tos:RT_TOS(iph->tos)|RTO_CONN,
#ifdef CONFIG_IP_ROUTE_FWMARK
/* 2002/8/29 Steve ++ */
//			      fwmark:skb->nfmark
			      fwmark:ct ? ct->routemark : 0
/* Steve -- */
#endif
			    };

	if (ip_route_output_key(&rt, &key) != 0) {
		printk("route_me_harder: No more route.\n");
		return -EINVAL;
	}

	/* Drop old route. */
	dst_release(skb->dst);

	skb->dst = &rt->u.dst;
	return 0;
}

static unsigned int
ipt_local_hook( unsigned int hook,
		struct sk_buff **pskb,
		const struct net_device *in,
		const struct net_device *out,
		int (*okfn)(struct sk_buff *))
{
	unsigned int ret;
	u_int8_t tos;
	u_int32_t saddr, daddr;
	unsigned long nfmark;
/* 2003/1/2 Steve ++ */
        enum ip_conntrack_info ctinfo;
        struct ip_conntrack *ct = ip_conntrack_get((struct sk_buff *)*pskb, &ctinfo);
/* Steve -- */
/* 2005/1/23 Steve ++ */
        struct sock *sk = (*pskb)->sk;

        if (ctinfo == IP_CT_NEW) {
                if (sk && sk->routemark && ct && !ct->routemark) {
                        ct->routemark = sk->routemark;
                }
        }
/* Steve -- */

	/* root is playing with raw sockets. */
	if ((*pskb)->len < sizeof(struct iphdr)
	    || (*pskb)->nh.iph->ihl * 4 < sizeof(struct iphdr)) {
		if (net_ratelimit())
			printk("ipt_hook: happy cracking.\n");
		return NF_ACCEPT;
	}

	/* Save things which could affect route */
	nfmark = (*pskb)->nfmark;
	saddr = (*pskb)->nh.iph->saddr;
	daddr = (*pskb)->nh.iph->daddr;
	tos = (*pskb)->nh.iph->tos;

	ret = ipt_do_table(pskb, hook, in, out, &packet_mangler, NULL);

/* 2002/1/2 Steve ++ 
 * Update TCPMSS options
 */
		update_tcpmss(pskb, ct);
/* Steve -- */

/* 2003/7/31 Steve ++
 * Log WAN port connection information
 */
        if (ct && !ct->indexwan && (ct->mark & URLBLOCK_MARK)) {
                unsigned int wanindex = 0;

                wanindex = ((ct->mark >> WAN_PORT_OFFSET) & 0xf);
                if (wanindex > 0 && wanindex <= MAX_WAN_PORT) {
                        ct->indexwan = 1;
                        wan_connection_num[wanindex-1]++;
                }
        }
/* Steve -- */

	/* Reroute for ANY change. */
	if (ret != NF_DROP && ret != NF_STOLEN && ret != NF_QUEUE
	    && ((*pskb)->nh.iph->saddr != saddr
		|| (*pskb)->nh.iph->daddr != daddr
		//		|| (*pskb)->nfmark != nfmark
		|| (*pskb)->nh.iph->tos != tos))
		return route_me_harder(*pskb) == 0 ? ret : NF_DROP;
	return ret;
}

/* 2002/12/11 Steve ++ */
static int ip_conntrack_update_pmtu(struct file *file, const char *input, unsigned long size, void *data)
{
	unsigned int mtu, len;
	char buffer[1024], *end;

	if (size > 1024)
		len = 1024;
	else
		len = size;

        READ_LOCK(&ip_conntrack_lock);
	if (copy_from_user(buffer, input, len)) {
		READ_UNLOCK(&ip_conntrack_lock);
		return -EFAULT;
	}
	buffer[len - 1] = '\0';
	mtu = simple_strtoul(buffer, &end, 10);
	if (mtu > 1500 || mtu <= 0)
		goto finish;

	pmtu = mtu;

finish:
        READ_UNLOCK(&ip_conntrack_lock);
        return len;
}

static int ip_conntrack_get_pmtu(char *buffer, char **start, off_t offset, int length)
{
        int size;
        unsigned int len = 0;
        off_t pos = 0;

	READ_LOCK(&ip_conntrack_lock);
	size = sprintf(buffer, "%lu", pmtu);
	pos += size;
	len += size;
	if (pos <= offset)
		len = 0;
	*start = buffer + len - (pos - offset);
	len = pos - offset;
	if (len > length)
		len = length;
	if (len < 0)
		len = 0;
	READ_UNLOCK(&ip_conntrack_lock);
	return len;
}
/* Steve -- */

static struct nf_hook_ops ipt_ops[]
= { 
    { { NULL, NULL }, ipt_route_hook, PF_INET, NF_IP_PRE_ROUTING, NF_IP_PRI_MANGLE },
    { { NULL, NULL }, ipt_local_hook, PF_INET, NF_IP_LOCAL_IN,		NF_IP_PRI_MANGLE },
/* 2002/2/22 Steve ++
 * Change FORWARD priority.
 * Let its lower than filter table.
 */
#if 0
    { { NULL, NULL }, ipt_route_hook, PF_INET, NF_IP_FORWARD,		NF_IP_PRI_MANGLE },
#endif
    { { NULL, NULL }, ipt_route_hook, PF_INET, NF_IP_FORWARD, NF_IP_PRI_MANGLE_FILTER},
#if 0
    { { NULL, NULL }, ipt_local_hook, PF_INET, NF_IP_LOCAL_OUT,		NF_IP_PRI_MANGLE },
#endif
    { { NULL, NULL }, ipt_local_hook, PF_INET, NF_IP_LOCAL_OUT, NF_IP_PRI_MANGLE_FILTER },
/* Steve -- */
    { { NULL, NULL }, ipt_route_hook, PF_INET, NF_IP_POST_ROUTING,	NF_IP_PRI_MANGLE }
};

static int __init init(void)
{
	int ret;
/* 2002/12/11 Steve ++ */
	struct proc_dir_entry *proc;

	proc = proc_net_create("ip_conntrack_pmtu", 0,
				ip_conntrack_get_pmtu);
	if (proc) {
		proc->owner = THIS_MODULE;
		proc->write_proc = ip_conntrack_update_pmtu;
	}
	pmtu = 1500;
/* Steve -- */
/* 2003/1/28 Steve ++ */
	if (!(ipfwsk = netlink_kernel_create(NETLINK_FIREWALL, NULL)))
		return -EINVAL;
/* Steve -- */
/* 2004/12/29 Steve ++ */
        if (!(iplogsk = netlink_kernel_create(NETLINK_NFLOG, NULL)))
                return -EINVAL;
/* Steve -- */

	/* Register table */
	ret = ipt_register_table(&packet_mangler);
	if (ret < 0)
		return ret;

	/* Register hooks */
	ret = nf_register_hook(&ipt_ops[0]);
	if (ret < 0)
		goto cleanup_table;

	ret = nf_register_hook(&ipt_ops[1]);
	if (ret < 0)
		goto cleanup_hook0;

	ret = nf_register_hook(&ipt_ops[2]);
	if (ret < 0)
		goto cleanup_hook1;

	ret = nf_register_hook(&ipt_ops[3]);
	if (ret < 0)
		goto cleanup_hook2;

	ret = nf_register_hook(&ipt_ops[4]);
	if (ret < 0)
		goto cleanup_hook3;

/* 2004/9/22 Steve ++ */
        sprintf(landev, "eth0");
        sprintf(dmzdev, "eth%d", (WAN_PORT+1));
/* Steve -- */

	return ret;

 cleanup_hook3:
	nf_unregister_hook(&ipt_ops[3]);
 cleanup_hook2:
	nf_unregister_hook(&ipt_ops[2]);
 cleanup_hook1:
	nf_unregister_hook(&ipt_ops[1]);
 cleanup_hook0:
	nf_unregister_hook(&ipt_ops[0]);
 cleanup_table:
	ipt_unregister_table(&packet_mangler);

	return ret;
}

static void __exit fini(void)
{
	unsigned int i;

/* 2003/1/28 Steve ++ */
	if(ipfwsk->socket) sock_release(ipfwsk->socket);
/* Steve -- */
/* 2004/12/29 Steve ++ */
        if (iplogsk->socket) sock_release(iplogsk->socket);
/* Steve -- */

	for (i = 0; i < sizeof(ipt_ops)/sizeof(struct nf_hook_ops); i++)
		nf_unregister_hook(&ipt_ops[i]);

	ipt_unregister_table(&packet_mangler);
}

module_init(init);
module_exit(fini);
MODULE_LICENSE("GPL");
